# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

require 'oci/common'
require 'oci/auth/auth'

# Top level module for the Oracle Cloud Infrastructure SDK
module OCI
  autoload :AiVision, 'oci/ai_vision/ai_vision'
  autoload :Budget, 'oci/budget/budget'
  autoload :DatabaseMigration, 'oci/database_migration/database_migration'
  autoload :DataLabelingServiceDataplane, 'oci/data_labeling_service_dataplane/data_labeling_service_dataplane'
  autoload :OperatorAccessControl, 'oci/operator_access_control/operator_access_control'
  autoload :Analytics, 'oci/analytics/analytics'
  autoload :Vault, 'oci/vault/vault'
  autoload :Waas, 'oci/waas/waas'
  autoload :ApmControlPlane, 'oci/apm_control_plane/apm_control_plane'
  autoload :AiSpeech, 'oci/ai_speech/ai_speech'
  autoload :Apigateway, 'oci/apigateway/apigateway'
  autoload :Oce, 'oci/oce/oce'
  autoload :Email, 'oci/email/email'
  autoload :Logging, 'oci/logging/logging'
  autoload :Nosql, 'oci/nosql/nosql'
  autoload :Rover, 'oci/rover/rover'
  autoload :ApmSynthetics, 'oci/apm_synthetics/apm_synthetics'
  autoload :Dns, 'oci/dns/dns'
  autoload :OspGateway, 'oci/osp_gateway/osp_gateway'
  autoload :LicenseManager, 'oci/license_manager/license_manager'
  autoload :Jms, 'oci/jms/jms'
  autoload :Autoscaling, 'oci/autoscaling/autoscaling'
  autoload :Events, 'oci/events/events'
  autoload :Monitoring, 'oci/monitoring/monitoring'
  autoload :AiLanguage, 'oci/ai_language/ai_language'
  autoload :ObjectStorage, 'oci/object_storage/object_storage'
  autoload :Loggingsearch, 'oci/loggingsearch/loggingsearch'
  autoload :ResourceSearch, 'oci/resource_search/resource_search'
  autoload :DataIntegration, 'oci/data_integration/data_integration'
  autoload :DatabaseManagement, 'oci/database_management/database_management'
  autoload :Opsi, 'oci/opsi/opsi'
  autoload :ServiceMesh, 'oci/service_mesh/service_mesh'
  autoload :Functions, 'oci/functions/functions'
  autoload :Waf, 'oci/waf/waf'
  autoload :ApmTraces, 'oci/apm_traces/apm_traces'
  autoload :OsubSubscription, 'oci/osub_subscription/osub_subscription'
  autoload :DatabaseTools, 'oci/database_tools/database_tools'
  autoload :Mysql, 'oci/mysql/mysql'
  autoload :GoldenGate, 'oci/golden_gate/golden_gate'
  autoload :Adm, 'oci/adm/adm'
  autoload :NetworkLoadBalancer, 'oci/network_load_balancer/network_load_balancer'
  autoload :Audit, 'oci/audit/audit'
  autoload :LogAnalytics, 'oci/log_analytics/log_analytics'
  autoload :DataFlow, 'oci/data_flow/data_flow'
  autoload :ApmConfig, 'oci/apm_config/apm_config'
  autoload :UsageApi, 'oci/usage_api/usage_api'
  autoload :AppmgmtControl, 'oci/appmgmt_control/appmgmt_control'
  autoload :ThreatIntelligence, 'oci/threat_intelligence/threat_intelligence'
  autoload :Blockchain, 'oci/blockchain/blockchain'
  autoload :ComputeInstanceAgent, 'oci/compute_instance_agent/compute_instance_agent'
  autoload :KeyManagement, 'oci/key_management/key_management'
  autoload :Streaming, 'oci/streaming/streaming'
  autoload :OsubOrganizationSubscription, 'oci/osub_organization_subscription/osub_organization_subscription'
  autoload :ServiceManagerProxy, 'oci/service_manager_proxy/service_manager_proxy'
  autoload :StackMonitoring, 'oci/stack_monitoring/stack_monitoring'
  autoload :ResourceManager, 'oci/resource_manager/resource_manager'
  autoload :VisualBuilder, 'oci/visual_builder/visual_builder'
  autoload :Certificates, 'oci/certificates/certificates'
  autoload :OsubBillingSchedule, 'oci/osub_billing_schedule/osub_billing_schedule'
  autoload :AnnouncementsService, 'oci/announcements_service/announcements_service'
  autoload :Usage, 'oci/usage/usage'
  autoload :AiAnomalyDetection, 'oci/ai_anomaly_detection/ai_anomaly_detection'
  autoload :ContainerEngine, 'oci/container_engine/container_engine'
  autoload :Cims, 'oci/cims/cims'
  autoload :Limits, 'oci/limits/limits'
  autoload :ManagementAgent, 'oci/management_agent/management_agent'
  autoload :Internal, 'oci/internal/internal'
  autoload :WorkRequests, 'oci/work_requests/work_requests'
  autoload :DataSafe, 'oci/data_safe/data_safe'
  autoload :ApplicationMigration, 'oci/application_migration/application_migration'
  autoload :DataLabelingService, 'oci/data_labeling_service/data_labeling_service'
  autoload :Marketplace, 'oci/marketplace/marketplace'
  autoload :DataScience, 'oci/data_science/data_science'
  autoload :FileStorage, 'oci/file_storage/file_storage'
  autoload :ServiceCatalog, 'oci/service_catalog/service_catalog'
  autoload :DataCatalog, 'oci/data_catalog/data_catalog'
  autoload :Secrets, 'oci/secrets/secrets'
  autoload :Sch, 'oci/sch/sch'
  autoload :Integration, 'oci/integration/integration'
  autoload :Bastion, 'oci/bastion/bastion'
  autoload :CloudGuard, 'oci/cloud_guard/cloud_guard'
  autoload :TenantManagerControlPlane, 'oci/tenant_manager_control_plane/tenant_manager_control_plane'
  autoload :Ocvp, 'oci/ocvp/ocvp'
  autoload :Bds, 'oci/bds/bds'
  autoload :Auth, 'oci/auth/auth'
  autoload :Optimizer, 'oci/optimizer/optimizer'
  autoload :Healthchecks, 'oci/healthchecks/healthchecks'
  autoload :Retry, 'oci/retry/retry'
  autoload :Identity, 'oci/identity/identity'
  autoload :Core, 'oci/core/core'
  autoload :DashboardService, 'oci/dashboard_service/dashboard_service'
  autoload :GenericArtifactsContent, 'oci/generic_artifacts_content/generic_artifacts_content'
  autoload :LoadBalancer, 'oci/load_balancer/load_balancer'
  autoload :Ons, 'oci/ons/ons'
  autoload :Oda, 'oci/oda/oda'
  autoload :Devops, 'oci/devops/devops'
  autoload :Loggingingestion, 'oci/loggingingestion/loggingingestion'
  autoload :Artifacts, 'oci/artifacts/artifacts'
  autoload :IdentityDataPlane, 'oci/identity_data_plane/identity_data_plane'
  autoload :ManagementDashboard, 'oci/management_dashboard/management_dashboard'
  autoload :Database, 'oci/database/database'
  autoload :Dts, 'oci/dts/dts'
  autoload :OsubUsage, 'oci/osub_usage/osub_usage'
  autoload :OsManagement, 'oci/os_management/os_management'
  autoload :VulnerabilityScanning, 'oci/vulnerability_scanning/vulnerability_scanning'
  autoload :CertificatesManagement, 'oci/certificates_management/certificates_management'
  autoload :GovernanceRulesControlPlane, 'oci/governance_rules_control_plane/governance_rules_control_plane'
  autoload :Waa, 'oci/waa/waa'
  autoload :Onesubscription, 'oci/onesubscription/onesubscription'
  autoload :NetworkFirewall, 'oci/network_firewall/network_firewall'
  autoload :EmWarehouse, 'oci/em_warehouse/em_warehouse'
  autoload :VnMonitoring, 'oci/vn_monitoring/vn_monitoring'
  autoload :Lockbox, 'oci/lockbox/lockbox'
  autoload :MediaServices, 'oci/media_services/media_services'
  autoload :Opa, 'oci/opa/opa'
  autoload :FusionApps, 'oci/fusion_apps/fusion_apps'
  autoload :Opensearch, 'oci/opensearch/opensearch'
  autoload :CloudMigrations, 'oci/cloud_migrations/cloud_migrations'
  autoload :CloudBridge, 'oci/cloud_bridge/cloud_bridge'
  autoload :DisasterRecovery, 'oci/disaster_recovery/disaster_recovery'
  autoload :ContainerInstances, 'oci/container_instances/container_instances'
  autoload :AiDocument, 'oci/ai_document/ai_document'
  autoload :Queue, 'oci/queue/queue'
  autoload :VbsInst, 'oci/vbs_inst/vbs_inst'
  autoload :Recovery, 'oci/recovery/recovery'
  autoload :IdentityDomains, 'oci/identity_domains/identity_domains'
  autoload :AccessGovernanceCp, 'oci/access_governance_cp/access_governance_cp'
  autoload :OciControlCenter, 'oci/oci_control_center/oci_control_center'
  autoload :OsManagementHub, 'oci/os_management_hub/os_management_hub'
  autoload :FleetSoftwareUpdate, 'oci/fleet_software_update/fleet_software_update'
  autoload :ComputeCloudAtCustomer, 'oci/compute_cloud_at_customer/compute_cloud_at_customer'
  autoload :Redis, 'oci/redis/redis'
  autoload :MarketplacePublisher, 'oci/marketplace_publisher/marketplace_publisher'
  autoload :JmsJavaDownloads, 'oci/jms_java_downloads/jms_java_downloads'
  autoload :Psql, 'oci/psql/psql'

  require 'oci/object_storage/transfer/transfer'
  class << self
  end
end

OracleBMC = OCI
