# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20210101
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Base class for the DetectAnomalies async call. It contains the identifier that is
  # used for deciding what type of request this is.
  #
  class AiAnomalyDetection::Models::CreateDetectAnomalyJobDetails
    # **[Required]** The OCID of the compartment that starts the job.
    # @return [String]
    attr_accessor :compartment_id

    # A short description of the detect anomaly job.
    # @return [String]
    attr_accessor :description

    # Detect anomaly job display name.
    # @return [String]
    attr_accessor :display_name

    # **[Required]** The OCID of the trained model.
    # @return [String]
    attr_accessor :model_id

    # The value that customer can adjust to control the sensitivity of anomaly detection
    # @return [Float]
    attr_accessor :sensitivity

    # Flag to enable the service to return estimates for all data points rather than just the anomalous data points.
    # @return [BOOLEAN]
    attr_accessor :are_all_estimates_required

    # This attribute is required.
    # @return [OCI::AiAnomalyDetection::Models::InputDetails]
    attr_accessor :input_details

    # This attribute is required.
    # @return [OCI::AiAnomalyDetection::Models::OutputDetails]
    attr_accessor :output_details

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'compartment_id': :'compartmentId',
        'description': :'description',
        'display_name': :'displayName',
        'model_id': :'modelId',
        'sensitivity': :'sensitivity',
        'are_all_estimates_required': :'areAllEstimatesRequired',
        'input_details': :'inputDetails',
        'output_details': :'outputDetails'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'compartment_id': :'String',
        'description': :'String',
        'display_name': :'String',
        'model_id': :'String',
        'sensitivity': :'Float',
        'are_all_estimates_required': :'BOOLEAN',
        'input_details': :'OCI::AiAnomalyDetection::Models::InputDetails',
        'output_details': :'OCI::AiAnomalyDetection::Models::OutputDetails'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :compartment_id The value to assign to the {#compartment_id} property
    # @option attributes [String] :description The value to assign to the {#description} property
    # @option attributes [String] :display_name The value to assign to the {#display_name} property
    # @option attributes [String] :model_id The value to assign to the {#model_id} property
    # @option attributes [Float] :sensitivity The value to assign to the {#sensitivity} property
    # @option attributes [BOOLEAN] :are_all_estimates_required The value to assign to the {#are_all_estimates_required} property
    # @option attributes [OCI::AiAnomalyDetection::Models::InputDetails] :input_details The value to assign to the {#input_details} property
    # @option attributes [OCI::AiAnomalyDetection::Models::OutputDetails] :output_details The value to assign to the {#output_details} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.compartment_id = attributes[:'compartmentId'] if attributes[:'compartmentId']

      raise 'You cannot provide both :compartmentId and :compartment_id' if attributes.key?(:'compartmentId') && attributes.key?(:'compartment_id')

      self.compartment_id = attributes[:'compartment_id'] if attributes[:'compartment_id']

      self.description = attributes[:'description'] if attributes[:'description']

      self.display_name = attributes[:'displayName'] if attributes[:'displayName']

      raise 'You cannot provide both :displayName and :display_name' if attributes.key?(:'displayName') && attributes.key?(:'display_name')

      self.display_name = attributes[:'display_name'] if attributes[:'display_name']

      self.model_id = attributes[:'modelId'] if attributes[:'modelId']

      raise 'You cannot provide both :modelId and :model_id' if attributes.key?(:'modelId') && attributes.key?(:'model_id')

      self.model_id = attributes[:'model_id'] if attributes[:'model_id']

      self.sensitivity = attributes[:'sensitivity'] if attributes[:'sensitivity']

      self.are_all_estimates_required = attributes[:'areAllEstimatesRequired'] unless attributes[:'areAllEstimatesRequired'].nil?

      raise 'You cannot provide both :areAllEstimatesRequired and :are_all_estimates_required' if attributes.key?(:'areAllEstimatesRequired') && attributes.key?(:'are_all_estimates_required')

      self.are_all_estimates_required = attributes[:'are_all_estimates_required'] unless attributes[:'are_all_estimates_required'].nil?

      self.input_details = attributes[:'inputDetails'] if attributes[:'inputDetails']

      raise 'You cannot provide both :inputDetails and :input_details' if attributes.key?(:'inputDetails') && attributes.key?(:'input_details')

      self.input_details = attributes[:'input_details'] if attributes[:'input_details']

      self.output_details = attributes[:'outputDetails'] if attributes[:'outputDetails']

      raise 'You cannot provide both :outputDetails and :output_details' if attributes.key?(:'outputDetails') && attributes.key?(:'output_details')

      self.output_details = attributes[:'output_details'] if attributes[:'output_details']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        compartment_id == other.compartment_id &&
        description == other.description &&
        display_name == other.display_name &&
        model_id == other.model_id &&
        sensitivity == other.sensitivity &&
        are_all_estimates_required == other.are_all_estimates_required &&
        input_details == other.input_details &&
        output_details == other.output_details
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [compartment_id, description, display_name, model_id, sensitivity, are_all_estimates_required, input_details, output_details].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
