# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20221001
require 'date'
require_relative 'evaluation_result_summary'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Possible NER model error analysis
  class AiLanguage::Models::NamedEntityRecognitionEvaluationResult < AiLanguage::Models::EvaluationResultSummary
    # **[Required]** For CSV format location is rowId(1 is header) and for JSONL location is jsonL line sequence(1 is metadata)
    # @return [String]
    attr_accessor :record

    # **[Required]** List of true(actual) entities in test data for NER model
    # @return [Array<OCI::AiLanguage::Models::EntityLabelErrorAnalysis>]
    attr_accessor :true_entities

    # **[Required]** List of true(actual) entities in test data for NER model
    # @return [Array<OCI::AiLanguage::Models::EntityLabelErrorAnalysis>]
    attr_accessor :predicted_entities

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'model_type': :'modelType',
        'freeform_tags': :'freeformTags',
        'defined_tags': :'definedTags',
        'record': :'record',
        'true_entities': :'trueEntities',
        'predicted_entities': :'predictedEntities'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'model_type': :'String',
        'freeform_tags': :'Hash<String, String>',
        'defined_tags': :'Hash<String, Hash<String, Object>>',
        'record': :'String',
        'true_entities': :'Array<OCI::AiLanguage::Models::EntityLabelErrorAnalysis>',
        'predicted_entities': :'Array<OCI::AiLanguage::Models::EntityLabelErrorAnalysis>'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [Hash<String, String>] :freeform_tags The value to assign to the {OCI::AiLanguage::Models::EvaluationResultSummary#freeform_tags #freeform_tags} proprety
    # @option attributes [Hash<String, Hash<String, Object>>] :defined_tags The value to assign to the {OCI::AiLanguage::Models::EvaluationResultSummary#defined_tags #defined_tags} proprety
    # @option attributes [String] :record The value to assign to the {#record} property
    # @option attributes [Array<OCI::AiLanguage::Models::EntityLabelErrorAnalysis>] :true_entities The value to assign to the {#true_entities} property
    # @option attributes [Array<OCI::AiLanguage::Models::EntityLabelErrorAnalysis>] :predicted_entities The value to assign to the {#predicted_entities} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      attributes['modelType'] = 'NAMED_ENTITY_RECOGNITION'

      super(attributes)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.record = attributes[:'record'] if attributes[:'record']

      self.true_entities = attributes[:'trueEntities'] if attributes[:'trueEntities']

      raise 'You cannot provide both :trueEntities and :true_entities' if attributes.key?(:'trueEntities') && attributes.key?(:'true_entities')

      self.true_entities = attributes[:'true_entities'] if attributes[:'true_entities']

      self.predicted_entities = attributes[:'predictedEntities'] if attributes[:'predictedEntities']

      raise 'You cannot provide both :predictedEntities and :predicted_entities' if attributes.key?(:'predictedEntities') && attributes.key?(:'predicted_entities')

      self.predicted_entities = attributes[:'predicted_entities'] if attributes[:'predicted_entities']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        model_type == other.model_type &&
        freeform_tags == other.freeform_tags &&
        defined_tags == other.defined_tags &&
        record == other.record &&
        true_entities == other.true_entities &&
        predicted_entities == other.predicted_entities
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [model_type, freeform_tags, defined_tags, record, true_entities, predicted_entities].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
