# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20221001
require 'date'
require_relative 'evaluation_results'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Named entity recognition model testing and evaluation results
  class AiLanguage::Models::NamedEntityRecognitionEvaluationResults < AiLanguage::Models::EvaluationResults
    # @return [OCI::AiLanguage::Models::NamedEntityRecognitionModelMetrics]
    attr_accessor :metrics

    # List of entity metrics
    # @return [Array<OCI::AiLanguage::Models::EntityMetrics>]
    attr_accessor :entity_metrics

    # class level confusion matrix
    # @return [Hash<String, OCI::AiLanguage::Models::ConfusionMatrixDetails>]
    attr_accessor :confusion_matrix

    # labels
    # @return [Array<String>]
    attr_accessor :labels

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'model_type': :'modelType',
        'metrics': :'metrics',
        'entity_metrics': :'entityMetrics',
        'confusion_matrix': :'confusionMatrix',
        'labels': :'labels'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'model_type': :'String',
        'metrics': :'OCI::AiLanguage::Models::NamedEntityRecognitionModelMetrics',
        'entity_metrics': :'Array<OCI::AiLanguage::Models::EntityMetrics>',
        'confusion_matrix': :'Hash<String, OCI::AiLanguage::Models::ConfusionMatrixDetails>',
        'labels': :'Array<String>'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [OCI::AiLanguage::Models::NamedEntityRecognitionModelMetrics] :metrics The value to assign to the {#metrics} property
    # @option attributes [Array<OCI::AiLanguage::Models::EntityMetrics>] :entity_metrics The value to assign to the {#entity_metrics} property
    # @option attributes [Hash<String, OCI::AiLanguage::Models::ConfusionMatrixDetails>] :confusion_matrix The value to assign to the {#confusion_matrix} property
    # @option attributes [Array<String>] :labels The value to assign to the {#labels} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      attributes['modelType'] = 'NAMED_ENTITY_RECOGNITION'

      super(attributes)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.metrics = attributes[:'metrics'] if attributes[:'metrics']

      self.entity_metrics = attributes[:'entityMetrics'] if attributes[:'entityMetrics']

      raise 'You cannot provide both :entityMetrics and :entity_metrics' if attributes.key?(:'entityMetrics') && attributes.key?(:'entity_metrics')

      self.entity_metrics = attributes[:'entity_metrics'] if attributes[:'entity_metrics']

      self.confusion_matrix = attributes[:'confusionMatrix'] if attributes[:'confusionMatrix']

      raise 'You cannot provide both :confusionMatrix and :confusion_matrix' if attributes.key?(:'confusionMatrix') && attributes.key?(:'confusion_matrix')

      self.confusion_matrix = attributes[:'confusion_matrix'] if attributes[:'confusion_matrix']

      self.labels = attributes[:'labels'] if attributes[:'labels']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        model_type == other.model_type &&
        metrics == other.metrics &&
        entity_metrics == other.entity_metrics &&
        confusion_matrix == other.confusion_matrix &&
        labels == other.labels
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [model_type, metrics, entity_metrics, confusion_matrix, labels].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
