# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20190531
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Time of day and vertical scaling configuration.
  class Bds::Models::TimeAndVerticalScalingConfig
    # Day/time recurrence (specified following RFC 5545) at which to trigger autoscaling action. Currently only WEEKLY frequency is supported. Days of the week are specified using BYDAY field. Time of the day is specified using BYHOUR and BYMINUTE fields. Other fields are not supported.
    #
    # @return [String]
    attr_accessor :time_recurrence

    # For nodes with [fixed compute shapes](https://docs.cloud.oracle.com/iaas/Content/bigdata/create-cluster.htm#cluster-plan-shape), this value is the desired shape of each node. This value is not used for nodes with flexible compute shapes.
    #
    # @return [String]
    attr_accessor :target_shape

    # For nodes with [flexible compute shapes](https://docs.cloud.oracle.com/iaas/Content/bigdata/create-cluster.htm#cluster-plan-shape), this value is the desired OCPUs count on each node. This value is not used for nodes with fixed compute shapes.
    #
    # @return [Integer]
    attr_accessor :target_ocpus_per_node

    # For nodes with [flexible compute shapes](https://docs.cloud.oracle.com/iaas/Content/bigdata/create-cluster.htm#cluster-plan-shape), this value is the desired memory in GBs on each node. This value is not used for nodes with fixed compute shapes.
    #
    # @return [Integer]
    attr_accessor :target_memory_per_node

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'time_recurrence': :'timeRecurrence',
        'target_shape': :'targetShape',
        'target_ocpus_per_node': :'targetOcpusPerNode',
        'target_memory_per_node': :'targetMemoryPerNode'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'time_recurrence': :'String',
        'target_shape': :'String',
        'target_ocpus_per_node': :'Integer',
        'target_memory_per_node': :'Integer'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :time_recurrence The value to assign to the {#time_recurrence} property
    # @option attributes [String] :target_shape The value to assign to the {#target_shape} property
    # @option attributes [Integer] :target_ocpus_per_node The value to assign to the {#target_ocpus_per_node} property
    # @option attributes [Integer] :target_memory_per_node The value to assign to the {#target_memory_per_node} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.time_recurrence = attributes[:'timeRecurrence'] if attributes[:'timeRecurrence']

      raise 'You cannot provide both :timeRecurrence and :time_recurrence' if attributes.key?(:'timeRecurrence') && attributes.key?(:'time_recurrence')

      self.time_recurrence = attributes[:'time_recurrence'] if attributes[:'time_recurrence']

      self.target_shape = attributes[:'targetShape'] if attributes[:'targetShape']

      raise 'You cannot provide both :targetShape and :target_shape' if attributes.key?(:'targetShape') && attributes.key?(:'target_shape')

      self.target_shape = attributes[:'target_shape'] if attributes[:'target_shape']

      self.target_ocpus_per_node = attributes[:'targetOcpusPerNode'] if attributes[:'targetOcpusPerNode']

      raise 'You cannot provide both :targetOcpusPerNode and :target_ocpus_per_node' if attributes.key?(:'targetOcpusPerNode') && attributes.key?(:'target_ocpus_per_node')

      self.target_ocpus_per_node = attributes[:'target_ocpus_per_node'] if attributes[:'target_ocpus_per_node']

      self.target_memory_per_node = attributes[:'targetMemoryPerNode'] if attributes[:'targetMemoryPerNode']

      raise 'You cannot provide both :targetMemoryPerNode and :target_memory_per_node' if attributes.key?(:'targetMemoryPerNode') && attributes.key?(:'target_memory_per_node')

      self.target_memory_per_node = attributes[:'target_memory_per_node'] if attributes[:'target_memory_per_node']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        time_recurrence == other.time_recurrence &&
        target_shape == other.target_shape &&
        target_ocpus_per_node == other.target_ocpus_per_node &&
        target_memory_per_node == other.target_memory_per_node
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [time_recurrence, target_shape, target_ocpus_per_node, target_memory_per_node].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
