# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20200131
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # The information about Event details of DataSource.
  class CloudGuard::Models::DataSourceEventSummary
    STATUS_ENUM = [
      STATUS_SUCCESS = 'SUCCESS'.freeze,
      STATUS_FAILURE = 'FAILURE'.freeze,
      STATUS_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** Data source event region
    # @return [String]
    attr_accessor :region

    # **[Required]** Data source event date time
    # @return [DateTime]
    attr_accessor :event_date

    # **[Required]** Attached data Source
    # @return [String]
    attr_accessor :data_source_id

    # **[Required]** Data source event created time
    # @return [DateTime]
    attr_accessor :time_created

    # Current data source event info status
    # @return [String]
    attr_reader :status

    # Data source event comments
    # @return [String]
    attr_accessor :comments

    # This attribute is required.
    # @return [OCI::CloudGuard::Models::DataSourceEventInfo]
    attr_accessor :event_info

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'region': :'region',
        'event_date': :'eventDate',
        'data_source_id': :'dataSourceId',
        'time_created': :'timeCreated',
        'status': :'status',
        'comments': :'comments',
        'event_info': :'eventInfo'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'region': :'String',
        'event_date': :'DateTime',
        'data_source_id': :'String',
        'time_created': :'DateTime',
        'status': :'String',
        'comments': :'String',
        'event_info': :'OCI::CloudGuard::Models::DataSourceEventInfo'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :region The value to assign to the {#region} property
    # @option attributes [DateTime] :event_date The value to assign to the {#event_date} property
    # @option attributes [String] :data_source_id The value to assign to the {#data_source_id} property
    # @option attributes [DateTime] :time_created The value to assign to the {#time_created} property
    # @option attributes [String] :status The value to assign to the {#status} property
    # @option attributes [String] :comments The value to assign to the {#comments} property
    # @option attributes [OCI::CloudGuard::Models::DataSourceEventInfo] :event_info The value to assign to the {#event_info} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.region = attributes[:'region'] if attributes[:'region']

      self.event_date = attributes[:'eventDate'] if attributes[:'eventDate']

      raise 'You cannot provide both :eventDate and :event_date' if attributes.key?(:'eventDate') && attributes.key?(:'event_date')

      self.event_date = attributes[:'event_date'] if attributes[:'event_date']

      self.data_source_id = attributes[:'dataSourceId'] if attributes[:'dataSourceId']

      raise 'You cannot provide both :dataSourceId and :data_source_id' if attributes.key?(:'dataSourceId') && attributes.key?(:'data_source_id')

      self.data_source_id = attributes[:'data_source_id'] if attributes[:'data_source_id']

      self.time_created = attributes[:'timeCreated'] if attributes[:'timeCreated']

      raise 'You cannot provide both :timeCreated and :time_created' if attributes.key?(:'timeCreated') && attributes.key?(:'time_created')

      self.time_created = attributes[:'time_created'] if attributes[:'time_created']

      self.status = attributes[:'status'] if attributes[:'status']

      self.comments = attributes[:'comments'] if attributes[:'comments']

      self.event_info = attributes[:'eventInfo'] if attributes[:'eventInfo']

      raise 'You cannot provide both :eventInfo and :event_info' if attributes.key?(:'eventInfo') && attributes.key?(:'event_info')

      self.event_info = attributes[:'event_info'] if attributes[:'event_info']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] status Object to be assigned
    def status=(status)
      # rubocop:disable Style/ConditionalAssignment
      if status && !STATUS_ENUM.include?(status)
        OCI.logger.debug("Unknown value for 'status' [" + status + "]. Mapping to 'STATUS_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @status = STATUS_UNKNOWN_ENUM_VALUE
      else
        @status = status
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        region == other.region &&
        event_date == other.event_date &&
        data_source_id == other.data_source_id &&
        time_created == other.time_created &&
        status == other.status &&
        comments == other.comments &&
        event_info == other.event_info
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [region, event_date, data_source_id, time_created, status, comments, event_info].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
