# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20220919
require 'date'
require 'logger'
require_relative 'resource_assessment_strategy'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # The strategy based on percentile usage.
  class CloudMigrations::Models::PercentileResourceAssessmentStrategy < CloudMigrations::Models::ResourceAssessmentStrategy
    PERCENTILE_ENUM = [
      PERCENTILE_P50 = 'P50'.freeze,
      PERCENTILE_P90 = 'P90'.freeze,
      PERCENTILE_P95 = 'P95'.freeze,
      PERCENTILE_P99 = 'P99'.freeze,
      PERCENTILE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    METRIC_TIME_WINDOW_ENUM = [
      METRIC_TIME_WINDOW_1D = '1d'.freeze,
      METRIC_TIME_WINDOW_7D = '7d'.freeze,
      METRIC_TIME_WINDOW_30D = '30d'.freeze,
      METRIC_TIME_WINDOW_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** Percentile value
    # @return [String]
    attr_reader :percentile

    # The real resource usage is multiplied to this number before making any recommendation.
    # @return [Float]
    attr_accessor :adjustment_multiplier

    # The current state of the migration plan.
    # @return [String]
    attr_reader :metric_time_window

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'resource_type': :'resourceType',
        'strategy_type': :'strategyType',
        'percentile': :'percentile',
        'adjustment_multiplier': :'adjustmentMultiplier',
        'metric_time_window': :'metricTimeWindow'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'resource_type': :'String',
        'strategy_type': :'String',
        'percentile': :'String',
        'adjustment_multiplier': :'Float',
        'metric_time_window': :'String'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :resource_type The value to assign to the {OCI::CloudMigrations::Models::ResourceAssessmentStrategy#resource_type #resource_type} proprety
    # @option attributes [String] :percentile The value to assign to the {#percentile} property
    # @option attributes [Float] :adjustment_multiplier The value to assign to the {#adjustment_multiplier} property
    # @option attributes [String] :metric_time_window The value to assign to the {#metric_time_window} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      attributes['strategyType'] = 'PERCENTILE'

      super(attributes)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.percentile = attributes[:'percentile'] if attributes[:'percentile']

      self.adjustment_multiplier = attributes[:'adjustmentMultiplier'] if attributes[:'adjustmentMultiplier']

      raise 'You cannot provide both :adjustmentMultiplier and :adjustment_multiplier' if attributes.key?(:'adjustmentMultiplier') && attributes.key?(:'adjustment_multiplier')

      self.adjustment_multiplier = attributes[:'adjustment_multiplier'] if attributes[:'adjustment_multiplier']

      self.metric_time_window = attributes[:'metricTimeWindow'] if attributes[:'metricTimeWindow']

      raise 'You cannot provide both :metricTimeWindow and :metric_time_window' if attributes.key?(:'metricTimeWindow') && attributes.key?(:'metric_time_window')

      self.metric_time_window = attributes[:'metric_time_window'] if attributes[:'metric_time_window']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] percentile Object to be assigned
    def percentile=(percentile)
      # rubocop:disable Style/ConditionalAssignment
      if percentile && !PERCENTILE_ENUM.include?(percentile)
        OCI.logger.debug("Unknown value for 'percentile' [" + percentile + "]. Mapping to 'PERCENTILE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @percentile = PERCENTILE_UNKNOWN_ENUM_VALUE
      else
        @percentile = percentile
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] metric_time_window Object to be assigned
    def metric_time_window=(metric_time_window)
      # rubocop:disable Style/ConditionalAssignment
      if metric_time_window && !METRIC_TIME_WINDOW_ENUM.include?(metric_time_window)
        OCI.logger.debug("Unknown value for 'metric_time_window' [" + metric_time_window + "]. Mapping to 'METRIC_TIME_WINDOW_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @metric_time_window = METRIC_TIME_WINDOW_UNKNOWN_ENUM_VALUE
      else
        @metric_time_window = metric_time_window
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        resource_type == other.resource_type &&
        strategy_type == other.strategy_type &&
        percentile == other.percentile &&
        adjustment_multiplier == other.adjustment_multiplier &&
        metric_time_window == other.metric_time_window
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [resource_type, strategy_type, percentile, adjustment_multiplier, metric_time_window].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
