# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20221208
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Dynamic routing information for the Compute Cloud@Customer infrastructure.
  #
  class ComputeCloudAtCustomer::Models::CccInfrastructureRoutingDynamicDetails
    BGP_TOPOLOGY_ENUM = [
      BGP_TOPOLOGY_TRIANGLE = 'TRIANGLE'.freeze,
      BGP_TOPOLOGY_SQUARE = 'SQUARE'.freeze,
      BGP_TOPOLOGY_MESH = 'MESH'.freeze,
      BGP_TOPOLOGY_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # The list of peer devices in the dynamic routing configuration.
    # @return [Array<OCI::ComputeCloudAtCustomer::Models::PeerInformation>]
    attr_accessor :peer_information

    # The Oracle Autonomous System Number (ASN) to control routing and exchange information
    # within the dynamic routing configuration.
    #
    # @return [Integer]
    attr_accessor :oracle_asn

    # The topology in use for the Border Gateway Protocol (BGP) configuration.
    #
    # @return [String]
    attr_reader :bgp_topology

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'peer_information': :'peerInformation',
        'oracle_asn': :'oracleAsn',
        'bgp_topology': :'bgpTopology'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'peer_information': :'Array<OCI::ComputeCloudAtCustomer::Models::PeerInformation>',
        'oracle_asn': :'Integer',
        'bgp_topology': :'String'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [Array<OCI::ComputeCloudAtCustomer::Models::PeerInformation>] :peer_information The value to assign to the {#peer_information} property
    # @option attributes [Integer] :oracle_asn The value to assign to the {#oracle_asn} property
    # @option attributes [String] :bgp_topology The value to assign to the {#bgp_topology} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.peer_information = attributes[:'peerInformation'] if attributes[:'peerInformation']

      raise 'You cannot provide both :peerInformation and :peer_information' if attributes.key?(:'peerInformation') && attributes.key?(:'peer_information')

      self.peer_information = attributes[:'peer_information'] if attributes[:'peer_information']

      self.oracle_asn = attributes[:'oracleAsn'] if attributes[:'oracleAsn']

      raise 'You cannot provide both :oracleAsn and :oracle_asn' if attributes.key?(:'oracleAsn') && attributes.key?(:'oracle_asn')

      self.oracle_asn = attributes[:'oracle_asn'] if attributes[:'oracle_asn']

      self.bgp_topology = attributes[:'bgpTopology'] if attributes[:'bgpTopology']
      self.bgp_topology = "TRIANGLE" if bgp_topology.nil? && !attributes.key?(:'bgpTopology') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :bgpTopology and :bgp_topology' if attributes.key?(:'bgpTopology') && attributes.key?(:'bgp_topology')

      self.bgp_topology = attributes[:'bgp_topology'] if attributes[:'bgp_topology']
      self.bgp_topology = "TRIANGLE" if bgp_topology.nil? && !attributes.key?(:'bgpTopology') && !attributes.key?(:'bgp_topology') # rubocop:disable Style/StringLiterals
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] bgp_topology Object to be assigned
    def bgp_topology=(bgp_topology)
      # rubocop:disable Style/ConditionalAssignment
      if bgp_topology && !BGP_TOPOLOGY_ENUM.include?(bgp_topology)
        OCI.logger.debug("Unknown value for 'bgp_topology' [" + bgp_topology + "]. Mapping to 'BGP_TOPOLOGY_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @bgp_topology = BGP_TOPOLOGY_UNKNOWN_ENUM_VALUE
      else
        @bgp_topology = bgp_topology
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        peer_information == other.peer_information &&
        oracle_asn == other.oracle_asn &&
        bgp_topology == other.bgp_topology
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [peer_information, oracle_asn, bgp_topology].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
