# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20210415
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Information to create a container instance.
  class ContainerInstances::Models::CreateContainerInstanceDetails
    # A user-friendly name. Does not have to be unique, and it's changeable. Avoid entering confidential information. If you don't provide a name, a name is generated automatically.
    #
    # @return [String]
    attr_accessor :display_name

    # **[Required]** The compartment OCID.
    # @return [String]
    attr_accessor :compartment_id

    # **[Required]** The availability domain where the container instance runs.
    # @return [String]
    attr_accessor :availability_domain

    # The fault domain where the container instance runs.
    #
    # @return [String]
    attr_accessor :fault_domain

    # **[Required]** The shape of the container instance. The shape determines the resources available to the container instance.
    # @return [String]
    attr_accessor :shape

    # This attribute is required.
    # @return [OCI::ContainerInstances::Models::CreateContainerInstanceShapeConfigDetails]
    attr_accessor :shape_config

    # A volume is a directory with data that is accessible across multiple containers in a
    # container instance.
    #
    # You can attach up to 32 volumes to single container instance.
    #
    # @return [Array<OCI::ContainerInstances::Models::CreateContainerVolumeDetails>]
    attr_accessor :volumes

    # **[Required]** The containers to create on this container instance.
    # @return [Array<OCI::ContainerInstances::Models::CreateContainerDetails>]
    attr_accessor :containers

    # **[Required]** The networks available to containers on this container instance.
    # @return [Array<OCI::ContainerInstances::Models::CreateContainerVnicDetails>]
    attr_accessor :vnics

    # @return [OCI::ContainerInstances::Models::CreateContainerDnsConfigDetails]
    attr_accessor :dns_config

    # The amount of time that processes in a container have to gracefully end when the container must be stopped. For example, when you delete a container instance. After the timeout is reached, the processes are sent a signal to be deleted.
    # @return [Integer]
    attr_accessor :graceful_shutdown_timeout_in_seconds

    # The image pulls secrets so you can access private registry to pull container images.
    # @return [Array<OCI::ContainerInstances::Models::CreateImagePullSecretDetails>]
    attr_accessor :image_pull_secrets

    # Container restart policy
    # @return [String]
    attr_accessor :container_restart_policy

    # Simple key-value pair that is applied without any predefined name, type or scope. Exists for cross-compatibility only.
    # Example: `{\"bar-key\": \"value\"}`
    #
    # @return [Hash<String, String>]
    attr_accessor :freeform_tags

    # Defined tags for this resource. Each key is predefined and scoped to a namespace.
    # Example: `{\"foo-namespace\": {\"bar-key\": \"value\"}}`.
    #
    # @return [Hash<String, Hash<String, Object>>]
    attr_accessor :defined_tags

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'display_name': :'displayName',
        'compartment_id': :'compartmentId',
        'availability_domain': :'availabilityDomain',
        'fault_domain': :'faultDomain',
        'shape': :'shape',
        'shape_config': :'shapeConfig',
        'volumes': :'volumes',
        'containers': :'containers',
        'vnics': :'vnics',
        'dns_config': :'dnsConfig',
        'graceful_shutdown_timeout_in_seconds': :'gracefulShutdownTimeoutInSeconds',
        'image_pull_secrets': :'imagePullSecrets',
        'container_restart_policy': :'containerRestartPolicy',
        'freeform_tags': :'freeformTags',
        'defined_tags': :'definedTags'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'display_name': :'String',
        'compartment_id': :'String',
        'availability_domain': :'String',
        'fault_domain': :'String',
        'shape': :'String',
        'shape_config': :'OCI::ContainerInstances::Models::CreateContainerInstanceShapeConfigDetails',
        'volumes': :'Array<OCI::ContainerInstances::Models::CreateContainerVolumeDetails>',
        'containers': :'Array<OCI::ContainerInstances::Models::CreateContainerDetails>',
        'vnics': :'Array<OCI::ContainerInstances::Models::CreateContainerVnicDetails>',
        'dns_config': :'OCI::ContainerInstances::Models::CreateContainerDnsConfigDetails',
        'graceful_shutdown_timeout_in_seconds': :'Integer',
        'image_pull_secrets': :'Array<OCI::ContainerInstances::Models::CreateImagePullSecretDetails>',
        'container_restart_policy': :'String',
        'freeform_tags': :'Hash<String, String>',
        'defined_tags': :'Hash<String, Hash<String, Object>>'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :display_name The value to assign to the {#display_name} property
    # @option attributes [String] :compartment_id The value to assign to the {#compartment_id} property
    # @option attributes [String] :availability_domain The value to assign to the {#availability_domain} property
    # @option attributes [String] :fault_domain The value to assign to the {#fault_domain} property
    # @option attributes [String] :shape The value to assign to the {#shape} property
    # @option attributes [OCI::ContainerInstances::Models::CreateContainerInstanceShapeConfigDetails] :shape_config The value to assign to the {#shape_config} property
    # @option attributes [Array<OCI::ContainerInstances::Models::CreateContainerVolumeDetails>] :volumes The value to assign to the {#volumes} property
    # @option attributes [Array<OCI::ContainerInstances::Models::CreateContainerDetails>] :containers The value to assign to the {#containers} property
    # @option attributes [Array<OCI::ContainerInstances::Models::CreateContainerVnicDetails>] :vnics The value to assign to the {#vnics} property
    # @option attributes [OCI::ContainerInstances::Models::CreateContainerDnsConfigDetails] :dns_config The value to assign to the {#dns_config} property
    # @option attributes [Integer] :graceful_shutdown_timeout_in_seconds The value to assign to the {#graceful_shutdown_timeout_in_seconds} property
    # @option attributes [Array<OCI::ContainerInstances::Models::CreateImagePullSecretDetails>] :image_pull_secrets The value to assign to the {#image_pull_secrets} property
    # @option attributes [String] :container_restart_policy The value to assign to the {#container_restart_policy} property
    # @option attributes [Hash<String, String>] :freeform_tags The value to assign to the {#freeform_tags} property
    # @option attributes [Hash<String, Hash<String, Object>>] :defined_tags The value to assign to the {#defined_tags} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.display_name = attributes[:'displayName'] if attributes[:'displayName']

      raise 'You cannot provide both :displayName and :display_name' if attributes.key?(:'displayName') && attributes.key?(:'display_name')

      self.display_name = attributes[:'display_name'] if attributes[:'display_name']

      self.compartment_id = attributes[:'compartmentId'] if attributes[:'compartmentId']

      raise 'You cannot provide both :compartmentId and :compartment_id' if attributes.key?(:'compartmentId') && attributes.key?(:'compartment_id')

      self.compartment_id = attributes[:'compartment_id'] if attributes[:'compartment_id']

      self.availability_domain = attributes[:'availabilityDomain'] if attributes[:'availabilityDomain']

      raise 'You cannot provide both :availabilityDomain and :availability_domain' if attributes.key?(:'availabilityDomain') && attributes.key?(:'availability_domain')

      self.availability_domain = attributes[:'availability_domain'] if attributes[:'availability_domain']

      self.fault_domain = attributes[:'faultDomain'] if attributes[:'faultDomain']

      raise 'You cannot provide both :faultDomain and :fault_domain' if attributes.key?(:'faultDomain') && attributes.key?(:'fault_domain')

      self.fault_domain = attributes[:'fault_domain'] if attributes[:'fault_domain']

      self.shape = attributes[:'shape'] if attributes[:'shape']

      self.shape_config = attributes[:'shapeConfig'] if attributes[:'shapeConfig']

      raise 'You cannot provide both :shapeConfig and :shape_config' if attributes.key?(:'shapeConfig') && attributes.key?(:'shape_config')

      self.shape_config = attributes[:'shape_config'] if attributes[:'shape_config']

      self.volumes = attributes[:'volumes'] if attributes[:'volumes']

      self.containers = attributes[:'containers'] if attributes[:'containers']

      self.vnics = attributes[:'vnics'] if attributes[:'vnics']

      self.dns_config = attributes[:'dnsConfig'] if attributes[:'dnsConfig']

      raise 'You cannot provide both :dnsConfig and :dns_config' if attributes.key?(:'dnsConfig') && attributes.key?(:'dns_config')

      self.dns_config = attributes[:'dns_config'] if attributes[:'dns_config']

      self.graceful_shutdown_timeout_in_seconds = attributes[:'gracefulShutdownTimeoutInSeconds'] if attributes[:'gracefulShutdownTimeoutInSeconds']
      self.graceful_shutdown_timeout_in_seconds = 0 if graceful_shutdown_timeout_in_seconds.nil? && !attributes.key?(:'gracefulShutdownTimeoutInSeconds') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :gracefulShutdownTimeoutInSeconds and :graceful_shutdown_timeout_in_seconds' if attributes.key?(:'gracefulShutdownTimeoutInSeconds') && attributes.key?(:'graceful_shutdown_timeout_in_seconds')

      self.graceful_shutdown_timeout_in_seconds = attributes[:'graceful_shutdown_timeout_in_seconds'] if attributes[:'graceful_shutdown_timeout_in_seconds']
      self.graceful_shutdown_timeout_in_seconds = 0 if graceful_shutdown_timeout_in_seconds.nil? && !attributes.key?(:'gracefulShutdownTimeoutInSeconds') && !attributes.key?(:'graceful_shutdown_timeout_in_seconds') # rubocop:disable Style/StringLiterals

      self.image_pull_secrets = attributes[:'imagePullSecrets'] if attributes[:'imagePullSecrets']

      raise 'You cannot provide both :imagePullSecrets and :image_pull_secrets' if attributes.key?(:'imagePullSecrets') && attributes.key?(:'image_pull_secrets')

      self.image_pull_secrets = attributes[:'image_pull_secrets'] if attributes[:'image_pull_secrets']

      self.container_restart_policy = attributes[:'containerRestartPolicy'] if attributes[:'containerRestartPolicy']
      self.container_restart_policy = "ALWAYS" if container_restart_policy.nil? && !attributes.key?(:'containerRestartPolicy') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :containerRestartPolicy and :container_restart_policy' if attributes.key?(:'containerRestartPolicy') && attributes.key?(:'container_restart_policy')

      self.container_restart_policy = attributes[:'container_restart_policy'] if attributes[:'container_restart_policy']
      self.container_restart_policy = "ALWAYS" if container_restart_policy.nil? && !attributes.key?(:'containerRestartPolicy') && !attributes.key?(:'container_restart_policy') # rubocop:disable Style/StringLiterals

      self.freeform_tags = attributes[:'freeformTags'] if attributes[:'freeformTags']

      raise 'You cannot provide both :freeformTags and :freeform_tags' if attributes.key?(:'freeformTags') && attributes.key?(:'freeform_tags')

      self.freeform_tags = attributes[:'freeform_tags'] if attributes[:'freeform_tags']

      self.defined_tags = attributes[:'definedTags'] if attributes[:'definedTags']

      raise 'You cannot provide both :definedTags and :defined_tags' if attributes.key?(:'definedTags') && attributes.key?(:'defined_tags')

      self.defined_tags = attributes[:'defined_tags'] if attributes[:'defined_tags']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        display_name == other.display_name &&
        compartment_id == other.compartment_id &&
        availability_domain == other.availability_domain &&
        fault_domain == other.fault_domain &&
        shape == other.shape &&
        shape_config == other.shape_config &&
        volumes == other.volumes &&
        containers == other.containers &&
        vnics == other.vnics &&
        dns_config == other.dns_config &&
        graceful_shutdown_timeout_in_seconds == other.graceful_shutdown_timeout_in_seconds &&
        image_pull_secrets == other.image_pull_secrets &&
        container_restart_policy == other.container_restart_policy &&
        freeform_tags == other.freeform_tags &&
        defined_tags == other.defined_tags
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [display_name, compartment_id, availability_domain, fault_domain, shape, shape_config, volumes, containers, vnics, dns_config, graceful_shutdown_timeout_in_seconds, image_pull_secrets, container_restart_policy, freeform_tags, defined_tags].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
