# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20200430
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Export metadata object response summary.
  class DataIntegration::Models::ExportRequestSummary
    STATUS_ENUM = [
      STATUS_SUCCESSFUL = 'SUCCESSFUL'.freeze,
      STATUS_FAILED = 'FAILED'.freeze,
      STATUS_IN_PROGRESS = 'IN_PROGRESS'.freeze,
      STATUS_TERMINATING = 'TERMINATING'.freeze,
      STATUS_TERMINATED = 'TERMINATED'.freeze,
      STATUS_QUEUED = 'QUEUED'.freeze,
      STATUS_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # Export object request key
    # @return [String]
    attr_accessor :key

    # The list of the objects to be exported
    # @return [Array<String>]
    attr_accessor :object_keys

    # The name of the Object Storage Bucket where the objects will be exported to
    # @return [String]
    attr_accessor :bucket_name

    # Name of the exported zip file.
    # @return [String]
    attr_accessor :file_name

    # Optional parameter to point to object storage tenancy (if using Object Storage of different tenancy)
    # @return [String]
    attr_accessor :object_storage_tenancy_id

    # Region of the object storage (if using object storage of different region)
    # @return [String]
    attr_accessor :object_storage_region

    # Controls if the references will be exported along with the objects
    # @return [BOOLEAN]
    attr_accessor :are_references_included

    # Flag to control whether to overwrite the object if it is already present at the provided object storage location.
    # @return [BOOLEAN]
    attr_accessor :is_object_overwrite_enabled

    # Export multiple objects based on filters.
    # @return [Array<String>]
    attr_accessor :filters

    # Export Objects request status.
    # @return [String]
    attr_reader :status

    # Name of the user who initiated export request.
    # @return [String]
    attr_accessor :created_by

    # Number of objects that are exported.
    # @return [Integer]
    attr_accessor :total_exported_object_count

    # Time at which the request started getting processed.
    # @return [Integer]
    attr_accessor :time_started_in_millis

    # Time at which the request was completely processed.
    # @return [Integer]
    attr_accessor :time_ended_in_millis

    # Contains key of the error
    # @return [Hash<String, String>]
    attr_accessor :error_messages

    # The array of exported object details.
    # @return [Array<OCI::DataIntegration::Models::ExportObjectMetadataSummary>]
    attr_accessor :exported_items

    # The array of exported referenced objects.
    # @return [String]
    attr_accessor :referenced_items

    # Name of the export request.
    # @return [String]
    attr_accessor :name

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'key': :'key',
        'object_keys': :'objectKeys',
        'bucket_name': :'bucketName',
        'file_name': :'fileName',
        'object_storage_tenancy_id': :'objectStorageTenancyId',
        'object_storage_region': :'objectStorageRegion',
        'are_references_included': :'areReferencesIncluded',
        'is_object_overwrite_enabled': :'isObjectOverwriteEnabled',
        'filters': :'filters',
        'status': :'status',
        'created_by': :'createdBy',
        'total_exported_object_count': :'totalExportedObjectCount',
        'time_started_in_millis': :'timeStartedInMillis',
        'time_ended_in_millis': :'timeEndedInMillis',
        'error_messages': :'errorMessages',
        'exported_items': :'exportedItems',
        'referenced_items': :'referencedItems',
        'name': :'name'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'key': :'String',
        'object_keys': :'Array<String>',
        'bucket_name': :'String',
        'file_name': :'String',
        'object_storage_tenancy_id': :'String',
        'object_storage_region': :'String',
        'are_references_included': :'BOOLEAN',
        'is_object_overwrite_enabled': :'BOOLEAN',
        'filters': :'Array<String>',
        'status': :'String',
        'created_by': :'String',
        'total_exported_object_count': :'Integer',
        'time_started_in_millis': :'Integer',
        'time_ended_in_millis': :'Integer',
        'error_messages': :'Hash<String, String>',
        'exported_items': :'Array<OCI::DataIntegration::Models::ExportObjectMetadataSummary>',
        'referenced_items': :'String',
        'name': :'String'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :key The value to assign to the {#key} property
    # @option attributes [Array<String>] :object_keys The value to assign to the {#object_keys} property
    # @option attributes [String] :bucket_name The value to assign to the {#bucket_name} property
    # @option attributes [String] :file_name The value to assign to the {#file_name} property
    # @option attributes [String] :object_storage_tenancy_id The value to assign to the {#object_storage_tenancy_id} property
    # @option attributes [String] :object_storage_region The value to assign to the {#object_storage_region} property
    # @option attributes [BOOLEAN] :are_references_included The value to assign to the {#are_references_included} property
    # @option attributes [BOOLEAN] :is_object_overwrite_enabled The value to assign to the {#is_object_overwrite_enabled} property
    # @option attributes [Array<String>] :filters The value to assign to the {#filters} property
    # @option attributes [String] :status The value to assign to the {#status} property
    # @option attributes [String] :created_by The value to assign to the {#created_by} property
    # @option attributes [Integer] :total_exported_object_count The value to assign to the {#total_exported_object_count} property
    # @option attributes [Integer] :time_started_in_millis The value to assign to the {#time_started_in_millis} property
    # @option attributes [Integer] :time_ended_in_millis The value to assign to the {#time_ended_in_millis} property
    # @option attributes [Hash<String, String>] :error_messages The value to assign to the {#error_messages} property
    # @option attributes [Array<OCI::DataIntegration::Models::ExportObjectMetadataSummary>] :exported_items The value to assign to the {#exported_items} property
    # @option attributes [String] :referenced_items The value to assign to the {#referenced_items} property
    # @option attributes [String] :name The value to assign to the {#name} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      self.key = attributes[:'key'] if attributes[:'key']

      self.object_keys = attributes[:'objectKeys'] if attributes[:'objectKeys']

      raise 'You cannot provide both :objectKeys and :object_keys' if attributes.key?(:'objectKeys') && attributes.key?(:'object_keys')

      self.object_keys = attributes[:'object_keys'] if attributes[:'object_keys']

      self.bucket_name = attributes[:'bucketName'] if attributes[:'bucketName']

      raise 'You cannot provide both :bucketName and :bucket_name' if attributes.key?(:'bucketName') && attributes.key?(:'bucket_name')

      self.bucket_name = attributes[:'bucket_name'] if attributes[:'bucket_name']

      self.file_name = attributes[:'fileName'] if attributes[:'fileName']

      raise 'You cannot provide both :fileName and :file_name' if attributes.key?(:'fileName') && attributes.key?(:'file_name')

      self.file_name = attributes[:'file_name'] if attributes[:'file_name']

      self.object_storage_tenancy_id = attributes[:'objectStorageTenancyId'] if attributes[:'objectStorageTenancyId']

      raise 'You cannot provide both :objectStorageTenancyId and :object_storage_tenancy_id' if attributes.key?(:'objectStorageTenancyId') && attributes.key?(:'object_storage_tenancy_id')

      self.object_storage_tenancy_id = attributes[:'object_storage_tenancy_id'] if attributes[:'object_storage_tenancy_id']

      self.object_storage_region = attributes[:'objectStorageRegion'] if attributes[:'objectStorageRegion']

      raise 'You cannot provide both :objectStorageRegion and :object_storage_region' if attributes.key?(:'objectStorageRegion') && attributes.key?(:'object_storage_region')

      self.object_storage_region = attributes[:'object_storage_region'] if attributes[:'object_storage_region']

      self.are_references_included = attributes[:'areReferencesIncluded'] unless attributes[:'areReferencesIncluded'].nil?
      self.are_references_included = true if are_references_included.nil? && !attributes.key?(:'areReferencesIncluded') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :areReferencesIncluded and :are_references_included' if attributes.key?(:'areReferencesIncluded') && attributes.key?(:'are_references_included')

      self.are_references_included = attributes[:'are_references_included'] unless attributes[:'are_references_included'].nil?
      self.are_references_included = true if are_references_included.nil? && !attributes.key?(:'areReferencesIncluded') && !attributes.key?(:'are_references_included') # rubocop:disable Style/StringLiterals

      self.is_object_overwrite_enabled = attributes[:'isObjectOverwriteEnabled'] unless attributes[:'isObjectOverwriteEnabled'].nil?

      raise 'You cannot provide both :isObjectOverwriteEnabled and :is_object_overwrite_enabled' if attributes.key?(:'isObjectOverwriteEnabled') && attributes.key?(:'is_object_overwrite_enabled')

      self.is_object_overwrite_enabled = attributes[:'is_object_overwrite_enabled'] unless attributes[:'is_object_overwrite_enabled'].nil?

      self.filters = attributes[:'filters'] if attributes[:'filters']

      self.status = attributes[:'status'] if attributes[:'status']

      self.created_by = attributes[:'createdBy'] if attributes[:'createdBy']

      raise 'You cannot provide both :createdBy and :created_by' if attributes.key?(:'createdBy') && attributes.key?(:'created_by')

      self.created_by = attributes[:'created_by'] if attributes[:'created_by']

      self.total_exported_object_count = attributes[:'totalExportedObjectCount'] if attributes[:'totalExportedObjectCount']

      raise 'You cannot provide both :totalExportedObjectCount and :total_exported_object_count' if attributes.key?(:'totalExportedObjectCount') && attributes.key?(:'total_exported_object_count')

      self.total_exported_object_count = attributes[:'total_exported_object_count'] if attributes[:'total_exported_object_count']

      self.time_started_in_millis = attributes[:'timeStartedInMillis'] if attributes[:'timeStartedInMillis']

      raise 'You cannot provide both :timeStartedInMillis and :time_started_in_millis' if attributes.key?(:'timeStartedInMillis') && attributes.key?(:'time_started_in_millis')

      self.time_started_in_millis = attributes[:'time_started_in_millis'] if attributes[:'time_started_in_millis']

      self.time_ended_in_millis = attributes[:'timeEndedInMillis'] if attributes[:'timeEndedInMillis']

      raise 'You cannot provide both :timeEndedInMillis and :time_ended_in_millis' if attributes.key?(:'timeEndedInMillis') && attributes.key?(:'time_ended_in_millis')

      self.time_ended_in_millis = attributes[:'time_ended_in_millis'] if attributes[:'time_ended_in_millis']

      self.error_messages = attributes[:'errorMessages'] if attributes[:'errorMessages']

      raise 'You cannot provide both :errorMessages and :error_messages' if attributes.key?(:'errorMessages') && attributes.key?(:'error_messages')

      self.error_messages = attributes[:'error_messages'] if attributes[:'error_messages']

      self.exported_items = attributes[:'exportedItems'] if attributes[:'exportedItems']

      raise 'You cannot provide both :exportedItems and :exported_items' if attributes.key?(:'exportedItems') && attributes.key?(:'exported_items')

      self.exported_items = attributes[:'exported_items'] if attributes[:'exported_items']

      self.referenced_items = attributes[:'referencedItems'] if attributes[:'referencedItems']

      raise 'You cannot provide both :referencedItems and :referenced_items' if attributes.key?(:'referencedItems') && attributes.key?(:'referenced_items')

      self.referenced_items = attributes[:'referenced_items'] if attributes[:'referenced_items']

      self.name = attributes[:'name'] if attributes[:'name']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] status Object to be assigned
    def status=(status)
      # rubocop:disable Style/ConditionalAssignment
      if status && !STATUS_ENUM.include?(status)
        OCI.logger.debug("Unknown value for 'status' [" + status + "]. Mapping to 'STATUS_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @status = STATUS_UNKNOWN_ENUM_VALUE
      else
        @status = status
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        key == other.key &&
        object_keys == other.object_keys &&
        bucket_name == other.bucket_name &&
        file_name == other.file_name &&
        object_storage_tenancy_id == other.object_storage_tenancy_id &&
        object_storage_region == other.object_storage_region &&
        are_references_included == other.are_references_included &&
        is_object_overwrite_enabled == other.is_object_overwrite_enabled &&
        filters == other.filters &&
        status == other.status &&
        created_by == other.created_by &&
        total_exported_object_count == other.total_exported_object_count &&
        time_started_in_millis == other.time_started_in_millis &&
        time_ended_in_millis == other.time_ended_in_millis &&
        error_messages == other.error_messages &&
        exported_items == other.exported_items &&
        referenced_items == other.referenced_items &&
        name == other.name
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [key, object_keys, bucket_name, file_name, object_storage_tenancy_id, object_storage_region, are_references_included, is_object_overwrite_enabled, filters, status, created_by, total_exported_object_count, time_started_in_millis, time_ended_in_millis, error_messages, exported_items, referenced_items, name].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
