# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20200430
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # The task lineage object provides information on the lineage information of a task after execution.
  class DataIntegration::Models::TaskRunLineageDetails
    TASK_EXECUTION_STATUS_ENUM = [
      TASK_EXECUTION_STATUS_SUCCESS = 'SUCCESS'.freeze,
      TASK_EXECUTION_STATUS_ERROR = 'ERROR'.freeze,
      TASK_EXECUTION_STATUS_TERMINATED = 'TERMINATED'.freeze
    ].freeze

    # The object key.
    # @return [String]
    attr_accessor :key

    # The object type.
    # @return [String]
    attr_accessor :model_type

    # The object's model version.
    # @return [String]
    attr_accessor :model_version

    # @return [OCI::DataIntegration::Models::ParentReference]
    attr_accessor :parent_ref

    # Free form text without any restriction on permitted characters. Name can have letters, numbers, and special characters. The value is editable and is restricted to 1000 characters.
    # @return [String]
    attr_accessor :name

    # Detailed description for the object.
    # @return [String]
    attr_accessor :description

    # The version of the object that is used to track changes in the object instance.
    # @return [Integer]
    attr_accessor :object_version

    # Task name
    # @return [String]
    attr_accessor :task_name

    # Task name
    # @return [String]
    attr_accessor :task_type

    # The object key.
    # @return [String]
    attr_accessor :task_key

    # This value is used to track if lineage generation for a task is completed or not.
    # @return [BOOLEAN]
    attr_accessor :is_lineage_gen_completed

    # The status of the task run.
    # @return [String]
    attr_reader :task_execution_status

    # @return [OCI::DataIntegration::Models::DataFlow]
    attr_accessor :flow

    # @return [OCI::DataIntegration::Models::ObjectMetadata]
    attr_accessor :metadata

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'key': :'key',
        'model_type': :'modelType',
        'model_version': :'modelVersion',
        'parent_ref': :'parentRef',
        'name': :'name',
        'description': :'description',
        'object_version': :'objectVersion',
        'task_name': :'taskName',
        'task_type': :'taskType',
        'task_key': :'taskKey',
        'is_lineage_gen_completed': :'isLineageGenCompleted',
        'task_execution_status': :'taskExecutionStatus',
        'flow': :'flow',
        'metadata': :'metadata'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'key': :'String',
        'model_type': :'String',
        'model_version': :'String',
        'parent_ref': :'OCI::DataIntegration::Models::ParentReference',
        'name': :'String',
        'description': :'String',
        'object_version': :'Integer',
        'task_name': :'String',
        'task_type': :'String',
        'task_key': :'String',
        'is_lineage_gen_completed': :'BOOLEAN',
        'task_execution_status': :'String',
        'flow': :'OCI::DataIntegration::Models::DataFlow',
        'metadata': :'OCI::DataIntegration::Models::ObjectMetadata'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :key The value to assign to the {#key} property
    # @option attributes [String] :model_type The value to assign to the {#model_type} property
    # @option attributes [String] :model_version The value to assign to the {#model_version} property
    # @option attributes [OCI::DataIntegration::Models::ParentReference] :parent_ref The value to assign to the {#parent_ref} property
    # @option attributes [String] :name The value to assign to the {#name} property
    # @option attributes [String] :description The value to assign to the {#description} property
    # @option attributes [Integer] :object_version The value to assign to the {#object_version} property
    # @option attributes [String] :task_name The value to assign to the {#task_name} property
    # @option attributes [String] :task_type The value to assign to the {#task_type} property
    # @option attributes [String] :task_key The value to assign to the {#task_key} property
    # @option attributes [BOOLEAN] :is_lineage_gen_completed The value to assign to the {#is_lineage_gen_completed} property
    # @option attributes [String] :task_execution_status The value to assign to the {#task_execution_status} property
    # @option attributes [OCI::DataIntegration::Models::DataFlow] :flow The value to assign to the {#flow} property
    # @option attributes [OCI::DataIntegration::Models::ObjectMetadata] :metadata The value to assign to the {#metadata} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.key = attributes[:'key'] if attributes[:'key']

      self.model_type = attributes[:'modelType'] if attributes[:'modelType']

      raise 'You cannot provide both :modelType and :model_type' if attributes.key?(:'modelType') && attributes.key?(:'model_type')

      self.model_type = attributes[:'model_type'] if attributes[:'model_type']

      self.model_version = attributes[:'modelVersion'] if attributes[:'modelVersion']

      raise 'You cannot provide both :modelVersion and :model_version' if attributes.key?(:'modelVersion') && attributes.key?(:'model_version')

      self.model_version = attributes[:'model_version'] if attributes[:'model_version']

      self.parent_ref = attributes[:'parentRef'] if attributes[:'parentRef']

      raise 'You cannot provide both :parentRef and :parent_ref' if attributes.key?(:'parentRef') && attributes.key?(:'parent_ref')

      self.parent_ref = attributes[:'parent_ref'] if attributes[:'parent_ref']

      self.name = attributes[:'name'] if attributes[:'name']

      self.description = attributes[:'description'] if attributes[:'description']

      self.object_version = attributes[:'objectVersion'] if attributes[:'objectVersion']

      raise 'You cannot provide both :objectVersion and :object_version' if attributes.key?(:'objectVersion') && attributes.key?(:'object_version')

      self.object_version = attributes[:'object_version'] if attributes[:'object_version']

      self.task_name = attributes[:'taskName'] if attributes[:'taskName']

      raise 'You cannot provide both :taskName and :task_name' if attributes.key?(:'taskName') && attributes.key?(:'task_name')

      self.task_name = attributes[:'task_name'] if attributes[:'task_name']

      self.task_type = attributes[:'taskType'] if attributes[:'taskType']

      raise 'You cannot provide both :taskType and :task_type' if attributes.key?(:'taskType') && attributes.key?(:'task_type')

      self.task_type = attributes[:'task_type'] if attributes[:'task_type']

      self.task_key = attributes[:'taskKey'] if attributes[:'taskKey']

      raise 'You cannot provide both :taskKey and :task_key' if attributes.key?(:'taskKey') && attributes.key?(:'task_key')

      self.task_key = attributes[:'task_key'] if attributes[:'task_key']

      self.is_lineage_gen_completed = attributes[:'isLineageGenCompleted'] unless attributes[:'isLineageGenCompleted'].nil?
      self.is_lineage_gen_completed = false if is_lineage_gen_completed.nil? && !attributes.key?(:'isLineageGenCompleted') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :isLineageGenCompleted and :is_lineage_gen_completed' if attributes.key?(:'isLineageGenCompleted') && attributes.key?(:'is_lineage_gen_completed')

      self.is_lineage_gen_completed = attributes[:'is_lineage_gen_completed'] unless attributes[:'is_lineage_gen_completed'].nil?
      self.is_lineage_gen_completed = false if is_lineage_gen_completed.nil? && !attributes.key?(:'isLineageGenCompleted') && !attributes.key?(:'is_lineage_gen_completed') # rubocop:disable Style/StringLiterals

      self.task_execution_status = attributes[:'taskExecutionStatus'] if attributes[:'taskExecutionStatus']

      raise 'You cannot provide both :taskExecutionStatus and :task_execution_status' if attributes.key?(:'taskExecutionStatus') && attributes.key?(:'task_execution_status')

      self.task_execution_status = attributes[:'task_execution_status'] if attributes[:'task_execution_status']

      self.flow = attributes[:'flow'] if attributes[:'flow']

      self.metadata = attributes[:'metadata'] if attributes[:'metadata']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] task_execution_status Object to be assigned
    def task_execution_status=(task_execution_status)
      raise "Invalid value for 'task_execution_status': this must be one of the values in TASK_EXECUTION_STATUS_ENUM." if task_execution_status && !TASK_EXECUTION_STATUS_ENUM.include?(task_execution_status)

      @task_execution_status = task_execution_status
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        key == other.key &&
        model_type == other.model_type &&
        model_version == other.model_version &&
        parent_ref == other.parent_ref &&
        name == other.name &&
        description == other.description &&
        object_version == other.object_version &&
        task_name == other.task_name &&
        task_type == other.task_type &&
        task_key == other.task_key &&
        is_lineage_gen_completed == other.is_lineage_gen_completed &&
        task_execution_status == other.task_execution_status &&
        flow == other.flow &&
        metadata == other.metadata
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [key, model_type, model_version, parent_ref, name, description, object_version, task_name, task_type, task_key, is_lineage_gen_completed, task_execution_status, flow, metadata].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
