# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20200430
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # A workspace is an organizational construct to keep multiple data integration solutions and their resources (data assets, data flows, tasks, and so on) separate from each other, helping you to stay organized. For example, you could have separate workspaces for development, testing, and production.
  class DataIntegration::Models::Workspace
    LIFECYCLE_STATE_ENUM = [
      LIFECYCLE_STATE_CREATING = 'CREATING'.freeze,
      LIFECYCLE_STATE_ACTIVE = 'ACTIVE'.freeze,
      LIFECYCLE_STATE_INACTIVE = 'INACTIVE'.freeze,
      LIFECYCLE_STATE_UPDATING = 'UPDATING'.freeze,
      LIFECYCLE_STATE_DELETING = 'DELETING'.freeze,
      LIFECYCLE_STATE_DELETED = 'DELETED'.freeze,
      LIFECYCLE_STATE_FAILED = 'FAILED'.freeze,
      LIFECYCLE_STATE_STARTING = 'STARTING'.freeze,
      LIFECYCLE_STATE_STOPPING = 'STOPPING'.freeze,
      LIFECYCLE_STATE_STOPPED = 'STOPPED'.freeze,
      LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # The OCID of the VCN the subnet is in.
    # @return [String]
    attr_accessor :vcn_id

    # The OCID of the subnet for customer connected databases.
    # @return [String]
    attr_accessor :subnet_id

    # The IP of the custom DNS.
    # @return [String]
    attr_accessor :dns_server_ip

    # The DNS zone of the custom DNS to use to resolve names.
    # @return [String]
    attr_accessor :dns_server_zone

    # Specifies whether the private network connection is enabled or disabled.
    # @return [BOOLEAN]
    attr_accessor :is_private_network_enabled

    # Free-form tags for this resource. Each tag is a simple key-value pair with no predefined name, type, or namespace. See [Resource Tags](https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm).
    # Example: `{\"Department\": \"Finance\"}`
    #
    # @return [Hash<String, String>]
    attr_accessor :freeform_tags

    # Defined tags for this resource. Each key is predefined and scoped to a namespace. See [Resource Tags](https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm).
    # Example: `{\"Operations\": {\"CostCenter\": \"42\"}}`
    #
    # @return [Hash<String, Hash<String, Object>>]
    attr_accessor :defined_tags

    # A detailed description for the workspace.
    # @return [String]
    attr_accessor :description

    # **[Required]** A user-friendly display name for the workspace. Does not have to be unique, and can be modified. Avoid entering confidential information.
    # @return [String]
    attr_accessor :display_name

    # The OCID of the compartment containing the workspace.
    # @return [String]
    attr_accessor :compartment_id

    # The date and time the workspace was created, in the timestamp format defined by RFC3339.
    #
    # @return [DateTime]
    attr_accessor :time_created

    # The date and time the workspace was updated, in the timestamp format defined by [RFC3339](https://tools.ietf.org/html/rfc3339).
    # @return [DateTime]
    attr_accessor :time_updated

    # Lifecycle states for workspaces in Data Integration Service
    # CREATING - The resource is being created and may not be usable until the entire metadata is defined
    # UPDATING - The resource is being updated and may not be usable until all changes are commited
    # DELETING - The resource is being deleted and might require deep cleanup of children.
    # ACTIVE   - The resource is valid and available for access
    # INACTIVE - The resource might be incomplete in its definition or might have been made unavailable for
    #          administrative reasons
    # DELETED  - The resource has been deleted and isn't available
    # FAILED   - The resource is in a failed state due to validation or other errors
    # STARTING - The resource is being started and may not be usable until becomes ACTIVE again
    # STOPPING - The resource is in the process of Stopping and may not be usable until it Stops or fails
    # STOPPED  - The resource is in Stopped state due to stop operation.
    #
    # @return [String]
    attr_reader :lifecycle_state

    # A message describing the current state in more detail. For example, can be used to provide actionable information for a resource in failed state.
    # @return [String]
    attr_accessor :state_message

    # **[Required]** A system-generated and immutable identifier assigned to the workspace upon creation.
    # @return [String]
    attr_accessor :id

    # OCID of the private endpoint associated with the container/workspace.
    # @return [String]
    attr_accessor :endpoint_id

    # Name of the private endpoint associated with the container/workspace.
    # @return [String]
    attr_accessor :endpoint_name

    # DCMS Registry ID associated with the container/workspace.
    # @return [String]
    attr_accessor :registry_id

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'vcn_id': :'vcnId',
        'subnet_id': :'subnetId',
        'dns_server_ip': :'dnsServerIp',
        'dns_server_zone': :'dnsServerZone',
        'is_private_network_enabled': :'isPrivateNetworkEnabled',
        'freeform_tags': :'freeformTags',
        'defined_tags': :'definedTags',
        'description': :'description',
        'display_name': :'displayName',
        'compartment_id': :'compartmentId',
        'time_created': :'timeCreated',
        'time_updated': :'timeUpdated',
        'lifecycle_state': :'lifecycleState',
        'state_message': :'stateMessage',
        'id': :'id',
        'endpoint_id': :'endpointId',
        'endpoint_name': :'endpointName',
        'registry_id': :'registryId'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'vcn_id': :'String',
        'subnet_id': :'String',
        'dns_server_ip': :'String',
        'dns_server_zone': :'String',
        'is_private_network_enabled': :'BOOLEAN',
        'freeform_tags': :'Hash<String, String>',
        'defined_tags': :'Hash<String, Hash<String, Object>>',
        'description': :'String',
        'display_name': :'String',
        'compartment_id': :'String',
        'time_created': :'DateTime',
        'time_updated': :'DateTime',
        'lifecycle_state': :'String',
        'state_message': :'String',
        'id': :'String',
        'endpoint_id': :'String',
        'endpoint_name': :'String',
        'registry_id': :'String'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :vcn_id The value to assign to the {#vcn_id} property
    # @option attributes [String] :subnet_id The value to assign to the {#subnet_id} property
    # @option attributes [String] :dns_server_ip The value to assign to the {#dns_server_ip} property
    # @option attributes [String] :dns_server_zone The value to assign to the {#dns_server_zone} property
    # @option attributes [BOOLEAN] :is_private_network_enabled The value to assign to the {#is_private_network_enabled} property
    # @option attributes [Hash<String, String>] :freeform_tags The value to assign to the {#freeform_tags} property
    # @option attributes [Hash<String, Hash<String, Object>>] :defined_tags The value to assign to the {#defined_tags} property
    # @option attributes [String] :description The value to assign to the {#description} property
    # @option attributes [String] :display_name The value to assign to the {#display_name} property
    # @option attributes [String] :compartment_id The value to assign to the {#compartment_id} property
    # @option attributes [DateTime] :time_created The value to assign to the {#time_created} property
    # @option attributes [DateTime] :time_updated The value to assign to the {#time_updated} property
    # @option attributes [String] :lifecycle_state The value to assign to the {#lifecycle_state} property
    # @option attributes [String] :state_message The value to assign to the {#state_message} property
    # @option attributes [String] :id The value to assign to the {#id} property
    # @option attributes [String] :endpoint_id The value to assign to the {#endpoint_id} property
    # @option attributes [String] :endpoint_name The value to assign to the {#endpoint_name} property
    # @option attributes [String] :registry_id The value to assign to the {#registry_id} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.vcn_id = attributes[:'vcnId'] if attributes[:'vcnId']

      raise 'You cannot provide both :vcnId and :vcn_id' if attributes.key?(:'vcnId') && attributes.key?(:'vcn_id')

      self.vcn_id = attributes[:'vcn_id'] if attributes[:'vcn_id']

      self.subnet_id = attributes[:'subnetId'] if attributes[:'subnetId']

      raise 'You cannot provide both :subnetId and :subnet_id' if attributes.key?(:'subnetId') && attributes.key?(:'subnet_id')

      self.subnet_id = attributes[:'subnet_id'] if attributes[:'subnet_id']

      self.dns_server_ip = attributes[:'dnsServerIp'] if attributes[:'dnsServerIp']

      raise 'You cannot provide both :dnsServerIp and :dns_server_ip' if attributes.key?(:'dnsServerIp') && attributes.key?(:'dns_server_ip')

      self.dns_server_ip = attributes[:'dns_server_ip'] if attributes[:'dns_server_ip']

      self.dns_server_zone = attributes[:'dnsServerZone'] if attributes[:'dnsServerZone']

      raise 'You cannot provide both :dnsServerZone and :dns_server_zone' if attributes.key?(:'dnsServerZone') && attributes.key?(:'dns_server_zone')

      self.dns_server_zone = attributes[:'dns_server_zone'] if attributes[:'dns_server_zone']

      self.is_private_network_enabled = attributes[:'isPrivateNetworkEnabled'] unless attributes[:'isPrivateNetworkEnabled'].nil?

      raise 'You cannot provide both :isPrivateNetworkEnabled and :is_private_network_enabled' if attributes.key?(:'isPrivateNetworkEnabled') && attributes.key?(:'is_private_network_enabled')

      self.is_private_network_enabled = attributes[:'is_private_network_enabled'] unless attributes[:'is_private_network_enabled'].nil?

      self.freeform_tags = attributes[:'freeformTags'] if attributes[:'freeformTags']

      raise 'You cannot provide both :freeformTags and :freeform_tags' if attributes.key?(:'freeformTags') && attributes.key?(:'freeform_tags')

      self.freeform_tags = attributes[:'freeform_tags'] if attributes[:'freeform_tags']

      self.defined_tags = attributes[:'definedTags'] if attributes[:'definedTags']

      raise 'You cannot provide both :definedTags and :defined_tags' if attributes.key?(:'definedTags') && attributes.key?(:'defined_tags')

      self.defined_tags = attributes[:'defined_tags'] if attributes[:'defined_tags']

      self.description = attributes[:'description'] if attributes[:'description']

      self.display_name = attributes[:'displayName'] if attributes[:'displayName']

      raise 'You cannot provide both :displayName and :display_name' if attributes.key?(:'displayName') && attributes.key?(:'display_name')

      self.display_name = attributes[:'display_name'] if attributes[:'display_name']

      self.compartment_id = attributes[:'compartmentId'] if attributes[:'compartmentId']

      raise 'You cannot provide both :compartmentId and :compartment_id' if attributes.key?(:'compartmentId') && attributes.key?(:'compartment_id')

      self.compartment_id = attributes[:'compartment_id'] if attributes[:'compartment_id']

      self.time_created = attributes[:'timeCreated'] if attributes[:'timeCreated']

      raise 'You cannot provide both :timeCreated and :time_created' if attributes.key?(:'timeCreated') && attributes.key?(:'time_created')

      self.time_created = attributes[:'time_created'] if attributes[:'time_created']

      self.time_updated = attributes[:'timeUpdated'] if attributes[:'timeUpdated']

      raise 'You cannot provide both :timeUpdated and :time_updated' if attributes.key?(:'timeUpdated') && attributes.key?(:'time_updated')

      self.time_updated = attributes[:'time_updated'] if attributes[:'time_updated']

      self.lifecycle_state = attributes[:'lifecycleState'] if attributes[:'lifecycleState']

      raise 'You cannot provide both :lifecycleState and :lifecycle_state' if attributes.key?(:'lifecycleState') && attributes.key?(:'lifecycle_state')

      self.lifecycle_state = attributes[:'lifecycle_state'] if attributes[:'lifecycle_state']

      self.state_message = attributes[:'stateMessage'] if attributes[:'stateMessage']

      raise 'You cannot provide both :stateMessage and :state_message' if attributes.key?(:'stateMessage') && attributes.key?(:'state_message')

      self.state_message = attributes[:'state_message'] if attributes[:'state_message']

      self.id = attributes[:'id'] if attributes[:'id']

      self.endpoint_id = attributes[:'endpointId'] if attributes[:'endpointId']

      raise 'You cannot provide both :endpointId and :endpoint_id' if attributes.key?(:'endpointId') && attributes.key?(:'endpoint_id')

      self.endpoint_id = attributes[:'endpoint_id'] if attributes[:'endpoint_id']

      self.endpoint_name = attributes[:'endpointName'] if attributes[:'endpointName']

      raise 'You cannot provide both :endpointName and :endpoint_name' if attributes.key?(:'endpointName') && attributes.key?(:'endpoint_name')

      self.endpoint_name = attributes[:'endpoint_name'] if attributes[:'endpoint_name']

      self.registry_id = attributes[:'registryId'] if attributes[:'registryId']

      raise 'You cannot provide both :registryId and :registry_id' if attributes.key?(:'registryId') && attributes.key?(:'registry_id')

      self.registry_id = attributes[:'registry_id'] if attributes[:'registry_id']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] lifecycle_state Object to be assigned
    def lifecycle_state=(lifecycle_state)
      # rubocop:disable Style/ConditionalAssignment
      if lifecycle_state && !LIFECYCLE_STATE_ENUM.include?(lifecycle_state)
        OCI.logger.debug("Unknown value for 'lifecycle_state' [" + lifecycle_state + "]. Mapping to 'LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @lifecycle_state = LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE
      else
        @lifecycle_state = lifecycle_state
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        vcn_id == other.vcn_id &&
        subnet_id == other.subnet_id &&
        dns_server_ip == other.dns_server_ip &&
        dns_server_zone == other.dns_server_zone &&
        is_private_network_enabled == other.is_private_network_enabled &&
        freeform_tags == other.freeform_tags &&
        defined_tags == other.defined_tags &&
        description == other.description &&
        display_name == other.display_name &&
        compartment_id == other.compartment_id &&
        time_created == other.time_created &&
        time_updated == other.time_updated &&
        lifecycle_state == other.lifecycle_state &&
        state_message == other.state_message &&
        id == other.id &&
        endpoint_id == other.endpoint_id &&
        endpoint_name == other.endpoint_name &&
        registry_id == other.registry_id
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [vcn_id, subnet_id, dns_server_ip, dns_server_zone, is_private_network_enabled, freeform_tags, defined_tags, description, display_name, compartment_id, time_created, time_updated, lifecycle_state, state_message, id, endpoint_id, endpoint_name, registry_id].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
