# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20181201
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Details to mask data.
  class DataSafe::Models::MaskDataDetails
    # The OCID of the target database to be masked. If it's not provided, the value of the
    # targetId attribute in the MaskingPolicy resource is used. The OCID of the target
    # database to be masked. If it's not provided, the value of the targetId attribute in
    # the MaskingPolicy resource is used.
    #
    # @return [String]
    attr_accessor :target_id

    # Indicates if the masking request is to decrypt the data values previously encrypted using Deterministic Encryption. Note that, to
    # correctly decrypt the encrypted data values, it requires the same seed value that was provided to encrypt those data values.
    #
    # @return [BOOLEAN]
    attr_accessor :is_decrypt

    # Indicates if the masking request is to rerun the previously failed masking steps. If a masking request is submitted with the
    # isIgnoreErrorsEnabled attribute set to true, the masking process tracks the failed masking steps. Another masking request can be
    # submitted with the isRun attribute set to true to rerun those failed masking steps. It helps save time by executing only the failed
    # masking steps and not doing the whole masking again.
    #
    # @return [BOOLEAN]
    attr_accessor :is_rerun

    # The tablespace that should be used to create the mapping tables, DMASK objects, and other temporary tables for data masking.
    # If no tablespace is provided, the DEFAULT tablespace is used.
    #
    # @return [String]
    attr_accessor :tablespace

    # Indicates if the masking process should continue on hitting an error. It provides fault tolerance support and is enabled by
    # default. In fault-tolerant mode, the masking process saves the failed step and continues. You can then submit another masking
    # request (with isRerun attribute set to true) to execute only the failed steps.
    #
    # @return [BOOLEAN]
    attr_accessor :is_ignore_errors_enabled

    # The seed value to be used in case of Deterministic Encryption and Deterministic Substitution masking formats.
    # @return [String]
    attr_accessor :seed

    # Indicates if the interim DMASK tables should be moved to the user-specified tablespace. As interim tables can be large in size,
    # set it to false if moving them causes performance overhead during masking.
    #
    # @return [BOOLEAN]
    attr_accessor :is_move_interim_tables_enabled

    # Indicates if data masking should be performed using a saved masking script. Setting this attribute to true skips masking script
    # generation and executes the masking script stored in the Data Safe repository. It helps save time if there are no changes in
    # the database tables and their dependencies.
    #
    # @return [BOOLEAN]
    attr_accessor :is_execute_saved_script_enabled

    # Indicates if the temporary tables created during a masking operation should be dropped after masking.
    # Set this attribute to false to preserve the temporary tables. Masking creates temporary tables that map the original sensitive
    # data values to mask values. These temporary tables are dropped after masking if this attribute is set as true. But, in some cases, you may want
    # to preserve this information to track how masking changed your data. Note that doing so compromises security. These tables
    # must be dropped before the database is available for unprivileged users.
    # If it's not provided, the value of the isDropTempTablesEnabled attribute in the MaskingPolicy resource is used.
    #
    # @return [BOOLEAN]
    attr_accessor :is_drop_temp_tables_enabled

    # Indicates if redo logging is enabled during a masking operation. Set this attribute to true to
    # enable redo logging. If set as flase, masking disables redo logging and flashback logging to purge any original unmasked
    # data from logs. However, in certain circumstances when you only want to test masking, rollback changes, and retry masking,
    # you could enable logging and use a flashback database to retrieve the original unmasked data after it has been masked.
    # If it's not provided, the value of the isRedoLoggingEnabled attribute in the MaskingPolicy resource is used.
    #
    # @return [BOOLEAN]
    attr_accessor :is_redo_logging_enabled

    # Indicates if statistics gathering is enabled. Set this attribute to false to disable statistics
    # gathering. The masking process gathers statistics on masked database tables after masking completes.
    # If it's not provided, the value of the isRefreshStatsEnabled attribute in the MaskingPolicy resource is used.
    #
    # @return [BOOLEAN]
    attr_accessor :is_refresh_stats_enabled

    # Specifies options to enable parallel execution when running data masking. Allowed values are 'NONE' (no parallelism),
    # 'DEFAULT' (the Oracle Database computes the optimum degree of parallelism) or an integer value to be used as the degree
    # of parallelism. Parallel execution helps effectively use multiple CPUs and improve masking performance. Refer to the
    # Oracle Database parallel execution framework when choosing an explicit degree of parallelism.
    # If it's not provided, the value of the parallelDegree attribute in the MaskingPolicy resource is used.
    #
    # @return [String]
    attr_accessor :parallel_degree

    # Specifies how to recompile invalid objects post data masking. Allowed values are 'SERIAL' (recompile in serial),
    # 'PARALLEL' (recompile in parallel), 'NONE' (do not recompile). If it's set to PARALLEL, the value of parallelDegree
    # attribute is used. Use the built-in UTL_RECOMP package to recompile any remaining invalid objects after masking completes.
    # If it's not provided, the value of the parallelDegree attribute in the MaskingPolicy resource is used.
    #
    # @return [String]
    attr_accessor :recompile

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'target_id': :'targetId',
        'is_decrypt': :'isDecrypt',
        'is_rerun': :'isRerun',
        'tablespace': :'tablespace',
        'is_ignore_errors_enabled': :'isIgnoreErrorsEnabled',
        'seed': :'seed',
        'is_move_interim_tables_enabled': :'isMoveInterimTablesEnabled',
        'is_execute_saved_script_enabled': :'isExecuteSavedScriptEnabled',
        'is_drop_temp_tables_enabled': :'isDropTempTablesEnabled',
        'is_redo_logging_enabled': :'isRedoLoggingEnabled',
        'is_refresh_stats_enabled': :'isRefreshStatsEnabled',
        'parallel_degree': :'parallelDegree',
        'recompile': :'recompile'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'target_id': :'String',
        'is_decrypt': :'BOOLEAN',
        'is_rerun': :'BOOLEAN',
        'tablespace': :'String',
        'is_ignore_errors_enabled': :'BOOLEAN',
        'seed': :'String',
        'is_move_interim_tables_enabled': :'BOOLEAN',
        'is_execute_saved_script_enabled': :'BOOLEAN',
        'is_drop_temp_tables_enabled': :'BOOLEAN',
        'is_redo_logging_enabled': :'BOOLEAN',
        'is_refresh_stats_enabled': :'BOOLEAN',
        'parallel_degree': :'String',
        'recompile': :'String'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :target_id The value to assign to the {#target_id} property
    # @option attributes [BOOLEAN] :is_decrypt The value to assign to the {#is_decrypt} property
    # @option attributes [BOOLEAN] :is_rerun The value to assign to the {#is_rerun} property
    # @option attributes [String] :tablespace The value to assign to the {#tablespace} property
    # @option attributes [BOOLEAN] :is_ignore_errors_enabled The value to assign to the {#is_ignore_errors_enabled} property
    # @option attributes [String] :seed The value to assign to the {#seed} property
    # @option attributes [BOOLEAN] :is_move_interim_tables_enabled The value to assign to the {#is_move_interim_tables_enabled} property
    # @option attributes [BOOLEAN] :is_execute_saved_script_enabled The value to assign to the {#is_execute_saved_script_enabled} property
    # @option attributes [BOOLEAN] :is_drop_temp_tables_enabled The value to assign to the {#is_drop_temp_tables_enabled} property
    # @option attributes [BOOLEAN] :is_redo_logging_enabled The value to assign to the {#is_redo_logging_enabled} property
    # @option attributes [BOOLEAN] :is_refresh_stats_enabled The value to assign to the {#is_refresh_stats_enabled} property
    # @option attributes [String] :parallel_degree The value to assign to the {#parallel_degree} property
    # @option attributes [String] :recompile The value to assign to the {#recompile} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.target_id = attributes[:'targetId'] if attributes[:'targetId']

      raise 'You cannot provide both :targetId and :target_id' if attributes.key?(:'targetId') && attributes.key?(:'target_id')

      self.target_id = attributes[:'target_id'] if attributes[:'target_id']

      self.is_decrypt = attributes[:'isDecrypt'] unless attributes[:'isDecrypt'].nil?
      self.is_decrypt = false if is_decrypt.nil? && !attributes.key?(:'isDecrypt') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :isDecrypt and :is_decrypt' if attributes.key?(:'isDecrypt') && attributes.key?(:'is_decrypt')

      self.is_decrypt = attributes[:'is_decrypt'] unless attributes[:'is_decrypt'].nil?
      self.is_decrypt = false if is_decrypt.nil? && !attributes.key?(:'isDecrypt') && !attributes.key?(:'is_decrypt') # rubocop:disable Style/StringLiterals

      self.is_rerun = attributes[:'isRerun'] unless attributes[:'isRerun'].nil?
      self.is_rerun = false if is_rerun.nil? && !attributes.key?(:'isRerun') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :isRerun and :is_rerun' if attributes.key?(:'isRerun') && attributes.key?(:'is_rerun')

      self.is_rerun = attributes[:'is_rerun'] unless attributes[:'is_rerun'].nil?
      self.is_rerun = false if is_rerun.nil? && !attributes.key?(:'isRerun') && !attributes.key?(:'is_rerun') # rubocop:disable Style/StringLiterals

      self.tablespace = attributes[:'tablespace'] if attributes[:'tablespace']

      self.is_ignore_errors_enabled = attributes[:'isIgnoreErrorsEnabled'] unless attributes[:'isIgnoreErrorsEnabled'].nil?
      self.is_ignore_errors_enabled = true if is_ignore_errors_enabled.nil? && !attributes.key?(:'isIgnoreErrorsEnabled') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :isIgnoreErrorsEnabled and :is_ignore_errors_enabled' if attributes.key?(:'isIgnoreErrorsEnabled') && attributes.key?(:'is_ignore_errors_enabled')

      self.is_ignore_errors_enabled = attributes[:'is_ignore_errors_enabled'] unless attributes[:'is_ignore_errors_enabled'].nil?
      self.is_ignore_errors_enabled = true if is_ignore_errors_enabled.nil? && !attributes.key?(:'isIgnoreErrorsEnabled') && !attributes.key?(:'is_ignore_errors_enabled') # rubocop:disable Style/StringLiterals

      self.seed = attributes[:'seed'] if attributes[:'seed']

      self.is_move_interim_tables_enabled = attributes[:'isMoveInterimTablesEnabled'] unless attributes[:'isMoveInterimTablesEnabled'].nil?
      self.is_move_interim_tables_enabled = true if is_move_interim_tables_enabled.nil? && !attributes.key?(:'isMoveInterimTablesEnabled') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :isMoveInterimTablesEnabled and :is_move_interim_tables_enabled' if attributes.key?(:'isMoveInterimTablesEnabled') && attributes.key?(:'is_move_interim_tables_enabled')

      self.is_move_interim_tables_enabled = attributes[:'is_move_interim_tables_enabled'] unless attributes[:'is_move_interim_tables_enabled'].nil?
      self.is_move_interim_tables_enabled = true if is_move_interim_tables_enabled.nil? && !attributes.key?(:'isMoveInterimTablesEnabled') && !attributes.key?(:'is_move_interim_tables_enabled') # rubocop:disable Style/StringLiterals

      self.is_execute_saved_script_enabled = attributes[:'isExecuteSavedScriptEnabled'] unless attributes[:'isExecuteSavedScriptEnabled'].nil?
      self.is_execute_saved_script_enabled = false if is_execute_saved_script_enabled.nil? && !attributes.key?(:'isExecuteSavedScriptEnabled') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :isExecuteSavedScriptEnabled and :is_execute_saved_script_enabled' if attributes.key?(:'isExecuteSavedScriptEnabled') && attributes.key?(:'is_execute_saved_script_enabled')

      self.is_execute_saved_script_enabled = attributes[:'is_execute_saved_script_enabled'] unless attributes[:'is_execute_saved_script_enabled'].nil?
      self.is_execute_saved_script_enabled = false if is_execute_saved_script_enabled.nil? && !attributes.key?(:'isExecuteSavedScriptEnabled') && !attributes.key?(:'is_execute_saved_script_enabled') # rubocop:disable Style/StringLiterals

      self.is_drop_temp_tables_enabled = attributes[:'isDropTempTablesEnabled'] unless attributes[:'isDropTempTablesEnabled'].nil?

      raise 'You cannot provide both :isDropTempTablesEnabled and :is_drop_temp_tables_enabled' if attributes.key?(:'isDropTempTablesEnabled') && attributes.key?(:'is_drop_temp_tables_enabled')

      self.is_drop_temp_tables_enabled = attributes[:'is_drop_temp_tables_enabled'] unless attributes[:'is_drop_temp_tables_enabled'].nil?

      self.is_redo_logging_enabled = attributes[:'isRedoLoggingEnabled'] unless attributes[:'isRedoLoggingEnabled'].nil?

      raise 'You cannot provide both :isRedoLoggingEnabled and :is_redo_logging_enabled' if attributes.key?(:'isRedoLoggingEnabled') && attributes.key?(:'is_redo_logging_enabled')

      self.is_redo_logging_enabled = attributes[:'is_redo_logging_enabled'] unless attributes[:'is_redo_logging_enabled'].nil?

      self.is_refresh_stats_enabled = attributes[:'isRefreshStatsEnabled'] unless attributes[:'isRefreshStatsEnabled'].nil?

      raise 'You cannot provide both :isRefreshStatsEnabled and :is_refresh_stats_enabled' if attributes.key?(:'isRefreshStatsEnabled') && attributes.key?(:'is_refresh_stats_enabled')

      self.is_refresh_stats_enabled = attributes[:'is_refresh_stats_enabled'] unless attributes[:'is_refresh_stats_enabled'].nil?

      self.parallel_degree = attributes[:'parallelDegree'] if attributes[:'parallelDegree']

      raise 'You cannot provide both :parallelDegree and :parallel_degree' if attributes.key?(:'parallelDegree') && attributes.key?(:'parallel_degree')

      self.parallel_degree = attributes[:'parallel_degree'] if attributes[:'parallel_degree']

      self.recompile = attributes[:'recompile'] if attributes[:'recompile']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        target_id == other.target_id &&
        is_decrypt == other.is_decrypt &&
        is_rerun == other.is_rerun &&
        tablespace == other.tablespace &&
        is_ignore_errors_enabled == other.is_ignore_errors_enabled &&
        seed == other.seed &&
        is_move_interim_tables_enabled == other.is_move_interim_tables_enabled &&
        is_execute_saved_script_enabled == other.is_execute_saved_script_enabled &&
        is_drop_temp_tables_enabled == other.is_drop_temp_tables_enabled &&
        is_redo_logging_enabled == other.is_redo_logging_enabled &&
        is_refresh_stats_enabled == other.is_refresh_stats_enabled &&
        parallel_degree == other.parallel_degree &&
        recompile == other.recompile
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [target_id, is_decrypt, is_rerun, tablespace, is_ignore_errors_enabled, seed, is_move_interim_tables_enabled, is_execute_saved_script_enabled, is_drop_temp_tables_enabled, is_redo_logging_enabled, is_refresh_stats_enabled, parallel_degree, recompile].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
