# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20190101
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Logging configuration for resource.
  #
  class DataScience::Models::JobLogConfigurationDetails
    # If customer logging is enabled for job runs.
    # @return [BOOLEAN]
    attr_accessor :enable_logging

    # If automatic on-behalf-of log object creation is enabled for job runs.
    #
    # @return [BOOLEAN]
    attr_accessor :enable_auto_log_creation

    # The log group id for where log objects are for job runs.
    #
    # @return [String]
    attr_accessor :log_group_id

    # The log id the job run will push logs too.
    #
    # @return [String]
    attr_accessor :log_id

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'enable_logging': :'enableLogging',
        'enable_auto_log_creation': :'enableAutoLogCreation',
        'log_group_id': :'logGroupId',
        'log_id': :'logId'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'enable_logging': :'BOOLEAN',
        'enable_auto_log_creation': :'BOOLEAN',
        'log_group_id': :'String',
        'log_id': :'String'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [BOOLEAN] :enable_logging The value to assign to the {#enable_logging} property
    # @option attributes [BOOLEAN] :enable_auto_log_creation The value to assign to the {#enable_auto_log_creation} property
    # @option attributes [String] :log_group_id The value to assign to the {#log_group_id} property
    # @option attributes [String] :log_id The value to assign to the {#log_id} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.enable_logging = attributes[:'enableLogging'] unless attributes[:'enableLogging'].nil?
      self.enable_logging = false if enable_logging.nil? && !attributes.key?(:'enableLogging') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :enableLogging and :enable_logging' if attributes.key?(:'enableLogging') && attributes.key?(:'enable_logging')

      self.enable_logging = attributes[:'enable_logging'] unless attributes[:'enable_logging'].nil?
      self.enable_logging = false if enable_logging.nil? && !attributes.key?(:'enableLogging') && !attributes.key?(:'enable_logging') # rubocop:disable Style/StringLiterals

      self.enable_auto_log_creation = attributes[:'enableAutoLogCreation'] unless attributes[:'enableAutoLogCreation'].nil?
      self.enable_auto_log_creation = false if enable_auto_log_creation.nil? && !attributes.key?(:'enableAutoLogCreation') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :enableAutoLogCreation and :enable_auto_log_creation' if attributes.key?(:'enableAutoLogCreation') && attributes.key?(:'enable_auto_log_creation')

      self.enable_auto_log_creation = attributes[:'enable_auto_log_creation'] unless attributes[:'enable_auto_log_creation'].nil?
      self.enable_auto_log_creation = false if enable_auto_log_creation.nil? && !attributes.key?(:'enableAutoLogCreation') && !attributes.key?(:'enable_auto_log_creation') # rubocop:disable Style/StringLiterals

      self.log_group_id = attributes[:'logGroupId'] if attributes[:'logGroupId']

      raise 'You cannot provide both :logGroupId and :log_group_id' if attributes.key?(:'logGroupId') && attributes.key?(:'log_group_id')

      self.log_group_id = attributes[:'log_group_id'] if attributes[:'log_group_id']

      self.log_id = attributes[:'logId'] if attributes[:'logId']

      raise 'You cannot provide both :logId and :log_id' if attributes.key?(:'logId') && attributes.key?(:'log_id')

      self.log_id = attributes[:'log_id'] if attributes[:'log_id']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        enable_logging == other.enable_logging &&
        enable_auto_log_creation == other.enable_auto_log_creation &&
        log_group_id == other.log_group_id &&
        log_id == other.log_id
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [enable_logging, enable_auto_log_creation, log_group_id, log_id].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
