# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20190101
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # A job run.
  class DataScience::Models::JobRun
    LIFECYCLE_STATE_ENUM = [
      LIFECYCLE_STATE_ACCEPTED = 'ACCEPTED'.freeze,
      LIFECYCLE_STATE_IN_PROGRESS = 'IN_PROGRESS'.freeze,
      LIFECYCLE_STATE_FAILED = 'FAILED'.freeze,
      LIFECYCLE_STATE_SUCCEEDED = 'SUCCEEDED'.freeze,
      LIFECYCLE_STATE_CANCELING = 'CANCELING'.freeze,
      LIFECYCLE_STATE_CANCELED = 'CANCELED'.freeze,
      LIFECYCLE_STATE_DELETED = 'DELETED'.freeze,
      LIFECYCLE_STATE_NEEDS_ATTENTION = 'NEEDS_ATTENTION'.freeze,
      LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the job run.
    # @return [String]
    attr_accessor :id

    # **[Required]** The date and time the job run was accepted in the timestamp format defined by [RFC3339](https://tools.ietf.org/html/rfc3339).
    # @return [DateTime]
    attr_accessor :time_accepted

    # The date and time the job run request was started in the timestamp format defined by [RFC3339](https://tools.ietf.org/html/rfc3339).
    # @return [DateTime]
    attr_accessor :time_started

    # The date and time the job run request was finished in the timestamp format defined by [RFC3339](https://tools.ietf.org/html/rfc3339).
    # @return [DateTime]
    attr_accessor :time_finished

    # **[Required]** The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the user who created the job run.
    # @return [String]
    attr_accessor :created_by

    # **[Required]** The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the project to associate the job with.
    # @return [String]
    attr_accessor :project_id

    # **[Required]** The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the compartment where you want to create the job.
    # @return [String]
    attr_accessor :compartment_id

    # **[Required]** The [OCID](https://docs.cloud.oracle.com/iaas/Content/General/Concepts/identifiers.htm) of the job run.
    # @return [String]
    attr_accessor :job_id

    # A user-friendly display name for the resource.
    # @return [String]
    attr_accessor :display_name

    # This attribute is required.
    # @return [OCI::DataScience::Models::JobConfigurationDetails]
    attr_accessor :job_configuration_override_details

    # This attribute is required.
    # @return [OCI::DataScience::Models::JobInfrastructureConfigurationDetails]
    attr_accessor :job_infrastructure_configuration_details

    # @return [OCI::DataScience::Models::JobLogConfigurationDetails]
    attr_accessor :job_log_configuration_override_details

    # Collection of JobStorageMountConfigurationDetails.
    # @return [Array<OCI::DataScience::Models::StorageMountConfigurationDetails>]
    attr_accessor :job_storage_mount_configuration_details_list

    # @return [OCI::DataScience::Models::JobRunLogDetails]
    attr_accessor :log_details

    # **[Required]** The state of the job run.
    # @return [String]
    attr_reader :lifecycle_state

    # Details of the state of the job run.
    # @return [String]
    attr_accessor :lifecycle_details

    # Free-form tags for this resource. Each tag is a simple key-value pair with no predefined name, type, or namespace. See [Resource Tags](https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm).
    # Example: `{\"Department\": \"Finance\"}`
    #
    # @return [Hash<String, String>]
    attr_accessor :freeform_tags

    # Defined tags for this resource. Each key is predefined and scoped to a namespace. See [Resource Tags](https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm).
    # Example: `{\"Operations\": {\"CostCenter\": \"42\"}}`
    #
    # @return [Hash<String, Hash<String, Object>>]
    attr_accessor :defined_tags

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'id',
        'time_accepted': :'timeAccepted',
        'time_started': :'timeStarted',
        'time_finished': :'timeFinished',
        'created_by': :'createdBy',
        'project_id': :'projectId',
        'compartment_id': :'compartmentId',
        'job_id': :'jobId',
        'display_name': :'displayName',
        'job_configuration_override_details': :'jobConfigurationOverrideDetails',
        'job_infrastructure_configuration_details': :'jobInfrastructureConfigurationDetails',
        'job_log_configuration_override_details': :'jobLogConfigurationOverrideDetails',
        'job_storage_mount_configuration_details_list': :'jobStorageMountConfigurationDetailsList',
        'log_details': :'logDetails',
        'lifecycle_state': :'lifecycleState',
        'lifecycle_details': :'lifecycleDetails',
        'freeform_tags': :'freeformTags',
        'defined_tags': :'definedTags'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'String',
        'time_accepted': :'DateTime',
        'time_started': :'DateTime',
        'time_finished': :'DateTime',
        'created_by': :'String',
        'project_id': :'String',
        'compartment_id': :'String',
        'job_id': :'String',
        'display_name': :'String',
        'job_configuration_override_details': :'OCI::DataScience::Models::JobConfigurationDetails',
        'job_infrastructure_configuration_details': :'OCI::DataScience::Models::JobInfrastructureConfigurationDetails',
        'job_log_configuration_override_details': :'OCI::DataScience::Models::JobLogConfigurationDetails',
        'job_storage_mount_configuration_details_list': :'Array<OCI::DataScience::Models::StorageMountConfigurationDetails>',
        'log_details': :'OCI::DataScience::Models::JobRunLogDetails',
        'lifecycle_state': :'String',
        'lifecycle_details': :'String',
        'freeform_tags': :'Hash<String, String>',
        'defined_tags': :'Hash<String, Hash<String, Object>>'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :id The value to assign to the {#id} property
    # @option attributes [DateTime] :time_accepted The value to assign to the {#time_accepted} property
    # @option attributes [DateTime] :time_started The value to assign to the {#time_started} property
    # @option attributes [DateTime] :time_finished The value to assign to the {#time_finished} property
    # @option attributes [String] :created_by The value to assign to the {#created_by} property
    # @option attributes [String] :project_id The value to assign to the {#project_id} property
    # @option attributes [String] :compartment_id The value to assign to the {#compartment_id} property
    # @option attributes [String] :job_id The value to assign to the {#job_id} property
    # @option attributes [String] :display_name The value to assign to the {#display_name} property
    # @option attributes [OCI::DataScience::Models::JobConfigurationDetails] :job_configuration_override_details The value to assign to the {#job_configuration_override_details} property
    # @option attributes [OCI::DataScience::Models::JobInfrastructureConfigurationDetails] :job_infrastructure_configuration_details The value to assign to the {#job_infrastructure_configuration_details} property
    # @option attributes [OCI::DataScience::Models::JobLogConfigurationDetails] :job_log_configuration_override_details The value to assign to the {#job_log_configuration_override_details} property
    # @option attributes [Array<OCI::DataScience::Models::StorageMountConfigurationDetails>] :job_storage_mount_configuration_details_list The value to assign to the {#job_storage_mount_configuration_details_list} property
    # @option attributes [OCI::DataScience::Models::JobRunLogDetails] :log_details The value to assign to the {#log_details} property
    # @option attributes [String] :lifecycle_state The value to assign to the {#lifecycle_state} property
    # @option attributes [String] :lifecycle_details The value to assign to the {#lifecycle_details} property
    # @option attributes [Hash<String, String>] :freeform_tags The value to assign to the {#freeform_tags} property
    # @option attributes [Hash<String, Hash<String, Object>>] :defined_tags The value to assign to the {#defined_tags} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.id = attributes[:'id'] if attributes[:'id']

      self.time_accepted = attributes[:'timeAccepted'] if attributes[:'timeAccepted']

      raise 'You cannot provide both :timeAccepted and :time_accepted' if attributes.key?(:'timeAccepted') && attributes.key?(:'time_accepted')

      self.time_accepted = attributes[:'time_accepted'] if attributes[:'time_accepted']

      self.time_started = attributes[:'timeStarted'] if attributes[:'timeStarted']

      raise 'You cannot provide both :timeStarted and :time_started' if attributes.key?(:'timeStarted') && attributes.key?(:'time_started')

      self.time_started = attributes[:'time_started'] if attributes[:'time_started']

      self.time_finished = attributes[:'timeFinished'] if attributes[:'timeFinished']

      raise 'You cannot provide both :timeFinished and :time_finished' if attributes.key?(:'timeFinished') && attributes.key?(:'time_finished')

      self.time_finished = attributes[:'time_finished'] if attributes[:'time_finished']

      self.created_by = attributes[:'createdBy'] if attributes[:'createdBy']

      raise 'You cannot provide both :createdBy and :created_by' if attributes.key?(:'createdBy') && attributes.key?(:'created_by')

      self.created_by = attributes[:'created_by'] if attributes[:'created_by']

      self.project_id = attributes[:'projectId'] if attributes[:'projectId']

      raise 'You cannot provide both :projectId and :project_id' if attributes.key?(:'projectId') && attributes.key?(:'project_id')

      self.project_id = attributes[:'project_id'] if attributes[:'project_id']

      self.compartment_id = attributes[:'compartmentId'] if attributes[:'compartmentId']

      raise 'You cannot provide both :compartmentId and :compartment_id' if attributes.key?(:'compartmentId') && attributes.key?(:'compartment_id')

      self.compartment_id = attributes[:'compartment_id'] if attributes[:'compartment_id']

      self.job_id = attributes[:'jobId'] if attributes[:'jobId']

      raise 'You cannot provide both :jobId and :job_id' if attributes.key?(:'jobId') && attributes.key?(:'job_id')

      self.job_id = attributes[:'job_id'] if attributes[:'job_id']

      self.display_name = attributes[:'displayName'] if attributes[:'displayName']

      raise 'You cannot provide both :displayName and :display_name' if attributes.key?(:'displayName') && attributes.key?(:'display_name')

      self.display_name = attributes[:'display_name'] if attributes[:'display_name']

      self.job_configuration_override_details = attributes[:'jobConfigurationOverrideDetails'] if attributes[:'jobConfigurationOverrideDetails']

      raise 'You cannot provide both :jobConfigurationOverrideDetails and :job_configuration_override_details' if attributes.key?(:'jobConfigurationOverrideDetails') && attributes.key?(:'job_configuration_override_details')

      self.job_configuration_override_details = attributes[:'job_configuration_override_details'] if attributes[:'job_configuration_override_details']

      self.job_infrastructure_configuration_details = attributes[:'jobInfrastructureConfigurationDetails'] if attributes[:'jobInfrastructureConfigurationDetails']

      raise 'You cannot provide both :jobInfrastructureConfigurationDetails and :job_infrastructure_configuration_details' if attributes.key?(:'jobInfrastructureConfigurationDetails') && attributes.key?(:'job_infrastructure_configuration_details')

      self.job_infrastructure_configuration_details = attributes[:'job_infrastructure_configuration_details'] if attributes[:'job_infrastructure_configuration_details']

      self.job_log_configuration_override_details = attributes[:'jobLogConfigurationOverrideDetails'] if attributes[:'jobLogConfigurationOverrideDetails']

      raise 'You cannot provide both :jobLogConfigurationOverrideDetails and :job_log_configuration_override_details' if attributes.key?(:'jobLogConfigurationOverrideDetails') && attributes.key?(:'job_log_configuration_override_details')

      self.job_log_configuration_override_details = attributes[:'job_log_configuration_override_details'] if attributes[:'job_log_configuration_override_details']

      self.job_storage_mount_configuration_details_list = attributes[:'jobStorageMountConfigurationDetailsList'] if attributes[:'jobStorageMountConfigurationDetailsList']

      raise 'You cannot provide both :jobStorageMountConfigurationDetailsList and :job_storage_mount_configuration_details_list' if attributes.key?(:'jobStorageMountConfigurationDetailsList') && attributes.key?(:'job_storage_mount_configuration_details_list')

      self.job_storage_mount_configuration_details_list = attributes[:'job_storage_mount_configuration_details_list'] if attributes[:'job_storage_mount_configuration_details_list']

      self.log_details = attributes[:'logDetails'] if attributes[:'logDetails']

      raise 'You cannot provide both :logDetails and :log_details' if attributes.key?(:'logDetails') && attributes.key?(:'log_details')

      self.log_details = attributes[:'log_details'] if attributes[:'log_details']

      self.lifecycle_state = attributes[:'lifecycleState'] if attributes[:'lifecycleState']

      raise 'You cannot provide both :lifecycleState and :lifecycle_state' if attributes.key?(:'lifecycleState') && attributes.key?(:'lifecycle_state')

      self.lifecycle_state = attributes[:'lifecycle_state'] if attributes[:'lifecycle_state']

      self.lifecycle_details = attributes[:'lifecycleDetails'] if attributes[:'lifecycleDetails']

      raise 'You cannot provide both :lifecycleDetails and :lifecycle_details' if attributes.key?(:'lifecycleDetails') && attributes.key?(:'lifecycle_details')

      self.lifecycle_details = attributes[:'lifecycle_details'] if attributes[:'lifecycle_details']

      self.freeform_tags = attributes[:'freeformTags'] if attributes[:'freeformTags']

      raise 'You cannot provide both :freeformTags and :freeform_tags' if attributes.key?(:'freeformTags') && attributes.key?(:'freeform_tags')

      self.freeform_tags = attributes[:'freeform_tags'] if attributes[:'freeform_tags']

      self.defined_tags = attributes[:'definedTags'] if attributes[:'definedTags']

      raise 'You cannot provide both :definedTags and :defined_tags' if attributes.key?(:'definedTags') && attributes.key?(:'defined_tags')

      self.defined_tags = attributes[:'defined_tags'] if attributes[:'defined_tags']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] lifecycle_state Object to be assigned
    def lifecycle_state=(lifecycle_state)
      # rubocop:disable Style/ConditionalAssignment
      if lifecycle_state && !LIFECYCLE_STATE_ENUM.include?(lifecycle_state)
        OCI.logger.debug("Unknown value for 'lifecycle_state' [" + lifecycle_state + "]. Mapping to 'LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @lifecycle_state = LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE
      else
        @lifecycle_state = lifecycle_state
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        id == other.id &&
        time_accepted == other.time_accepted &&
        time_started == other.time_started &&
        time_finished == other.time_finished &&
        created_by == other.created_by &&
        project_id == other.project_id &&
        compartment_id == other.compartment_id &&
        job_id == other.job_id &&
        display_name == other.display_name &&
        job_configuration_override_details == other.job_configuration_override_details &&
        job_infrastructure_configuration_details == other.job_infrastructure_configuration_details &&
        job_log_configuration_override_details == other.job_log_configuration_override_details &&
        job_storage_mount_configuration_details_list == other.job_storage_mount_configuration_details_list &&
        log_details == other.log_details &&
        lifecycle_state == other.lifecycle_state &&
        lifecycle_details == other.lifecycle_details &&
        freeform_tags == other.freeform_tags &&
        defined_tags == other.defined_tags
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [id, time_accepted, time_started, time_finished, created_by, project_id, compartment_id, job_id, display_name, job_configuration_override_details, job_infrastructure_configuration_details, job_log_configuration_override_details, job_storage_mount_configuration_details_list, log_details, lifecycle_state, lifecycle_details, freeform_tags, defined_tags].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
