# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20190101
require 'date'
require_relative 'job_infrastructure_configuration_details'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # The standalone job infrastructure configuration.
  class DataScience::Models::StandaloneJobInfrastructureConfigurationDetails < DataScience::Models::JobInfrastructureConfigurationDetails
    # **[Required]** The shape used to launch the job run instances.
    # @return [String]
    attr_accessor :shape_name

    # **[Required]** The subnet to create a secondary vnic in to attach to the instance running the job
    #
    # @return [String]
    attr_accessor :subnet_id

    # **[Required]** The size of the block storage volume to attach to the instance running the job
    #
    # @return [Integer]
    attr_accessor :block_storage_size_in_gbs

    # @return [OCI::DataScience::Models::JobShapeConfigDetails]
    attr_accessor :job_shape_config_details

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'job_infrastructure_type': :'jobInfrastructureType',
        'shape_name': :'shapeName',
        'subnet_id': :'subnetId',
        'block_storage_size_in_gbs': :'blockStorageSizeInGBs',
        'job_shape_config_details': :'jobShapeConfigDetails'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'job_infrastructure_type': :'String',
        'shape_name': :'String',
        'subnet_id': :'String',
        'block_storage_size_in_gbs': :'Integer',
        'job_shape_config_details': :'OCI::DataScience::Models::JobShapeConfigDetails'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :shape_name The value to assign to the {#shape_name} property
    # @option attributes [String] :subnet_id The value to assign to the {#subnet_id} property
    # @option attributes [Integer] :block_storage_size_in_gbs The value to assign to the {#block_storage_size_in_gbs} property
    # @option attributes [OCI::DataScience::Models::JobShapeConfigDetails] :job_shape_config_details The value to assign to the {#job_shape_config_details} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      attributes['jobInfrastructureType'] = 'STANDALONE'

      super(attributes)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.shape_name = attributes[:'shapeName'] if attributes[:'shapeName']

      raise 'You cannot provide both :shapeName and :shape_name' if attributes.key?(:'shapeName') && attributes.key?(:'shape_name')

      self.shape_name = attributes[:'shape_name'] if attributes[:'shape_name']

      self.subnet_id = attributes[:'subnetId'] if attributes[:'subnetId']

      raise 'You cannot provide both :subnetId and :subnet_id' if attributes.key?(:'subnetId') && attributes.key?(:'subnet_id')

      self.subnet_id = attributes[:'subnet_id'] if attributes[:'subnet_id']

      self.block_storage_size_in_gbs = attributes[:'blockStorageSizeInGBs'] if attributes[:'blockStorageSizeInGBs']

      raise 'You cannot provide both :blockStorageSizeInGBs and :block_storage_size_in_gbs' if attributes.key?(:'blockStorageSizeInGBs') && attributes.key?(:'block_storage_size_in_gbs')

      self.block_storage_size_in_gbs = attributes[:'block_storage_size_in_gbs'] if attributes[:'block_storage_size_in_gbs']

      self.job_shape_config_details = attributes[:'jobShapeConfigDetails'] if attributes[:'jobShapeConfigDetails']

      raise 'You cannot provide both :jobShapeConfigDetails and :job_shape_config_details' if attributes.key?(:'jobShapeConfigDetails') && attributes.key?(:'job_shape_config_details')

      self.job_shape_config_details = attributes[:'job_shape_config_details'] if attributes[:'job_shape_config_details']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        job_infrastructure_type == other.job_infrastructure_type &&
        shape_name == other.shape_name &&
        subnet_id == other.subnet_id &&
        block_storage_size_in_gbs == other.block_storage_size_in_gbs &&
        job_shape_config_details == other.job_shape_config_details
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [job_infrastructure_type, shape_name, subnet_id, block_storage_size_in_gbs, job_shape_config_details].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
