# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20160918
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Details of the cloud Autonomous VM cluster.
  #
  class Database::Models::CloudAutonomousVmCluster
    LIFECYCLE_STATE_ENUM = [
      LIFECYCLE_STATE_PROVISIONING = 'PROVISIONING'.freeze,
      LIFECYCLE_STATE_AVAILABLE = 'AVAILABLE'.freeze,
      LIFECYCLE_STATE_UPDATING = 'UPDATING'.freeze,
      LIFECYCLE_STATE_TERMINATING = 'TERMINATING'.freeze,
      LIFECYCLE_STATE_TERMINATED = 'TERMINATED'.freeze,
      LIFECYCLE_STATE_FAILED = 'FAILED'.freeze,
      LIFECYCLE_STATE_MAINTENANCE_IN_PROGRESS = 'MAINTENANCE_IN_PROGRESS'.freeze,
      LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    COMPUTE_MODEL_ENUM = [
      COMPUTE_MODEL_ECPU = 'ECPU'.freeze,
      COMPUTE_MODEL_OCPU = 'OCPU'.freeze,
      COMPUTE_MODEL_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    LICENSE_MODEL_ENUM = [
      LICENSE_MODEL_LICENSE_INCLUDED = 'LICENSE_INCLUDED'.freeze,
      LICENSE_MODEL_BRING_YOUR_OWN_LICENSE = 'BRING_YOUR_OWN_LICENSE'.freeze,
      LICENSE_MODEL_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the Cloud Autonomous VM cluster.
    # @return [String]
    attr_accessor :id

    # **[Required]** The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the compartment.
    # @return [String]
    attr_accessor :compartment_id

    # User defined description of the cloud Autonomous VM cluster.
    # @return [String]
    attr_accessor :description

    # **[Required]** The name of the availability domain that the cloud Autonomous VM cluster is located in.
    # @return [String]
    attr_accessor :availability_domain

    # **[Required]** The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the subnet the cloud Autonomous VM Cluster is associated with.
    #
    # **Subnet Restrictions:**
    # - For Exadata and virtual machine 2-node RAC DB systems, do not use a subnet that overlaps with 192.168.128.0/20.
    #
    # These subnets are used by the Oracle Clusterware private interconnect on the database instance.
    # Specifying an overlapping subnet will cause the private interconnect to malfunction.
    # This restriction applies to both the client subnet and backup subnet.
    #
    # @return [String]
    attr_accessor :subnet_id

    # The list of [OCIDs](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) for the network security groups (NSGs) to which this resource belongs. Setting this to an empty list removes all resources from all NSGs. For more information about NSGs, see [Security Rules](https://docs.cloud.oracle.com/Content/Network/Concepts/securityrules.htm).
    # **NsgIds restrictions:**
    # - A network security group (NSG) is optional for Autonomous Databases with private access. The nsgIds list can be empty.
    #
    # @return [Array<String>]
    attr_accessor :nsg_ids

    # The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the last maintenance update history. This value is updated when a maintenance update starts.
    # @return [String]
    attr_accessor :last_update_history_entry_id

    # **[Required]** The current state of the cloud Autonomous VM cluster.
    # @return [String]
    attr_reader :lifecycle_state

    # **[Required]** The user-friendly name for the cloud Autonomous VM cluster. The name does not need to be unique.
    # @return [String]
    attr_accessor :display_name

    # The date and time that the cloud Autonomous VM cluster was created.
    # @return [DateTime]
    attr_accessor :time_created

    # The last date and time that the cloud Autonomous VM cluster was updated.
    # @return [DateTime]
    attr_accessor :time_updated

    # The time zone of the Cloud Autonomous VM Cluster.
    # @return [String]
    attr_accessor :cluster_time_zone

    # Additional information about the current lifecycle state.
    # @return [String]
    attr_accessor :lifecycle_details

    # The hostname for the cloud Autonomous VM cluster.
    # @return [String]
    attr_accessor :hostname

    # The domain name for the cloud Autonomous VM cluster.
    # @return [String]
    attr_accessor :domain

    # **[Required]** The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the cloud Exadata infrastructure.
    # @return [String]
    attr_accessor :cloud_exadata_infrastructure_id

    # The model name of the Exadata hardware running the cloud Autonomous VM cluster.
    #
    # @return [String]
    attr_accessor :shape

    # The number of database servers in the cloud VM cluster.
    #
    # @return [Integer]
    attr_accessor :node_count

    # The total data storage allocated, in terabytes (TB).
    # @return [Float]
    attr_accessor :data_storage_size_in_tbs

    # The total data storage allocated, in gigabytes (GB).
    # @return [Float]
    attr_accessor :data_storage_size_in_gbs

    # The number of CPU cores on the cloud Autonomous VM cluster.
    # @return [Integer]
    attr_accessor :cpu_core_count

    # The number of CPU cores on the cloud Autonomous VM cluster. Only 1 decimal place is allowed for the fractional part.
    # @return [Float]
    attr_accessor :ocpu_count

    # The compute model of the Cloud Autonomous VM Cluster.
    #
    # @return [String]
    attr_reader :compute_model

    # Enable mutual TLS(mTLS) authentication for database at time of provisioning a VMCluster. This is applicable to database TLS Certificates only. Default is TLS
    # @return [BOOLEAN]
    attr_accessor :is_mtls_enabled_vm_cluster

    # The number of CPU cores enabled per VM cluster node.
    # @return [Integer]
    attr_accessor :cpu_core_count_per_node

    # The memory allocated in GBs.
    # @return [Integer]
    attr_accessor :memory_size_in_gbs

    # The Oracle license model that applies to the Oracle Autonomous Database. Bring your own license (BYOL) allows you to apply your current on-premises Oracle software licenses to equivalent, highly automated Oracle services in the cloud.
    # License Included allows you to subscribe to new Oracle Database software licenses and the Oracle Database service.
    # Note that when provisioning an [Autonomous Database on dedicated Exadata infrastructure](https://docs.oracle.com/en/cloud/paas/autonomous-database/index.html), this attribute must be null. It is already set at the
    # Autonomous Exadata Infrastructure level. When provisioning an [Autonomous Database Serverless] (https://docs.oracle.com/en/cloud/paas/autonomous-database/index.html) database, if a value is not specified, the system defaults the value to `BRING_YOUR_OWN_LICENSE`.
    #
    # This cannot be updated in parallel with any of the following: cpuCoreCount, computeCount, maxCpuCoreCount, dataStorageSizeInTBs, adminPassword, isMTLSConnectionRequired, dbWorkload, privateEndpointLabel, nsgIds, dbVersion, dbName, scheduledOperations, dbToolsDetails, or isFreeTier.
    #
    # @return [String]
    attr_reader :license_model

    # The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the last maintenance run.
    # @return [String]
    attr_accessor :last_maintenance_run_id

    # The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the next maintenance run.
    # @return [String]
    attr_accessor :next_maintenance_run_id

    # @return [OCI::Database::Models::MaintenanceWindow]
    attr_accessor :maintenance_window

    # The SCAN Listenenr TLS port. Default is 2484.
    # @return [Integer]
    attr_accessor :scan_listener_port_tls

    # The SCAN Listener Non TLS port. Default is 1521.
    # @return [Integer]
    attr_accessor :scan_listener_port_non_tls

    # Free-form tags for this resource. Each tag is a simple key-value pair with no predefined name, type, or namespace.
    # For more information, see [Resource Tags](https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm).
    #
    # Example: `{\"Department\": \"Finance\"}`
    #
    # @return [Hash<String, String>]
    attr_accessor :freeform_tags

    # Defined tags for this resource. Each key is predefined and scoped to a namespace.
    # For more information, see [Resource Tags](https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm).
    #
    # @return [Hash<String, Hash<String, Object>>]
    attr_accessor :defined_tags

    # The date and time of Database SSL certificate expiration.
    # @return [DateTime]
    attr_accessor :time_database_ssl_certificate_expires

    # The date and time of ORDS certificate expiration.
    # @return [DateTime]
    attr_accessor :time_ords_certificate_expires

    # CPU cores available for allocation to Autonomous Databases.
    # @return [Float]
    attr_accessor :available_cpus

    # For Autonomous Databases on Dedicated Exadata Infrastructure:
    # - These are the CPUs that continue to be included in the count of CPUs available to the Autonomous Container Database even after one of its Autonomous Database is terminated or scaled down. You can release them to the available CPUs at its parent Autonomous VM Cluster level by restarting the Autonomous Container Database.
    # - The CPU type (OCPUs or ECPUs) is determined by the parent Autonomous Exadata VM Cluster's compute model.
    #
    # @return [Float]
    attr_accessor :reclaimable_cpus

    # The number of Autonomous Container Databases that can be created with the currently available local storage.
    # @return [Integer]
    attr_accessor :available_container_databases

    # The total number of Autonomous Container Databases that can be created with the allocated local storage.
    # @return [Integer]
    attr_accessor :total_container_databases

    # The data disk group size available for Autonomous Databases, in TBs.
    # @return [Float]
    attr_accessor :available_autonomous_data_storage_size_in_tbs

    # The data disk group size allocated for Autonomous Databases, in TBs.
    # @return [Float]
    attr_accessor :autonomous_data_storage_size_in_tbs

    # The local node storage allocated in GBs.
    # @return [Integer]
    attr_accessor :db_node_storage_size_in_gbs

    # The amount of memory (in GBs) enabled per OCPU or ECPU.
    #
    # @return [Integer]
    attr_accessor :memory_per_oracle_compute_unit_in_gbs

    # The list of [OCIDs](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the Db servers.
    # @return [Array<String>]
    attr_accessor :db_servers

    # The percentage of total number of CPUs used in an Autonomous VM Cluster.
    # @return [Float]
    attr_accessor :cpu_percentage

    # The percentage of the data storage used for the Autonomous Databases in an Autonomous VM Cluster.
    # @return [Float]
    attr_accessor :autonomous_data_storage_percentage

    # The number of CPUs provisioned in an Autonomous VM Cluster.
    # @return [Float]
    attr_accessor :provisioned_cpus

    # The total number of CPUs in an Autonomous VM Cluster.
    # @return [Float]
    attr_accessor :total_cpus

    # The total data disk group size for Autonomous Databases, in TBs.
    # @return [Float]
    attr_accessor :total_autonomous_data_storage_in_tbs

    # The number of CPUs reserved in an Autonomous VM Cluster.
    # @return [Float]
    attr_accessor :reserved_cpus

    # The number of provisionable Autonomous Container Databases in an Autonomous VM Cluster.
    # @return [Integer]
    attr_accessor :provisionable_autonomous_container_databases

    # The number of provisioned Autonomous Container Databases in an Autonomous VM Cluster.
    # @return [Integer]
    attr_accessor :provisioned_autonomous_container_databases

    # The number of non-provisionable Autonomous Container Databases in an Autonomous VM Cluster.
    # @return [Integer]
    attr_accessor :non_provisionable_autonomous_container_databases

    # The lowest value to which exadataStorage in TBs can be scaled down.
    # @return [Float]
    attr_accessor :exadata_storage_in_tbs_lowest_scaled_value

    # The lowest value to which ocpus can be scaled down.
    # @return [Integer]
    attr_accessor :ocpus_lowest_scaled_value

    # The lowest value to which ACDs can be scaled down.
    # @return [Integer]
    attr_accessor :max_acds_lowest_scaled_value

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'id',
        'compartment_id': :'compartmentId',
        'description': :'description',
        'availability_domain': :'availabilityDomain',
        'subnet_id': :'subnetId',
        'nsg_ids': :'nsgIds',
        'last_update_history_entry_id': :'lastUpdateHistoryEntryId',
        'lifecycle_state': :'lifecycleState',
        'display_name': :'displayName',
        'time_created': :'timeCreated',
        'time_updated': :'timeUpdated',
        'cluster_time_zone': :'clusterTimeZone',
        'lifecycle_details': :'lifecycleDetails',
        'hostname': :'hostname',
        'domain': :'domain',
        'cloud_exadata_infrastructure_id': :'cloudExadataInfrastructureId',
        'shape': :'shape',
        'node_count': :'nodeCount',
        'data_storage_size_in_tbs': :'dataStorageSizeInTBs',
        'data_storage_size_in_gbs': :'dataStorageSizeInGBs',
        'cpu_core_count': :'cpuCoreCount',
        'ocpu_count': :'ocpuCount',
        'compute_model': :'computeModel',
        'is_mtls_enabled_vm_cluster': :'isMtlsEnabledVmCluster',
        'cpu_core_count_per_node': :'cpuCoreCountPerNode',
        'memory_size_in_gbs': :'memorySizeInGBs',
        'license_model': :'licenseModel',
        'last_maintenance_run_id': :'lastMaintenanceRunId',
        'next_maintenance_run_id': :'nextMaintenanceRunId',
        'maintenance_window': :'maintenanceWindow',
        'scan_listener_port_tls': :'scanListenerPortTls',
        'scan_listener_port_non_tls': :'scanListenerPortNonTls',
        'freeform_tags': :'freeformTags',
        'defined_tags': :'definedTags',
        'time_database_ssl_certificate_expires': :'timeDatabaseSslCertificateExpires',
        'time_ords_certificate_expires': :'timeOrdsCertificateExpires',
        'available_cpus': :'availableCpus',
        'reclaimable_cpus': :'reclaimableCpus',
        'available_container_databases': :'availableContainerDatabases',
        'total_container_databases': :'totalContainerDatabases',
        'available_autonomous_data_storage_size_in_tbs': :'availableAutonomousDataStorageSizeInTBs',
        'autonomous_data_storage_size_in_tbs': :'autonomousDataStorageSizeInTBs',
        'db_node_storage_size_in_gbs': :'dbNodeStorageSizeInGBs',
        'memory_per_oracle_compute_unit_in_gbs': :'memoryPerOracleComputeUnitInGBs',
        'db_servers': :'dbServers',
        'cpu_percentage': :'cpuPercentage',
        'autonomous_data_storage_percentage': :'autonomousDataStoragePercentage',
        'provisioned_cpus': :'provisionedCpus',
        'total_cpus': :'totalCpus',
        'total_autonomous_data_storage_in_tbs': :'totalAutonomousDataStorageInTBs',
        'reserved_cpus': :'reservedCpus',
        'provisionable_autonomous_container_databases': :'provisionableAutonomousContainerDatabases',
        'provisioned_autonomous_container_databases': :'provisionedAutonomousContainerDatabases',
        'non_provisionable_autonomous_container_databases': :'nonProvisionableAutonomousContainerDatabases',
        'exadata_storage_in_tbs_lowest_scaled_value': :'exadataStorageInTBsLowestScaledValue',
        'ocpus_lowest_scaled_value': :'ocpusLowestScaledValue',
        'max_acds_lowest_scaled_value': :'maxAcdsLowestScaledValue'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'String',
        'compartment_id': :'String',
        'description': :'String',
        'availability_domain': :'String',
        'subnet_id': :'String',
        'nsg_ids': :'Array<String>',
        'last_update_history_entry_id': :'String',
        'lifecycle_state': :'String',
        'display_name': :'String',
        'time_created': :'DateTime',
        'time_updated': :'DateTime',
        'cluster_time_zone': :'String',
        'lifecycle_details': :'String',
        'hostname': :'String',
        'domain': :'String',
        'cloud_exadata_infrastructure_id': :'String',
        'shape': :'String',
        'node_count': :'Integer',
        'data_storage_size_in_tbs': :'Float',
        'data_storage_size_in_gbs': :'Float',
        'cpu_core_count': :'Integer',
        'ocpu_count': :'Float',
        'compute_model': :'String',
        'is_mtls_enabled_vm_cluster': :'BOOLEAN',
        'cpu_core_count_per_node': :'Integer',
        'memory_size_in_gbs': :'Integer',
        'license_model': :'String',
        'last_maintenance_run_id': :'String',
        'next_maintenance_run_id': :'String',
        'maintenance_window': :'OCI::Database::Models::MaintenanceWindow',
        'scan_listener_port_tls': :'Integer',
        'scan_listener_port_non_tls': :'Integer',
        'freeform_tags': :'Hash<String, String>',
        'defined_tags': :'Hash<String, Hash<String, Object>>',
        'time_database_ssl_certificate_expires': :'DateTime',
        'time_ords_certificate_expires': :'DateTime',
        'available_cpus': :'Float',
        'reclaimable_cpus': :'Float',
        'available_container_databases': :'Integer',
        'total_container_databases': :'Integer',
        'available_autonomous_data_storage_size_in_tbs': :'Float',
        'autonomous_data_storage_size_in_tbs': :'Float',
        'db_node_storage_size_in_gbs': :'Integer',
        'memory_per_oracle_compute_unit_in_gbs': :'Integer',
        'db_servers': :'Array<String>',
        'cpu_percentage': :'Float',
        'autonomous_data_storage_percentage': :'Float',
        'provisioned_cpus': :'Float',
        'total_cpus': :'Float',
        'total_autonomous_data_storage_in_tbs': :'Float',
        'reserved_cpus': :'Float',
        'provisionable_autonomous_container_databases': :'Integer',
        'provisioned_autonomous_container_databases': :'Integer',
        'non_provisionable_autonomous_container_databases': :'Integer',
        'exadata_storage_in_tbs_lowest_scaled_value': :'Float',
        'ocpus_lowest_scaled_value': :'Integer',
        'max_acds_lowest_scaled_value': :'Integer'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :id The value to assign to the {#id} property
    # @option attributes [String] :compartment_id The value to assign to the {#compartment_id} property
    # @option attributes [String] :description The value to assign to the {#description} property
    # @option attributes [String] :availability_domain The value to assign to the {#availability_domain} property
    # @option attributes [String] :subnet_id The value to assign to the {#subnet_id} property
    # @option attributes [Array<String>] :nsg_ids The value to assign to the {#nsg_ids} property
    # @option attributes [String] :last_update_history_entry_id The value to assign to the {#last_update_history_entry_id} property
    # @option attributes [String] :lifecycle_state The value to assign to the {#lifecycle_state} property
    # @option attributes [String] :display_name The value to assign to the {#display_name} property
    # @option attributes [DateTime] :time_created The value to assign to the {#time_created} property
    # @option attributes [DateTime] :time_updated The value to assign to the {#time_updated} property
    # @option attributes [String] :cluster_time_zone The value to assign to the {#cluster_time_zone} property
    # @option attributes [String] :lifecycle_details The value to assign to the {#lifecycle_details} property
    # @option attributes [String] :hostname The value to assign to the {#hostname} property
    # @option attributes [String] :domain The value to assign to the {#domain} property
    # @option attributes [String] :cloud_exadata_infrastructure_id The value to assign to the {#cloud_exadata_infrastructure_id} property
    # @option attributes [String] :shape The value to assign to the {#shape} property
    # @option attributes [Integer] :node_count The value to assign to the {#node_count} property
    # @option attributes [Float] :data_storage_size_in_tbs The value to assign to the {#data_storage_size_in_tbs} property
    # @option attributes [Float] :data_storage_size_in_gbs The value to assign to the {#data_storage_size_in_gbs} property
    # @option attributes [Integer] :cpu_core_count The value to assign to the {#cpu_core_count} property
    # @option attributes [Float] :ocpu_count The value to assign to the {#ocpu_count} property
    # @option attributes [String] :compute_model The value to assign to the {#compute_model} property
    # @option attributes [BOOLEAN] :is_mtls_enabled_vm_cluster The value to assign to the {#is_mtls_enabled_vm_cluster} property
    # @option attributes [Integer] :cpu_core_count_per_node The value to assign to the {#cpu_core_count_per_node} property
    # @option attributes [Integer] :memory_size_in_gbs The value to assign to the {#memory_size_in_gbs} property
    # @option attributes [String] :license_model The value to assign to the {#license_model} property
    # @option attributes [String] :last_maintenance_run_id The value to assign to the {#last_maintenance_run_id} property
    # @option attributes [String] :next_maintenance_run_id The value to assign to the {#next_maintenance_run_id} property
    # @option attributes [OCI::Database::Models::MaintenanceWindow] :maintenance_window The value to assign to the {#maintenance_window} property
    # @option attributes [Integer] :scan_listener_port_tls The value to assign to the {#scan_listener_port_tls} property
    # @option attributes [Integer] :scan_listener_port_non_tls The value to assign to the {#scan_listener_port_non_tls} property
    # @option attributes [Hash<String, String>] :freeform_tags The value to assign to the {#freeform_tags} property
    # @option attributes [Hash<String, Hash<String, Object>>] :defined_tags The value to assign to the {#defined_tags} property
    # @option attributes [DateTime] :time_database_ssl_certificate_expires The value to assign to the {#time_database_ssl_certificate_expires} property
    # @option attributes [DateTime] :time_ords_certificate_expires The value to assign to the {#time_ords_certificate_expires} property
    # @option attributes [Float] :available_cpus The value to assign to the {#available_cpus} property
    # @option attributes [Float] :reclaimable_cpus The value to assign to the {#reclaimable_cpus} property
    # @option attributes [Integer] :available_container_databases The value to assign to the {#available_container_databases} property
    # @option attributes [Integer] :total_container_databases The value to assign to the {#total_container_databases} property
    # @option attributes [Float] :available_autonomous_data_storage_size_in_tbs The value to assign to the {#available_autonomous_data_storage_size_in_tbs} property
    # @option attributes [Float] :autonomous_data_storage_size_in_tbs The value to assign to the {#autonomous_data_storage_size_in_tbs} property
    # @option attributes [Integer] :db_node_storage_size_in_gbs The value to assign to the {#db_node_storage_size_in_gbs} property
    # @option attributes [Integer] :memory_per_oracle_compute_unit_in_gbs The value to assign to the {#memory_per_oracle_compute_unit_in_gbs} property
    # @option attributes [Array<String>] :db_servers The value to assign to the {#db_servers} property
    # @option attributes [Float] :cpu_percentage The value to assign to the {#cpu_percentage} property
    # @option attributes [Float] :autonomous_data_storage_percentage The value to assign to the {#autonomous_data_storage_percentage} property
    # @option attributes [Float] :provisioned_cpus The value to assign to the {#provisioned_cpus} property
    # @option attributes [Float] :total_cpus The value to assign to the {#total_cpus} property
    # @option attributes [Float] :total_autonomous_data_storage_in_tbs The value to assign to the {#total_autonomous_data_storage_in_tbs} property
    # @option attributes [Float] :reserved_cpus The value to assign to the {#reserved_cpus} property
    # @option attributes [Integer] :provisionable_autonomous_container_databases The value to assign to the {#provisionable_autonomous_container_databases} property
    # @option attributes [Integer] :provisioned_autonomous_container_databases The value to assign to the {#provisioned_autonomous_container_databases} property
    # @option attributes [Integer] :non_provisionable_autonomous_container_databases The value to assign to the {#non_provisionable_autonomous_container_databases} property
    # @option attributes [Float] :exadata_storage_in_tbs_lowest_scaled_value The value to assign to the {#exadata_storage_in_tbs_lowest_scaled_value} property
    # @option attributes [Integer] :ocpus_lowest_scaled_value The value to assign to the {#ocpus_lowest_scaled_value} property
    # @option attributes [Integer] :max_acds_lowest_scaled_value The value to assign to the {#max_acds_lowest_scaled_value} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      self.id = attributes[:'id'] if attributes[:'id']

      self.compartment_id = attributes[:'compartmentId'] if attributes[:'compartmentId']

      raise 'You cannot provide both :compartmentId and :compartment_id' if attributes.key?(:'compartmentId') && attributes.key?(:'compartment_id')

      self.compartment_id = attributes[:'compartment_id'] if attributes[:'compartment_id']

      self.description = attributes[:'description'] if attributes[:'description']

      self.availability_domain = attributes[:'availabilityDomain'] if attributes[:'availabilityDomain']

      raise 'You cannot provide both :availabilityDomain and :availability_domain' if attributes.key?(:'availabilityDomain') && attributes.key?(:'availability_domain')

      self.availability_domain = attributes[:'availability_domain'] if attributes[:'availability_domain']

      self.subnet_id = attributes[:'subnetId'] if attributes[:'subnetId']

      raise 'You cannot provide both :subnetId and :subnet_id' if attributes.key?(:'subnetId') && attributes.key?(:'subnet_id')

      self.subnet_id = attributes[:'subnet_id'] if attributes[:'subnet_id']

      self.nsg_ids = attributes[:'nsgIds'] if attributes[:'nsgIds']

      raise 'You cannot provide both :nsgIds and :nsg_ids' if attributes.key?(:'nsgIds') && attributes.key?(:'nsg_ids')

      self.nsg_ids = attributes[:'nsg_ids'] if attributes[:'nsg_ids']

      self.last_update_history_entry_id = attributes[:'lastUpdateHistoryEntryId'] if attributes[:'lastUpdateHistoryEntryId']

      raise 'You cannot provide both :lastUpdateHistoryEntryId and :last_update_history_entry_id' if attributes.key?(:'lastUpdateHistoryEntryId') && attributes.key?(:'last_update_history_entry_id')

      self.last_update_history_entry_id = attributes[:'last_update_history_entry_id'] if attributes[:'last_update_history_entry_id']

      self.lifecycle_state = attributes[:'lifecycleState'] if attributes[:'lifecycleState']

      raise 'You cannot provide both :lifecycleState and :lifecycle_state' if attributes.key?(:'lifecycleState') && attributes.key?(:'lifecycle_state')

      self.lifecycle_state = attributes[:'lifecycle_state'] if attributes[:'lifecycle_state']

      self.display_name = attributes[:'displayName'] if attributes[:'displayName']

      raise 'You cannot provide both :displayName and :display_name' if attributes.key?(:'displayName') && attributes.key?(:'display_name')

      self.display_name = attributes[:'display_name'] if attributes[:'display_name']

      self.time_created = attributes[:'timeCreated'] if attributes[:'timeCreated']

      raise 'You cannot provide both :timeCreated and :time_created' if attributes.key?(:'timeCreated') && attributes.key?(:'time_created')

      self.time_created = attributes[:'time_created'] if attributes[:'time_created']

      self.time_updated = attributes[:'timeUpdated'] if attributes[:'timeUpdated']

      raise 'You cannot provide both :timeUpdated and :time_updated' if attributes.key?(:'timeUpdated') && attributes.key?(:'time_updated')

      self.time_updated = attributes[:'time_updated'] if attributes[:'time_updated']

      self.cluster_time_zone = attributes[:'clusterTimeZone'] if attributes[:'clusterTimeZone']

      raise 'You cannot provide both :clusterTimeZone and :cluster_time_zone' if attributes.key?(:'clusterTimeZone') && attributes.key?(:'cluster_time_zone')

      self.cluster_time_zone = attributes[:'cluster_time_zone'] if attributes[:'cluster_time_zone']

      self.lifecycle_details = attributes[:'lifecycleDetails'] if attributes[:'lifecycleDetails']

      raise 'You cannot provide both :lifecycleDetails and :lifecycle_details' if attributes.key?(:'lifecycleDetails') && attributes.key?(:'lifecycle_details')

      self.lifecycle_details = attributes[:'lifecycle_details'] if attributes[:'lifecycle_details']

      self.hostname = attributes[:'hostname'] if attributes[:'hostname']

      self.domain = attributes[:'domain'] if attributes[:'domain']

      self.cloud_exadata_infrastructure_id = attributes[:'cloudExadataInfrastructureId'] if attributes[:'cloudExadataInfrastructureId']

      raise 'You cannot provide both :cloudExadataInfrastructureId and :cloud_exadata_infrastructure_id' if attributes.key?(:'cloudExadataInfrastructureId') && attributes.key?(:'cloud_exadata_infrastructure_id')

      self.cloud_exadata_infrastructure_id = attributes[:'cloud_exadata_infrastructure_id'] if attributes[:'cloud_exadata_infrastructure_id']

      self.shape = attributes[:'shape'] if attributes[:'shape']

      self.node_count = attributes[:'nodeCount'] if attributes[:'nodeCount']

      raise 'You cannot provide both :nodeCount and :node_count' if attributes.key?(:'nodeCount') && attributes.key?(:'node_count')

      self.node_count = attributes[:'node_count'] if attributes[:'node_count']

      self.data_storage_size_in_tbs = attributes[:'dataStorageSizeInTBs'] if attributes[:'dataStorageSizeInTBs']

      raise 'You cannot provide both :dataStorageSizeInTBs and :data_storage_size_in_tbs' if attributes.key?(:'dataStorageSizeInTBs') && attributes.key?(:'data_storage_size_in_tbs')

      self.data_storage_size_in_tbs = attributes[:'data_storage_size_in_tbs'] if attributes[:'data_storage_size_in_tbs']

      self.data_storage_size_in_gbs = attributes[:'dataStorageSizeInGBs'] if attributes[:'dataStorageSizeInGBs']

      raise 'You cannot provide both :dataStorageSizeInGBs and :data_storage_size_in_gbs' if attributes.key?(:'dataStorageSizeInGBs') && attributes.key?(:'data_storage_size_in_gbs')

      self.data_storage_size_in_gbs = attributes[:'data_storage_size_in_gbs'] if attributes[:'data_storage_size_in_gbs']

      self.cpu_core_count = attributes[:'cpuCoreCount'] if attributes[:'cpuCoreCount']

      raise 'You cannot provide both :cpuCoreCount and :cpu_core_count' if attributes.key?(:'cpuCoreCount') && attributes.key?(:'cpu_core_count')

      self.cpu_core_count = attributes[:'cpu_core_count'] if attributes[:'cpu_core_count']

      self.ocpu_count = attributes[:'ocpuCount'] if attributes[:'ocpuCount']

      raise 'You cannot provide both :ocpuCount and :ocpu_count' if attributes.key?(:'ocpuCount') && attributes.key?(:'ocpu_count')

      self.ocpu_count = attributes[:'ocpu_count'] if attributes[:'ocpu_count']

      self.compute_model = attributes[:'computeModel'] if attributes[:'computeModel']

      raise 'You cannot provide both :computeModel and :compute_model' if attributes.key?(:'computeModel') && attributes.key?(:'compute_model')

      self.compute_model = attributes[:'compute_model'] if attributes[:'compute_model']

      self.is_mtls_enabled_vm_cluster = attributes[:'isMtlsEnabledVmCluster'] unless attributes[:'isMtlsEnabledVmCluster'].nil?
      self.is_mtls_enabled_vm_cluster = false if is_mtls_enabled_vm_cluster.nil? && !attributes.key?(:'isMtlsEnabledVmCluster') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :isMtlsEnabledVmCluster and :is_mtls_enabled_vm_cluster' if attributes.key?(:'isMtlsEnabledVmCluster') && attributes.key?(:'is_mtls_enabled_vm_cluster')

      self.is_mtls_enabled_vm_cluster = attributes[:'is_mtls_enabled_vm_cluster'] unless attributes[:'is_mtls_enabled_vm_cluster'].nil?
      self.is_mtls_enabled_vm_cluster = false if is_mtls_enabled_vm_cluster.nil? && !attributes.key?(:'isMtlsEnabledVmCluster') && !attributes.key?(:'is_mtls_enabled_vm_cluster') # rubocop:disable Style/StringLiterals

      self.cpu_core_count_per_node = attributes[:'cpuCoreCountPerNode'] if attributes[:'cpuCoreCountPerNode']

      raise 'You cannot provide both :cpuCoreCountPerNode and :cpu_core_count_per_node' if attributes.key?(:'cpuCoreCountPerNode') && attributes.key?(:'cpu_core_count_per_node')

      self.cpu_core_count_per_node = attributes[:'cpu_core_count_per_node'] if attributes[:'cpu_core_count_per_node']

      self.memory_size_in_gbs = attributes[:'memorySizeInGBs'] if attributes[:'memorySizeInGBs']

      raise 'You cannot provide both :memorySizeInGBs and :memory_size_in_gbs' if attributes.key?(:'memorySizeInGBs') && attributes.key?(:'memory_size_in_gbs')

      self.memory_size_in_gbs = attributes[:'memory_size_in_gbs'] if attributes[:'memory_size_in_gbs']

      self.license_model = attributes[:'licenseModel'] if attributes[:'licenseModel']

      raise 'You cannot provide both :licenseModel and :license_model' if attributes.key?(:'licenseModel') && attributes.key?(:'license_model')

      self.license_model = attributes[:'license_model'] if attributes[:'license_model']

      self.last_maintenance_run_id = attributes[:'lastMaintenanceRunId'] if attributes[:'lastMaintenanceRunId']

      raise 'You cannot provide both :lastMaintenanceRunId and :last_maintenance_run_id' if attributes.key?(:'lastMaintenanceRunId') && attributes.key?(:'last_maintenance_run_id')

      self.last_maintenance_run_id = attributes[:'last_maintenance_run_id'] if attributes[:'last_maintenance_run_id']

      self.next_maintenance_run_id = attributes[:'nextMaintenanceRunId'] if attributes[:'nextMaintenanceRunId']

      raise 'You cannot provide both :nextMaintenanceRunId and :next_maintenance_run_id' if attributes.key?(:'nextMaintenanceRunId') && attributes.key?(:'next_maintenance_run_id')

      self.next_maintenance_run_id = attributes[:'next_maintenance_run_id'] if attributes[:'next_maintenance_run_id']

      self.maintenance_window = attributes[:'maintenanceWindow'] if attributes[:'maintenanceWindow']

      raise 'You cannot provide both :maintenanceWindow and :maintenance_window' if attributes.key?(:'maintenanceWindow') && attributes.key?(:'maintenance_window')

      self.maintenance_window = attributes[:'maintenance_window'] if attributes[:'maintenance_window']

      self.scan_listener_port_tls = attributes[:'scanListenerPortTls'] if attributes[:'scanListenerPortTls']

      raise 'You cannot provide both :scanListenerPortTls and :scan_listener_port_tls' if attributes.key?(:'scanListenerPortTls') && attributes.key?(:'scan_listener_port_tls')

      self.scan_listener_port_tls = attributes[:'scan_listener_port_tls'] if attributes[:'scan_listener_port_tls']

      self.scan_listener_port_non_tls = attributes[:'scanListenerPortNonTls'] if attributes[:'scanListenerPortNonTls']

      raise 'You cannot provide both :scanListenerPortNonTls and :scan_listener_port_non_tls' if attributes.key?(:'scanListenerPortNonTls') && attributes.key?(:'scan_listener_port_non_tls')

      self.scan_listener_port_non_tls = attributes[:'scan_listener_port_non_tls'] if attributes[:'scan_listener_port_non_tls']

      self.freeform_tags = attributes[:'freeformTags'] if attributes[:'freeformTags']

      raise 'You cannot provide both :freeformTags and :freeform_tags' if attributes.key?(:'freeformTags') && attributes.key?(:'freeform_tags')

      self.freeform_tags = attributes[:'freeform_tags'] if attributes[:'freeform_tags']

      self.defined_tags = attributes[:'definedTags'] if attributes[:'definedTags']

      raise 'You cannot provide both :definedTags and :defined_tags' if attributes.key?(:'definedTags') && attributes.key?(:'defined_tags')

      self.defined_tags = attributes[:'defined_tags'] if attributes[:'defined_tags']

      self.time_database_ssl_certificate_expires = attributes[:'timeDatabaseSslCertificateExpires'] if attributes[:'timeDatabaseSslCertificateExpires']

      raise 'You cannot provide both :timeDatabaseSslCertificateExpires and :time_database_ssl_certificate_expires' if attributes.key?(:'timeDatabaseSslCertificateExpires') && attributes.key?(:'time_database_ssl_certificate_expires')

      self.time_database_ssl_certificate_expires = attributes[:'time_database_ssl_certificate_expires'] if attributes[:'time_database_ssl_certificate_expires']

      self.time_ords_certificate_expires = attributes[:'timeOrdsCertificateExpires'] if attributes[:'timeOrdsCertificateExpires']

      raise 'You cannot provide both :timeOrdsCertificateExpires and :time_ords_certificate_expires' if attributes.key?(:'timeOrdsCertificateExpires') && attributes.key?(:'time_ords_certificate_expires')

      self.time_ords_certificate_expires = attributes[:'time_ords_certificate_expires'] if attributes[:'time_ords_certificate_expires']

      self.available_cpus = attributes[:'availableCpus'] if attributes[:'availableCpus']

      raise 'You cannot provide both :availableCpus and :available_cpus' if attributes.key?(:'availableCpus') && attributes.key?(:'available_cpus')

      self.available_cpus = attributes[:'available_cpus'] if attributes[:'available_cpus']

      self.reclaimable_cpus = attributes[:'reclaimableCpus'] if attributes[:'reclaimableCpus']

      raise 'You cannot provide both :reclaimableCpus and :reclaimable_cpus' if attributes.key?(:'reclaimableCpus') && attributes.key?(:'reclaimable_cpus')

      self.reclaimable_cpus = attributes[:'reclaimable_cpus'] if attributes[:'reclaimable_cpus']

      self.available_container_databases = attributes[:'availableContainerDatabases'] if attributes[:'availableContainerDatabases']

      raise 'You cannot provide both :availableContainerDatabases and :available_container_databases' if attributes.key?(:'availableContainerDatabases') && attributes.key?(:'available_container_databases')

      self.available_container_databases = attributes[:'available_container_databases'] if attributes[:'available_container_databases']

      self.total_container_databases = attributes[:'totalContainerDatabases'] if attributes[:'totalContainerDatabases']

      raise 'You cannot provide both :totalContainerDatabases and :total_container_databases' if attributes.key?(:'totalContainerDatabases') && attributes.key?(:'total_container_databases')

      self.total_container_databases = attributes[:'total_container_databases'] if attributes[:'total_container_databases']

      self.available_autonomous_data_storage_size_in_tbs = attributes[:'availableAutonomousDataStorageSizeInTBs'] if attributes[:'availableAutonomousDataStorageSizeInTBs']

      raise 'You cannot provide both :availableAutonomousDataStorageSizeInTBs and :available_autonomous_data_storage_size_in_tbs' if attributes.key?(:'availableAutonomousDataStorageSizeInTBs') && attributes.key?(:'available_autonomous_data_storage_size_in_tbs')

      self.available_autonomous_data_storage_size_in_tbs = attributes[:'available_autonomous_data_storage_size_in_tbs'] if attributes[:'available_autonomous_data_storage_size_in_tbs']

      self.autonomous_data_storage_size_in_tbs = attributes[:'autonomousDataStorageSizeInTBs'] if attributes[:'autonomousDataStorageSizeInTBs']

      raise 'You cannot provide both :autonomousDataStorageSizeInTBs and :autonomous_data_storage_size_in_tbs' if attributes.key?(:'autonomousDataStorageSizeInTBs') && attributes.key?(:'autonomous_data_storage_size_in_tbs')

      self.autonomous_data_storage_size_in_tbs = attributes[:'autonomous_data_storage_size_in_tbs'] if attributes[:'autonomous_data_storage_size_in_tbs']

      self.db_node_storage_size_in_gbs = attributes[:'dbNodeStorageSizeInGBs'] if attributes[:'dbNodeStorageSizeInGBs']

      raise 'You cannot provide both :dbNodeStorageSizeInGBs and :db_node_storage_size_in_gbs' if attributes.key?(:'dbNodeStorageSizeInGBs') && attributes.key?(:'db_node_storage_size_in_gbs')

      self.db_node_storage_size_in_gbs = attributes[:'db_node_storage_size_in_gbs'] if attributes[:'db_node_storage_size_in_gbs']

      self.memory_per_oracle_compute_unit_in_gbs = attributes[:'memoryPerOracleComputeUnitInGBs'] if attributes[:'memoryPerOracleComputeUnitInGBs']

      raise 'You cannot provide both :memoryPerOracleComputeUnitInGBs and :memory_per_oracle_compute_unit_in_gbs' if attributes.key?(:'memoryPerOracleComputeUnitInGBs') && attributes.key?(:'memory_per_oracle_compute_unit_in_gbs')

      self.memory_per_oracle_compute_unit_in_gbs = attributes[:'memory_per_oracle_compute_unit_in_gbs'] if attributes[:'memory_per_oracle_compute_unit_in_gbs']

      self.db_servers = attributes[:'dbServers'] if attributes[:'dbServers']

      raise 'You cannot provide both :dbServers and :db_servers' if attributes.key?(:'dbServers') && attributes.key?(:'db_servers')

      self.db_servers = attributes[:'db_servers'] if attributes[:'db_servers']

      self.cpu_percentage = attributes[:'cpuPercentage'] if attributes[:'cpuPercentage']

      raise 'You cannot provide both :cpuPercentage and :cpu_percentage' if attributes.key?(:'cpuPercentage') && attributes.key?(:'cpu_percentage')

      self.cpu_percentage = attributes[:'cpu_percentage'] if attributes[:'cpu_percentage']

      self.autonomous_data_storage_percentage = attributes[:'autonomousDataStoragePercentage'] if attributes[:'autonomousDataStoragePercentage']

      raise 'You cannot provide both :autonomousDataStoragePercentage and :autonomous_data_storage_percentage' if attributes.key?(:'autonomousDataStoragePercentage') && attributes.key?(:'autonomous_data_storage_percentage')

      self.autonomous_data_storage_percentage = attributes[:'autonomous_data_storage_percentage'] if attributes[:'autonomous_data_storage_percentage']

      self.provisioned_cpus = attributes[:'provisionedCpus'] if attributes[:'provisionedCpus']

      raise 'You cannot provide both :provisionedCpus and :provisioned_cpus' if attributes.key?(:'provisionedCpus') && attributes.key?(:'provisioned_cpus')

      self.provisioned_cpus = attributes[:'provisioned_cpus'] if attributes[:'provisioned_cpus']

      self.total_cpus = attributes[:'totalCpus'] if attributes[:'totalCpus']

      raise 'You cannot provide both :totalCpus and :total_cpus' if attributes.key?(:'totalCpus') && attributes.key?(:'total_cpus')

      self.total_cpus = attributes[:'total_cpus'] if attributes[:'total_cpus']

      self.total_autonomous_data_storage_in_tbs = attributes[:'totalAutonomousDataStorageInTBs'] if attributes[:'totalAutonomousDataStorageInTBs']

      raise 'You cannot provide both :totalAutonomousDataStorageInTBs and :total_autonomous_data_storage_in_tbs' if attributes.key?(:'totalAutonomousDataStorageInTBs') && attributes.key?(:'total_autonomous_data_storage_in_tbs')

      self.total_autonomous_data_storage_in_tbs = attributes[:'total_autonomous_data_storage_in_tbs'] if attributes[:'total_autonomous_data_storage_in_tbs']

      self.reserved_cpus = attributes[:'reservedCpus'] if attributes[:'reservedCpus']

      raise 'You cannot provide both :reservedCpus and :reserved_cpus' if attributes.key?(:'reservedCpus') && attributes.key?(:'reserved_cpus')

      self.reserved_cpus = attributes[:'reserved_cpus'] if attributes[:'reserved_cpus']

      self.provisionable_autonomous_container_databases = attributes[:'provisionableAutonomousContainerDatabases'] if attributes[:'provisionableAutonomousContainerDatabases']

      raise 'You cannot provide both :provisionableAutonomousContainerDatabases and :provisionable_autonomous_container_databases' if attributes.key?(:'provisionableAutonomousContainerDatabases') && attributes.key?(:'provisionable_autonomous_container_databases')

      self.provisionable_autonomous_container_databases = attributes[:'provisionable_autonomous_container_databases'] if attributes[:'provisionable_autonomous_container_databases']

      self.provisioned_autonomous_container_databases = attributes[:'provisionedAutonomousContainerDatabases'] if attributes[:'provisionedAutonomousContainerDatabases']

      raise 'You cannot provide both :provisionedAutonomousContainerDatabases and :provisioned_autonomous_container_databases' if attributes.key?(:'provisionedAutonomousContainerDatabases') && attributes.key?(:'provisioned_autonomous_container_databases')

      self.provisioned_autonomous_container_databases = attributes[:'provisioned_autonomous_container_databases'] if attributes[:'provisioned_autonomous_container_databases']

      self.non_provisionable_autonomous_container_databases = attributes[:'nonProvisionableAutonomousContainerDatabases'] if attributes[:'nonProvisionableAutonomousContainerDatabases']

      raise 'You cannot provide both :nonProvisionableAutonomousContainerDatabases and :non_provisionable_autonomous_container_databases' if attributes.key?(:'nonProvisionableAutonomousContainerDatabases') && attributes.key?(:'non_provisionable_autonomous_container_databases')

      self.non_provisionable_autonomous_container_databases = attributes[:'non_provisionable_autonomous_container_databases'] if attributes[:'non_provisionable_autonomous_container_databases']

      self.exadata_storage_in_tbs_lowest_scaled_value = attributes[:'exadataStorageInTBsLowestScaledValue'] if attributes[:'exadataStorageInTBsLowestScaledValue']

      raise 'You cannot provide both :exadataStorageInTBsLowestScaledValue and :exadata_storage_in_tbs_lowest_scaled_value' if attributes.key?(:'exadataStorageInTBsLowestScaledValue') && attributes.key?(:'exadata_storage_in_tbs_lowest_scaled_value')

      self.exadata_storage_in_tbs_lowest_scaled_value = attributes[:'exadata_storage_in_tbs_lowest_scaled_value'] if attributes[:'exadata_storage_in_tbs_lowest_scaled_value']

      self.ocpus_lowest_scaled_value = attributes[:'ocpusLowestScaledValue'] if attributes[:'ocpusLowestScaledValue']

      raise 'You cannot provide both :ocpusLowestScaledValue and :ocpus_lowest_scaled_value' if attributes.key?(:'ocpusLowestScaledValue') && attributes.key?(:'ocpus_lowest_scaled_value')

      self.ocpus_lowest_scaled_value = attributes[:'ocpus_lowest_scaled_value'] if attributes[:'ocpus_lowest_scaled_value']

      self.max_acds_lowest_scaled_value = attributes[:'maxAcdsLowestScaledValue'] if attributes[:'maxAcdsLowestScaledValue']

      raise 'You cannot provide both :maxAcdsLowestScaledValue and :max_acds_lowest_scaled_value' if attributes.key?(:'maxAcdsLowestScaledValue') && attributes.key?(:'max_acds_lowest_scaled_value')

      self.max_acds_lowest_scaled_value = attributes[:'max_acds_lowest_scaled_value'] if attributes[:'max_acds_lowest_scaled_value']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] lifecycle_state Object to be assigned
    def lifecycle_state=(lifecycle_state)
      # rubocop:disable Style/ConditionalAssignment
      if lifecycle_state && !LIFECYCLE_STATE_ENUM.include?(lifecycle_state)
        OCI.logger.debug("Unknown value for 'lifecycle_state' [" + lifecycle_state + "]. Mapping to 'LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @lifecycle_state = LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE
      else
        @lifecycle_state = lifecycle_state
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] compute_model Object to be assigned
    def compute_model=(compute_model)
      # rubocop:disable Style/ConditionalAssignment
      if compute_model && !COMPUTE_MODEL_ENUM.include?(compute_model)
        OCI.logger.debug("Unknown value for 'compute_model' [" + compute_model + "]. Mapping to 'COMPUTE_MODEL_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @compute_model = COMPUTE_MODEL_UNKNOWN_ENUM_VALUE
      else
        @compute_model = compute_model
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] license_model Object to be assigned
    def license_model=(license_model)
      # rubocop:disable Style/ConditionalAssignment
      if license_model && !LICENSE_MODEL_ENUM.include?(license_model)
        OCI.logger.debug("Unknown value for 'license_model' [" + license_model + "]. Mapping to 'LICENSE_MODEL_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @license_model = LICENSE_MODEL_UNKNOWN_ENUM_VALUE
      else
        @license_model = license_model
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        id == other.id &&
        compartment_id == other.compartment_id &&
        description == other.description &&
        availability_domain == other.availability_domain &&
        subnet_id == other.subnet_id &&
        nsg_ids == other.nsg_ids &&
        last_update_history_entry_id == other.last_update_history_entry_id &&
        lifecycle_state == other.lifecycle_state &&
        display_name == other.display_name &&
        time_created == other.time_created &&
        time_updated == other.time_updated &&
        cluster_time_zone == other.cluster_time_zone &&
        lifecycle_details == other.lifecycle_details &&
        hostname == other.hostname &&
        domain == other.domain &&
        cloud_exadata_infrastructure_id == other.cloud_exadata_infrastructure_id &&
        shape == other.shape &&
        node_count == other.node_count &&
        data_storage_size_in_tbs == other.data_storage_size_in_tbs &&
        data_storage_size_in_gbs == other.data_storage_size_in_gbs &&
        cpu_core_count == other.cpu_core_count &&
        ocpu_count == other.ocpu_count &&
        compute_model == other.compute_model &&
        is_mtls_enabled_vm_cluster == other.is_mtls_enabled_vm_cluster &&
        cpu_core_count_per_node == other.cpu_core_count_per_node &&
        memory_size_in_gbs == other.memory_size_in_gbs &&
        license_model == other.license_model &&
        last_maintenance_run_id == other.last_maintenance_run_id &&
        next_maintenance_run_id == other.next_maintenance_run_id &&
        maintenance_window == other.maintenance_window &&
        scan_listener_port_tls == other.scan_listener_port_tls &&
        scan_listener_port_non_tls == other.scan_listener_port_non_tls &&
        freeform_tags == other.freeform_tags &&
        defined_tags == other.defined_tags &&
        time_database_ssl_certificate_expires == other.time_database_ssl_certificate_expires &&
        time_ords_certificate_expires == other.time_ords_certificate_expires &&
        available_cpus == other.available_cpus &&
        reclaimable_cpus == other.reclaimable_cpus &&
        available_container_databases == other.available_container_databases &&
        total_container_databases == other.total_container_databases &&
        available_autonomous_data_storage_size_in_tbs == other.available_autonomous_data_storage_size_in_tbs &&
        autonomous_data_storage_size_in_tbs == other.autonomous_data_storage_size_in_tbs &&
        db_node_storage_size_in_gbs == other.db_node_storage_size_in_gbs &&
        memory_per_oracle_compute_unit_in_gbs == other.memory_per_oracle_compute_unit_in_gbs &&
        db_servers == other.db_servers &&
        cpu_percentage == other.cpu_percentage &&
        autonomous_data_storage_percentage == other.autonomous_data_storage_percentage &&
        provisioned_cpus == other.provisioned_cpus &&
        total_cpus == other.total_cpus &&
        total_autonomous_data_storage_in_tbs == other.total_autonomous_data_storage_in_tbs &&
        reserved_cpus == other.reserved_cpus &&
        provisionable_autonomous_container_databases == other.provisionable_autonomous_container_databases &&
        provisioned_autonomous_container_databases == other.provisioned_autonomous_container_databases &&
        non_provisionable_autonomous_container_databases == other.non_provisionable_autonomous_container_databases &&
        exadata_storage_in_tbs_lowest_scaled_value == other.exadata_storage_in_tbs_lowest_scaled_value &&
        ocpus_lowest_scaled_value == other.ocpus_lowest_scaled_value &&
        max_acds_lowest_scaled_value == other.max_acds_lowest_scaled_value
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [id, compartment_id, description, availability_domain, subnet_id, nsg_ids, last_update_history_entry_id, lifecycle_state, display_name, time_created, time_updated, cluster_time_zone, lifecycle_details, hostname, domain, cloud_exadata_infrastructure_id, shape, node_count, data_storage_size_in_tbs, data_storage_size_in_gbs, cpu_core_count, ocpu_count, compute_model, is_mtls_enabled_vm_cluster, cpu_core_count_per_node, memory_size_in_gbs, license_model, last_maintenance_run_id, next_maintenance_run_id, maintenance_window, scan_listener_port_tls, scan_listener_port_non_tls, freeform_tags, defined_tags, time_database_ssl_certificate_expires, time_ords_certificate_expires, available_cpus, reclaimable_cpus, available_container_databases, total_container_databases, available_autonomous_data_storage_size_in_tbs, autonomous_data_storage_size_in_tbs, db_node_storage_size_in_gbs, memory_per_oracle_compute_unit_in_gbs, db_servers, cpu_percentage, autonomous_data_storage_percentage, provisioned_cpus, total_cpus, total_autonomous_data_storage_in_tbs, reserved_cpus, provisionable_autonomous_container_databases, provisioned_autonomous_container_databases, non_provisionable_autonomous_container_databases, exadata_storage_in_tbs_lowest_scaled_value, ocpus_lowest_scaled_value, max_acds_lowest_scaled_value].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
