# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20160918
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # The summary for the record of an OS upgrade action on a DB system.
  #
  class Database::Models::DbSystemUpgradeHistoryEntrySummary
    ACTION_ENUM = [
      ACTION_PRECHECK = 'PRECHECK'.freeze,
      ACTION_ROLLBACK = 'ROLLBACK'.freeze,
      ACTION_UPDATE_SNAPSHOT_RETENTION_DAYS = 'UPDATE_SNAPSHOT_RETENTION_DAYS'.freeze,
      ACTION_UPGRADE = 'UPGRADE'.freeze,
      ACTION_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    LIFECYCLE_STATE_ENUM = [
      LIFECYCLE_STATE_IN_PROGRESS = 'IN_PROGRESS'.freeze,
      LIFECYCLE_STATE_SUCCEEDED = 'SUCCEEDED'.freeze,
      LIFECYCLE_STATE_FAILED = 'FAILED'.freeze,
      LIFECYCLE_STATE_NEEDS_ATTENTION = 'NEEDS_ATTENTION'.freeze,
      LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the upgrade history entry.
    # @return [String]
    attr_accessor :id

    # **[Required]** The operating system upgrade action.
    # @return [String]
    attr_reader :action

    # A valid Oracle Grid Infrastructure (GI) software version.
    # @return [String]
    attr_accessor :new_gi_version

    # A valid Oracle Grid Infrastructure (GI) software version.
    # @return [String]
    attr_accessor :old_gi_version

    # A valid Oracle Software (OS) version eg. Oracle Linux Server release 8
    # @return [String]
    attr_accessor :old_os_version

    # A valid Oracle Software (OS) version eg. Oracle Linux Server release 8
    # @return [String]
    attr_accessor :new_os_version

    # **[Required]** The retention period, in days, for the snapshot that allows you to perform a rollback of the upgrade operation. After this number of days passes, you cannot roll back the upgrade.
    # @return [Integer]
    attr_accessor :snapshot_retention_period_in_days

    # **[Required]** The current state of the action.
    # @return [String]
    attr_reader :lifecycle_state

    # A descriptive text associated with the lifecycleState.
    # Typically contains additional displayable text.
    #
    # @return [String]
    attr_accessor :lifecycle_details

    # **[Required]** The date and time when the upgrade action started.
    # @return [DateTime]
    attr_accessor :time_started

    # The date and time when the upgrade action completed
    # @return [DateTime]
    attr_accessor :time_ended

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'id',
        'action': :'action',
        'new_gi_version': :'newGiVersion',
        'old_gi_version': :'oldGiVersion',
        'old_os_version': :'oldOsVersion',
        'new_os_version': :'newOsVersion',
        'snapshot_retention_period_in_days': :'snapshotRetentionPeriodInDays',
        'lifecycle_state': :'lifecycleState',
        'lifecycle_details': :'lifecycleDetails',
        'time_started': :'timeStarted',
        'time_ended': :'timeEnded'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'String',
        'action': :'String',
        'new_gi_version': :'String',
        'old_gi_version': :'String',
        'old_os_version': :'String',
        'new_os_version': :'String',
        'snapshot_retention_period_in_days': :'Integer',
        'lifecycle_state': :'String',
        'lifecycle_details': :'String',
        'time_started': :'DateTime',
        'time_ended': :'DateTime'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :id The value to assign to the {#id} property
    # @option attributes [String] :action The value to assign to the {#action} property
    # @option attributes [String] :new_gi_version The value to assign to the {#new_gi_version} property
    # @option attributes [String] :old_gi_version The value to assign to the {#old_gi_version} property
    # @option attributes [String] :old_os_version The value to assign to the {#old_os_version} property
    # @option attributes [String] :new_os_version The value to assign to the {#new_os_version} property
    # @option attributes [Integer] :snapshot_retention_period_in_days The value to assign to the {#snapshot_retention_period_in_days} property
    # @option attributes [String] :lifecycle_state The value to assign to the {#lifecycle_state} property
    # @option attributes [String] :lifecycle_details The value to assign to the {#lifecycle_details} property
    # @option attributes [DateTime] :time_started The value to assign to the {#time_started} property
    # @option attributes [DateTime] :time_ended The value to assign to the {#time_ended} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.id = attributes[:'id'] if attributes[:'id']

      self.action = attributes[:'action'] if attributes[:'action']

      self.new_gi_version = attributes[:'newGiVersion'] if attributes[:'newGiVersion']

      raise 'You cannot provide both :newGiVersion and :new_gi_version' if attributes.key?(:'newGiVersion') && attributes.key?(:'new_gi_version')

      self.new_gi_version = attributes[:'new_gi_version'] if attributes[:'new_gi_version']

      self.old_gi_version = attributes[:'oldGiVersion'] if attributes[:'oldGiVersion']

      raise 'You cannot provide both :oldGiVersion and :old_gi_version' if attributes.key?(:'oldGiVersion') && attributes.key?(:'old_gi_version')

      self.old_gi_version = attributes[:'old_gi_version'] if attributes[:'old_gi_version']

      self.old_os_version = attributes[:'oldOsVersion'] if attributes[:'oldOsVersion']

      raise 'You cannot provide both :oldOsVersion and :old_os_version' if attributes.key?(:'oldOsVersion') && attributes.key?(:'old_os_version')

      self.old_os_version = attributes[:'old_os_version'] if attributes[:'old_os_version']

      self.new_os_version = attributes[:'newOsVersion'] if attributes[:'newOsVersion']

      raise 'You cannot provide both :newOsVersion and :new_os_version' if attributes.key?(:'newOsVersion') && attributes.key?(:'new_os_version')

      self.new_os_version = attributes[:'new_os_version'] if attributes[:'new_os_version']

      self.snapshot_retention_period_in_days = attributes[:'snapshotRetentionPeriodInDays'] if attributes[:'snapshotRetentionPeriodInDays']

      raise 'You cannot provide both :snapshotRetentionPeriodInDays and :snapshot_retention_period_in_days' if attributes.key?(:'snapshotRetentionPeriodInDays') && attributes.key?(:'snapshot_retention_period_in_days')

      self.snapshot_retention_period_in_days = attributes[:'snapshot_retention_period_in_days'] if attributes[:'snapshot_retention_period_in_days']

      self.lifecycle_state = attributes[:'lifecycleState'] if attributes[:'lifecycleState']

      raise 'You cannot provide both :lifecycleState and :lifecycle_state' if attributes.key?(:'lifecycleState') && attributes.key?(:'lifecycle_state')

      self.lifecycle_state = attributes[:'lifecycle_state'] if attributes[:'lifecycle_state']

      self.lifecycle_details = attributes[:'lifecycleDetails'] if attributes[:'lifecycleDetails']

      raise 'You cannot provide both :lifecycleDetails and :lifecycle_details' if attributes.key?(:'lifecycleDetails') && attributes.key?(:'lifecycle_details')

      self.lifecycle_details = attributes[:'lifecycle_details'] if attributes[:'lifecycle_details']

      self.time_started = attributes[:'timeStarted'] if attributes[:'timeStarted']

      raise 'You cannot provide both :timeStarted and :time_started' if attributes.key?(:'timeStarted') && attributes.key?(:'time_started')

      self.time_started = attributes[:'time_started'] if attributes[:'time_started']

      self.time_ended = attributes[:'timeEnded'] if attributes[:'timeEnded']

      raise 'You cannot provide both :timeEnded and :time_ended' if attributes.key?(:'timeEnded') && attributes.key?(:'time_ended')

      self.time_ended = attributes[:'time_ended'] if attributes[:'time_ended']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] action Object to be assigned
    def action=(action)
      # rubocop:disable Style/ConditionalAssignment
      if action && !ACTION_ENUM.include?(action)
        OCI.logger.debug("Unknown value for 'action' [" + action + "]. Mapping to 'ACTION_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @action = ACTION_UNKNOWN_ENUM_VALUE
      else
        @action = action
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] lifecycle_state Object to be assigned
    def lifecycle_state=(lifecycle_state)
      # rubocop:disable Style/ConditionalAssignment
      if lifecycle_state && !LIFECYCLE_STATE_ENUM.include?(lifecycle_state)
        OCI.logger.debug("Unknown value for 'lifecycle_state' [" + lifecycle_state + "]. Mapping to 'LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @lifecycle_state = LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE
      else
        @lifecycle_state = lifecycle_state
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        id == other.id &&
        action == other.action &&
        new_gi_version == other.new_gi_version &&
        old_gi_version == other.old_gi_version &&
        old_os_version == other.old_os_version &&
        new_os_version == other.new_os_version &&
        snapshot_retention_period_in_days == other.snapshot_retention_period_in_days &&
        lifecycle_state == other.lifecycle_state &&
        lifecycle_details == other.lifecycle_details &&
        time_started == other.time_started &&
        time_ended == other.time_ended
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [id, action, new_gi_version, old_gi_version, old_os_version, new_os_version, snapshot_retention_period_in_days, lifecycle_state, lifecycle_details, time_started, time_ended].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
