# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20201101
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # The database metric details.
  class DatabaseManagement::Models::FleetMetricDefinition
    # The name of the metric.
    # @return [String]
    attr_accessor :metric_name

    # The baseline value of the metric.
    # @return [Float]
    attr_accessor :baseline_value

    # The target value of the metric.
    # @return [Float]
    attr_accessor :target_value

    # The unit of the value.
    # @return [String]
    attr_accessor :unit

    # The data point date and time in UTC in ISO-8601 format, which is \"yyyy-MM-dd'T'hh:mm:ss.sss'Z'\".
    # @return [DateTime]
    attr_accessor :timestamp

    # The percentage change in the metric aggregated value compared to the baseline value.
    # @return [Float]
    attr_accessor :percentage_change

    # The dimensions of the metric.
    # @return [Array<OCI::DatabaseManagement::Models::MetricDimensionDefinition>]
    attr_accessor :dimensions

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'metric_name': :'metricName',
        'baseline_value': :'baselineValue',
        'target_value': :'targetValue',
        'unit': :'unit',
        'timestamp': :'timestamp',
        'percentage_change': :'percentageChange',
        'dimensions': :'dimensions'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'metric_name': :'String',
        'baseline_value': :'Float',
        'target_value': :'Float',
        'unit': :'String',
        'timestamp': :'DateTime',
        'percentage_change': :'Float',
        'dimensions': :'Array<OCI::DatabaseManagement::Models::MetricDimensionDefinition>'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :metric_name The value to assign to the {#metric_name} property
    # @option attributes [Float] :baseline_value The value to assign to the {#baseline_value} property
    # @option attributes [Float] :target_value The value to assign to the {#target_value} property
    # @option attributes [String] :unit The value to assign to the {#unit} property
    # @option attributes [DateTime] :timestamp The value to assign to the {#timestamp} property
    # @option attributes [Float] :percentage_change The value to assign to the {#percentage_change} property
    # @option attributes [Array<OCI::DatabaseManagement::Models::MetricDimensionDefinition>] :dimensions The value to assign to the {#dimensions} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.metric_name = attributes[:'metricName'] if attributes[:'metricName']

      raise 'You cannot provide both :metricName and :metric_name' if attributes.key?(:'metricName') && attributes.key?(:'metric_name')

      self.metric_name = attributes[:'metric_name'] if attributes[:'metric_name']

      self.baseline_value = attributes[:'baselineValue'] if attributes[:'baselineValue']

      raise 'You cannot provide both :baselineValue and :baseline_value' if attributes.key?(:'baselineValue') && attributes.key?(:'baseline_value')

      self.baseline_value = attributes[:'baseline_value'] if attributes[:'baseline_value']

      self.target_value = attributes[:'targetValue'] if attributes[:'targetValue']

      raise 'You cannot provide both :targetValue and :target_value' if attributes.key?(:'targetValue') && attributes.key?(:'target_value')

      self.target_value = attributes[:'target_value'] if attributes[:'target_value']

      self.unit = attributes[:'unit'] if attributes[:'unit']

      self.timestamp = attributes[:'timestamp'] if attributes[:'timestamp']

      self.percentage_change = attributes[:'percentageChange'] if attributes[:'percentageChange']

      raise 'You cannot provide both :percentageChange and :percentage_change' if attributes.key?(:'percentageChange') && attributes.key?(:'percentage_change')

      self.percentage_change = attributes[:'percentage_change'] if attributes[:'percentage_change']

      self.dimensions = attributes[:'dimensions'] if attributes[:'dimensions']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        metric_name == other.metric_name &&
        baseline_value == other.baseline_value &&
        target_value == other.target_value &&
        unit == other.unit &&
        timestamp == other.timestamp &&
        percentage_change == other.percentage_change &&
        dimensions == other.dimensions
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [metric_name, baseline_value, target_value, unit, timestamp, percentage_change, dimensions].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
