# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20201101
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # The details required to load the Sql statements into the Sql tuning set.
  #
  class DatabaseManagement::Models::LoadSqlTuningSetDetails
    LOAD_TYPE_ENUM = [
      LOAD_TYPE_INCREMENTAL_CURSOR_CACHE = 'INCREMENTAL_CURSOR_CACHE'.freeze,
      LOAD_TYPE_CURRENT_CURSOR_CACHE = 'CURRENT_CURSOR_CACHE'.freeze,
      LOAD_TYPE_AWR = 'AWR'.freeze
    ].freeze

    RECURSIVE_SQL_ENUM = [
      RECURSIVE_SQL_HAS_RECURSIVE_SQL = 'HAS_RECURSIVE_SQL'.freeze,
      RECURSIVE_SQL_NO_RECURSIVE_SQL = 'NO_RECURSIVE_SQL'.freeze
    ].freeze

    RANKING_MEASURE1_ENUM = [
      RANKING_MEASURE1_ELAPSED_TIME = 'ELAPSED_TIME'.freeze,
      RANKING_MEASURE1_CPU_TIME = 'CPU_TIME'.freeze,
      RANKING_MEASURE1_OPTIMIZER_COST = 'OPTIMIZER_COST'.freeze,
      RANKING_MEASURE1_BUFFER_GETS = 'BUFFER_GETS'.freeze,
      RANKING_MEASURE1_DISK_READS = 'DISK_READS'.freeze,
      RANKING_MEASURE1_DIRECT_WRITES = 'DIRECT_WRITES'.freeze
    ].freeze

    RANKING_MEASURE2_ENUM = [
      RANKING_MEASURE2_ELAPSED_TIME = 'ELAPSED_TIME'.freeze,
      RANKING_MEASURE2_CPU_TIME = 'CPU_TIME'.freeze,
      RANKING_MEASURE2_OPTIMIZER_COST = 'OPTIMIZER_COST'.freeze,
      RANKING_MEASURE2_BUFFER_GETS = 'BUFFER_GETS'.freeze,
      RANKING_MEASURE2_DISK_READS = 'DISK_READS'.freeze,
      RANKING_MEASURE2_DIRECT_WRITES = 'DIRECT_WRITES'.freeze
    ].freeze

    RANKING_MEASURE3_ENUM = [
      RANKING_MEASURE3_ELAPSED_TIME = 'ELAPSED_TIME'.freeze,
      RANKING_MEASURE3_CPU_TIME = 'CPU_TIME'.freeze,
      RANKING_MEASURE3_OPTIMIZER_COST = 'OPTIMIZER_COST'.freeze,
      RANKING_MEASURE3_BUFFER_GETS = 'BUFFER_GETS'.freeze,
      RANKING_MEASURE3_DISK_READS = 'DISK_READS'.freeze,
      RANKING_MEASURE3_DIRECT_WRITES = 'DIRECT_WRITES'.freeze
    ].freeze

    CAPTURE_OPTION_ENUM = [
      CAPTURE_OPTION_INSERT = 'INSERT'.freeze,
      CAPTURE_OPTION_UPDATE = 'UPDATE'.freeze,
      CAPTURE_OPTION_MERGE = 'MERGE'.freeze
    ].freeze

    CAPTURE_MODE_ENUM = [
      CAPTURE_MODE_MODE_REPLACE_OLD_STATS = 'MODE_REPLACE_OLD_STATS'.freeze,
      CAPTURE_MODE_MODE_ACCUMULATE_STATS = 'MODE_ACCUMULATE_STATS'.freeze
    ].freeze

    LOAD_OPTION_ENUM = [
      LOAD_OPTION_INSERT = 'INSERT'.freeze,
      LOAD_OPTION_UPDATE = 'UPDATE'.freeze,
      LOAD_OPTION_MERGE = 'MERGE'.freeze
    ].freeze

    UPDATE_OPTION_ENUM = [
      UPDATE_OPTION_REPLACE = 'REPLACE'.freeze,
      UPDATE_OPTION_ACCUMULATE = 'ACCUMULATE'.freeze
    ].freeze

    UPDATE_CONDITION_ENUM = [
      UPDATE_CONDITION_OLD = 'OLD'.freeze,
      UPDATE_CONDITION_NEW = 'NEW'.freeze
    ].freeze

    # This attribute is required.
    # @return [OCI::DatabaseManagement::Models::SqlTuningSetAdminCredentialDetails]
    attr_accessor :credential_details

    # Flag to indicate whether to create the Sql tuning set or just display the plsql used to create Sql tuning set.
    #
    # @return [Integer]
    attr_accessor :show_sql_only

    # The owner of the Sql tuning set.
    # @return [String]
    attr_accessor :owner

    # **[Required]** The name of the Sql tuning set.
    # @return [String]
    attr_accessor :name

    # **[Required]** Specifies the loading method into the Sql tuning set.
    # @return [String]
    attr_reader :load_type

    # Specifies the Sql predicate to filter the Sql from the Sql tuning set defined on attributes of the SQLSET_ROW.
    # User could use any combination of the following columns with appropriate values as Sql predicate
    # Refer to the documentation https://docs.oracle.com/en/database/oracle/oracle-database/18/arpls/DBMS_SQLTUNE.html#GUID-1F4AFB03-7B29-46FC-B3F2-CB01EC36326C
    #
    # @return [String]
    attr_accessor :basic_filter

    # Specifies that the filter must include recursive Sql in the Sql tuning set.
    # @return [String]
    attr_reader :recursive_sql

    # Specifies a filter that picks the top n% according to the supplied ranking measure.
    # Note that this parameter applies only if one ranking measure is supplied.
    #
    # @return [Float]
    attr_accessor :result_percentage

    # The top limit Sql from the filtered source, ranked by the ranking measure.
    # @return [Integer]
    attr_accessor :result_limit

    # Specifies an ORDER BY clause on the selected Sql. User can specify upto three ranking measures.
    #
    # @return [String]
    attr_reader :ranking_measure1

    # Specifies an ORDER BY clause on the selected Sql. User can specify upto three ranking measures.
    #
    # @return [String]
    attr_reader :ranking_measure2

    # Specifies an ORDER BY clause on the selected Sql. User can specify upto three ranking measures.
    #
    # @return [String]
    attr_reader :ranking_measure3

    # Defines the total amount of time, in seconds, to execute.
    # @return [Integer]
    attr_accessor :total_time_limit

    # Defines the amount of time, in seconds, to pause between sampling.
    # @return [Integer]
    attr_accessor :repeat_interval

    # Specifies whether to insert new statements, update existing statements, or both.
    # @return [String]
    attr_reader :capture_option

    # Specifies the capture mode. Note that this parameter is applicable only for UPDATE and MERGE capture options.
    # Capture mode can take one of the following values
    #  - MODE_REPLACE_OLD_STATS
    #      Replaces statistics when the number of executions is greater than the number stored in the Sql tuning set
    #  - MODE_ACCUMULATE_STATS
    #      Adds new values to current values for Sql that is already stored.
    #      Note that this mode detects if a statement has been aged out, so the final value for a statistics is the sum of the statistics of all cursors that statement existed under.
    #
    # @return [String]
    attr_reader :capture_mode

    # Specifies the list of Sql statement attributes to return in the result.
    # Note that this parameter cannot be made an enum since custom value can take a list of comma separated attribute names.
    # Attribute list can take one of the following values.
    #  TYPICAL - Specifies BASIC plus Sql plan (without row source statistics) and without object reference list (default).
    #  BASIC - Specifies all attributes (such as execution statistics and binds) except the plans. The execution context is always part of the result.
    #  ALL - Specifies all attributes.
    #  CUSTOM - Comma-separated list of the following attribute names.
    #           - EXECUTION_STATISTICS
    #           - BIND_LIST
    #           - OBJECT_LIST
    #           - SQL_PLAN
    #           - SQL_PLAN_STATISTICS
    # Usage examples:
    #   1. \"attributeList\": \"TYPICAL\"
    #   2. \"attributeList\": \"ALL\"
    #   3. \"attributeList\": \"EXECUTION_STATISTICS,OBJECT_LIST,SQL_PLAN\"
    #
    # @return [String]
    attr_accessor :attribute_list

    # Specifies which statements are loaded into the Sql tuning set.
    # The possible values are.
    #  - INSERT (default)
    #       Adds only new statements.
    #  - UPDATE
    #       Updates existing the Sql statements and ignores any new statements.
    #  - MERGE
    #       Inserts new statements and updates the information of the existing ones.
    #
    # @return [String]
    attr_reader :load_option

    # Specifies how existing Sql statements are updated.
    # This parameter is applicable only if load_option is specified with UPDATE or MERGE as an option.
    # Update option can take one of the following values.
    #    REPLACE (default) - Updates the statement using the new statistics, bind list, object list, and so on.
    #    ACCUMULATE - Combines attributes when possible (for example, statistics such as elapsed_time), otherwise replaces the existing values (for example, module and action) with the provided values.
    #    Following Sql statement attributes can be accumulated.
    #        elapsed_time
    #        buffer_gets
    #        direct_writes
    #        disk_reads
    #        row_processed
    #        fetches
    #        executions
    #        end_of_fetch_count
    #        stat_period
    #        active_stat_period
    #
    # @return [String]
    attr_reader :update_option

    # Specifies the list of Sql statement attributes to update during a merge or update.
    # Note that this parameter cannot be made an enum since custom value can take a list of comma separated attribute names.
    # Update attributes can take one of the following values.
    #    NULL (default) - Specifies the content of the input cursor except the execution context. On other terms, it is equivalent to ALL without execution contexts such as module and action.
    #    BASIC - Specifies statistics and binds only.
    #    TYPICAL - Specifies BASIC with Sql plans (without row source statistics) and without an object reference list.
    #    ALL - Specifies all attributes, including the execution context attributes such as module and action.
    #    CUSTOM - List of comma separated attribute names to update
    #        EXECUTION_CONTEXT
    #        EXECUTION_STATISTICS
    #        SQL_BINDS
    #        SQL_PLAN
    #        SQL_PLAN_STATISTICS (similar to SQL_PLAN with added row source statistics)
    # Usage examples:
    #   1. \"updateAttributes\": \"TYPICAL\"
    #   2. \"updateAttributes\": \"BASIC\"
    #   3. \"updateAttributes\": \"EXECUTION_STATISTICS,SQL_PLAN_STATISTICS,SQL_PLAN\"
    #   4. \"updateAttributes\": \"EXECUTION_STATISTICS,SQL_PLAN\"
    #
    # @return [String]
    attr_accessor :update_attributes

    # Specifies when to perform the update.
    # The procedure only performs the update when the specified condition is satisfied.
    # The condition can refer to either the data source or destination.
    # The condition must use the following prefixes to refer to attributes from the source or the destination:
    # OLD  \u2014 Refers to statement attributes from the SQL tuning set (destination).
    # NEW  \u2014 Refers to statement attributes from the input statements (source).
    # NULL \u2014 No updates are performed.
    #
    # @return [String]
    attr_reader :update_condition

    # Specifies whether to update attributes when the new value is NULL.
    # If TRUE, then the procedure does not update an attribute when the new value is NULL.
    # That is, do not override with NULL values unless intentional.
    # Possible values - true or false
    #
    # @return [BOOLEAN]
    attr_accessor :is_ignore_null

    # Specifies whether to commit statements after DML.
    # If a value is provided, then the load commits after each specified number of statements is inserted.
    # If NULL is provided, then the load commits only once, at the end of the operation.
    #
    # @return [Integer]
    attr_accessor :commit_rows

    # Defines the beginning AWR snapshot (non-inclusive).
    # @return [Integer]
    attr_accessor :begin_snapshot

    # Defines the ending AWR snapshot (inclusive).
    # @return [Integer]
    attr_accessor :end_snapshot

    # Specifies the name of the AWR baseline period.
    # When loading the sql statements from AWR, following inputs has to be provided:
    # beginSnapshot and endSnapshot
    # OR
    # baselineName
    #
    # @return [String]
    attr_accessor :baseline_name

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'credential_details': :'credentialDetails',
        'show_sql_only': :'showSqlOnly',
        'owner': :'owner',
        'name': :'name',
        'load_type': :'loadType',
        'basic_filter': :'basicFilter',
        'recursive_sql': :'recursiveSql',
        'result_percentage': :'resultPercentage',
        'result_limit': :'resultLimit',
        'ranking_measure1': :'rankingMeasure1',
        'ranking_measure2': :'rankingMeasure2',
        'ranking_measure3': :'rankingMeasure3',
        'total_time_limit': :'totalTimeLimit',
        'repeat_interval': :'repeatInterval',
        'capture_option': :'captureOption',
        'capture_mode': :'captureMode',
        'attribute_list': :'attributeList',
        'load_option': :'loadOption',
        'update_option': :'updateOption',
        'update_attributes': :'updateAttributes',
        'update_condition': :'updateCondition',
        'is_ignore_null': :'isIgnoreNull',
        'commit_rows': :'commitRows',
        'begin_snapshot': :'beginSnapshot',
        'end_snapshot': :'endSnapshot',
        'baseline_name': :'baselineName'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'credential_details': :'OCI::DatabaseManagement::Models::SqlTuningSetAdminCredentialDetails',
        'show_sql_only': :'Integer',
        'owner': :'String',
        'name': :'String',
        'load_type': :'String',
        'basic_filter': :'String',
        'recursive_sql': :'String',
        'result_percentage': :'Float',
        'result_limit': :'Integer',
        'ranking_measure1': :'String',
        'ranking_measure2': :'String',
        'ranking_measure3': :'String',
        'total_time_limit': :'Integer',
        'repeat_interval': :'Integer',
        'capture_option': :'String',
        'capture_mode': :'String',
        'attribute_list': :'String',
        'load_option': :'String',
        'update_option': :'String',
        'update_attributes': :'String',
        'update_condition': :'String',
        'is_ignore_null': :'BOOLEAN',
        'commit_rows': :'Integer',
        'begin_snapshot': :'Integer',
        'end_snapshot': :'Integer',
        'baseline_name': :'String'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [OCI::DatabaseManagement::Models::SqlTuningSetAdminCredentialDetails] :credential_details The value to assign to the {#credential_details} property
    # @option attributes [Integer] :show_sql_only The value to assign to the {#show_sql_only} property
    # @option attributes [String] :owner The value to assign to the {#owner} property
    # @option attributes [String] :name The value to assign to the {#name} property
    # @option attributes [String] :load_type The value to assign to the {#load_type} property
    # @option attributes [String] :basic_filter The value to assign to the {#basic_filter} property
    # @option attributes [String] :recursive_sql The value to assign to the {#recursive_sql} property
    # @option attributes [Float] :result_percentage The value to assign to the {#result_percentage} property
    # @option attributes [Integer] :result_limit The value to assign to the {#result_limit} property
    # @option attributes [String] :ranking_measure1 The value to assign to the {#ranking_measure1} property
    # @option attributes [String] :ranking_measure2 The value to assign to the {#ranking_measure2} property
    # @option attributes [String] :ranking_measure3 The value to assign to the {#ranking_measure3} property
    # @option attributes [Integer] :total_time_limit The value to assign to the {#total_time_limit} property
    # @option attributes [Integer] :repeat_interval The value to assign to the {#repeat_interval} property
    # @option attributes [String] :capture_option The value to assign to the {#capture_option} property
    # @option attributes [String] :capture_mode The value to assign to the {#capture_mode} property
    # @option attributes [String] :attribute_list The value to assign to the {#attribute_list} property
    # @option attributes [String] :load_option The value to assign to the {#load_option} property
    # @option attributes [String] :update_option The value to assign to the {#update_option} property
    # @option attributes [String] :update_attributes The value to assign to the {#update_attributes} property
    # @option attributes [String] :update_condition The value to assign to the {#update_condition} property
    # @option attributes [BOOLEAN] :is_ignore_null The value to assign to the {#is_ignore_null} property
    # @option attributes [Integer] :commit_rows The value to assign to the {#commit_rows} property
    # @option attributes [Integer] :begin_snapshot The value to assign to the {#begin_snapshot} property
    # @option attributes [Integer] :end_snapshot The value to assign to the {#end_snapshot} property
    # @option attributes [String] :baseline_name The value to assign to the {#baseline_name} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.credential_details = attributes[:'credentialDetails'] if attributes[:'credentialDetails']

      raise 'You cannot provide both :credentialDetails and :credential_details' if attributes.key?(:'credentialDetails') && attributes.key?(:'credential_details')

      self.credential_details = attributes[:'credential_details'] if attributes[:'credential_details']

      self.show_sql_only = attributes[:'showSqlOnly'] if attributes[:'showSqlOnly']

      raise 'You cannot provide both :showSqlOnly and :show_sql_only' if attributes.key?(:'showSqlOnly') && attributes.key?(:'show_sql_only')

      self.show_sql_only = attributes[:'show_sql_only'] if attributes[:'show_sql_only']

      self.owner = attributes[:'owner'] if attributes[:'owner']
      self.owner = "null" if owner.nil? && !attributes.key?(:'owner') # rubocop:disable Style/StringLiterals

      self.name = attributes[:'name'] if attributes[:'name']

      self.load_type = attributes[:'loadType'] if attributes[:'loadType']
      self.load_type = "INCREMENTAL_CURSOR_CACHE" if load_type.nil? && !attributes.key?(:'loadType') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :loadType and :load_type' if attributes.key?(:'loadType') && attributes.key?(:'load_type')

      self.load_type = attributes[:'load_type'] if attributes[:'load_type']
      self.load_type = "INCREMENTAL_CURSOR_CACHE" if load_type.nil? && !attributes.key?(:'loadType') && !attributes.key?(:'load_type') # rubocop:disable Style/StringLiterals

      self.basic_filter = attributes[:'basicFilter'] if attributes[:'basicFilter']
      self.basic_filter = "null" if basic_filter.nil? && !attributes.key?(:'basicFilter') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :basicFilter and :basic_filter' if attributes.key?(:'basicFilter') && attributes.key?(:'basic_filter')

      self.basic_filter = attributes[:'basic_filter'] if attributes[:'basic_filter']
      self.basic_filter = "null" if basic_filter.nil? && !attributes.key?(:'basicFilter') && !attributes.key?(:'basic_filter') # rubocop:disable Style/StringLiterals

      self.recursive_sql = attributes[:'recursiveSql'] if attributes[:'recursiveSql']
      self.recursive_sql = "HAS_RECURSIVE_SQL" if recursive_sql.nil? && !attributes.key?(:'recursiveSql') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :recursiveSql and :recursive_sql' if attributes.key?(:'recursiveSql') && attributes.key?(:'recursive_sql')

      self.recursive_sql = attributes[:'recursive_sql'] if attributes[:'recursive_sql']
      self.recursive_sql = "HAS_RECURSIVE_SQL" if recursive_sql.nil? && !attributes.key?(:'recursiveSql') && !attributes.key?(:'recursive_sql') # rubocop:disable Style/StringLiterals

      self.result_percentage = attributes[:'resultPercentage'] if attributes[:'resultPercentage']
      self.result_percentage = 1.0 if result_percentage.nil? && !attributes.key?(:'resultPercentage') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :resultPercentage and :result_percentage' if attributes.key?(:'resultPercentage') && attributes.key?(:'result_percentage')

      self.result_percentage = attributes[:'result_percentage'] if attributes[:'result_percentage']
      self.result_percentage = 1.0 if result_percentage.nil? && !attributes.key?(:'resultPercentage') && !attributes.key?(:'result_percentage') # rubocop:disable Style/StringLiterals

      self.result_limit = attributes[:'resultLimit'] if attributes[:'resultLimit']

      raise 'You cannot provide both :resultLimit and :result_limit' if attributes.key?(:'resultLimit') && attributes.key?(:'result_limit')

      self.result_limit = attributes[:'result_limit'] if attributes[:'result_limit']

      self.ranking_measure1 = attributes[:'rankingMeasure1'] if attributes[:'rankingMeasure1']
      self.ranking_measure1 = "null" if ranking_measure1.nil? && !attributes.key?(:'rankingMeasure1') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :rankingMeasure1 and :ranking_measure1' if attributes.key?(:'rankingMeasure1') && attributes.key?(:'ranking_measure1')

      self.ranking_measure1 = attributes[:'ranking_measure1'] if attributes[:'ranking_measure1']
      self.ranking_measure1 = "null" if ranking_measure1.nil? && !attributes.key?(:'rankingMeasure1') && !attributes.key?(:'ranking_measure1') # rubocop:disable Style/StringLiterals

      self.ranking_measure2 = attributes[:'rankingMeasure2'] if attributes[:'rankingMeasure2']
      self.ranking_measure2 = "null" if ranking_measure2.nil? && !attributes.key?(:'rankingMeasure2') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :rankingMeasure2 and :ranking_measure2' if attributes.key?(:'rankingMeasure2') && attributes.key?(:'ranking_measure2')

      self.ranking_measure2 = attributes[:'ranking_measure2'] if attributes[:'ranking_measure2']
      self.ranking_measure2 = "null" if ranking_measure2.nil? && !attributes.key?(:'rankingMeasure2') && !attributes.key?(:'ranking_measure2') # rubocop:disable Style/StringLiterals

      self.ranking_measure3 = attributes[:'rankingMeasure3'] if attributes[:'rankingMeasure3']
      self.ranking_measure3 = "null" if ranking_measure3.nil? && !attributes.key?(:'rankingMeasure3') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :rankingMeasure3 and :ranking_measure3' if attributes.key?(:'rankingMeasure3') && attributes.key?(:'ranking_measure3')

      self.ranking_measure3 = attributes[:'ranking_measure3'] if attributes[:'ranking_measure3']
      self.ranking_measure3 = "null" if ranking_measure3.nil? && !attributes.key?(:'rankingMeasure3') && !attributes.key?(:'ranking_measure3') # rubocop:disable Style/StringLiterals

      self.total_time_limit = attributes[:'totalTimeLimit'] if attributes[:'totalTimeLimit']

      raise 'You cannot provide both :totalTimeLimit and :total_time_limit' if attributes.key?(:'totalTimeLimit') && attributes.key?(:'total_time_limit')

      self.total_time_limit = attributes[:'total_time_limit'] if attributes[:'total_time_limit']

      self.repeat_interval = attributes[:'repeatInterval'] if attributes[:'repeatInterval']

      raise 'You cannot provide both :repeatInterval and :repeat_interval' if attributes.key?(:'repeatInterval') && attributes.key?(:'repeat_interval')

      self.repeat_interval = attributes[:'repeat_interval'] if attributes[:'repeat_interval']

      self.capture_option = attributes[:'captureOption'] if attributes[:'captureOption']
      self.capture_option = "MERGE" if capture_option.nil? && !attributes.key?(:'captureOption') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :captureOption and :capture_option' if attributes.key?(:'captureOption') && attributes.key?(:'capture_option')

      self.capture_option = attributes[:'capture_option'] if attributes[:'capture_option']
      self.capture_option = "MERGE" if capture_option.nil? && !attributes.key?(:'captureOption') && !attributes.key?(:'capture_option') # rubocop:disable Style/StringLiterals

      self.capture_mode = attributes[:'captureMode'] if attributes[:'captureMode']
      self.capture_mode = "MODE_REPLACE_OLD_STATS" if capture_mode.nil? && !attributes.key?(:'captureMode') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :captureMode and :capture_mode' if attributes.key?(:'captureMode') && attributes.key?(:'capture_mode')

      self.capture_mode = attributes[:'capture_mode'] if attributes[:'capture_mode']
      self.capture_mode = "MODE_REPLACE_OLD_STATS" if capture_mode.nil? && !attributes.key?(:'captureMode') && !attributes.key?(:'capture_mode') # rubocop:disable Style/StringLiterals

      self.attribute_list = attributes[:'attributeList'] if attributes[:'attributeList']
      self.attribute_list = "TYPICAL" if attribute_list.nil? && !attributes.key?(:'attributeList') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :attributeList and :attribute_list' if attributes.key?(:'attributeList') && attributes.key?(:'attribute_list')

      self.attribute_list = attributes[:'attribute_list'] if attributes[:'attribute_list']
      self.attribute_list = "TYPICAL" if attribute_list.nil? && !attributes.key?(:'attributeList') && !attributes.key?(:'attribute_list') # rubocop:disable Style/StringLiterals

      self.load_option = attributes[:'loadOption'] if attributes[:'loadOption']
      self.load_option = "INSERT" if load_option.nil? && !attributes.key?(:'loadOption') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :loadOption and :load_option' if attributes.key?(:'loadOption') && attributes.key?(:'load_option')

      self.load_option = attributes[:'load_option'] if attributes[:'load_option']
      self.load_option = "INSERT" if load_option.nil? && !attributes.key?(:'loadOption') && !attributes.key?(:'load_option') # rubocop:disable Style/StringLiterals

      self.update_option = attributes[:'updateOption'] if attributes[:'updateOption']
      self.update_option = "REPLACE" if update_option.nil? && !attributes.key?(:'updateOption') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :updateOption and :update_option' if attributes.key?(:'updateOption') && attributes.key?(:'update_option')

      self.update_option = attributes[:'update_option'] if attributes[:'update_option']
      self.update_option = "REPLACE" if update_option.nil? && !attributes.key?(:'updateOption') && !attributes.key?(:'update_option') # rubocop:disable Style/StringLiterals

      self.update_attributes = attributes[:'updateAttributes'] if attributes[:'updateAttributes']
      self.update_attributes = "null" if update_attributes.nil? && !attributes.key?(:'updateAttributes') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :updateAttributes and :update_attributes' if attributes.key?(:'updateAttributes') && attributes.key?(:'update_attributes')

      self.update_attributes = attributes[:'update_attributes'] if attributes[:'update_attributes']
      self.update_attributes = "null" if update_attributes.nil? && !attributes.key?(:'updateAttributes') && !attributes.key?(:'update_attributes') # rubocop:disable Style/StringLiterals

      self.update_condition = attributes[:'updateCondition'] if attributes[:'updateCondition']
      self.update_condition = "null" if update_condition.nil? && !attributes.key?(:'updateCondition') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :updateCondition and :update_condition' if attributes.key?(:'updateCondition') && attributes.key?(:'update_condition')

      self.update_condition = attributes[:'update_condition'] if attributes[:'update_condition']
      self.update_condition = "null" if update_condition.nil? && !attributes.key?(:'updateCondition') && !attributes.key?(:'update_condition') # rubocop:disable Style/StringLiterals

      self.is_ignore_null = attributes[:'isIgnoreNull'] unless attributes[:'isIgnoreNull'].nil?
      self.is_ignore_null = true if is_ignore_null.nil? && !attributes.key?(:'isIgnoreNull') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :isIgnoreNull and :is_ignore_null' if attributes.key?(:'isIgnoreNull') && attributes.key?(:'is_ignore_null')

      self.is_ignore_null = attributes[:'is_ignore_null'] unless attributes[:'is_ignore_null'].nil?
      self.is_ignore_null = true if is_ignore_null.nil? && !attributes.key?(:'isIgnoreNull') && !attributes.key?(:'is_ignore_null') # rubocop:disable Style/StringLiterals

      self.commit_rows = attributes[:'commitRows'] if attributes[:'commitRows']

      raise 'You cannot provide both :commitRows and :commit_rows' if attributes.key?(:'commitRows') && attributes.key?(:'commit_rows')

      self.commit_rows = attributes[:'commit_rows'] if attributes[:'commit_rows']

      self.begin_snapshot = attributes[:'beginSnapshot'] if attributes[:'beginSnapshot']
      self.begin_snapshot = 1 if begin_snapshot.nil? && !attributes.key?(:'beginSnapshot') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :beginSnapshot and :begin_snapshot' if attributes.key?(:'beginSnapshot') && attributes.key?(:'begin_snapshot')

      self.begin_snapshot = attributes[:'begin_snapshot'] if attributes[:'begin_snapshot']
      self.begin_snapshot = 1 if begin_snapshot.nil? && !attributes.key?(:'beginSnapshot') && !attributes.key?(:'begin_snapshot') # rubocop:disable Style/StringLiterals

      self.end_snapshot = attributes[:'endSnapshot'] if attributes[:'endSnapshot']
      self.end_snapshot = 2 if end_snapshot.nil? && !attributes.key?(:'endSnapshot') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :endSnapshot and :end_snapshot' if attributes.key?(:'endSnapshot') && attributes.key?(:'end_snapshot')

      self.end_snapshot = attributes[:'end_snapshot'] if attributes[:'end_snapshot']
      self.end_snapshot = 2 if end_snapshot.nil? && !attributes.key?(:'endSnapshot') && !attributes.key?(:'end_snapshot') # rubocop:disable Style/StringLiterals

      self.baseline_name = attributes[:'baselineName'] if attributes[:'baselineName']
      self.baseline_name = "null" if baseline_name.nil? && !attributes.key?(:'baselineName') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :baselineName and :baseline_name' if attributes.key?(:'baselineName') && attributes.key?(:'baseline_name')

      self.baseline_name = attributes[:'baseline_name'] if attributes[:'baseline_name']
      self.baseline_name = "null" if baseline_name.nil? && !attributes.key?(:'baselineName') && !attributes.key?(:'baseline_name') # rubocop:disable Style/StringLiterals
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] load_type Object to be assigned
    def load_type=(load_type)
      raise "Invalid value for 'load_type': this must be one of the values in LOAD_TYPE_ENUM." if load_type && !LOAD_TYPE_ENUM.include?(load_type)

      @load_type = load_type
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] recursive_sql Object to be assigned
    def recursive_sql=(recursive_sql)
      raise "Invalid value for 'recursive_sql': this must be one of the values in RECURSIVE_SQL_ENUM." if recursive_sql && !RECURSIVE_SQL_ENUM.include?(recursive_sql)

      @recursive_sql = recursive_sql
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] ranking_measure1 Object to be assigned
    def ranking_measure1=(ranking_measure1)
      raise "Invalid value for 'ranking_measure1': this must be one of the values in RANKING_MEASURE1_ENUM." if ranking_measure1 && !RANKING_MEASURE1_ENUM.include?(ranking_measure1)

      @ranking_measure1 = ranking_measure1
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] ranking_measure2 Object to be assigned
    def ranking_measure2=(ranking_measure2)
      raise "Invalid value for 'ranking_measure2': this must be one of the values in RANKING_MEASURE2_ENUM." if ranking_measure2 && !RANKING_MEASURE2_ENUM.include?(ranking_measure2)

      @ranking_measure2 = ranking_measure2
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] ranking_measure3 Object to be assigned
    def ranking_measure3=(ranking_measure3)
      raise "Invalid value for 'ranking_measure3': this must be one of the values in RANKING_MEASURE3_ENUM." if ranking_measure3 && !RANKING_MEASURE3_ENUM.include?(ranking_measure3)

      @ranking_measure3 = ranking_measure3
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] capture_option Object to be assigned
    def capture_option=(capture_option)
      raise "Invalid value for 'capture_option': this must be one of the values in CAPTURE_OPTION_ENUM." if capture_option && !CAPTURE_OPTION_ENUM.include?(capture_option)

      @capture_option = capture_option
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] capture_mode Object to be assigned
    def capture_mode=(capture_mode)
      raise "Invalid value for 'capture_mode': this must be one of the values in CAPTURE_MODE_ENUM." if capture_mode && !CAPTURE_MODE_ENUM.include?(capture_mode)

      @capture_mode = capture_mode
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] load_option Object to be assigned
    def load_option=(load_option)
      raise "Invalid value for 'load_option': this must be one of the values in LOAD_OPTION_ENUM." if load_option && !LOAD_OPTION_ENUM.include?(load_option)

      @load_option = load_option
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] update_option Object to be assigned
    def update_option=(update_option)
      raise "Invalid value for 'update_option': this must be one of the values in UPDATE_OPTION_ENUM." if update_option && !UPDATE_OPTION_ENUM.include?(update_option)

      @update_option = update_option
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] update_condition Object to be assigned
    def update_condition=(update_condition)
      raise "Invalid value for 'update_condition': this must be one of the values in UPDATE_CONDITION_ENUM." if update_condition && !UPDATE_CONDITION_ENUM.include?(update_condition)

      @update_condition = update_condition
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        credential_details == other.credential_details &&
        show_sql_only == other.show_sql_only &&
        owner == other.owner &&
        name == other.name &&
        load_type == other.load_type &&
        basic_filter == other.basic_filter &&
        recursive_sql == other.recursive_sql &&
        result_percentage == other.result_percentage &&
        result_limit == other.result_limit &&
        ranking_measure1 == other.ranking_measure1 &&
        ranking_measure2 == other.ranking_measure2 &&
        ranking_measure3 == other.ranking_measure3 &&
        total_time_limit == other.total_time_limit &&
        repeat_interval == other.repeat_interval &&
        capture_option == other.capture_option &&
        capture_mode == other.capture_mode &&
        attribute_list == other.attribute_list &&
        load_option == other.load_option &&
        update_option == other.update_option &&
        update_attributes == other.update_attributes &&
        update_condition == other.update_condition &&
        is_ignore_null == other.is_ignore_null &&
        commit_rows == other.commit_rows &&
        begin_snapshot == other.begin_snapshot &&
        end_snapshot == other.end_snapshot &&
        baseline_name == other.baseline_name
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [credential_details, show_sql_only, owner, name, load_type, basic_filter, recursive_sql, result_percentage, result_limit, ranking_measure1, ranking_measure2, ranking_measure3, total_time_limit, repeat_interval, capture_option, capture_mode, attribute_list, load_option, update_option, update_attributes, update_condition, is_ignore_null, commit_rows, begin_snapshot, end_snapshot, baseline_name].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
