# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20210630
require 'date'
require_relative 'build_pipeline_stage'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Specifies the Trigger Deployment stage, which runs another pipeline of the application.
  class Devops::Models::TriggerDeploymentStage < Devops::Models::BuildPipelineStage
    # **[Required]** A target deployment pipeline OCID that will run in this stage.
    # @return [String]
    attr_accessor :deploy_pipeline_id

    # **[Required]** A boolean flag that specifies whether all the parameters must be passed when the deployment is triggered.
    # @return [BOOLEAN]
    attr_accessor :is_pass_all_parameters_enabled

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'id',
        'display_name': :'displayName',
        'description': :'description',
        'project_id': :'projectId',
        'build_pipeline_id': :'buildPipelineId',
        'compartment_id': :'compartmentId',
        'build_pipeline_stage_type': :'buildPipelineStageType',
        'time_created': :'timeCreated',
        'time_updated': :'timeUpdated',
        'lifecycle_state': :'lifecycleState',
        'lifecycle_details': :'lifecycleDetails',
        'build_pipeline_stage_predecessor_collection': :'buildPipelineStagePredecessorCollection',
        'freeform_tags': :'freeformTags',
        'defined_tags': :'definedTags',
        'system_tags': :'systemTags',
        'deploy_pipeline_id': :'deployPipelineId',
        'is_pass_all_parameters_enabled': :'isPassAllParametersEnabled'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'String',
        'display_name': :'String',
        'description': :'String',
        'project_id': :'String',
        'build_pipeline_id': :'String',
        'compartment_id': :'String',
        'build_pipeline_stage_type': :'String',
        'time_created': :'DateTime',
        'time_updated': :'DateTime',
        'lifecycle_state': :'String',
        'lifecycle_details': :'String',
        'build_pipeline_stage_predecessor_collection': :'OCI::Devops::Models::BuildPipelineStagePredecessorCollection',
        'freeform_tags': :'Hash<String, String>',
        'defined_tags': :'Hash<String, Hash<String, Object>>',
        'system_tags': :'Hash<String, Hash<String, Object>>',
        'deploy_pipeline_id': :'String',
        'is_pass_all_parameters_enabled': :'BOOLEAN'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :id The value to assign to the {OCI::Devops::Models::BuildPipelineStage#id #id} proprety
    # @option attributes [String] :display_name The value to assign to the {OCI::Devops::Models::BuildPipelineStage#display_name #display_name} proprety
    # @option attributes [String] :description The value to assign to the {OCI::Devops::Models::BuildPipelineStage#description #description} proprety
    # @option attributes [String] :project_id The value to assign to the {OCI::Devops::Models::BuildPipelineStage#project_id #project_id} proprety
    # @option attributes [String] :build_pipeline_id The value to assign to the {OCI::Devops::Models::BuildPipelineStage#build_pipeline_id #build_pipeline_id} proprety
    # @option attributes [String] :compartment_id The value to assign to the {OCI::Devops::Models::BuildPipelineStage#compartment_id #compartment_id} proprety
    # @option attributes [DateTime] :time_created The value to assign to the {OCI::Devops::Models::BuildPipelineStage#time_created #time_created} proprety
    # @option attributes [DateTime] :time_updated The value to assign to the {OCI::Devops::Models::BuildPipelineStage#time_updated #time_updated} proprety
    # @option attributes [String] :lifecycle_state The value to assign to the {OCI::Devops::Models::BuildPipelineStage#lifecycle_state #lifecycle_state} proprety
    # @option attributes [String] :lifecycle_details The value to assign to the {OCI::Devops::Models::BuildPipelineStage#lifecycle_details #lifecycle_details} proprety
    # @option attributes [OCI::Devops::Models::BuildPipelineStagePredecessorCollection] :build_pipeline_stage_predecessor_collection The value to assign to the {OCI::Devops::Models::BuildPipelineStage#build_pipeline_stage_predecessor_collection #build_pipeline_stage_predecessor_collection} proprety
    # @option attributes [Hash<String, String>] :freeform_tags The value to assign to the {OCI::Devops::Models::BuildPipelineStage#freeform_tags #freeform_tags} proprety
    # @option attributes [Hash<String, Hash<String, Object>>] :defined_tags The value to assign to the {OCI::Devops::Models::BuildPipelineStage#defined_tags #defined_tags} proprety
    # @option attributes [Hash<String, Hash<String, Object>>] :system_tags The value to assign to the {OCI::Devops::Models::BuildPipelineStage#system_tags #system_tags} proprety
    # @option attributes [String] :deploy_pipeline_id The value to assign to the {#deploy_pipeline_id} property
    # @option attributes [BOOLEAN] :is_pass_all_parameters_enabled The value to assign to the {#is_pass_all_parameters_enabled} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      attributes['buildPipelineStageType'] = 'TRIGGER_DEPLOYMENT_PIPELINE'

      super(attributes)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.deploy_pipeline_id = attributes[:'deployPipelineId'] if attributes[:'deployPipelineId']

      raise 'You cannot provide both :deployPipelineId and :deploy_pipeline_id' if attributes.key?(:'deployPipelineId') && attributes.key?(:'deploy_pipeline_id')

      self.deploy_pipeline_id = attributes[:'deploy_pipeline_id'] if attributes[:'deploy_pipeline_id']

      self.is_pass_all_parameters_enabled = attributes[:'isPassAllParametersEnabled'] unless attributes[:'isPassAllParametersEnabled'].nil?

      raise 'You cannot provide both :isPassAllParametersEnabled and :is_pass_all_parameters_enabled' if attributes.key?(:'isPassAllParametersEnabled') && attributes.key?(:'is_pass_all_parameters_enabled')

      self.is_pass_all_parameters_enabled = attributes[:'is_pass_all_parameters_enabled'] unless attributes[:'is_pass_all_parameters_enabled'].nil?
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        id == other.id &&
        display_name == other.display_name &&
        description == other.description &&
        project_id == other.project_id &&
        build_pipeline_id == other.build_pipeline_id &&
        compartment_id == other.compartment_id &&
        build_pipeline_stage_type == other.build_pipeline_stage_type &&
        time_created == other.time_created &&
        time_updated == other.time_updated &&
        lifecycle_state == other.lifecycle_state &&
        lifecycle_details == other.lifecycle_details &&
        build_pipeline_stage_predecessor_collection == other.build_pipeline_stage_predecessor_collection &&
        freeform_tags == other.freeform_tags &&
        defined_tags == other.defined_tags &&
        system_tags == other.system_tags &&
        deploy_pipeline_id == other.deploy_pipeline_id &&
        is_pass_all_parameters_enabled == other.is_pass_all_parameters_enabled
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [id, display_name, description, project_id, build_pipeline_id, compartment_id, build_pipeline_stage_type, time_created, time_updated, lifecycle_state, lifecycle_details, build_pipeline_stage_predecessor_collection, freeform_tags, defined_tags, system_tags, deploy_pipeline_id, is_pass_all_parameters_enabled].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
