# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20171215
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Details for creating the export.
  class FileStorage::Models::CreateExportDetails
    # Export options for the new export. If left unspecified,
    # defaults to:
    #
    #        [
    #          {
    #             \"source\" : \"0.0.0.0/0\",
    #             \"requirePrivilegedSourcePort\" : false,
    #             \"access\": \"READ_WRITE\",
    #             \"identitySquash\": \"NONE\",
    #             \"anonymousUid\": 65534,
    #             \"anonymousGid\": 65534,
    #             \"isAnonymousAccessAllowed\": false,
    #             \"allowedAuth\": [\"SYS\"]
    #           }
    #        ]
    #
    #   **Note:** Mount targets do not have Internet-routable IP
    #   addresses.  Therefore they will not be reachable from the
    #   Internet, even if an associated `ClientOptions` item has
    #   a source of `0.0.0.0/0`.
    #
    #   **If set to the empty array then the export will not be
    #   visible to any clients.**
    #
    #   The export's `exportOptions` can be changed after creation
    #   using the `UpdateExport` operation.
    #
    # @return [Array<OCI::FileStorage::Models::ClientOptions>]
    attr_accessor :export_options

    # **[Required]** The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of this export's export set.
    # @return [String]
    attr_accessor :export_set_id

    # **[Required]** The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of this export's file system.
    # @return [String]
    attr_accessor :file_system_id

    # **[Required]** Path used to access the associated file system.
    #
    # Avoid entering confidential information.
    #
    # Example: `/mediafiles`
    #
    # @return [String]
    attr_accessor :path

    # Whether or not the export should use ID mapping for Unix groups rather than the group list provided within an NFS request's RPC header. When this flag is true the Unix UID from the RPC header is used to retrieve the list of secondary groups from a the ID mapping subsystem. The primary GID is always taken from the RPC header. If ID mapping is not configured, incorrectly configured, unavailable, or cannot be used to determine a list of secondary groups then an empty secondary group list is used for authorization. If the number of groups exceeds the limit of 256 groups, the list retrieved from LDAP is truncated to the first 256 groups read.
    # @return [BOOLEAN]
    attr_accessor :is_idmap_groups_for_sys_auth

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'export_options': :'exportOptions',
        'export_set_id': :'exportSetId',
        'file_system_id': :'fileSystemId',
        'path': :'path',
        'is_idmap_groups_for_sys_auth': :'isIdmapGroupsForSysAuth'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'export_options': :'Array<OCI::FileStorage::Models::ClientOptions>',
        'export_set_id': :'String',
        'file_system_id': :'String',
        'path': :'String',
        'is_idmap_groups_for_sys_auth': :'BOOLEAN'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [Array<OCI::FileStorage::Models::ClientOptions>] :export_options The value to assign to the {#export_options} property
    # @option attributes [String] :export_set_id The value to assign to the {#export_set_id} property
    # @option attributes [String] :file_system_id The value to assign to the {#file_system_id} property
    # @option attributes [String] :path The value to assign to the {#path} property
    # @option attributes [BOOLEAN] :is_idmap_groups_for_sys_auth The value to assign to the {#is_idmap_groups_for_sys_auth} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.export_options = attributes[:'exportOptions'] if attributes[:'exportOptions']

      raise 'You cannot provide both :exportOptions and :export_options' if attributes.key?(:'exportOptions') && attributes.key?(:'export_options')

      self.export_options = attributes[:'export_options'] if attributes[:'export_options']

      self.export_set_id = attributes[:'exportSetId'] if attributes[:'exportSetId']

      raise 'You cannot provide both :exportSetId and :export_set_id' if attributes.key?(:'exportSetId') && attributes.key?(:'export_set_id')

      self.export_set_id = attributes[:'export_set_id'] if attributes[:'export_set_id']

      self.file_system_id = attributes[:'fileSystemId'] if attributes[:'fileSystemId']

      raise 'You cannot provide both :fileSystemId and :file_system_id' if attributes.key?(:'fileSystemId') && attributes.key?(:'file_system_id')

      self.file_system_id = attributes[:'file_system_id'] if attributes[:'file_system_id']

      self.path = attributes[:'path'] if attributes[:'path']

      self.is_idmap_groups_for_sys_auth = attributes[:'isIdmapGroupsForSysAuth'] unless attributes[:'isIdmapGroupsForSysAuth'].nil?

      raise 'You cannot provide both :isIdmapGroupsForSysAuth and :is_idmap_groups_for_sys_auth' if attributes.key?(:'isIdmapGroupsForSysAuth') && attributes.key?(:'is_idmap_groups_for_sys_auth')

      self.is_idmap_groups_for_sys_auth = attributes[:'is_idmap_groups_for_sys_auth'] unless attributes[:'is_idmap_groups_for_sys_auth'].nil?
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        export_options == other.export_options &&
        export_set_id == other.export_set_id &&
        file_system_id == other.file_system_id &&
        path == other.path &&
        is_idmap_groups_for_sys_auth == other.is_idmap_groups_for_sys_auth
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [export_options, export_set_id, file_system_id, path, is_idmap_groups_for_sys_auth].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
