# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20171215
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Replications are the primary resource that governs the policy of cross-region replication between source
  # and target file systems. Replications are associated with a secondary resource called a {ReplicationTarget}
  # located in another availability domain in the same or different region.
  # The replication retrieves the delta of data between two snapshots of a source file system
  # and sends it to the associated `ReplicationTarget`, which applies it to the target
  # file system. For more information, see [File System Replication](https://docs.cloud.oracle.com/iaas/Content/File/Tasks/FSreplication.htm).
  #
  class FileStorage::Models::Replication
    LIFECYCLE_STATE_ENUM = [
      LIFECYCLE_STATE_CREATING = 'CREATING'.freeze,
      LIFECYCLE_STATE_ACTIVE = 'ACTIVE'.freeze,
      LIFECYCLE_STATE_DELETING = 'DELETING'.freeze,
      LIFECYCLE_STATE_DELETED = 'DELETED'.freeze,
      LIFECYCLE_STATE_FAILED = 'FAILED'.freeze,
      LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    DELTA_STATUS_ENUM = [
      DELTA_STATUS_IDLE = 'IDLE'.freeze,
      DELTA_STATUS_CAPTURING = 'CAPTURING'.freeze,
      DELTA_STATUS_APPLYING = 'APPLYING'.freeze,
      DELTA_STATUS_SERVICE_ERROR = 'SERVICE_ERROR'.freeze,
      DELTA_STATUS_USER_ERROR = 'USER_ERROR'.freeze,
      DELTA_STATUS_FAILED = 'FAILED'.freeze,
      DELTA_STATUS_TRANSFERRING = 'TRANSFERRING'.freeze,
      DELTA_STATUS_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the compartment that contains the replication.
    # @return [String]
    attr_accessor :compartment_id

    # The availability domain that contains the replication. May be unset as a blank or `NULL` value.
    # Example: `Uocm:PHX-AD-2`
    #
    # @return [String]
    attr_accessor :availability_domain

    # **[Required]** The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the replication.
    # @return [String]
    attr_accessor :id

    # **[Required]** The current lifecycle state of the replication.
    # @return [String]
    attr_reader :lifecycle_state

    # **[Required]** A user-friendly name. It does not have to be unique, and it is changeable.
    # Avoid entering confidential information.
    #
    # Example: `My replication`
    #
    # @return [String]
    attr_accessor :display_name

    # **[Required]** The date and time the replication was created
    # in [RFC 3339](https://tools.ietf.org/rfc/rfc3339) timestamp format.
    #
    # Example: `2021-01-04T20:01:29.100Z`
    #
    # @return [DateTime]
    attr_accessor :time_created

    # **[Required]** The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the source file system.
    #
    # @return [String]
    attr_accessor :source_id

    # **[Required]** The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the target file system.
    #
    # @return [String]
    attr_accessor :target_id

    # **[Required]** The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the {ReplicationTarget}.
    #
    # @return [String]
    attr_accessor :replication_target_id

    # Duration in minutes between replication snapshots.
    # @return [Integer]
    attr_accessor :replication_interval

    # The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the last snapshot that has been replicated completely.
    # Empty if the copy of the initial snapshot is not complete.
    #
    # @return [String]
    attr_accessor :last_snapshot_id

    # The {#snapshot_time snapshot_time} of the most recent recoverable replication snapshot
    # in [RFC 3339](https://tools.ietf.org/rfc/rfc3339) timestamp format.
    # Example: `2021-04-04T20:01:29.100Z`
    #
    # @return [DateTime]
    attr_accessor :recovery_point_time

    # The current state of the snapshot during replication operations.
    # @return [String]
    attr_reader :delta_status

    # Additional information about the current 'lifecycleState'.
    # @return [String]
    attr_accessor :lifecycle_details

    # Percentage progress of the current replication cycle.
    #
    # @return [Integer]
    attr_accessor :delta_progress

    # Free-form tags for this resource. Each tag is a simple key-value pair
    #  with no predefined name, type, or namespace.
    # For more information, see [Resource Tags](https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm).
    # Example: `{\"Department\": \"Finance\"}`
    #
    # @return [Hash<String, String>]
    attr_accessor :freeform_tags

    # Defined tags for this resource. Each key is predefined and scoped to a namespace.
    # For more information, see [Resource Tags](https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm).
    # Example: `{\"Operations\": {\"CostCenter\": \"42\"}}`
    #
    # @return [Hash<String, Hash<String, Object>>]
    attr_accessor :defined_tags

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'compartment_id': :'compartmentId',
        'availability_domain': :'availabilityDomain',
        'id': :'id',
        'lifecycle_state': :'lifecycleState',
        'display_name': :'displayName',
        'time_created': :'timeCreated',
        'source_id': :'sourceId',
        'target_id': :'targetId',
        'replication_target_id': :'replicationTargetId',
        'replication_interval': :'replicationInterval',
        'last_snapshot_id': :'lastSnapshotId',
        'recovery_point_time': :'recoveryPointTime',
        'delta_status': :'deltaStatus',
        'lifecycle_details': :'lifecycleDetails',
        'delta_progress': :'deltaProgress',
        'freeform_tags': :'freeformTags',
        'defined_tags': :'definedTags'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'compartment_id': :'String',
        'availability_domain': :'String',
        'id': :'String',
        'lifecycle_state': :'String',
        'display_name': :'String',
        'time_created': :'DateTime',
        'source_id': :'String',
        'target_id': :'String',
        'replication_target_id': :'String',
        'replication_interval': :'Integer',
        'last_snapshot_id': :'String',
        'recovery_point_time': :'DateTime',
        'delta_status': :'String',
        'lifecycle_details': :'String',
        'delta_progress': :'Integer',
        'freeform_tags': :'Hash<String, String>',
        'defined_tags': :'Hash<String, Hash<String, Object>>'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :compartment_id The value to assign to the {#compartment_id} property
    # @option attributes [String] :availability_domain The value to assign to the {#availability_domain} property
    # @option attributes [String] :id The value to assign to the {#id} property
    # @option attributes [String] :lifecycle_state The value to assign to the {#lifecycle_state} property
    # @option attributes [String] :display_name The value to assign to the {#display_name} property
    # @option attributes [DateTime] :time_created The value to assign to the {#time_created} property
    # @option attributes [String] :source_id The value to assign to the {#source_id} property
    # @option attributes [String] :target_id The value to assign to the {#target_id} property
    # @option attributes [String] :replication_target_id The value to assign to the {#replication_target_id} property
    # @option attributes [Integer] :replication_interval The value to assign to the {#replication_interval} property
    # @option attributes [String] :last_snapshot_id The value to assign to the {#last_snapshot_id} property
    # @option attributes [DateTime] :recovery_point_time The value to assign to the {#recovery_point_time} property
    # @option attributes [String] :delta_status The value to assign to the {#delta_status} property
    # @option attributes [String] :lifecycle_details The value to assign to the {#lifecycle_details} property
    # @option attributes [Integer] :delta_progress The value to assign to the {#delta_progress} property
    # @option attributes [Hash<String, String>] :freeform_tags The value to assign to the {#freeform_tags} property
    # @option attributes [Hash<String, Hash<String, Object>>] :defined_tags The value to assign to the {#defined_tags} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.compartment_id = attributes[:'compartmentId'] if attributes[:'compartmentId']

      raise 'You cannot provide both :compartmentId and :compartment_id' if attributes.key?(:'compartmentId') && attributes.key?(:'compartment_id')

      self.compartment_id = attributes[:'compartment_id'] if attributes[:'compartment_id']

      self.availability_domain = attributes[:'availabilityDomain'] if attributes[:'availabilityDomain']

      raise 'You cannot provide both :availabilityDomain and :availability_domain' if attributes.key?(:'availabilityDomain') && attributes.key?(:'availability_domain')

      self.availability_domain = attributes[:'availability_domain'] if attributes[:'availability_domain']

      self.id = attributes[:'id'] if attributes[:'id']

      self.lifecycle_state = attributes[:'lifecycleState'] if attributes[:'lifecycleState']

      raise 'You cannot provide both :lifecycleState and :lifecycle_state' if attributes.key?(:'lifecycleState') && attributes.key?(:'lifecycle_state')

      self.lifecycle_state = attributes[:'lifecycle_state'] if attributes[:'lifecycle_state']

      self.display_name = attributes[:'displayName'] if attributes[:'displayName']

      raise 'You cannot provide both :displayName and :display_name' if attributes.key?(:'displayName') && attributes.key?(:'display_name')

      self.display_name = attributes[:'display_name'] if attributes[:'display_name']

      self.time_created = attributes[:'timeCreated'] if attributes[:'timeCreated']

      raise 'You cannot provide both :timeCreated and :time_created' if attributes.key?(:'timeCreated') && attributes.key?(:'time_created')

      self.time_created = attributes[:'time_created'] if attributes[:'time_created']

      self.source_id = attributes[:'sourceId'] if attributes[:'sourceId']

      raise 'You cannot provide both :sourceId and :source_id' if attributes.key?(:'sourceId') && attributes.key?(:'source_id')

      self.source_id = attributes[:'source_id'] if attributes[:'source_id']

      self.target_id = attributes[:'targetId'] if attributes[:'targetId']

      raise 'You cannot provide both :targetId and :target_id' if attributes.key?(:'targetId') && attributes.key?(:'target_id')

      self.target_id = attributes[:'target_id'] if attributes[:'target_id']

      self.replication_target_id = attributes[:'replicationTargetId'] if attributes[:'replicationTargetId']

      raise 'You cannot provide both :replicationTargetId and :replication_target_id' if attributes.key?(:'replicationTargetId') && attributes.key?(:'replication_target_id')

      self.replication_target_id = attributes[:'replication_target_id'] if attributes[:'replication_target_id']

      self.replication_interval = attributes[:'replicationInterval'] if attributes[:'replicationInterval']

      raise 'You cannot provide both :replicationInterval and :replication_interval' if attributes.key?(:'replicationInterval') && attributes.key?(:'replication_interval')

      self.replication_interval = attributes[:'replication_interval'] if attributes[:'replication_interval']

      self.last_snapshot_id = attributes[:'lastSnapshotId'] if attributes[:'lastSnapshotId']

      raise 'You cannot provide both :lastSnapshotId and :last_snapshot_id' if attributes.key?(:'lastSnapshotId') && attributes.key?(:'last_snapshot_id')

      self.last_snapshot_id = attributes[:'last_snapshot_id'] if attributes[:'last_snapshot_id']

      self.recovery_point_time = attributes[:'recoveryPointTime'] if attributes[:'recoveryPointTime']

      raise 'You cannot provide both :recoveryPointTime and :recovery_point_time' if attributes.key?(:'recoveryPointTime') && attributes.key?(:'recovery_point_time')

      self.recovery_point_time = attributes[:'recovery_point_time'] if attributes[:'recovery_point_time']

      self.delta_status = attributes[:'deltaStatus'] if attributes[:'deltaStatus']

      raise 'You cannot provide both :deltaStatus and :delta_status' if attributes.key?(:'deltaStatus') && attributes.key?(:'delta_status')

      self.delta_status = attributes[:'delta_status'] if attributes[:'delta_status']

      self.lifecycle_details = attributes[:'lifecycleDetails'] if attributes[:'lifecycleDetails']

      raise 'You cannot provide both :lifecycleDetails and :lifecycle_details' if attributes.key?(:'lifecycleDetails') && attributes.key?(:'lifecycle_details')

      self.lifecycle_details = attributes[:'lifecycle_details'] if attributes[:'lifecycle_details']

      self.delta_progress = attributes[:'deltaProgress'] if attributes[:'deltaProgress']

      raise 'You cannot provide both :deltaProgress and :delta_progress' if attributes.key?(:'deltaProgress') && attributes.key?(:'delta_progress')

      self.delta_progress = attributes[:'delta_progress'] if attributes[:'delta_progress']

      self.freeform_tags = attributes[:'freeformTags'] if attributes[:'freeformTags']

      raise 'You cannot provide both :freeformTags and :freeform_tags' if attributes.key?(:'freeformTags') && attributes.key?(:'freeform_tags')

      self.freeform_tags = attributes[:'freeform_tags'] if attributes[:'freeform_tags']

      self.defined_tags = attributes[:'definedTags'] if attributes[:'definedTags']

      raise 'You cannot provide both :definedTags and :defined_tags' if attributes.key?(:'definedTags') && attributes.key?(:'defined_tags')

      self.defined_tags = attributes[:'defined_tags'] if attributes[:'defined_tags']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] lifecycle_state Object to be assigned
    def lifecycle_state=(lifecycle_state)
      # rubocop:disable Style/ConditionalAssignment
      if lifecycle_state && !LIFECYCLE_STATE_ENUM.include?(lifecycle_state)
        OCI.logger.debug("Unknown value for 'lifecycle_state' [" + lifecycle_state + "]. Mapping to 'LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @lifecycle_state = LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE
      else
        @lifecycle_state = lifecycle_state
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] delta_status Object to be assigned
    def delta_status=(delta_status)
      # rubocop:disable Style/ConditionalAssignment
      if delta_status && !DELTA_STATUS_ENUM.include?(delta_status)
        OCI.logger.debug("Unknown value for 'delta_status' [" + delta_status + "]. Mapping to 'DELTA_STATUS_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @delta_status = DELTA_STATUS_UNKNOWN_ENUM_VALUE
      else
        @delta_status = delta_status
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        compartment_id == other.compartment_id &&
        availability_domain == other.availability_domain &&
        id == other.id &&
        lifecycle_state == other.lifecycle_state &&
        display_name == other.display_name &&
        time_created == other.time_created &&
        source_id == other.source_id &&
        target_id == other.target_id &&
        replication_target_id == other.replication_target_id &&
        replication_interval == other.replication_interval &&
        last_snapshot_id == other.last_snapshot_id &&
        recovery_point_time == other.recovery_point_time &&
        delta_status == other.delta_status &&
        lifecycle_details == other.lifecycle_details &&
        delta_progress == other.delta_progress &&
        freeform_tags == other.freeform_tags &&
        defined_tags == other.defined_tags
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [compartment_id, availability_domain, id, lifecycle_state, display_name, time_created, source_id, target_id, replication_target_id, replication_interval, last_snapshot_id, recovery_point_time, delta_status, lifecycle_details, delta_progress, freeform_tags, defined_tags].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
