# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20220528
require 'date'
require 'logger'
require_relative 'discovery_details'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Details to create a 'GI' type Exadata Fleet Update Discovery.
  #
  class FleetSoftwareUpdate::Models::GiDiscoveryDetails < FleetSoftwareUpdate::Models::DiscoveryDetails
    SOURCE_MAJOR_VERSION_ENUM = [
      SOURCE_MAJOR_VERSION_GI_18 = 'GI_18'.freeze,
      SOURCE_MAJOR_VERSION_GI_19 = 'GI_19'.freeze,
      SOURCE_MAJOR_VERSION_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** Grid Infrastructure Major Version of targets to be included in the Exadata Fleet Update Discovery results.
    # Only GI targets that match the version specified in this value would be added to the Exadata Fleet Update Discovery results.
    #
    # @return [String]
    attr_reader :source_major_version

    # This attribute is required.
    # @return [OCI::FleetSoftwareUpdate::Models::GiFleetDiscoveryDetails]
    attr_accessor :criteria

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'type': :'type',
        'service_type': :'serviceType',
        'source_major_version': :'sourceMajorVersion',
        'criteria': :'criteria'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'type': :'String',
        'service_type': :'String',
        'source_major_version': :'String',
        'criteria': :'OCI::FleetSoftwareUpdate::Models::GiFleetDiscoveryDetails'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :service_type The value to assign to the {OCI::FleetSoftwareUpdate::Models::DiscoveryDetails#service_type #service_type} proprety
    # @option attributes [String] :source_major_version The value to assign to the {#source_major_version} property
    # @option attributes [OCI::FleetSoftwareUpdate::Models::GiFleetDiscoveryDetails] :criteria The value to assign to the {#criteria} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      attributes['type'] = 'GI'

      super(attributes)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.source_major_version = attributes[:'sourceMajorVersion'] if attributes[:'sourceMajorVersion']

      raise 'You cannot provide both :sourceMajorVersion and :source_major_version' if attributes.key?(:'sourceMajorVersion') && attributes.key?(:'source_major_version')

      self.source_major_version = attributes[:'source_major_version'] if attributes[:'source_major_version']

      self.criteria = attributes[:'criteria'] if attributes[:'criteria']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] source_major_version Object to be assigned
    def source_major_version=(source_major_version)
      # rubocop:disable Style/ConditionalAssignment
      if source_major_version && !SOURCE_MAJOR_VERSION_ENUM.include?(source_major_version)
        OCI.logger.debug("Unknown value for 'source_major_version' [" + source_major_version + "]. Mapping to 'SOURCE_MAJOR_VERSION_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @source_major_version = SOURCE_MAJOR_VERSION_UNKNOWN_ENUM_VALUE
      else
        @source_major_version = source_major_version
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        type == other.type &&
        service_type == other.service_type &&
        source_major_version == other.source_major_version &&
        criteria == other.criteria
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [type, service_type, source_major_version, criteria].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
