# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20211201
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Status of capabilities that can be enabled for an environment family.
  class FusionApps::Models::Capabilities
    # Indicates whether data masking is enabled for the environment family. When enabled, data masking activities are supported.
    # @return [BOOLEAN]
    attr_accessor :is_data_masking_enabled

    # Indicates whether Break Glass is enabled for the environment family.
    # @return [BOOLEAN]
    attr_accessor :is_break_glass_enabled

    # Indicates whether customers can use their own encryption keys.
    # @return [BOOLEAN]
    attr_accessor :is_byok_enabled

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'is_data_masking_enabled': :'isDataMaskingEnabled',
        'is_break_glass_enabled': :'isBreakGlassEnabled',
        'is_byok_enabled': :'isByokEnabled'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'is_data_masking_enabled': :'BOOLEAN',
        'is_break_glass_enabled': :'BOOLEAN',
        'is_byok_enabled': :'BOOLEAN'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [BOOLEAN] :is_data_masking_enabled The value to assign to the {#is_data_masking_enabled} property
    # @option attributes [BOOLEAN] :is_break_glass_enabled The value to assign to the {#is_break_glass_enabled} property
    # @option attributes [BOOLEAN] :is_byok_enabled The value to assign to the {#is_byok_enabled} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.is_data_masking_enabled = attributes[:'isDataMaskingEnabled'] unless attributes[:'isDataMaskingEnabled'].nil?
      self.is_data_masking_enabled = false if is_data_masking_enabled.nil? && !attributes.key?(:'isDataMaskingEnabled') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :isDataMaskingEnabled and :is_data_masking_enabled' if attributes.key?(:'isDataMaskingEnabled') && attributes.key?(:'is_data_masking_enabled')

      self.is_data_masking_enabled = attributes[:'is_data_masking_enabled'] unless attributes[:'is_data_masking_enabled'].nil?
      self.is_data_masking_enabled = false if is_data_masking_enabled.nil? && !attributes.key?(:'isDataMaskingEnabled') && !attributes.key?(:'is_data_masking_enabled') # rubocop:disable Style/StringLiterals

      self.is_break_glass_enabled = attributes[:'isBreakGlassEnabled'] unless attributes[:'isBreakGlassEnabled'].nil?
      self.is_break_glass_enabled = false if is_break_glass_enabled.nil? && !attributes.key?(:'isBreakGlassEnabled') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :isBreakGlassEnabled and :is_break_glass_enabled' if attributes.key?(:'isBreakGlassEnabled') && attributes.key?(:'is_break_glass_enabled')

      self.is_break_glass_enabled = attributes[:'is_break_glass_enabled'] unless attributes[:'is_break_glass_enabled'].nil?
      self.is_break_glass_enabled = false if is_break_glass_enabled.nil? && !attributes.key?(:'isBreakGlassEnabled') && !attributes.key?(:'is_break_glass_enabled') # rubocop:disable Style/StringLiterals

      self.is_byok_enabled = attributes[:'isByokEnabled'] unless attributes[:'isByokEnabled'].nil?
      self.is_byok_enabled = false if is_byok_enabled.nil? && !attributes.key?(:'isByokEnabled') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :isByokEnabled and :is_byok_enabled' if attributes.key?(:'isByokEnabled') && attributes.key?(:'is_byok_enabled')

      self.is_byok_enabled = attributes[:'is_byok_enabled'] unless attributes[:'is_byok_enabled'].nil?
      self.is_byok_enabled = false if is_byok_enabled.nil? && !attributes.key?(:'isByokEnabled') && !attributes.key?(:'is_byok_enabled') # rubocop:disable Style/StringLiterals
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        is_data_masking_enabled == other.is_data_masking_enabled &&
        is_break_glass_enabled == other.is_break_glass_enabled &&
        is_byok_enabled == other.is_byok_enabled
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [is_data_masking_enabled, is_break_glass_enabled, is_byok_enabled].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
