# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20220126
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Description of Lockbox.
  class Lockbox::Models::Lockbox
    LOCKBOX_PARTNER_ENUM = [
      LOCKBOX_PARTNER_FAAAS = 'FAAAS'.freeze,
      LOCKBOX_PARTNER_CANARY = 'CANARY'.freeze,
      LOCKBOX_PARTNER_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    LIFECYCLE_STATE_ENUM = [
      LIFECYCLE_STATE_ACTIVE = 'ACTIVE'.freeze,
      LIFECYCLE_STATE_CREATING = 'CREATING'.freeze,
      LIFECYCLE_STATE_UPDATING = 'UPDATING'.freeze,
      LIFECYCLE_STATE_DELETING = 'DELETING'.freeze,
      LIFECYCLE_STATE_DELETED = 'DELETED'.freeze,
      LIFECYCLE_STATE_FAILED = 'FAILED'.freeze,
      LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** Unique identifier that is immutable on creation
    # @return [String]
    attr_accessor :id

    # **[Required]** Lockbox Identifier, can be renamed
    # @return [String]
    attr_accessor :display_name

    # **[Required]** Compartment Identifier
    # @return [String]
    attr_accessor :compartment_id

    # The unique identifier (OCID) of partner resource using this lockbox to lock a resource
    # @return [String]
    attr_accessor :partner_id

    # The unique identifier (OCID) of lockbox resource using to reference parent lockbox in hybrid oma setup
    # @return [String]
    attr_accessor :parent_lockbox_id

    # Compartment Identifier
    # @return [String]
    attr_accessor :partner_compartment_id

    # **[Required]** The unique identifier (OCID) of associated resource that the lockbox is created for.
    # @return [String]
    attr_accessor :resource_id

    # The partner using this lockbox to lock a resource.
    # @return [String]
    attr_reader :lockbox_partner

    # **[Required]** The time the the Lockbox was created. An RFC3339 formatted datetime string
    # @return [DateTime]
    attr_accessor :time_created

    # The time the Lockbox was updated. An RFC3339 formatted datetime string
    # @return [DateTime]
    attr_accessor :time_updated

    # **[Required]** The current state of the Lockbox.
    # @return [String]
    attr_reader :lifecycle_state

    # @return [OCI::Lockbox::Models::AccessContextAttributeCollection]
    attr_accessor :access_context_attributes

    # Approval template ID
    # @return [String]
    attr_accessor :approval_template_id

    # The maximum amount of time operator has access to associated resources.
    # @return [String]
    attr_accessor :max_access_duration

    # A message describing the current state in more detail. For example, can be used to provide actionable information for a resource in Failed state.
    # @return [String]
    attr_accessor :lifecycle_details

    # **[Required]** Simple key-value pair that is applied without any predefined name, type or scope. Exists for cross-compatibility only.
    # Example: `{\"bar-key\": \"value\"}`
    #
    # @return [Hash<String, String>]
    attr_accessor :freeform_tags

    # **[Required]** Defined tags for this resource. Each key is predefined and scoped to a namespace.
    # Example: `{\"foo-namespace\": {\"bar-key\": \"value\"}}`
    #
    # @return [Hash<String, Hash<String, Object>>]
    attr_accessor :defined_tags

    # Usage of system tag keys. These predefined keys are scoped to namespaces.
    # Example: `{\"orcl-cloud\": {\"free-tier-retained\": \"true\"}}`
    #
    # @return [Hash<String, Hash<String, Object>>]
    attr_accessor :system_tags

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'id',
        'display_name': :'displayName',
        'compartment_id': :'compartmentId',
        'partner_id': :'partnerId',
        'parent_lockbox_id': :'parentLockboxId',
        'partner_compartment_id': :'partnerCompartmentId',
        'resource_id': :'resourceId',
        'lockbox_partner': :'lockboxPartner',
        'time_created': :'timeCreated',
        'time_updated': :'timeUpdated',
        'lifecycle_state': :'lifecycleState',
        'access_context_attributes': :'accessContextAttributes',
        'approval_template_id': :'approvalTemplateId',
        'max_access_duration': :'maxAccessDuration',
        'lifecycle_details': :'lifecycleDetails',
        'freeform_tags': :'freeformTags',
        'defined_tags': :'definedTags',
        'system_tags': :'systemTags'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'String',
        'display_name': :'String',
        'compartment_id': :'String',
        'partner_id': :'String',
        'parent_lockbox_id': :'String',
        'partner_compartment_id': :'String',
        'resource_id': :'String',
        'lockbox_partner': :'String',
        'time_created': :'DateTime',
        'time_updated': :'DateTime',
        'lifecycle_state': :'String',
        'access_context_attributes': :'OCI::Lockbox::Models::AccessContextAttributeCollection',
        'approval_template_id': :'String',
        'max_access_duration': :'String',
        'lifecycle_details': :'String',
        'freeform_tags': :'Hash<String, String>',
        'defined_tags': :'Hash<String, Hash<String, Object>>',
        'system_tags': :'Hash<String, Hash<String, Object>>'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :id The value to assign to the {#id} property
    # @option attributes [String] :display_name The value to assign to the {#display_name} property
    # @option attributes [String] :compartment_id The value to assign to the {#compartment_id} property
    # @option attributes [String] :partner_id The value to assign to the {#partner_id} property
    # @option attributes [String] :parent_lockbox_id The value to assign to the {#parent_lockbox_id} property
    # @option attributes [String] :partner_compartment_id The value to assign to the {#partner_compartment_id} property
    # @option attributes [String] :resource_id The value to assign to the {#resource_id} property
    # @option attributes [String] :lockbox_partner The value to assign to the {#lockbox_partner} property
    # @option attributes [DateTime] :time_created The value to assign to the {#time_created} property
    # @option attributes [DateTime] :time_updated The value to assign to the {#time_updated} property
    # @option attributes [String] :lifecycle_state The value to assign to the {#lifecycle_state} property
    # @option attributes [OCI::Lockbox::Models::AccessContextAttributeCollection] :access_context_attributes The value to assign to the {#access_context_attributes} property
    # @option attributes [String] :approval_template_id The value to assign to the {#approval_template_id} property
    # @option attributes [String] :max_access_duration The value to assign to the {#max_access_duration} property
    # @option attributes [String] :lifecycle_details The value to assign to the {#lifecycle_details} property
    # @option attributes [Hash<String, String>] :freeform_tags The value to assign to the {#freeform_tags} property
    # @option attributes [Hash<String, Hash<String, Object>>] :defined_tags The value to assign to the {#defined_tags} property
    # @option attributes [Hash<String, Hash<String, Object>>] :system_tags The value to assign to the {#system_tags} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.id = attributes[:'id'] if attributes[:'id']

      self.display_name = attributes[:'displayName'] if attributes[:'displayName']

      raise 'You cannot provide both :displayName and :display_name' if attributes.key?(:'displayName') && attributes.key?(:'display_name')

      self.display_name = attributes[:'display_name'] if attributes[:'display_name']

      self.compartment_id = attributes[:'compartmentId'] if attributes[:'compartmentId']

      raise 'You cannot provide both :compartmentId and :compartment_id' if attributes.key?(:'compartmentId') && attributes.key?(:'compartment_id')

      self.compartment_id = attributes[:'compartment_id'] if attributes[:'compartment_id']

      self.partner_id = attributes[:'partnerId'] if attributes[:'partnerId']

      raise 'You cannot provide both :partnerId and :partner_id' if attributes.key?(:'partnerId') && attributes.key?(:'partner_id')

      self.partner_id = attributes[:'partner_id'] if attributes[:'partner_id']

      self.parent_lockbox_id = attributes[:'parentLockboxId'] if attributes[:'parentLockboxId']

      raise 'You cannot provide both :parentLockboxId and :parent_lockbox_id' if attributes.key?(:'parentLockboxId') && attributes.key?(:'parent_lockbox_id')

      self.parent_lockbox_id = attributes[:'parent_lockbox_id'] if attributes[:'parent_lockbox_id']

      self.partner_compartment_id = attributes[:'partnerCompartmentId'] if attributes[:'partnerCompartmentId']

      raise 'You cannot provide both :partnerCompartmentId and :partner_compartment_id' if attributes.key?(:'partnerCompartmentId') && attributes.key?(:'partner_compartment_id')

      self.partner_compartment_id = attributes[:'partner_compartment_id'] if attributes[:'partner_compartment_id']

      self.resource_id = attributes[:'resourceId'] if attributes[:'resourceId']

      raise 'You cannot provide both :resourceId and :resource_id' if attributes.key?(:'resourceId') && attributes.key?(:'resource_id')

      self.resource_id = attributes[:'resource_id'] if attributes[:'resource_id']

      self.lockbox_partner = attributes[:'lockboxPartner'] if attributes[:'lockboxPartner']

      raise 'You cannot provide both :lockboxPartner and :lockbox_partner' if attributes.key?(:'lockboxPartner') && attributes.key?(:'lockbox_partner')

      self.lockbox_partner = attributes[:'lockbox_partner'] if attributes[:'lockbox_partner']

      self.time_created = attributes[:'timeCreated'] if attributes[:'timeCreated']

      raise 'You cannot provide both :timeCreated and :time_created' if attributes.key?(:'timeCreated') && attributes.key?(:'time_created')

      self.time_created = attributes[:'time_created'] if attributes[:'time_created']

      self.time_updated = attributes[:'timeUpdated'] if attributes[:'timeUpdated']

      raise 'You cannot provide both :timeUpdated and :time_updated' if attributes.key?(:'timeUpdated') && attributes.key?(:'time_updated')

      self.time_updated = attributes[:'time_updated'] if attributes[:'time_updated']

      self.lifecycle_state = attributes[:'lifecycleState'] if attributes[:'lifecycleState']

      raise 'You cannot provide both :lifecycleState and :lifecycle_state' if attributes.key?(:'lifecycleState') && attributes.key?(:'lifecycle_state')

      self.lifecycle_state = attributes[:'lifecycle_state'] if attributes[:'lifecycle_state']

      self.access_context_attributes = attributes[:'accessContextAttributes'] if attributes[:'accessContextAttributes']

      raise 'You cannot provide both :accessContextAttributes and :access_context_attributes' if attributes.key?(:'accessContextAttributes') && attributes.key?(:'access_context_attributes')

      self.access_context_attributes = attributes[:'access_context_attributes'] if attributes[:'access_context_attributes']

      self.approval_template_id = attributes[:'approvalTemplateId'] if attributes[:'approvalTemplateId']

      raise 'You cannot provide both :approvalTemplateId and :approval_template_id' if attributes.key?(:'approvalTemplateId') && attributes.key?(:'approval_template_id')

      self.approval_template_id = attributes[:'approval_template_id'] if attributes[:'approval_template_id']

      self.max_access_duration = attributes[:'maxAccessDuration'] if attributes[:'maxAccessDuration']

      raise 'You cannot provide both :maxAccessDuration and :max_access_duration' if attributes.key?(:'maxAccessDuration') && attributes.key?(:'max_access_duration')

      self.max_access_duration = attributes[:'max_access_duration'] if attributes[:'max_access_duration']

      self.lifecycle_details = attributes[:'lifecycleDetails'] if attributes[:'lifecycleDetails']

      raise 'You cannot provide both :lifecycleDetails and :lifecycle_details' if attributes.key?(:'lifecycleDetails') && attributes.key?(:'lifecycle_details')

      self.lifecycle_details = attributes[:'lifecycle_details'] if attributes[:'lifecycle_details']

      self.freeform_tags = attributes[:'freeformTags'] if attributes[:'freeformTags']

      raise 'You cannot provide both :freeformTags and :freeform_tags' if attributes.key?(:'freeformTags') && attributes.key?(:'freeform_tags')

      self.freeform_tags = attributes[:'freeform_tags'] if attributes[:'freeform_tags']

      self.defined_tags = attributes[:'definedTags'] if attributes[:'definedTags']

      raise 'You cannot provide both :definedTags and :defined_tags' if attributes.key?(:'definedTags') && attributes.key?(:'defined_tags')

      self.defined_tags = attributes[:'defined_tags'] if attributes[:'defined_tags']

      self.system_tags = attributes[:'systemTags'] if attributes[:'systemTags']

      raise 'You cannot provide both :systemTags and :system_tags' if attributes.key?(:'systemTags') && attributes.key?(:'system_tags')

      self.system_tags = attributes[:'system_tags'] if attributes[:'system_tags']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] lockbox_partner Object to be assigned
    def lockbox_partner=(lockbox_partner)
      # rubocop:disable Style/ConditionalAssignment
      if lockbox_partner && !LOCKBOX_PARTNER_ENUM.include?(lockbox_partner)
        OCI.logger.debug("Unknown value for 'lockbox_partner' [" + lockbox_partner + "]. Mapping to 'LOCKBOX_PARTNER_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @lockbox_partner = LOCKBOX_PARTNER_UNKNOWN_ENUM_VALUE
      else
        @lockbox_partner = lockbox_partner
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] lifecycle_state Object to be assigned
    def lifecycle_state=(lifecycle_state)
      # rubocop:disable Style/ConditionalAssignment
      if lifecycle_state && !LIFECYCLE_STATE_ENUM.include?(lifecycle_state)
        OCI.logger.debug("Unknown value for 'lifecycle_state' [" + lifecycle_state + "]. Mapping to 'LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @lifecycle_state = LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE
      else
        @lifecycle_state = lifecycle_state
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        id == other.id &&
        display_name == other.display_name &&
        compartment_id == other.compartment_id &&
        partner_id == other.partner_id &&
        parent_lockbox_id == other.parent_lockbox_id &&
        partner_compartment_id == other.partner_compartment_id &&
        resource_id == other.resource_id &&
        lockbox_partner == other.lockbox_partner &&
        time_created == other.time_created &&
        time_updated == other.time_updated &&
        lifecycle_state == other.lifecycle_state &&
        access_context_attributes == other.access_context_attributes &&
        approval_template_id == other.approval_template_id &&
        max_access_duration == other.max_access_duration &&
        lifecycle_details == other.lifecycle_details &&
        freeform_tags == other.freeform_tags &&
        defined_tags == other.defined_tags &&
        system_tags == other.system_tags
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [id, display_name, compartment_id, partner_id, parent_lockbox_id, partner_compartment_id, resource_id, lockbox_partner, time_created, time_updated, lifecycle_state, access_context_attributes, approval_template_id, max_access_duration, lifecycle_details, freeform_tags, defined_tags, system_tags].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
