# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20211101
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # The declaration of a type of task that can be used in a MediaWorkflow.
  class MediaServices::Models::MediaWorkflowTaskDeclaration
    # **[Required]** MediaWorkflowTaskDeclaration identifier. The name and version should be unique among
    # MediaWorkflowTaskDeclarations.
    #
    # @return [String]
    attr_accessor :name

    # **[Required]** The version of MediaWorkflowTaskDeclaration, incremented whenever the team implementing the task processor
    # modifies the JSON schema of this declaration's definitions, parameters or list of required parameters.
    #
    # @return [Integer]
    attr_accessor :version

    # **[Required]** JSON schema specifying the parameters supported by this type of task. This is used to validate tasks'
    # parameters when jobs are created.
    #
    # @return [Hash<String, Object>]
    attr_accessor :parameters_schema

    # **[Required]** JSON schema similar to the parameterSchema, but permits parameter values to refer to other parameters using the
    # ${/path/to/another/parmeter} syntax.  This is used to validate task parameters when workflows are created.
    #
    # @return [Hash<String, Object>]
    attr_accessor :parameters_schema_allowing_references

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'name': :'name',
        'version': :'version',
        'parameters_schema': :'parametersSchema',
        'parameters_schema_allowing_references': :'parametersSchemaAllowingReferences'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'name': :'String',
        'version': :'Integer',
        'parameters_schema': :'Hash<String, Object>',
        'parameters_schema_allowing_references': :'Hash<String, Object>'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :name The value to assign to the {#name} property
    # @option attributes [Integer] :version The value to assign to the {#version} property
    # @option attributes [Hash<String, Object>] :parameters_schema The value to assign to the {#parameters_schema} property
    # @option attributes [Hash<String, Object>] :parameters_schema_allowing_references The value to assign to the {#parameters_schema_allowing_references} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.name = attributes[:'name'] if attributes[:'name']

      self.version = attributes[:'version'] if attributes[:'version']

      self.parameters_schema = attributes[:'parametersSchema'] if attributes[:'parametersSchema']

      raise 'You cannot provide both :parametersSchema and :parameters_schema' if attributes.key?(:'parametersSchema') && attributes.key?(:'parameters_schema')

      self.parameters_schema = attributes[:'parameters_schema'] if attributes[:'parameters_schema']

      self.parameters_schema_allowing_references = attributes[:'parametersSchemaAllowingReferences'] if attributes[:'parametersSchemaAllowingReferences']

      raise 'You cannot provide both :parametersSchemaAllowingReferences and :parameters_schema_allowing_references' if attributes.key?(:'parametersSchemaAllowingReferences') && attributes.key?(:'parameters_schema_allowing_references')

      self.parameters_schema_allowing_references = attributes[:'parameters_schema_allowing_references'] if attributes[:'parameters_schema_allowing_references']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        name == other.name &&
        version == other.version &&
        parameters_schema == other.parameters_schema &&
        parameters_schema_allowing_references == other.parameters_schema_allowing_references
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [name, version, parameters_schema, parameters_schema_allowing_references].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
