# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20190415
require 'date'
require_relative 'anonymous_transactions_handling'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Enables assignment of IDs on the target to anonymous transactions coming from the source. The target server UUID
  # is added as a prefix to the ID.
  #
  class Mysql::Models::AssignTargetUuidHandling < Mysql::Models::AnonymousTransactionsHandling
    # Specifies one of the coordinates (file) at which the replica should begin
    # reading the source's log. As this value specifies the point where replication
    # starts from, it is only used once, when it starts. It is never used again,
    # unless a new UpdateChannel operation modifies it.
    #
    # @return [String]
    attr_accessor :last_configured_log_filename

    # Specifies one of the coordinates (offset) at which the replica should begin
    # reading the source's log. As this value specifies the point where replication
    # starts from, it is only used once, when it starts. It is never used again,
    # unless a new UpdateChannel operation modifies it.
    #
    # @return [Integer]
    attr_accessor :last_configured_log_offset

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'policy': :'policy',
        'last_configured_log_filename': :'lastConfiguredLogFilename',
        'last_configured_log_offset': :'lastConfiguredLogOffset'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'policy': :'String',
        'last_configured_log_filename': :'String',
        'last_configured_log_offset': :'Integer'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :last_configured_log_filename The value to assign to the {#last_configured_log_filename} property
    # @option attributes [Integer] :last_configured_log_offset The value to assign to the {#last_configured_log_offset} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      attributes['policy'] = 'ASSIGN_TARGET_UUID'

      super(attributes)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.last_configured_log_filename = attributes[:'lastConfiguredLogFilename'] if attributes[:'lastConfiguredLogFilename']

      raise 'You cannot provide both :lastConfiguredLogFilename and :last_configured_log_filename' if attributes.key?(:'lastConfiguredLogFilename') && attributes.key?(:'last_configured_log_filename')

      self.last_configured_log_filename = attributes[:'last_configured_log_filename'] if attributes[:'last_configured_log_filename']

      self.last_configured_log_offset = attributes[:'lastConfiguredLogOffset'] if attributes[:'lastConfiguredLogOffset']

      raise 'You cannot provide both :lastConfiguredLogOffset and :last_configured_log_offset' if attributes.key?(:'lastConfiguredLogOffset') && attributes.key?(:'last_configured_log_offset')

      self.last_configured_log_offset = attributes[:'last_configured_log_offset'] if attributes[:'last_configured_log_offset']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        policy == other.policy &&
        last_configured_log_filename == other.last_configured_log_filename &&
        last_configured_log_offset == other.last_configured_log_offset
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [policy, last_configured_log_filename, last_configured_log_offset].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
