# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20190415
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # User-defined service variables.
  class Mysql::Models::ConfigurationVariables
    COMPLETION_TYPE_ENUM = [
      COMPLETION_TYPE_NO_CHAIN = 'NO_CHAIN'.freeze,
      COMPLETION_TYPE_CHAIN = 'CHAIN'.freeze,
      COMPLETION_TYPE_RELEASE = 'RELEASE'.freeze,
      COMPLETION_TYPE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    DEFAULT_AUTHENTICATION_PLUGIN_ENUM = [
      DEFAULT_AUTHENTICATION_PLUGIN_MYSQL_NATIVE_PASSWORD = 'mysql_native_password'.freeze,
      DEFAULT_AUTHENTICATION_PLUGIN_SHA256_PASSWORD = 'sha256_password'.freeze,
      DEFAULT_AUTHENTICATION_PLUGIN_CACHING_SHA2_PASSWORD = 'caching_sha2_password'.freeze,
      DEFAULT_AUTHENTICATION_PLUGIN_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    TRANSACTION_ISOLATION_ENUM = [
      TRANSACTION_ISOLATION_READ_UNCOMMITTED = 'READ-UNCOMMITTED'.freeze,
      TRANSACTION_ISOLATION_READ_COMMITED = 'READ-COMMITED'.freeze,
      TRANSACTION_ISOLATION_READ_COMMITTED = 'READ-COMMITTED'.freeze,
      TRANSACTION_ISOLATION_REPEATABLE_READ = 'REPEATABLE-READ'.freeze,
      TRANSACTION_ISOLATION_SERIALIZABLE = 'SERIALIZABLE'.freeze,
      TRANSACTION_ISOLATION_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    GROUP_REPLICATION_CONSISTENCY_ENUM = [
      GROUP_REPLICATION_CONSISTENCY_EVENTUAL = 'EVENTUAL'.freeze,
      GROUP_REPLICATION_CONSISTENCY_BEFORE_ON_PRIMARY_FAILOVER = 'BEFORE_ON_PRIMARY_FAILOVER'.freeze,
      GROUP_REPLICATION_CONSISTENCY_BEFORE = 'BEFORE'.freeze,
      GROUP_REPLICATION_CONSISTENCY_AFTER = 'AFTER'.freeze,
      GROUP_REPLICATION_CONSISTENCY_BEFORE_AND_AFTER = 'BEFORE_AND_AFTER'.freeze,
      GROUP_REPLICATION_CONSISTENCY_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    BINLOG_ROW_METADATA_ENUM = [
      BINLOG_ROW_METADATA_FULL = 'FULL'.freeze,
      BINLOG_ROW_METADATA_MINIMAL = 'MINIMAL'.freeze,
      BINLOG_ROW_METADATA_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # (\"completion_type\")
    # @return [String]
    attr_reader :completion_type

    # If enabled, the server stores all temporary tables on disk rather than in memory.
    #
    # bigTables corresponds to the MySQL server variable [big_tables](https://dev.mysql.com/doc/refman/en/server-system-variables.html#sysvar_big_tables).
    #
    # @return [BOOLEAN]
    attr_accessor :big_tables

    # Set the chunking size for updates to the global memory usage counter Global_connection_memory.
    #
    # connectionMemoryChunkSize corresponds to the MySQL system variable [connection_memory_chunk_size](https://dev.mysql.com/doc/refman/en/server-system-variables.html#sysvar_connection_memory_chunk_size).
    #
    # @return [Integer]
    attr_accessor :connection_memory_chunk_size

    # Set the maximum amount of memory that can be used by a single user connection.
    #
    # connectionMemoryLimit corresponds to the MySQL system variable [connection_memory_limit](https://dev.mysql.com/doc/refman/en/server-system-variables.html#sysvar_connection_memory_limit).
    #
    # @return [Integer]
    attr_accessor :connection_memory_limit

    # (\"default_authentication_plugin\")
    # @return [String]
    attr_reader :default_authentication_plugin

    # Set the total amount of memory that can be used by all user connections.
    #
    # globalConnectionMemoryLimit corresponds to the MySQL system variable [global_connection_memory_limit](https://dev.mysql.com/doc/refman/en/server-system-variables.html#sysvar_global_connection_memory_limit).
    #
    # @return [Integer]
    attr_accessor :global_connection_memory_limit

    # Determines whether the MySQL server calculates Global_connection_memory.
    #
    # globalConnectionMemoryTracking corresponds to the MySQL system variable [global_connection_memory_tracking](https://dev.mysql.com/doc/refman/en/server-system-variables.html#sysvar_global_connection_memory_tracking).
    #
    # @return [BOOLEAN]
    attr_accessor :global_connection_memory_tracking

    # (\"transaction_isolation\")
    # @return [String]
    attr_reader :transaction_isolation

    # (\"innodb_ft_server_stopword_table\")
    # @return [String]
    attr_accessor :innodb_ft_server_stopword_table

    # (\"mandatory_roles\")
    # @return [String]
    attr_accessor :mandatory_roles

    # (\"autocommit\")
    # @return [BOOLEAN]
    attr_accessor :autocommit

    # (\"foreign_key_checks\")
    # @return [BOOLEAN]
    attr_accessor :foreign_key_checks

    # - EVENTUAL:
    #     Both RO and RW transactions do not wait for preceding transactions to be applied before executing.
    #     A RW transaction does not wait for other members to apply a transaction. This means that a transaction
    #     could be externalized on one member before the others. This also means that in the event of a primary failover,
    #     the new primary can accept new RO and RW transactions before the previous primary transactions are all applied.
    #     RO transactions could result in outdated values, RW transactions could result in a rollback due to conflicts.
    # - BEFORE_ON_PRIMARY_FAILOVER:
    #     New RO or RW transactions with a newly elected primary that is applying backlog from the old
    #     primary are held (not applied) until any backlog has been applied. This ensures that when a primary failover happens,
    #     intentionally or not, clients always see the latest value on the primary. This guarantees consistency, but means that
    #     clients must be able to handle the delay in the event that a backlog is being applied. Usually this delay should be minimal,
    #     but does depend on the size of the backlog.
    # - BEFORE:
    #     A RW transaction waits for all preceding transactions to complete before being applied. A RO transaction waits for all preceding
    #     transactions to complete before being executed. This ensures that this transaction reads the latest value by only affecting the
    #     latency of the transaction. This reduces the overhead of synchronization on every RW transaction, by ensuring synchronization is
    #     used only on RO transactions. This consistency level also includes the consistency guarantees provided by BEFORE_ON_PRIMARY_FAILOVER.
    # - AFTER:
    #     A RW transaction waits until its changes have been applied to all of the other members. This value has no effect on RO transactions.
    #     This mode ensures that when a transaction is committed on the local member, any subsequent transaction reads the written value or
    #     a more recent value on any group member. Use this mode with a group that is used for predominantly RO operations to ensure that
    #     applied RW transactions are applied everywhere once they commit. This could be used by your application to ensure that subsequent
    #     reads fetch the latest data which includes the latest writes. This reduces the overhead of synchronization on every RO transaction,
    #     by ensuring synchronization is used only on RW transactions. This consistency level also includes the consistency guarantees
    #     provided by BEFORE_ON_PRIMARY_FAILOVER.
    # - BEFORE_AND_AFTER:
    #     A RW transaction waits for 1) all preceding transactions to complete before being applied and 2) until its changes have been
    #     applied on other members. A RO transaction waits for all preceding transactions to complete before execution takes place.
    #     This consistency level also includes the consistency guarantees provided by BEFORE_ON_PRIMARY_FAILOVER.
    #
    # @return [String]
    attr_reader :group_replication_consistency

    # (\"innodb_ft_enable_stopword\")
    # @return [BOOLEAN]
    attr_accessor :innodb_ft_enable_stopword

    # Enables dedicated log writer threads for writing redo log records from the log buffer to the system buffers and flushing the system buffers to the redo log files.
    #
    # This is the MySQL variable \"innodb_log_writer_threads\". For more information, please see the [MySQL documentation](https://dev.mysql.com/doc/refman/8.0/en/innodb-parameters.html#sysvar_innodb_log_writer_threads)
    #
    # @return [BOOLEAN]
    attr_accessor :innodb_log_writer_threads

    # (\"local_infile\")
    # @return [BOOLEAN]
    attr_accessor :local_infile

    # (\"mysql_firewall_mode\")
    # @return [BOOLEAN]
    attr_accessor :mysql_firewall_mode

    # (\"mysqlx_enable_hello_notice\") DEPRECATED -- variable should not be settable and will be ignored
    # @return [BOOLEAN]
    attr_accessor :mysqlx_enable_hello_notice

    # (\"sql_require_primary_key\")
    # @return [BOOLEAN]
    attr_accessor :sql_require_primary_key

    # (\"sql_warnings\")
    # @return [BOOLEAN]
    attr_accessor :sql_warnings

    # Sets the binary log expiration period in seconds.
    # binlogExpireLogsSeconds corresponds to the MySQL binary logging system variable [binlog_expire_logs_seconds](https://dev.mysql.com/doc/refman/8.0/en/replication-options-binary-log.html#sysvar_binlog_expire_logs_seconds).
    #
    # @return [Integer]
    attr_accessor :binlog_expire_logs_seconds

    # Configures the amount of table metadata added to the binary log when using row-based logging.
    # binlogRowMetadata corresponds to the MySQL binary logging system variable [binlog_row_metadata](https://dev.mysql.com/doc/refman/8.0/en/replication-options-binary-log.html#sysvar_binlog_row_metadata).
    #
    # @return [String]
    attr_reader :binlog_row_metadata

    # When set to PARTIAL_JSON, this enables use of a space-efficient binary log format for updates that modify only a small portion of a JSON document.
    # binlogRowValueOptions corresponds to the MySQL binary logging system variable [binlog_row_value_options](https://dev.mysql.com/doc/refman/8.0/en/replication-options-binary-log.html#sysvar_binlog_row_value_options).
    #
    # @return [String]
    attr_accessor :binlog_row_value_options

    # Enables compression for transactions that are written to binary log files on this server.
    # binlogTransactionCompression corresponds to the MySQL binary logging system variable [binlog_transaction_compression](https://dev.mysql.com/doc/refman/8.0/en/replication-options-binary-log.html#sysvar_binlog_transaction_compression).
    #
    # @return [BOOLEAN]
    attr_accessor :binlog_transaction_compression

    # The size (in bytes) of the buffer pool, that is, the memory area where InnoDB caches table and index data.
    #
    # innodbBufferPoolSize corresponds to the MySQL server system variable
    # [innodb_buffer_pool_size](https://dev.mysql.com/doc/refman/en/innodb-parameters.html#sysvar_innodb_buffer_pool_size).
    #
    # The default and maximum values depend on the amount of RAM provisioned by the shape.
    # See [Default User Variables](https://docs.cloud.oracle.com/mysql-database/doc/configuring-db-system.html#GUID-B5504C19-F6F4-4DAB-8506-189A4E8F4A6A).
    #
    # @return [Integer]
    attr_accessor :innodb_buffer_pool_size

    # (\"innodb_ft_result_cache_limit\")
    # @return [Integer]
    attr_accessor :innodb_ft_result_cache_limit

    # Sets the size of the transaction cache.
    #
    # maxBinlogCacheSize corresponds to the MySQL server system variable [max_binlog_cache_size](https://dev.mysql.com/doc/refman/8.0/en/replication-options-binary-log.html#sysvar_max_binlog_cache_size).
    #
    # @return [Integer]
    attr_accessor :max_binlog_cache_size

    # (\"max_connect_errors\")
    # @return [Integer]
    attr_accessor :max_connect_errors

    # This variable sets the maximum size to which user-created MEMORY tables are permitted to grow.
    #
    # maxHeapTableSize corresponds to the MySQL system variable
    # [max_heap_table_size](https://dev.mysql.com/doc/refman/8.0/en/server-system-variables.html#sysvar_max_heap_table_size)
    #
    # @return [Integer]
    attr_accessor :max_heap_table_size

    # (\"max_connections\")
    # @return [Integer]
    attr_accessor :max_connections

    # (\"max_prepared_stmt_count\")
    # @return [Integer]
    attr_accessor :max_prepared_stmt_count

    # The number of seconds that the mysqld server waits for a connect packet before responding with Bad handshake.
    #
    # connectTimeout corresponds to the MySQL system variable
    # [connect_timeout](https://dev.mysql.com/doc/refman/8.0/en/server-system-variables.html#sysvar_connect_timeout)
    #
    # Increasing the connect_timeout value might help if clients frequently encounter errors of the form
    # \"Lost connection to MySQL server at 'XXX', system error: errno\".
    #
    # @return [Integer]
    attr_accessor :connect_timeout

    # (\"cte_max_recursion_depth\")
    # @return [Integer]
    attr_accessor :cte_max_recursion_depth

    # (\"generated_random_password_length\") DEPRECATED -- variable should not be settable and will be ignored
    # @return [Integer]
    attr_accessor :generated_random_password_length

    # (\"information_schema_stats_expiry\")
    # @return [Integer]
    attr_accessor :information_schema_stats_expiry

    # Specifies the percentage of the most recently used pages for each buffer pool to read out and dump.
    #
    # innodbBufferPoolDumpPct corresponds to the MySQL InnoDB system variable
    # [innodb_buffer_pool_dump_pct](https://dev.mysql.com/doc/refman/8.0/en/innodb-parameters.html#sysvar_innodb_buffer_pool_dump_pct).
    #
    # The range is 1 to 100. The default value is 25.
    #
    # For example, if there are 4 buffer pools with 100 pages each, and innodb_buffer_pool_dump_pct is set to 25,
    # the 25 most recently used pages from each buffer pool are dumped.
    #
    # @return [Integer]
    attr_accessor :innodb_buffer_pool_dump_pct

    # (\"innodb_buffer_pool_instances\")
    # @return [Integer]
    attr_accessor :innodb_buffer_pool_instances

    # innodbDdlBufferSize corresponds to the MySQL system variable [innodb_ddl_buffer_size] (https://dev.mysql.com/doc/refman/8.0/en/innodb-parameters.html#sysvar_innodb_ddl_buffer_size)
    #
    # @return [Integer]
    attr_accessor :innodb_ddl_buffer_size

    # innodbDdlThreads corresponds to the MySQL system variable [innodb_ddl_threads] (https://dev.mysql.com/doc/refman/8.0/en/innodb-parameters.html#sysvar_innodb_ddl_threads)
    #
    # @return [Integer]
    attr_accessor :innodb_ddl_threads

    # (\"innodb_ft_max_token_size\")
    # @return [Integer]
    attr_accessor :innodb_ft_max_token_size

    # (\"innodb_ft_min_token_size\")
    # @return [Integer]
    attr_accessor :innodb_ft_min_token_size

    # (\"innodb_ft_num_word_optimize\")
    # @return [Integer]
    attr_accessor :innodb_ft_num_word_optimize

    # (\"innodb_lock_wait_timeout\")
    # @return [Integer]
    attr_accessor :innodb_lock_wait_timeout

    # The desired maximum purge lag in terms of transactions.
    #
    # InnoDB maintains a list of transactions that have index records delete-marked by UPDATE or DELETE operations. The length of the list is the purge lag.
    #
    # If this value is exceeded, a delay is imposed on INSERT, UPDATE, and DELETE operations to allow time for purge to catch up.
    #
    # The default value is 0, which means there is no maximum purge lag and no delay.
    #
    # innodbMaxPurgeLag corresponds to the MySQL server system variable
    # [innodb_max_purge_lag](https://dev.mysql.com/doc/refman/en/innodb-parameters.html#sysvar_innodb_max_purge_lag).
    #
    # @return [Integer]
    attr_accessor :innodb_max_purge_lag

    # The maximum delay in microseconds for the delay imposed when the innodb_max_purge_lag threshold is exceeded.
    #
    # The specified innodb_max_purge_lag_delay value is an upper limit on the delay period.
    #
    # innodbMaxPurgeLagDelay corresponds to the MySQL server system variable
    # [innodb_max_purge_lag_delay](https://dev.mysql.com/doc/refman/en/innodb-parameters.html#sysvar_innodb_max_purge_lag_delay).
    #
    # @return [Integer]
    attr_accessor :innodb_max_purge_lag_delay

    # The number of seconds the server waits for activity on an interactive connection before closing it.
    #
    # interactiveTimeout corresponds to the MySQL system variable.
    # [interactive_timeout](https://dev.mysql.com/doc/refman/8.0/en/server-system-variables.html#sysvar_interactive_timeout)
    #
    # @return [Integer]
    attr_accessor :interactive_timeout

    # The number of index pages to sample when estimating cardinality and other statistics for an indexed column,
    # such as those calculated by ANALYZE TABLE.
    #
    # innodbStatsPersistentSamplePages corresponds to the MySQL InnoDB system variable
    # [innodb_stats_persistent_sample_pages](https://dev.mysql.com/doc/refman/8.0/en/innodb-parameters.html#sysvar_innodb_stats_persistent_sample_pages)
    #
    # innodb_stats_persistent_sample_pages only applies when innodb_stats_persistent is enabled for a table;
    # when innodb_stats_persistent is disabled, innodb_stats_transient_sample_pages applies instead.
    #
    # @return [Integer]
    attr_accessor :innodb_stats_persistent_sample_pages

    # The number of index pages to sample when estimating cardinality and other statistics for an indexed column,
    # such as those calculated by [ANALYZE TABLE](https://dev.mysql.com/doc/refman/8.0/en/analyze-table.html).
    #
    # innodbStatsTransientSamplePages corresponds to the MySQL InnoDB system variable
    # [innodb_stats_transient_sample_pages](https://dev.mysql.com/doc/refman/8.0/en/innodb-parameters.html#sysvar_innodb_stats_transient_sample_pages)
    #
    # innodb_stats_transient_sample_pages only applies when innodb_stats_persistent is disabled for a table;
    # when innodb_stats_persistent is enabled, innodb_stats_persistent_sample_pages applies instead.
    #
    # innodb_stats_persistent is ON by default and cannot be changed. It is possible to override it using the
    # STATS_PERSISTENT clause of the [CREATE TABLE](https://dev.mysql.com/doc/refman/8.0/en/create-table.html) and
    # [ALTER TABLE](https://dev.mysql.com/doc/refman/8.0/en/alter-table.html) statements.
    #
    # @return [Integer]
    attr_accessor :innodb_stats_transient_sample_pages

    # The maximum size of one packet or any generated/intermediate string.
    #
    # This is the mysql variable \"max_allowed_packet\".
    #
    # @return [Integer]
    attr_accessor :max_allowed_packet

    # (\"max_execution_time\")
    # @return [Integer]
    attr_accessor :max_execution_time

    # The number of seconds X Plugin waits for the first packet to be received from newly connected clients.
    #
    # mysqlxConnectTimeout corresponds to the MySQL X Plugin system variable
    # [mysqlx_connect_timeout](https://dev.mysql.com/doc/refman/8.0/en/x-plugin-options-system-variables.html#sysvar_mysqlx_connect_timeout)
    #
    # @return [Integer]
    attr_accessor :mysqlx_connect_timeout

    # (\"mysqlx_document_id_unique_prefix\") DEPRECATED -- variable should not be settable and will be ignored
    # @return [Integer]
    attr_accessor :mysqlx_document_id_unique_prefix

    # (\"mysqlx_idle_worker_thread_timeout\") DEPRECATED -- variable should not be settable and will be ignored
    # @return [Integer]
    attr_accessor :mysqlx_idle_worker_thread_timeout

    # The number of seconds to wait for interactive clients to timeout.
    #
    # mysqlxInteractiveTimeout corresponds to the MySQL X Plugin system variable.
    # [mysqlx_interactive_timeout](https://dev.mysql.com/doc/refman/8.0/en/x-plugin-options-system-variables.html#sysvar_mysqlx_interactive_timeout)
    #
    # @return [Integer]
    attr_accessor :mysqlx_interactive_timeout

    # The maximum size of network packets that can be received by X Plugin.
    #
    # This is the mysql variable \"mysqlx_max_allowed_packet\".
    #
    # @return [Integer]
    attr_accessor :mysqlx_max_allowed_packet

    # (\"mysqlx_min_worker_threads\") DEPRECATED -- variable should not be settable and will be ignored
    # @return [Integer]
    attr_accessor :mysqlx_min_worker_threads

    # The number of seconds that X Plugin waits for blocking read operations to complete. After this time, if the
    # read operation is not successful, X Plugin closes the connection and returns a warning notice with the error
    # code ER_IO_READ_ERROR to the client application.
    #
    # mysqlxReadTimeout corresponds to the MySQL X Plugin system variable
    # [mysqlx_read_timeout](https://dev.mysql.com/doc/refman/8.0/en/x-plugin-options-system-variables.html#sysvar_mysqlx_read_timeout)
    #
    # @return [Integer]
    attr_accessor :mysqlx_read_timeout

    # The number of seconds that X Plugin waits for activity on a connection.
    #
    # mysqlxWaitTimeout corresponds to the MySQL X Plugin system variable.
    # [mysqlx_wait_timeout](https://dev.mysql.com/doc/refman/8.0/en/x-plugin-options-system-variables.html#sysvar_mysqlx_wait_timeout)
    #
    # @return [Integer]
    attr_accessor :mysqlx_wait_timeout

    # The number of seconds that X Plugin waits for blocking write operations to complete. After this time, if the
    # write operation is not successful, X Plugin closes the connection.
    #
    # mysqlxReadmysqlxWriteTimeoutTimeout corresponds to the MySQL X Plugin system variable
    # [mysqlx_write_timeout](https://dev.mysql.com/doc/refman/8.0/en/x-plugin-options-system-variables.html#sysvar_mysqlx_write_timeout)
    #
    # @return [Integer]
    attr_accessor :mysqlx_write_timeout

    # The number of seconds to wait for more data from a connection before aborting the read.
    #
    # netReadTimeout corresponds to the MySQL system variable
    # [net_read_timeout](https://dev.mysql.com/doc/refman/8.0/en/server-system-variables.html#sysvar_net_read_timeout)
    #
    # @return [Integer]
    attr_accessor :net_read_timeout

    # The number of seconds to wait for a block to be written to a connection before aborting the write.
    #
    # netWriteTimeout corresponds to the MySQL system variable
    # [net_write_timeout](https://dev.mysql.com/doc/refman/8.0/en/server-system-variables.html#sysvar_net_write_timeout)
    #
    # @return [Integer]
    attr_accessor :net_write_timeout

    # (\"parser_max_mem_size\")
    # @return [Integer]
    attr_accessor :parser_max_mem_size

    # (\"query_alloc_block_size\") DEPRECATED -- variable should not be settable and will be ignored
    # @return [Integer]
    attr_accessor :query_alloc_block_size

    # (\"query_prealloc_size\") DEPRECATED -- variable should not be settable and will be ignored
    # @return [Integer]
    attr_accessor :query_prealloc_size

    # regexpTimeLimit corresponds to the MySQL system variable [regexp_time_limit] (https://dev.mysql.com/doc/refman/8.0/en/server-system-variables.html#sysvar_regexp_time_limit)
    #
    # @return [Integer]
    attr_accessor :regexp_time_limit

    # (\"sql_mode\")
    # @return [String]
    attr_accessor :sql_mode

    # The maximum size of internal in-memory temporary tables. This variable does not apply to user-created MEMORY tables.
    #
    # tmp_table_size corresponds to the MySQL system variable
    # [tmp_table_size](https://dev.mysql.com/doc/refman/8.0/en/server-system-variables.html#sysvar_tmp_table_size)
    #
    # @return [Integer]
    attr_accessor :tmp_table_size

    # Set the default compression level for the deflate algorithm. (\"mysqlx_deflate_default_compression_level\")
    # @return [Integer]
    attr_accessor :mysqlx_deflate_default_compression_level

    # Limit the upper bound of accepted compression levels for the deflate algorithm. (\"mysqlx_deflate_max_client_compression_level\")
    # @return [Integer]
    attr_accessor :mysqlx_deflate_max_client_compression_level

    # Limit the upper bound of accepted compression levels for the lz4 algorithm. (\"mysqlx_lz4_max_client_compression_level\")
    # @return [Integer]
    attr_accessor :mysqlx_lz4_max_client_compression_level

    # Set the default compression level for the lz4 algorithm. (\"mysqlx_lz4_default_compression_level\")
    # @return [Integer]
    attr_accessor :mysqlx_lz4_default_compression_level

    # Limit the upper bound of accepted compression levels for the zstd algorithm. (\"mysqlx_zstd_max_client_compression_level\")
    # @return [Integer]
    attr_accessor :mysqlx_zstd_max_client_compression_level

    # Set the default compression level for the zstd algorithm. (\"mysqlx_zstd_default_compression_level\")
    # @return [Integer]
    attr_accessor :mysqlx_zstd_default_compression_level

    # DEPRECATED -- typo of mysqlx_zstd_default_compression_level. variable will be ignored.
    # @return [Integer]
    attr_accessor :mysql_zstd_default_compression_level

    # Each session that must perform a sort allocates a buffer of this size.
    #
    # sortBufferSize corresponds to the MySQL system variable [sort_buffer_size](https://dev.mysql.com/doc/refman/en/server-system-variables.html#sysvar_sort_buffer_size)
    #
    # @return [Integer]
    attr_accessor :sort_buffer_size

    # The number of seconds the server waits for activity on a noninteractive connection before closing it.
    #
    # waitTimeout corresponds to the MySQL system variable.
    # [wait_timeout](https://dev.mysql.com/doc/refman/8.0/en/server-system-variables.html#sysvar_wait_timeout)
    #
    # @return [Integer]
    attr_accessor :wait_timeout

    # Controls whether the thread pool uses dedicated listener threads. If enabled, a listener thread in each thread group is dedicated to the task of listening
    # for network events from clients, ensuring that the maximum number of query worker threads is no more than the value specified by threadPoolMaxTransactionsLimit.
    # threadPoolDedicatedListeners corresponds to the MySQL Database Service-specific system variable thread_pool_dedicated_listeners.
    #
    # @return [BOOLEAN]
    attr_accessor :thread_pool_dedicated_listeners

    # Limits the maximum number of open transactions to the defined value. The default value is 0, which enforces no limit.
    # threadPoolMaxTransactionsLimit corresponds to the MySQL Database Service-specific system variable thread_pool_max_transactions_limit.
    #
    # @return [Integer]
    attr_accessor :thread_pool_max_transactions_limit

    # Initializes the time zone for each client that connects.
    #
    # This corresponds to the MySQL System Variable \"time_zone\".
    #
    # The values can be given in one of the following formats, none of which are case-sensitive:
    #
    # - As a string indicating an offset from UTC of the form [H]H:MM, prefixed with a + or -, such as '+10:00', '-6:00', or '+05:30'. The permitted range is '-13:59' to '+14:00', inclusive.
    # - As a named time zone, as defined by the \"IANA Time Zone database\", such as 'Europe/Helsinki', 'US/Eastern', 'MET', or 'UTC'.
    #
    # @return [String]
    attr_accessor :time_zone

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'completion_type': :'completionType',
        'big_tables': :'bigTables',
        'connection_memory_chunk_size': :'connectionMemoryChunkSize',
        'connection_memory_limit': :'connectionMemoryLimit',
        'default_authentication_plugin': :'defaultAuthenticationPlugin',
        'global_connection_memory_limit': :'globalConnectionMemoryLimit',
        'global_connection_memory_tracking': :'globalConnectionMemoryTracking',
        'transaction_isolation': :'transactionIsolation',
        'innodb_ft_server_stopword_table': :'innodbFtServerStopwordTable',
        'mandatory_roles': :'mandatoryRoles',
        'autocommit': :'autocommit',
        'foreign_key_checks': :'foreignKeyChecks',
        'group_replication_consistency': :'groupReplicationConsistency',
        'innodb_ft_enable_stopword': :'innodbFtEnableStopword',
        'innodb_log_writer_threads': :'innodbLogWriterThreads',
        'local_infile': :'localInfile',
        'mysql_firewall_mode': :'mysqlFirewallMode',
        'mysqlx_enable_hello_notice': :'mysqlxEnableHelloNotice',
        'sql_require_primary_key': :'sqlRequirePrimaryKey',
        'sql_warnings': :'sqlWarnings',
        'binlog_expire_logs_seconds': :'binlogExpireLogsSeconds',
        'binlog_row_metadata': :'binlogRowMetadata',
        'binlog_row_value_options': :'binlogRowValueOptions',
        'binlog_transaction_compression': :'binlogTransactionCompression',
        'innodb_buffer_pool_size': :'innodbBufferPoolSize',
        'innodb_ft_result_cache_limit': :'innodbFtResultCacheLimit',
        'max_binlog_cache_size': :'maxBinlogCacheSize',
        'max_connect_errors': :'maxConnectErrors',
        'max_heap_table_size': :'maxHeapTableSize',
        'max_connections': :'maxConnections',
        'max_prepared_stmt_count': :'maxPreparedStmtCount',
        'connect_timeout': :'connectTimeout',
        'cte_max_recursion_depth': :'cteMaxRecursionDepth',
        'generated_random_password_length': :'generatedRandomPasswordLength',
        'information_schema_stats_expiry': :'informationSchemaStatsExpiry',
        'innodb_buffer_pool_dump_pct': :'innodbBufferPoolDumpPct',
        'innodb_buffer_pool_instances': :'innodbBufferPoolInstances',
        'innodb_ddl_buffer_size': :'innodbDdlBufferSize',
        'innodb_ddl_threads': :'innodbDdlThreads',
        'innodb_ft_max_token_size': :'innodbFtMaxTokenSize',
        'innodb_ft_min_token_size': :'innodbFtMinTokenSize',
        'innodb_ft_num_word_optimize': :'innodbFtNumWordOptimize',
        'innodb_lock_wait_timeout': :'innodbLockWaitTimeout',
        'innodb_max_purge_lag': :'innodbMaxPurgeLag',
        'innodb_max_purge_lag_delay': :'innodbMaxPurgeLagDelay',
        'interactive_timeout': :'interactiveTimeout',
        'innodb_stats_persistent_sample_pages': :'innodbStatsPersistentSamplePages',
        'innodb_stats_transient_sample_pages': :'innodbStatsTransientSamplePages',
        'max_allowed_packet': :'maxAllowedPacket',
        'max_execution_time': :'maxExecutionTime',
        'mysqlx_connect_timeout': :'mysqlxConnectTimeout',
        'mysqlx_document_id_unique_prefix': :'mysqlxDocumentIdUniquePrefix',
        'mysqlx_idle_worker_thread_timeout': :'mysqlxIdleWorkerThreadTimeout',
        'mysqlx_interactive_timeout': :'mysqlxInteractiveTimeout',
        'mysqlx_max_allowed_packet': :'mysqlxMaxAllowedPacket',
        'mysqlx_min_worker_threads': :'mysqlxMinWorkerThreads',
        'mysqlx_read_timeout': :'mysqlxReadTimeout',
        'mysqlx_wait_timeout': :'mysqlxWaitTimeout',
        'mysqlx_write_timeout': :'mysqlxWriteTimeout',
        'net_read_timeout': :'netReadTimeout',
        'net_write_timeout': :'netWriteTimeout',
        'parser_max_mem_size': :'parserMaxMemSize',
        'query_alloc_block_size': :'queryAllocBlockSize',
        'query_prealloc_size': :'queryPreallocSize',
        'regexp_time_limit': :'regexpTimeLimit',
        'sql_mode': :'sqlMode',
        'tmp_table_size': :'tmpTableSize',
        'mysqlx_deflate_default_compression_level': :'mysqlxDeflateDefaultCompressionLevel',
        'mysqlx_deflate_max_client_compression_level': :'mysqlxDeflateMaxClientCompressionLevel',
        'mysqlx_lz4_max_client_compression_level': :'mysqlxLz4MaxClientCompressionLevel',
        'mysqlx_lz4_default_compression_level': :'mysqlxLz4DefaultCompressionLevel',
        'mysqlx_zstd_max_client_compression_level': :'mysqlxZstdMaxClientCompressionLevel',
        'mysqlx_zstd_default_compression_level': :'mysqlxZstdDefaultCompressionLevel',
        'mysql_zstd_default_compression_level': :'mysqlZstdDefaultCompressionLevel',
        'sort_buffer_size': :'sortBufferSize',
        'wait_timeout': :'waitTimeout',
        'thread_pool_dedicated_listeners': :'threadPoolDedicatedListeners',
        'thread_pool_max_transactions_limit': :'threadPoolMaxTransactionsLimit',
        'time_zone': :'timeZone'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'completion_type': :'String',
        'big_tables': :'BOOLEAN',
        'connection_memory_chunk_size': :'Integer',
        'connection_memory_limit': :'Integer',
        'default_authentication_plugin': :'String',
        'global_connection_memory_limit': :'Integer',
        'global_connection_memory_tracking': :'BOOLEAN',
        'transaction_isolation': :'String',
        'innodb_ft_server_stopword_table': :'String',
        'mandatory_roles': :'String',
        'autocommit': :'BOOLEAN',
        'foreign_key_checks': :'BOOLEAN',
        'group_replication_consistency': :'String',
        'innodb_ft_enable_stopword': :'BOOLEAN',
        'innodb_log_writer_threads': :'BOOLEAN',
        'local_infile': :'BOOLEAN',
        'mysql_firewall_mode': :'BOOLEAN',
        'mysqlx_enable_hello_notice': :'BOOLEAN',
        'sql_require_primary_key': :'BOOLEAN',
        'sql_warnings': :'BOOLEAN',
        'binlog_expire_logs_seconds': :'Integer',
        'binlog_row_metadata': :'String',
        'binlog_row_value_options': :'String',
        'binlog_transaction_compression': :'BOOLEAN',
        'innodb_buffer_pool_size': :'Integer',
        'innodb_ft_result_cache_limit': :'Integer',
        'max_binlog_cache_size': :'Integer',
        'max_connect_errors': :'Integer',
        'max_heap_table_size': :'Integer',
        'max_connections': :'Integer',
        'max_prepared_stmt_count': :'Integer',
        'connect_timeout': :'Integer',
        'cte_max_recursion_depth': :'Integer',
        'generated_random_password_length': :'Integer',
        'information_schema_stats_expiry': :'Integer',
        'innodb_buffer_pool_dump_pct': :'Integer',
        'innodb_buffer_pool_instances': :'Integer',
        'innodb_ddl_buffer_size': :'Integer',
        'innodb_ddl_threads': :'Integer',
        'innodb_ft_max_token_size': :'Integer',
        'innodb_ft_min_token_size': :'Integer',
        'innodb_ft_num_word_optimize': :'Integer',
        'innodb_lock_wait_timeout': :'Integer',
        'innodb_max_purge_lag': :'Integer',
        'innodb_max_purge_lag_delay': :'Integer',
        'interactive_timeout': :'Integer',
        'innodb_stats_persistent_sample_pages': :'Integer',
        'innodb_stats_transient_sample_pages': :'Integer',
        'max_allowed_packet': :'Integer',
        'max_execution_time': :'Integer',
        'mysqlx_connect_timeout': :'Integer',
        'mysqlx_document_id_unique_prefix': :'Integer',
        'mysqlx_idle_worker_thread_timeout': :'Integer',
        'mysqlx_interactive_timeout': :'Integer',
        'mysqlx_max_allowed_packet': :'Integer',
        'mysqlx_min_worker_threads': :'Integer',
        'mysqlx_read_timeout': :'Integer',
        'mysqlx_wait_timeout': :'Integer',
        'mysqlx_write_timeout': :'Integer',
        'net_read_timeout': :'Integer',
        'net_write_timeout': :'Integer',
        'parser_max_mem_size': :'Integer',
        'query_alloc_block_size': :'Integer',
        'query_prealloc_size': :'Integer',
        'regexp_time_limit': :'Integer',
        'sql_mode': :'String',
        'tmp_table_size': :'Integer',
        'mysqlx_deflate_default_compression_level': :'Integer',
        'mysqlx_deflate_max_client_compression_level': :'Integer',
        'mysqlx_lz4_max_client_compression_level': :'Integer',
        'mysqlx_lz4_default_compression_level': :'Integer',
        'mysqlx_zstd_max_client_compression_level': :'Integer',
        'mysqlx_zstd_default_compression_level': :'Integer',
        'mysql_zstd_default_compression_level': :'Integer',
        'sort_buffer_size': :'Integer',
        'wait_timeout': :'Integer',
        'thread_pool_dedicated_listeners': :'BOOLEAN',
        'thread_pool_max_transactions_limit': :'Integer',
        'time_zone': :'String'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :completion_type The value to assign to the {#completion_type} property
    # @option attributes [BOOLEAN] :big_tables The value to assign to the {#big_tables} property
    # @option attributes [Integer] :connection_memory_chunk_size The value to assign to the {#connection_memory_chunk_size} property
    # @option attributes [Integer] :connection_memory_limit The value to assign to the {#connection_memory_limit} property
    # @option attributes [String] :default_authentication_plugin The value to assign to the {#default_authentication_plugin} property
    # @option attributes [Integer] :global_connection_memory_limit The value to assign to the {#global_connection_memory_limit} property
    # @option attributes [BOOLEAN] :global_connection_memory_tracking The value to assign to the {#global_connection_memory_tracking} property
    # @option attributes [String] :transaction_isolation The value to assign to the {#transaction_isolation} property
    # @option attributes [String] :innodb_ft_server_stopword_table The value to assign to the {#innodb_ft_server_stopword_table} property
    # @option attributes [String] :mandatory_roles The value to assign to the {#mandatory_roles} property
    # @option attributes [BOOLEAN] :autocommit The value to assign to the {#autocommit} property
    # @option attributes [BOOLEAN] :foreign_key_checks The value to assign to the {#foreign_key_checks} property
    # @option attributes [String] :group_replication_consistency The value to assign to the {#group_replication_consistency} property
    # @option attributes [BOOLEAN] :innodb_ft_enable_stopword The value to assign to the {#innodb_ft_enable_stopword} property
    # @option attributes [BOOLEAN] :innodb_log_writer_threads The value to assign to the {#innodb_log_writer_threads} property
    # @option attributes [BOOLEAN] :local_infile The value to assign to the {#local_infile} property
    # @option attributes [BOOLEAN] :mysql_firewall_mode The value to assign to the {#mysql_firewall_mode} property
    # @option attributes [BOOLEAN] :mysqlx_enable_hello_notice The value to assign to the {#mysqlx_enable_hello_notice} property
    # @option attributes [BOOLEAN] :sql_require_primary_key The value to assign to the {#sql_require_primary_key} property
    # @option attributes [BOOLEAN] :sql_warnings The value to assign to the {#sql_warnings} property
    # @option attributes [Integer] :binlog_expire_logs_seconds The value to assign to the {#binlog_expire_logs_seconds} property
    # @option attributes [String] :binlog_row_metadata The value to assign to the {#binlog_row_metadata} property
    # @option attributes [String] :binlog_row_value_options The value to assign to the {#binlog_row_value_options} property
    # @option attributes [BOOLEAN] :binlog_transaction_compression The value to assign to the {#binlog_transaction_compression} property
    # @option attributes [Integer] :innodb_buffer_pool_size The value to assign to the {#innodb_buffer_pool_size} property
    # @option attributes [Integer] :innodb_ft_result_cache_limit The value to assign to the {#innodb_ft_result_cache_limit} property
    # @option attributes [Integer] :max_binlog_cache_size The value to assign to the {#max_binlog_cache_size} property
    # @option attributes [Integer] :max_connect_errors The value to assign to the {#max_connect_errors} property
    # @option attributes [Integer] :max_heap_table_size The value to assign to the {#max_heap_table_size} property
    # @option attributes [Integer] :max_connections The value to assign to the {#max_connections} property
    # @option attributes [Integer] :max_prepared_stmt_count The value to assign to the {#max_prepared_stmt_count} property
    # @option attributes [Integer] :connect_timeout The value to assign to the {#connect_timeout} property
    # @option attributes [Integer] :cte_max_recursion_depth The value to assign to the {#cte_max_recursion_depth} property
    # @option attributes [Integer] :generated_random_password_length The value to assign to the {#generated_random_password_length} property
    # @option attributes [Integer] :information_schema_stats_expiry The value to assign to the {#information_schema_stats_expiry} property
    # @option attributes [Integer] :innodb_buffer_pool_dump_pct The value to assign to the {#innodb_buffer_pool_dump_pct} property
    # @option attributes [Integer] :innodb_buffer_pool_instances The value to assign to the {#innodb_buffer_pool_instances} property
    # @option attributes [Integer] :innodb_ddl_buffer_size The value to assign to the {#innodb_ddl_buffer_size} property
    # @option attributes [Integer] :innodb_ddl_threads The value to assign to the {#innodb_ddl_threads} property
    # @option attributes [Integer] :innodb_ft_max_token_size The value to assign to the {#innodb_ft_max_token_size} property
    # @option attributes [Integer] :innodb_ft_min_token_size The value to assign to the {#innodb_ft_min_token_size} property
    # @option attributes [Integer] :innodb_ft_num_word_optimize The value to assign to the {#innodb_ft_num_word_optimize} property
    # @option attributes [Integer] :innodb_lock_wait_timeout The value to assign to the {#innodb_lock_wait_timeout} property
    # @option attributes [Integer] :innodb_max_purge_lag The value to assign to the {#innodb_max_purge_lag} property
    # @option attributes [Integer] :innodb_max_purge_lag_delay The value to assign to the {#innodb_max_purge_lag_delay} property
    # @option attributes [Integer] :interactive_timeout The value to assign to the {#interactive_timeout} property
    # @option attributes [Integer] :innodb_stats_persistent_sample_pages The value to assign to the {#innodb_stats_persistent_sample_pages} property
    # @option attributes [Integer] :innodb_stats_transient_sample_pages The value to assign to the {#innodb_stats_transient_sample_pages} property
    # @option attributes [Integer] :max_allowed_packet The value to assign to the {#max_allowed_packet} property
    # @option attributes [Integer] :max_execution_time The value to assign to the {#max_execution_time} property
    # @option attributes [Integer] :mysqlx_connect_timeout The value to assign to the {#mysqlx_connect_timeout} property
    # @option attributes [Integer] :mysqlx_document_id_unique_prefix The value to assign to the {#mysqlx_document_id_unique_prefix} property
    # @option attributes [Integer] :mysqlx_idle_worker_thread_timeout The value to assign to the {#mysqlx_idle_worker_thread_timeout} property
    # @option attributes [Integer] :mysqlx_interactive_timeout The value to assign to the {#mysqlx_interactive_timeout} property
    # @option attributes [Integer] :mysqlx_max_allowed_packet The value to assign to the {#mysqlx_max_allowed_packet} property
    # @option attributes [Integer] :mysqlx_min_worker_threads The value to assign to the {#mysqlx_min_worker_threads} property
    # @option attributes [Integer] :mysqlx_read_timeout The value to assign to the {#mysqlx_read_timeout} property
    # @option attributes [Integer] :mysqlx_wait_timeout The value to assign to the {#mysqlx_wait_timeout} property
    # @option attributes [Integer] :mysqlx_write_timeout The value to assign to the {#mysqlx_write_timeout} property
    # @option attributes [Integer] :net_read_timeout The value to assign to the {#net_read_timeout} property
    # @option attributes [Integer] :net_write_timeout The value to assign to the {#net_write_timeout} property
    # @option attributes [Integer] :parser_max_mem_size The value to assign to the {#parser_max_mem_size} property
    # @option attributes [Integer] :query_alloc_block_size The value to assign to the {#query_alloc_block_size} property
    # @option attributes [Integer] :query_prealloc_size The value to assign to the {#query_prealloc_size} property
    # @option attributes [Integer] :regexp_time_limit The value to assign to the {#regexp_time_limit} property
    # @option attributes [String] :sql_mode The value to assign to the {#sql_mode} property
    # @option attributes [Integer] :tmp_table_size The value to assign to the {#tmp_table_size} property
    # @option attributes [Integer] :mysqlx_deflate_default_compression_level The value to assign to the {#mysqlx_deflate_default_compression_level} property
    # @option attributes [Integer] :mysqlx_deflate_max_client_compression_level The value to assign to the {#mysqlx_deflate_max_client_compression_level} property
    # @option attributes [Integer] :mysqlx_lz4_max_client_compression_level The value to assign to the {#mysqlx_lz4_max_client_compression_level} property
    # @option attributes [Integer] :mysqlx_lz4_default_compression_level The value to assign to the {#mysqlx_lz4_default_compression_level} property
    # @option attributes [Integer] :mysqlx_zstd_max_client_compression_level The value to assign to the {#mysqlx_zstd_max_client_compression_level} property
    # @option attributes [Integer] :mysqlx_zstd_default_compression_level The value to assign to the {#mysqlx_zstd_default_compression_level} property
    # @option attributes [Integer] :mysql_zstd_default_compression_level The value to assign to the {#mysql_zstd_default_compression_level} property
    # @option attributes [Integer] :sort_buffer_size The value to assign to the {#sort_buffer_size} property
    # @option attributes [Integer] :wait_timeout The value to assign to the {#wait_timeout} property
    # @option attributes [BOOLEAN] :thread_pool_dedicated_listeners The value to assign to the {#thread_pool_dedicated_listeners} property
    # @option attributes [Integer] :thread_pool_max_transactions_limit The value to assign to the {#thread_pool_max_transactions_limit} property
    # @option attributes [String] :time_zone The value to assign to the {#time_zone} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.completion_type = attributes[:'completionType'] if attributes[:'completionType']
      self.completion_type = "NO_CHAIN" if completion_type.nil? && !attributes.key?(:'completionType') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :completionType and :completion_type' if attributes.key?(:'completionType') && attributes.key?(:'completion_type')

      self.completion_type = attributes[:'completion_type'] if attributes[:'completion_type']
      self.completion_type = "NO_CHAIN" if completion_type.nil? && !attributes.key?(:'completionType') && !attributes.key?(:'completion_type') # rubocop:disable Style/StringLiterals

      self.big_tables = attributes[:'bigTables'] unless attributes[:'bigTables'].nil?
      self.big_tables = false if big_tables.nil? && !attributes.key?(:'bigTables') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :bigTables and :big_tables' if attributes.key?(:'bigTables') && attributes.key?(:'big_tables')

      self.big_tables = attributes[:'big_tables'] unless attributes[:'big_tables'].nil?
      self.big_tables = false if big_tables.nil? && !attributes.key?(:'bigTables') && !attributes.key?(:'big_tables') # rubocop:disable Style/StringLiterals

      self.connection_memory_chunk_size = attributes[:'connectionMemoryChunkSize'] if attributes[:'connectionMemoryChunkSize']

      raise 'You cannot provide both :connectionMemoryChunkSize and :connection_memory_chunk_size' if attributes.key?(:'connectionMemoryChunkSize') && attributes.key?(:'connection_memory_chunk_size')

      self.connection_memory_chunk_size = attributes[:'connection_memory_chunk_size'] if attributes[:'connection_memory_chunk_size']

      self.connection_memory_limit = attributes[:'connectionMemoryLimit'] if attributes[:'connectionMemoryLimit']

      raise 'You cannot provide both :connectionMemoryLimit and :connection_memory_limit' if attributes.key?(:'connectionMemoryLimit') && attributes.key?(:'connection_memory_limit')

      self.connection_memory_limit = attributes[:'connection_memory_limit'] if attributes[:'connection_memory_limit']

      self.default_authentication_plugin = attributes[:'defaultAuthenticationPlugin'] if attributes[:'defaultAuthenticationPlugin']
      self.default_authentication_plugin = "caching_sha2_password" if default_authentication_plugin.nil? && !attributes.key?(:'defaultAuthenticationPlugin') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :defaultAuthenticationPlugin and :default_authentication_plugin' if attributes.key?(:'defaultAuthenticationPlugin') && attributes.key?(:'default_authentication_plugin')

      self.default_authentication_plugin = attributes[:'default_authentication_plugin'] if attributes[:'default_authentication_plugin']
      self.default_authentication_plugin = "caching_sha2_password" if default_authentication_plugin.nil? && !attributes.key?(:'defaultAuthenticationPlugin') && !attributes.key?(:'default_authentication_plugin') # rubocop:disable Style/StringLiterals

      self.global_connection_memory_limit = attributes[:'globalConnectionMemoryLimit'] if attributes[:'globalConnectionMemoryLimit']

      raise 'You cannot provide both :globalConnectionMemoryLimit and :global_connection_memory_limit' if attributes.key?(:'globalConnectionMemoryLimit') && attributes.key?(:'global_connection_memory_limit')

      self.global_connection_memory_limit = attributes[:'global_connection_memory_limit'] if attributes[:'global_connection_memory_limit']

      self.global_connection_memory_tracking = attributes[:'globalConnectionMemoryTracking'] unless attributes[:'globalConnectionMemoryTracking'].nil?
      self.global_connection_memory_tracking = false if global_connection_memory_tracking.nil? && !attributes.key?(:'globalConnectionMemoryTracking') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :globalConnectionMemoryTracking and :global_connection_memory_tracking' if attributes.key?(:'globalConnectionMemoryTracking') && attributes.key?(:'global_connection_memory_tracking')

      self.global_connection_memory_tracking = attributes[:'global_connection_memory_tracking'] unless attributes[:'global_connection_memory_tracking'].nil?
      self.global_connection_memory_tracking = false if global_connection_memory_tracking.nil? && !attributes.key?(:'globalConnectionMemoryTracking') && !attributes.key?(:'global_connection_memory_tracking') # rubocop:disable Style/StringLiterals

      self.transaction_isolation = attributes[:'transactionIsolation'] if attributes[:'transactionIsolation']
      self.transaction_isolation = "REPEATABLE-READ" if transaction_isolation.nil? && !attributes.key?(:'transactionIsolation') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :transactionIsolation and :transaction_isolation' if attributes.key?(:'transactionIsolation') && attributes.key?(:'transaction_isolation')

      self.transaction_isolation = attributes[:'transaction_isolation'] if attributes[:'transaction_isolation']
      self.transaction_isolation = "REPEATABLE-READ" if transaction_isolation.nil? && !attributes.key?(:'transactionIsolation') && !attributes.key?(:'transaction_isolation') # rubocop:disable Style/StringLiterals

      self.innodb_ft_server_stopword_table = attributes[:'innodbFtServerStopwordTable'] if attributes[:'innodbFtServerStopwordTable']
      self.innodb_ft_server_stopword_table = "null" if innodb_ft_server_stopword_table.nil? && !attributes.key?(:'innodbFtServerStopwordTable') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :innodbFtServerStopwordTable and :innodb_ft_server_stopword_table' if attributes.key?(:'innodbFtServerStopwordTable') && attributes.key?(:'innodb_ft_server_stopword_table')

      self.innodb_ft_server_stopword_table = attributes[:'innodb_ft_server_stopword_table'] if attributes[:'innodb_ft_server_stopword_table']
      self.innodb_ft_server_stopword_table = "null" if innodb_ft_server_stopword_table.nil? && !attributes.key?(:'innodbFtServerStopwordTable') && !attributes.key?(:'innodb_ft_server_stopword_table') # rubocop:disable Style/StringLiterals

      self.mandatory_roles = attributes[:'mandatoryRoles'] if attributes[:'mandatoryRoles']
      self.mandatory_roles = "" if mandatory_roles.nil? && !attributes.key?(:'mandatoryRoles') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :mandatoryRoles and :mandatory_roles' if attributes.key?(:'mandatoryRoles') && attributes.key?(:'mandatory_roles')

      self.mandatory_roles = attributes[:'mandatory_roles'] if attributes[:'mandatory_roles']
      self.mandatory_roles = "" if mandatory_roles.nil? && !attributes.key?(:'mandatoryRoles') && !attributes.key?(:'mandatory_roles') # rubocop:disable Style/StringLiterals

      self.autocommit = attributes[:'autocommit'] unless attributes[:'autocommit'].nil?
      self.autocommit = true if autocommit.nil? && !attributes.key?(:'autocommit') # rubocop:disable Style/StringLiterals

      self.foreign_key_checks = attributes[:'foreignKeyChecks'] unless attributes[:'foreignKeyChecks'].nil?
      self.foreign_key_checks = true if foreign_key_checks.nil? && !attributes.key?(:'foreignKeyChecks') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :foreignKeyChecks and :foreign_key_checks' if attributes.key?(:'foreignKeyChecks') && attributes.key?(:'foreign_key_checks')

      self.foreign_key_checks = attributes[:'foreign_key_checks'] unless attributes[:'foreign_key_checks'].nil?
      self.foreign_key_checks = true if foreign_key_checks.nil? && !attributes.key?(:'foreignKeyChecks') && !attributes.key?(:'foreign_key_checks') # rubocop:disable Style/StringLiterals

      self.group_replication_consistency = attributes[:'groupReplicationConsistency'] if attributes[:'groupReplicationConsistency']
      self.group_replication_consistency = "BEFORE_ON_PRIMARY_FAILOVER" if group_replication_consistency.nil? && !attributes.key?(:'groupReplicationConsistency') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :groupReplicationConsistency and :group_replication_consistency' if attributes.key?(:'groupReplicationConsistency') && attributes.key?(:'group_replication_consistency')

      self.group_replication_consistency = attributes[:'group_replication_consistency'] if attributes[:'group_replication_consistency']
      self.group_replication_consistency = "BEFORE_ON_PRIMARY_FAILOVER" if group_replication_consistency.nil? && !attributes.key?(:'groupReplicationConsistency') && !attributes.key?(:'group_replication_consistency') # rubocop:disable Style/StringLiterals

      self.innodb_ft_enable_stopword = attributes[:'innodbFtEnableStopword'] unless attributes[:'innodbFtEnableStopword'].nil?
      self.innodb_ft_enable_stopword = true if innodb_ft_enable_stopword.nil? && !attributes.key?(:'innodbFtEnableStopword') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :innodbFtEnableStopword and :innodb_ft_enable_stopword' if attributes.key?(:'innodbFtEnableStopword') && attributes.key?(:'innodb_ft_enable_stopword')

      self.innodb_ft_enable_stopword = attributes[:'innodb_ft_enable_stopword'] unless attributes[:'innodb_ft_enable_stopword'].nil?
      self.innodb_ft_enable_stopword = true if innodb_ft_enable_stopword.nil? && !attributes.key?(:'innodbFtEnableStopword') && !attributes.key?(:'innodb_ft_enable_stopword') # rubocop:disable Style/StringLiterals

      self.innodb_log_writer_threads = attributes[:'innodbLogWriterThreads'] unless attributes[:'innodbLogWriterThreads'].nil?
      self.innodb_log_writer_threads = true if innodb_log_writer_threads.nil? && !attributes.key?(:'innodbLogWriterThreads') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :innodbLogWriterThreads and :innodb_log_writer_threads' if attributes.key?(:'innodbLogWriterThreads') && attributes.key?(:'innodb_log_writer_threads')

      self.innodb_log_writer_threads = attributes[:'innodb_log_writer_threads'] unless attributes[:'innodb_log_writer_threads'].nil?
      self.innodb_log_writer_threads = true if innodb_log_writer_threads.nil? && !attributes.key?(:'innodbLogWriterThreads') && !attributes.key?(:'innodb_log_writer_threads') # rubocop:disable Style/StringLiterals

      self.local_infile = attributes[:'localInfile'] unless attributes[:'localInfile'].nil?
      self.local_infile = false if local_infile.nil? && !attributes.key?(:'localInfile') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :localInfile and :local_infile' if attributes.key?(:'localInfile') && attributes.key?(:'local_infile')

      self.local_infile = attributes[:'local_infile'] unless attributes[:'local_infile'].nil?
      self.local_infile = false if local_infile.nil? && !attributes.key?(:'localInfile') && !attributes.key?(:'local_infile') # rubocop:disable Style/StringLiterals

      self.mysql_firewall_mode = attributes[:'mysqlFirewallMode'] unless attributes[:'mysqlFirewallMode'].nil?
      self.mysql_firewall_mode = true if mysql_firewall_mode.nil? && !attributes.key?(:'mysqlFirewallMode') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :mysqlFirewallMode and :mysql_firewall_mode' if attributes.key?(:'mysqlFirewallMode') && attributes.key?(:'mysql_firewall_mode')

      self.mysql_firewall_mode = attributes[:'mysql_firewall_mode'] unless attributes[:'mysql_firewall_mode'].nil?
      self.mysql_firewall_mode = true if mysql_firewall_mode.nil? && !attributes.key?(:'mysqlFirewallMode') && !attributes.key?(:'mysql_firewall_mode') # rubocop:disable Style/StringLiterals

      self.mysqlx_enable_hello_notice = attributes[:'mysqlxEnableHelloNotice'] unless attributes[:'mysqlxEnableHelloNotice'].nil?
      self.mysqlx_enable_hello_notice = true if mysqlx_enable_hello_notice.nil? && !attributes.key?(:'mysqlxEnableHelloNotice') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :mysqlxEnableHelloNotice and :mysqlx_enable_hello_notice' if attributes.key?(:'mysqlxEnableHelloNotice') && attributes.key?(:'mysqlx_enable_hello_notice')

      self.mysqlx_enable_hello_notice = attributes[:'mysqlx_enable_hello_notice'] unless attributes[:'mysqlx_enable_hello_notice'].nil?
      self.mysqlx_enable_hello_notice = true if mysqlx_enable_hello_notice.nil? && !attributes.key?(:'mysqlxEnableHelloNotice') && !attributes.key?(:'mysqlx_enable_hello_notice') # rubocop:disable Style/StringLiterals

      self.sql_require_primary_key = attributes[:'sqlRequirePrimaryKey'] unless attributes[:'sqlRequirePrimaryKey'].nil?
      self.sql_require_primary_key = false if sql_require_primary_key.nil? && !attributes.key?(:'sqlRequirePrimaryKey') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :sqlRequirePrimaryKey and :sql_require_primary_key' if attributes.key?(:'sqlRequirePrimaryKey') && attributes.key?(:'sql_require_primary_key')

      self.sql_require_primary_key = attributes[:'sql_require_primary_key'] unless attributes[:'sql_require_primary_key'].nil?
      self.sql_require_primary_key = false if sql_require_primary_key.nil? && !attributes.key?(:'sqlRequirePrimaryKey') && !attributes.key?(:'sql_require_primary_key') # rubocop:disable Style/StringLiterals

      self.sql_warnings = attributes[:'sqlWarnings'] unless attributes[:'sqlWarnings'].nil?
      self.sql_warnings = false if sql_warnings.nil? && !attributes.key?(:'sqlWarnings') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :sqlWarnings and :sql_warnings' if attributes.key?(:'sqlWarnings') && attributes.key?(:'sql_warnings')

      self.sql_warnings = attributes[:'sql_warnings'] unless attributes[:'sql_warnings'].nil?
      self.sql_warnings = false if sql_warnings.nil? && !attributes.key?(:'sqlWarnings') && !attributes.key?(:'sql_warnings') # rubocop:disable Style/StringLiterals

      self.binlog_expire_logs_seconds = attributes[:'binlogExpireLogsSeconds'] if attributes[:'binlogExpireLogsSeconds']

      raise 'You cannot provide both :binlogExpireLogsSeconds and :binlog_expire_logs_seconds' if attributes.key?(:'binlogExpireLogsSeconds') && attributes.key?(:'binlog_expire_logs_seconds')

      self.binlog_expire_logs_seconds = attributes[:'binlog_expire_logs_seconds'] if attributes[:'binlog_expire_logs_seconds']

      self.binlog_row_metadata = attributes[:'binlogRowMetadata'] if attributes[:'binlogRowMetadata']
      self.binlog_row_metadata = "MINIMAL" if binlog_row_metadata.nil? && !attributes.key?(:'binlogRowMetadata') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :binlogRowMetadata and :binlog_row_metadata' if attributes.key?(:'binlogRowMetadata') && attributes.key?(:'binlog_row_metadata')

      self.binlog_row_metadata = attributes[:'binlog_row_metadata'] if attributes[:'binlog_row_metadata']
      self.binlog_row_metadata = "MINIMAL" if binlog_row_metadata.nil? && !attributes.key?(:'binlogRowMetadata') && !attributes.key?(:'binlog_row_metadata') # rubocop:disable Style/StringLiterals

      self.binlog_row_value_options = attributes[:'binlogRowValueOptions'] if attributes[:'binlogRowValueOptions']
      self.binlog_row_value_options = "PARTIAL_JSON" if binlog_row_value_options.nil? && !attributes.key?(:'binlogRowValueOptions') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :binlogRowValueOptions and :binlog_row_value_options' if attributes.key?(:'binlogRowValueOptions') && attributes.key?(:'binlog_row_value_options')

      self.binlog_row_value_options = attributes[:'binlog_row_value_options'] if attributes[:'binlog_row_value_options']
      self.binlog_row_value_options = "PARTIAL_JSON" if binlog_row_value_options.nil? && !attributes.key?(:'binlogRowValueOptions') && !attributes.key?(:'binlog_row_value_options') # rubocop:disable Style/StringLiterals

      self.binlog_transaction_compression = attributes[:'binlogTransactionCompression'] unless attributes[:'binlogTransactionCompression'].nil?
      self.binlog_transaction_compression = true if binlog_transaction_compression.nil? && !attributes.key?(:'binlogTransactionCompression') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :binlogTransactionCompression and :binlog_transaction_compression' if attributes.key?(:'binlogTransactionCompression') && attributes.key?(:'binlog_transaction_compression')

      self.binlog_transaction_compression = attributes[:'binlog_transaction_compression'] unless attributes[:'binlog_transaction_compression'].nil?
      self.binlog_transaction_compression = true if binlog_transaction_compression.nil? && !attributes.key?(:'binlogTransactionCompression') && !attributes.key?(:'binlog_transaction_compression') # rubocop:disable Style/StringLiterals

      self.innodb_buffer_pool_size = attributes[:'innodbBufferPoolSize'] if attributes[:'innodbBufferPoolSize']
      self.innodb_buffer_pool_size = 134217728 if innodb_buffer_pool_size.nil? && !attributes.key?(:'innodbBufferPoolSize') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :innodbBufferPoolSize and :innodb_buffer_pool_size' if attributes.key?(:'innodbBufferPoolSize') && attributes.key?(:'innodb_buffer_pool_size')

      self.innodb_buffer_pool_size = attributes[:'innodb_buffer_pool_size'] if attributes[:'innodb_buffer_pool_size']
      self.innodb_buffer_pool_size = 134217728 if innodb_buffer_pool_size.nil? && !attributes.key?(:'innodbBufferPoolSize') && !attributes.key?(:'innodb_buffer_pool_size') # rubocop:disable Style/StringLiterals

      self.innodb_ft_result_cache_limit = attributes[:'innodbFtResultCacheLimit'] if attributes[:'innodbFtResultCacheLimit']
      self.innodb_ft_result_cache_limit = 2000000000 if innodb_ft_result_cache_limit.nil? && !attributes.key?(:'innodbFtResultCacheLimit') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :innodbFtResultCacheLimit and :innodb_ft_result_cache_limit' if attributes.key?(:'innodbFtResultCacheLimit') && attributes.key?(:'innodb_ft_result_cache_limit')

      self.innodb_ft_result_cache_limit = attributes[:'innodb_ft_result_cache_limit'] if attributes[:'innodb_ft_result_cache_limit']
      self.innodb_ft_result_cache_limit = 2000000000 if innodb_ft_result_cache_limit.nil? && !attributes.key?(:'innodbFtResultCacheLimit') && !attributes.key?(:'innodb_ft_result_cache_limit') # rubocop:disable Style/StringLiterals

      self.max_binlog_cache_size = attributes[:'maxBinlogCacheSize'] if attributes[:'maxBinlogCacheSize']

      raise 'You cannot provide both :maxBinlogCacheSize and :max_binlog_cache_size' if attributes.key?(:'maxBinlogCacheSize') && attributes.key?(:'max_binlog_cache_size')

      self.max_binlog_cache_size = attributes[:'max_binlog_cache_size'] if attributes[:'max_binlog_cache_size']

      self.max_connect_errors = attributes[:'maxConnectErrors'] if attributes[:'maxConnectErrors']

      raise 'You cannot provide both :maxConnectErrors and :max_connect_errors' if attributes.key?(:'maxConnectErrors') && attributes.key?(:'max_connect_errors')

      self.max_connect_errors = attributes[:'max_connect_errors'] if attributes[:'max_connect_errors']

      self.max_heap_table_size = attributes[:'maxHeapTableSize'] if attributes[:'maxHeapTableSize']
      self.max_heap_table_size = 16777216 if max_heap_table_size.nil? && !attributes.key?(:'maxHeapTableSize') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :maxHeapTableSize and :max_heap_table_size' if attributes.key?(:'maxHeapTableSize') && attributes.key?(:'max_heap_table_size')

      self.max_heap_table_size = attributes[:'max_heap_table_size'] if attributes[:'max_heap_table_size']
      self.max_heap_table_size = 16777216 if max_heap_table_size.nil? && !attributes.key?(:'maxHeapTableSize') && !attributes.key?(:'max_heap_table_size') # rubocop:disable Style/StringLiterals

      self.max_connections = attributes[:'maxConnections'] if attributes[:'maxConnections']

      raise 'You cannot provide both :maxConnections and :max_connections' if attributes.key?(:'maxConnections') && attributes.key?(:'max_connections')

      self.max_connections = attributes[:'max_connections'] if attributes[:'max_connections']

      self.max_prepared_stmt_count = attributes[:'maxPreparedStmtCount'] if attributes[:'maxPreparedStmtCount']

      raise 'You cannot provide both :maxPreparedStmtCount and :max_prepared_stmt_count' if attributes.key?(:'maxPreparedStmtCount') && attributes.key?(:'max_prepared_stmt_count')

      self.max_prepared_stmt_count = attributes[:'max_prepared_stmt_count'] if attributes[:'max_prepared_stmt_count']

      self.connect_timeout = attributes[:'connectTimeout'] if attributes[:'connectTimeout']

      raise 'You cannot provide both :connectTimeout and :connect_timeout' if attributes.key?(:'connectTimeout') && attributes.key?(:'connect_timeout')

      self.connect_timeout = attributes[:'connect_timeout'] if attributes[:'connect_timeout']

      self.cte_max_recursion_depth = attributes[:'cteMaxRecursionDepth'] if attributes[:'cteMaxRecursionDepth']
      self.cte_max_recursion_depth = 1000 if cte_max_recursion_depth.nil? && !attributes.key?(:'cteMaxRecursionDepth') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :cteMaxRecursionDepth and :cte_max_recursion_depth' if attributes.key?(:'cteMaxRecursionDepth') && attributes.key?(:'cte_max_recursion_depth')

      self.cte_max_recursion_depth = attributes[:'cte_max_recursion_depth'] if attributes[:'cte_max_recursion_depth']
      self.cte_max_recursion_depth = 1000 if cte_max_recursion_depth.nil? && !attributes.key?(:'cteMaxRecursionDepth') && !attributes.key?(:'cte_max_recursion_depth') # rubocop:disable Style/StringLiterals

      self.generated_random_password_length = attributes[:'generatedRandomPasswordLength'] if attributes[:'generatedRandomPasswordLength']

      raise 'You cannot provide both :generatedRandomPasswordLength and :generated_random_password_length' if attributes.key?(:'generatedRandomPasswordLength') && attributes.key?(:'generated_random_password_length')

      self.generated_random_password_length = attributes[:'generated_random_password_length'] if attributes[:'generated_random_password_length']

      self.information_schema_stats_expiry = attributes[:'informationSchemaStatsExpiry'] if attributes[:'informationSchemaStatsExpiry']

      raise 'You cannot provide both :informationSchemaStatsExpiry and :information_schema_stats_expiry' if attributes.key?(:'informationSchemaStatsExpiry') && attributes.key?(:'information_schema_stats_expiry')

      self.information_schema_stats_expiry = attributes[:'information_schema_stats_expiry'] if attributes[:'information_schema_stats_expiry']

      self.innodb_buffer_pool_dump_pct = attributes[:'innodbBufferPoolDumpPct'] if attributes[:'innodbBufferPoolDumpPct']

      raise 'You cannot provide both :innodbBufferPoolDumpPct and :innodb_buffer_pool_dump_pct' if attributes.key?(:'innodbBufferPoolDumpPct') && attributes.key?(:'innodb_buffer_pool_dump_pct')

      self.innodb_buffer_pool_dump_pct = attributes[:'innodb_buffer_pool_dump_pct'] if attributes[:'innodb_buffer_pool_dump_pct']

      self.innodb_buffer_pool_instances = attributes[:'innodbBufferPoolInstances'] if attributes[:'innodbBufferPoolInstances']

      raise 'You cannot provide both :innodbBufferPoolInstances and :innodb_buffer_pool_instances' if attributes.key?(:'innodbBufferPoolInstances') && attributes.key?(:'innodb_buffer_pool_instances')

      self.innodb_buffer_pool_instances = attributes[:'innodb_buffer_pool_instances'] if attributes[:'innodb_buffer_pool_instances']

      self.innodb_ddl_buffer_size = attributes[:'innodbDdlBufferSize'] if attributes[:'innodbDdlBufferSize']
      self.innodb_ddl_buffer_size = 1048576 if innodb_ddl_buffer_size.nil? && !attributes.key?(:'innodbDdlBufferSize') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :innodbDdlBufferSize and :innodb_ddl_buffer_size' if attributes.key?(:'innodbDdlBufferSize') && attributes.key?(:'innodb_ddl_buffer_size')

      self.innodb_ddl_buffer_size = attributes[:'innodb_ddl_buffer_size'] if attributes[:'innodb_ddl_buffer_size']
      self.innodb_ddl_buffer_size = 1048576 if innodb_ddl_buffer_size.nil? && !attributes.key?(:'innodbDdlBufferSize') && !attributes.key?(:'innodb_ddl_buffer_size') # rubocop:disable Style/StringLiterals

      self.innodb_ddl_threads = attributes[:'innodbDdlThreads'] if attributes[:'innodbDdlThreads']

      raise 'You cannot provide both :innodbDdlThreads and :innodb_ddl_threads' if attributes.key?(:'innodbDdlThreads') && attributes.key?(:'innodb_ddl_threads')

      self.innodb_ddl_threads = attributes[:'innodb_ddl_threads'] if attributes[:'innodb_ddl_threads']

      self.innodb_ft_max_token_size = attributes[:'innodbFtMaxTokenSize'] if attributes[:'innodbFtMaxTokenSize']

      raise 'You cannot provide both :innodbFtMaxTokenSize and :innodb_ft_max_token_size' if attributes.key?(:'innodbFtMaxTokenSize') && attributes.key?(:'innodb_ft_max_token_size')

      self.innodb_ft_max_token_size = attributes[:'innodb_ft_max_token_size'] if attributes[:'innodb_ft_max_token_size']

      self.innodb_ft_min_token_size = attributes[:'innodbFtMinTokenSize'] if attributes[:'innodbFtMinTokenSize']

      raise 'You cannot provide both :innodbFtMinTokenSize and :innodb_ft_min_token_size' if attributes.key?(:'innodbFtMinTokenSize') && attributes.key?(:'innodb_ft_min_token_size')

      self.innodb_ft_min_token_size = attributes[:'innodb_ft_min_token_size'] if attributes[:'innodb_ft_min_token_size']

      self.innodb_ft_num_word_optimize = attributes[:'innodbFtNumWordOptimize'] if attributes[:'innodbFtNumWordOptimize']

      raise 'You cannot provide both :innodbFtNumWordOptimize and :innodb_ft_num_word_optimize' if attributes.key?(:'innodbFtNumWordOptimize') && attributes.key?(:'innodb_ft_num_word_optimize')

      self.innodb_ft_num_word_optimize = attributes[:'innodb_ft_num_word_optimize'] if attributes[:'innodb_ft_num_word_optimize']

      self.innodb_lock_wait_timeout = attributes[:'innodbLockWaitTimeout'] if attributes[:'innodbLockWaitTimeout']

      raise 'You cannot provide both :innodbLockWaitTimeout and :innodb_lock_wait_timeout' if attributes.key?(:'innodbLockWaitTimeout') && attributes.key?(:'innodb_lock_wait_timeout')

      self.innodb_lock_wait_timeout = attributes[:'innodb_lock_wait_timeout'] if attributes[:'innodb_lock_wait_timeout']

      self.innodb_max_purge_lag = attributes[:'innodbMaxPurgeLag'] if attributes[:'innodbMaxPurgeLag']
      self.innodb_max_purge_lag = 0 if innodb_max_purge_lag.nil? && !attributes.key?(:'innodbMaxPurgeLag') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :innodbMaxPurgeLag and :innodb_max_purge_lag' if attributes.key?(:'innodbMaxPurgeLag') && attributes.key?(:'innodb_max_purge_lag')

      self.innodb_max_purge_lag = attributes[:'innodb_max_purge_lag'] if attributes[:'innodb_max_purge_lag']
      self.innodb_max_purge_lag = 0 if innodb_max_purge_lag.nil? && !attributes.key?(:'innodbMaxPurgeLag') && !attributes.key?(:'innodb_max_purge_lag') # rubocop:disable Style/StringLiterals

      self.innodb_max_purge_lag_delay = attributes[:'innodbMaxPurgeLagDelay'] if attributes[:'innodbMaxPurgeLagDelay']

      raise 'You cannot provide both :innodbMaxPurgeLagDelay and :innodb_max_purge_lag_delay' if attributes.key?(:'innodbMaxPurgeLagDelay') && attributes.key?(:'innodb_max_purge_lag_delay')

      self.innodb_max_purge_lag_delay = attributes[:'innodb_max_purge_lag_delay'] if attributes[:'innodb_max_purge_lag_delay']

      self.interactive_timeout = attributes[:'interactiveTimeout'] if attributes[:'interactiveTimeout']

      raise 'You cannot provide both :interactiveTimeout and :interactive_timeout' if attributes.key?(:'interactiveTimeout') && attributes.key?(:'interactive_timeout')

      self.interactive_timeout = attributes[:'interactive_timeout'] if attributes[:'interactive_timeout']

      self.innodb_stats_persistent_sample_pages = attributes[:'innodbStatsPersistentSamplePages'] if attributes[:'innodbStatsPersistentSamplePages']
      self.innodb_stats_persistent_sample_pages = 20 if innodb_stats_persistent_sample_pages.nil? && !attributes.key?(:'innodbStatsPersistentSamplePages') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :innodbStatsPersistentSamplePages and :innodb_stats_persistent_sample_pages' if attributes.key?(:'innodbStatsPersistentSamplePages') && attributes.key?(:'innodb_stats_persistent_sample_pages')

      self.innodb_stats_persistent_sample_pages = attributes[:'innodb_stats_persistent_sample_pages'] if attributes[:'innodb_stats_persistent_sample_pages']
      self.innodb_stats_persistent_sample_pages = 20 if innodb_stats_persistent_sample_pages.nil? && !attributes.key?(:'innodbStatsPersistentSamplePages') && !attributes.key?(:'innodb_stats_persistent_sample_pages') # rubocop:disable Style/StringLiterals

      self.innodb_stats_transient_sample_pages = attributes[:'innodbStatsTransientSamplePages'] if attributes[:'innodbStatsTransientSamplePages']
      self.innodb_stats_transient_sample_pages = 8 if innodb_stats_transient_sample_pages.nil? && !attributes.key?(:'innodbStatsTransientSamplePages') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :innodbStatsTransientSamplePages and :innodb_stats_transient_sample_pages' if attributes.key?(:'innodbStatsTransientSamplePages') && attributes.key?(:'innodb_stats_transient_sample_pages')

      self.innodb_stats_transient_sample_pages = attributes[:'innodb_stats_transient_sample_pages'] if attributes[:'innodb_stats_transient_sample_pages']
      self.innodb_stats_transient_sample_pages = 8 if innodb_stats_transient_sample_pages.nil? && !attributes.key?(:'innodbStatsTransientSamplePages') && !attributes.key?(:'innodb_stats_transient_sample_pages') # rubocop:disable Style/StringLiterals

      self.max_allowed_packet = attributes[:'maxAllowedPacket'] if attributes[:'maxAllowedPacket']

      raise 'You cannot provide both :maxAllowedPacket and :max_allowed_packet' if attributes.key?(:'maxAllowedPacket') && attributes.key?(:'max_allowed_packet')

      self.max_allowed_packet = attributes[:'max_allowed_packet'] if attributes[:'max_allowed_packet']

      self.max_execution_time = attributes[:'maxExecutionTime'] if attributes[:'maxExecutionTime']
      self.max_execution_time = 0 if max_execution_time.nil? && !attributes.key?(:'maxExecutionTime') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :maxExecutionTime and :max_execution_time' if attributes.key?(:'maxExecutionTime') && attributes.key?(:'max_execution_time')

      self.max_execution_time = attributes[:'max_execution_time'] if attributes[:'max_execution_time']
      self.max_execution_time = 0 if max_execution_time.nil? && !attributes.key?(:'maxExecutionTime') && !attributes.key?(:'max_execution_time') # rubocop:disable Style/StringLiterals

      self.mysqlx_connect_timeout = attributes[:'mysqlxConnectTimeout'] if attributes[:'mysqlxConnectTimeout']

      raise 'You cannot provide both :mysqlxConnectTimeout and :mysqlx_connect_timeout' if attributes.key?(:'mysqlxConnectTimeout') && attributes.key?(:'mysqlx_connect_timeout')

      self.mysqlx_connect_timeout = attributes[:'mysqlx_connect_timeout'] if attributes[:'mysqlx_connect_timeout']

      self.mysqlx_document_id_unique_prefix = attributes[:'mysqlxDocumentIdUniquePrefix'] if attributes[:'mysqlxDocumentIdUniquePrefix']

      raise 'You cannot provide both :mysqlxDocumentIdUniquePrefix and :mysqlx_document_id_unique_prefix' if attributes.key?(:'mysqlxDocumentIdUniquePrefix') && attributes.key?(:'mysqlx_document_id_unique_prefix')

      self.mysqlx_document_id_unique_prefix = attributes[:'mysqlx_document_id_unique_prefix'] if attributes[:'mysqlx_document_id_unique_prefix']

      self.mysqlx_idle_worker_thread_timeout = attributes[:'mysqlxIdleWorkerThreadTimeout'] if attributes[:'mysqlxIdleWorkerThreadTimeout']

      raise 'You cannot provide both :mysqlxIdleWorkerThreadTimeout and :mysqlx_idle_worker_thread_timeout' if attributes.key?(:'mysqlxIdleWorkerThreadTimeout') && attributes.key?(:'mysqlx_idle_worker_thread_timeout')

      self.mysqlx_idle_worker_thread_timeout = attributes[:'mysqlx_idle_worker_thread_timeout'] if attributes[:'mysqlx_idle_worker_thread_timeout']

      self.mysqlx_interactive_timeout = attributes[:'mysqlxInteractiveTimeout'] if attributes[:'mysqlxInteractiveTimeout']

      raise 'You cannot provide both :mysqlxInteractiveTimeout and :mysqlx_interactive_timeout' if attributes.key?(:'mysqlxInteractiveTimeout') && attributes.key?(:'mysqlx_interactive_timeout')

      self.mysqlx_interactive_timeout = attributes[:'mysqlx_interactive_timeout'] if attributes[:'mysqlx_interactive_timeout']

      self.mysqlx_max_allowed_packet = attributes[:'mysqlxMaxAllowedPacket'] if attributes[:'mysqlxMaxAllowedPacket']

      raise 'You cannot provide both :mysqlxMaxAllowedPacket and :mysqlx_max_allowed_packet' if attributes.key?(:'mysqlxMaxAllowedPacket') && attributes.key?(:'mysqlx_max_allowed_packet')

      self.mysqlx_max_allowed_packet = attributes[:'mysqlx_max_allowed_packet'] if attributes[:'mysqlx_max_allowed_packet']

      self.mysqlx_min_worker_threads = attributes[:'mysqlxMinWorkerThreads'] if attributes[:'mysqlxMinWorkerThreads']

      raise 'You cannot provide both :mysqlxMinWorkerThreads and :mysqlx_min_worker_threads' if attributes.key?(:'mysqlxMinWorkerThreads') && attributes.key?(:'mysqlx_min_worker_threads')

      self.mysqlx_min_worker_threads = attributes[:'mysqlx_min_worker_threads'] if attributes[:'mysqlx_min_worker_threads']

      self.mysqlx_read_timeout = attributes[:'mysqlxReadTimeout'] if attributes[:'mysqlxReadTimeout']

      raise 'You cannot provide both :mysqlxReadTimeout and :mysqlx_read_timeout' if attributes.key?(:'mysqlxReadTimeout') && attributes.key?(:'mysqlx_read_timeout')

      self.mysqlx_read_timeout = attributes[:'mysqlx_read_timeout'] if attributes[:'mysqlx_read_timeout']

      self.mysqlx_wait_timeout = attributes[:'mysqlxWaitTimeout'] if attributes[:'mysqlxWaitTimeout']

      raise 'You cannot provide both :mysqlxWaitTimeout and :mysqlx_wait_timeout' if attributes.key?(:'mysqlxWaitTimeout') && attributes.key?(:'mysqlx_wait_timeout')

      self.mysqlx_wait_timeout = attributes[:'mysqlx_wait_timeout'] if attributes[:'mysqlx_wait_timeout']

      self.mysqlx_write_timeout = attributes[:'mysqlxWriteTimeout'] if attributes[:'mysqlxWriteTimeout']

      raise 'You cannot provide both :mysqlxWriteTimeout and :mysqlx_write_timeout' if attributes.key?(:'mysqlxWriteTimeout') && attributes.key?(:'mysqlx_write_timeout')

      self.mysqlx_write_timeout = attributes[:'mysqlx_write_timeout'] if attributes[:'mysqlx_write_timeout']

      self.net_read_timeout = attributes[:'netReadTimeout'] if attributes[:'netReadTimeout']

      raise 'You cannot provide both :netReadTimeout and :net_read_timeout' if attributes.key?(:'netReadTimeout') && attributes.key?(:'net_read_timeout')

      self.net_read_timeout = attributes[:'net_read_timeout'] if attributes[:'net_read_timeout']

      self.net_write_timeout = attributes[:'netWriteTimeout'] if attributes[:'netWriteTimeout']

      raise 'You cannot provide both :netWriteTimeout and :net_write_timeout' if attributes.key?(:'netWriteTimeout') && attributes.key?(:'net_write_timeout')

      self.net_write_timeout = attributes[:'net_write_timeout'] if attributes[:'net_write_timeout']

      self.parser_max_mem_size = attributes[:'parserMaxMemSize'] if attributes[:'parserMaxMemSize']

      raise 'You cannot provide both :parserMaxMemSize and :parser_max_mem_size' if attributes.key?(:'parserMaxMemSize') && attributes.key?(:'parser_max_mem_size')

      self.parser_max_mem_size = attributes[:'parser_max_mem_size'] if attributes[:'parser_max_mem_size']

      self.query_alloc_block_size = attributes[:'queryAllocBlockSize'] if attributes[:'queryAllocBlockSize']
      self.query_alloc_block_size = 8192 if query_alloc_block_size.nil? && !attributes.key?(:'queryAllocBlockSize') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :queryAllocBlockSize and :query_alloc_block_size' if attributes.key?(:'queryAllocBlockSize') && attributes.key?(:'query_alloc_block_size')

      self.query_alloc_block_size = attributes[:'query_alloc_block_size'] if attributes[:'query_alloc_block_size']
      self.query_alloc_block_size = 8192 if query_alloc_block_size.nil? && !attributes.key?(:'queryAllocBlockSize') && !attributes.key?(:'query_alloc_block_size') # rubocop:disable Style/StringLiterals

      self.query_prealloc_size = attributes[:'queryPreallocSize'] if attributes[:'queryPreallocSize']
      self.query_prealloc_size = 8192 if query_prealloc_size.nil? && !attributes.key?(:'queryPreallocSize') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :queryPreallocSize and :query_prealloc_size' if attributes.key?(:'queryPreallocSize') && attributes.key?(:'query_prealloc_size')

      self.query_prealloc_size = attributes[:'query_prealloc_size'] if attributes[:'query_prealloc_size']
      self.query_prealloc_size = 8192 if query_prealloc_size.nil? && !attributes.key?(:'queryPreallocSize') && !attributes.key?(:'query_prealloc_size') # rubocop:disable Style/StringLiterals

      self.regexp_time_limit = attributes[:'regexpTimeLimit'] if attributes[:'regexpTimeLimit']

      raise 'You cannot provide both :regexpTimeLimit and :regexp_time_limit' if attributes.key?(:'regexpTimeLimit') && attributes.key?(:'regexp_time_limit')

      self.regexp_time_limit = attributes[:'regexp_time_limit'] if attributes[:'regexp_time_limit']

      self.sql_mode = attributes[:'sqlMode'] if attributes[:'sqlMode']
      self.sql_mode = "ONLY_FULL_GROUP_BY,\nSTRICT_TRANS_TABLES,\nNO_ZERO_IN_DATE,\nNO_ZERO_DATE,\nERROR_FOR_DIVISION_BY_ZERO,\nNO_ENGINE_SUBSTITUTION\n" if sql_mode.nil? && !attributes.key?(:'sqlMode') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :sqlMode and :sql_mode' if attributes.key?(:'sqlMode') && attributes.key?(:'sql_mode')

      self.sql_mode = attributes[:'sql_mode'] if attributes[:'sql_mode']
      self.sql_mode = "ONLY_FULL_GROUP_BY,\nSTRICT_TRANS_TABLES,\nNO_ZERO_IN_DATE,\nNO_ZERO_DATE,\nERROR_FOR_DIVISION_BY_ZERO,\nNO_ENGINE_SUBSTITUTION\n" if sql_mode.nil? && !attributes.key?(:'sqlMode') && !attributes.key?(:'sql_mode') # rubocop:disable Style/StringLiterals

      self.tmp_table_size = attributes[:'tmpTableSize'] if attributes[:'tmpTableSize']
      self.tmp_table_size = 16777216 if tmp_table_size.nil? && !attributes.key?(:'tmpTableSize') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :tmpTableSize and :tmp_table_size' if attributes.key?(:'tmpTableSize') && attributes.key?(:'tmp_table_size')

      self.tmp_table_size = attributes[:'tmp_table_size'] if attributes[:'tmp_table_size']
      self.tmp_table_size = 16777216 if tmp_table_size.nil? && !attributes.key?(:'tmpTableSize') && !attributes.key?(:'tmp_table_size') # rubocop:disable Style/StringLiterals

      self.mysqlx_deflate_default_compression_level = attributes[:'mysqlxDeflateDefaultCompressionLevel'] if attributes[:'mysqlxDeflateDefaultCompressionLevel']

      raise 'You cannot provide both :mysqlxDeflateDefaultCompressionLevel and :mysqlx_deflate_default_compression_level' if attributes.key?(:'mysqlxDeflateDefaultCompressionLevel') && attributes.key?(:'mysqlx_deflate_default_compression_level')

      self.mysqlx_deflate_default_compression_level = attributes[:'mysqlx_deflate_default_compression_level'] if attributes[:'mysqlx_deflate_default_compression_level']

      self.mysqlx_deflate_max_client_compression_level = attributes[:'mysqlxDeflateMaxClientCompressionLevel'] if attributes[:'mysqlxDeflateMaxClientCompressionLevel']

      raise 'You cannot provide both :mysqlxDeflateMaxClientCompressionLevel and :mysqlx_deflate_max_client_compression_level' if attributes.key?(:'mysqlxDeflateMaxClientCompressionLevel') && attributes.key?(:'mysqlx_deflate_max_client_compression_level')

      self.mysqlx_deflate_max_client_compression_level = attributes[:'mysqlx_deflate_max_client_compression_level'] if attributes[:'mysqlx_deflate_max_client_compression_level']

      self.mysqlx_lz4_max_client_compression_level = attributes[:'mysqlxLz4MaxClientCompressionLevel'] if attributes[:'mysqlxLz4MaxClientCompressionLevel']

      raise 'You cannot provide both :mysqlxLz4MaxClientCompressionLevel and :mysqlx_lz4_max_client_compression_level' if attributes.key?(:'mysqlxLz4MaxClientCompressionLevel') && attributes.key?(:'mysqlx_lz4_max_client_compression_level')

      self.mysqlx_lz4_max_client_compression_level = attributes[:'mysqlx_lz4_max_client_compression_level'] if attributes[:'mysqlx_lz4_max_client_compression_level']

      self.mysqlx_lz4_default_compression_level = attributes[:'mysqlxLz4DefaultCompressionLevel'] if attributes[:'mysqlxLz4DefaultCompressionLevel']

      raise 'You cannot provide both :mysqlxLz4DefaultCompressionLevel and :mysqlx_lz4_default_compression_level' if attributes.key?(:'mysqlxLz4DefaultCompressionLevel') && attributes.key?(:'mysqlx_lz4_default_compression_level')

      self.mysqlx_lz4_default_compression_level = attributes[:'mysqlx_lz4_default_compression_level'] if attributes[:'mysqlx_lz4_default_compression_level']

      self.mysqlx_zstd_max_client_compression_level = attributes[:'mysqlxZstdMaxClientCompressionLevel'] if attributes[:'mysqlxZstdMaxClientCompressionLevel']

      raise 'You cannot provide both :mysqlxZstdMaxClientCompressionLevel and :mysqlx_zstd_max_client_compression_level' if attributes.key?(:'mysqlxZstdMaxClientCompressionLevel') && attributes.key?(:'mysqlx_zstd_max_client_compression_level')

      self.mysqlx_zstd_max_client_compression_level = attributes[:'mysqlx_zstd_max_client_compression_level'] if attributes[:'mysqlx_zstd_max_client_compression_level']

      self.mysqlx_zstd_default_compression_level = attributes[:'mysqlxZstdDefaultCompressionLevel'] if attributes[:'mysqlxZstdDefaultCompressionLevel']

      raise 'You cannot provide both :mysqlxZstdDefaultCompressionLevel and :mysqlx_zstd_default_compression_level' if attributes.key?(:'mysqlxZstdDefaultCompressionLevel') && attributes.key?(:'mysqlx_zstd_default_compression_level')

      self.mysqlx_zstd_default_compression_level = attributes[:'mysqlx_zstd_default_compression_level'] if attributes[:'mysqlx_zstd_default_compression_level']

      self.mysql_zstd_default_compression_level = attributes[:'mysqlZstdDefaultCompressionLevel'] if attributes[:'mysqlZstdDefaultCompressionLevel']

      raise 'You cannot provide both :mysqlZstdDefaultCompressionLevel and :mysql_zstd_default_compression_level' if attributes.key?(:'mysqlZstdDefaultCompressionLevel') && attributes.key?(:'mysql_zstd_default_compression_level')

      self.mysql_zstd_default_compression_level = attributes[:'mysql_zstd_default_compression_level'] if attributes[:'mysql_zstd_default_compression_level']

      self.sort_buffer_size = attributes[:'sortBufferSize'] if attributes[:'sortBufferSize']
      self.sort_buffer_size = 262144 if sort_buffer_size.nil? && !attributes.key?(:'sortBufferSize') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :sortBufferSize and :sort_buffer_size' if attributes.key?(:'sortBufferSize') && attributes.key?(:'sort_buffer_size')

      self.sort_buffer_size = attributes[:'sort_buffer_size'] if attributes[:'sort_buffer_size']
      self.sort_buffer_size = 262144 if sort_buffer_size.nil? && !attributes.key?(:'sortBufferSize') && !attributes.key?(:'sort_buffer_size') # rubocop:disable Style/StringLiterals

      self.wait_timeout = attributes[:'waitTimeout'] if attributes[:'waitTimeout']

      raise 'You cannot provide both :waitTimeout and :wait_timeout' if attributes.key?(:'waitTimeout') && attributes.key?(:'wait_timeout')

      self.wait_timeout = attributes[:'wait_timeout'] if attributes[:'wait_timeout']

      self.thread_pool_dedicated_listeners = attributes[:'threadPoolDedicatedListeners'] unless attributes[:'threadPoolDedicatedListeners'].nil?
      self.thread_pool_dedicated_listeners = false if thread_pool_dedicated_listeners.nil? && !attributes.key?(:'threadPoolDedicatedListeners') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :threadPoolDedicatedListeners and :thread_pool_dedicated_listeners' if attributes.key?(:'threadPoolDedicatedListeners') && attributes.key?(:'thread_pool_dedicated_listeners')

      self.thread_pool_dedicated_listeners = attributes[:'thread_pool_dedicated_listeners'] unless attributes[:'thread_pool_dedicated_listeners'].nil?
      self.thread_pool_dedicated_listeners = false if thread_pool_dedicated_listeners.nil? && !attributes.key?(:'threadPoolDedicatedListeners') && !attributes.key?(:'thread_pool_dedicated_listeners') # rubocop:disable Style/StringLiterals

      self.thread_pool_max_transactions_limit = attributes[:'threadPoolMaxTransactionsLimit'] if attributes[:'threadPoolMaxTransactionsLimit']

      raise 'You cannot provide both :threadPoolMaxTransactionsLimit and :thread_pool_max_transactions_limit' if attributes.key?(:'threadPoolMaxTransactionsLimit') && attributes.key?(:'thread_pool_max_transactions_limit')

      self.thread_pool_max_transactions_limit = attributes[:'thread_pool_max_transactions_limit'] if attributes[:'thread_pool_max_transactions_limit']

      self.time_zone = attributes[:'timeZone'] if attributes[:'timeZone']
      self.time_zone = "UTC" if time_zone.nil? && !attributes.key?(:'timeZone') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :timeZone and :time_zone' if attributes.key?(:'timeZone') && attributes.key?(:'time_zone')

      self.time_zone = attributes[:'time_zone'] if attributes[:'time_zone']
      self.time_zone = "UTC" if time_zone.nil? && !attributes.key?(:'timeZone') && !attributes.key?(:'time_zone') # rubocop:disable Style/StringLiterals
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] completion_type Object to be assigned
    def completion_type=(completion_type)
      # rubocop:disable Style/ConditionalAssignment
      if completion_type && !COMPLETION_TYPE_ENUM.include?(completion_type)
        OCI.logger.debug("Unknown value for 'completion_type' [" + completion_type + "]. Mapping to 'COMPLETION_TYPE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @completion_type = COMPLETION_TYPE_UNKNOWN_ENUM_VALUE
      else
        @completion_type = completion_type
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] default_authentication_plugin Object to be assigned
    def default_authentication_plugin=(default_authentication_plugin)
      # rubocop:disable Style/ConditionalAssignment
      if default_authentication_plugin && !DEFAULT_AUTHENTICATION_PLUGIN_ENUM.include?(default_authentication_plugin)
        OCI.logger.debug("Unknown value for 'default_authentication_plugin' [" + default_authentication_plugin + "]. Mapping to 'DEFAULT_AUTHENTICATION_PLUGIN_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @default_authentication_plugin = DEFAULT_AUTHENTICATION_PLUGIN_UNKNOWN_ENUM_VALUE
      else
        @default_authentication_plugin = default_authentication_plugin
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] transaction_isolation Object to be assigned
    def transaction_isolation=(transaction_isolation)
      # rubocop:disable Style/ConditionalAssignment
      if transaction_isolation && !TRANSACTION_ISOLATION_ENUM.include?(transaction_isolation)
        OCI.logger.debug("Unknown value for 'transaction_isolation' [" + transaction_isolation + "]. Mapping to 'TRANSACTION_ISOLATION_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @transaction_isolation = TRANSACTION_ISOLATION_UNKNOWN_ENUM_VALUE
      else
        @transaction_isolation = transaction_isolation
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] group_replication_consistency Object to be assigned
    def group_replication_consistency=(group_replication_consistency)
      # rubocop:disable Style/ConditionalAssignment
      if group_replication_consistency && !GROUP_REPLICATION_CONSISTENCY_ENUM.include?(group_replication_consistency)
        OCI.logger.debug("Unknown value for 'group_replication_consistency' [" + group_replication_consistency + "]. Mapping to 'GROUP_REPLICATION_CONSISTENCY_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @group_replication_consistency = GROUP_REPLICATION_CONSISTENCY_UNKNOWN_ENUM_VALUE
      else
        @group_replication_consistency = group_replication_consistency
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] binlog_row_metadata Object to be assigned
    def binlog_row_metadata=(binlog_row_metadata)
      # rubocop:disable Style/ConditionalAssignment
      if binlog_row_metadata && !BINLOG_ROW_METADATA_ENUM.include?(binlog_row_metadata)
        OCI.logger.debug("Unknown value for 'binlog_row_metadata' [" + binlog_row_metadata + "]. Mapping to 'BINLOG_ROW_METADATA_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @binlog_row_metadata = BINLOG_ROW_METADATA_UNKNOWN_ENUM_VALUE
      else
        @binlog_row_metadata = binlog_row_metadata
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        completion_type == other.completion_type &&
        big_tables == other.big_tables &&
        connection_memory_chunk_size == other.connection_memory_chunk_size &&
        connection_memory_limit == other.connection_memory_limit &&
        default_authentication_plugin == other.default_authentication_plugin &&
        global_connection_memory_limit == other.global_connection_memory_limit &&
        global_connection_memory_tracking == other.global_connection_memory_tracking &&
        transaction_isolation == other.transaction_isolation &&
        innodb_ft_server_stopword_table == other.innodb_ft_server_stopword_table &&
        mandatory_roles == other.mandatory_roles &&
        autocommit == other.autocommit &&
        foreign_key_checks == other.foreign_key_checks &&
        group_replication_consistency == other.group_replication_consistency &&
        innodb_ft_enable_stopword == other.innodb_ft_enable_stopword &&
        innodb_log_writer_threads == other.innodb_log_writer_threads &&
        local_infile == other.local_infile &&
        mysql_firewall_mode == other.mysql_firewall_mode &&
        mysqlx_enable_hello_notice == other.mysqlx_enable_hello_notice &&
        sql_require_primary_key == other.sql_require_primary_key &&
        sql_warnings == other.sql_warnings &&
        binlog_expire_logs_seconds == other.binlog_expire_logs_seconds &&
        binlog_row_metadata == other.binlog_row_metadata &&
        binlog_row_value_options == other.binlog_row_value_options &&
        binlog_transaction_compression == other.binlog_transaction_compression &&
        innodb_buffer_pool_size == other.innodb_buffer_pool_size &&
        innodb_ft_result_cache_limit == other.innodb_ft_result_cache_limit &&
        max_binlog_cache_size == other.max_binlog_cache_size &&
        max_connect_errors == other.max_connect_errors &&
        max_heap_table_size == other.max_heap_table_size &&
        max_connections == other.max_connections &&
        max_prepared_stmt_count == other.max_prepared_stmt_count &&
        connect_timeout == other.connect_timeout &&
        cte_max_recursion_depth == other.cte_max_recursion_depth &&
        generated_random_password_length == other.generated_random_password_length &&
        information_schema_stats_expiry == other.information_schema_stats_expiry &&
        innodb_buffer_pool_dump_pct == other.innodb_buffer_pool_dump_pct &&
        innodb_buffer_pool_instances == other.innodb_buffer_pool_instances &&
        innodb_ddl_buffer_size == other.innodb_ddl_buffer_size &&
        innodb_ddl_threads == other.innodb_ddl_threads &&
        innodb_ft_max_token_size == other.innodb_ft_max_token_size &&
        innodb_ft_min_token_size == other.innodb_ft_min_token_size &&
        innodb_ft_num_word_optimize == other.innodb_ft_num_word_optimize &&
        innodb_lock_wait_timeout == other.innodb_lock_wait_timeout &&
        innodb_max_purge_lag == other.innodb_max_purge_lag &&
        innodb_max_purge_lag_delay == other.innodb_max_purge_lag_delay &&
        interactive_timeout == other.interactive_timeout &&
        innodb_stats_persistent_sample_pages == other.innodb_stats_persistent_sample_pages &&
        innodb_stats_transient_sample_pages == other.innodb_stats_transient_sample_pages &&
        max_allowed_packet == other.max_allowed_packet &&
        max_execution_time == other.max_execution_time &&
        mysqlx_connect_timeout == other.mysqlx_connect_timeout &&
        mysqlx_document_id_unique_prefix == other.mysqlx_document_id_unique_prefix &&
        mysqlx_idle_worker_thread_timeout == other.mysqlx_idle_worker_thread_timeout &&
        mysqlx_interactive_timeout == other.mysqlx_interactive_timeout &&
        mysqlx_max_allowed_packet == other.mysqlx_max_allowed_packet &&
        mysqlx_min_worker_threads == other.mysqlx_min_worker_threads &&
        mysqlx_read_timeout == other.mysqlx_read_timeout &&
        mysqlx_wait_timeout == other.mysqlx_wait_timeout &&
        mysqlx_write_timeout == other.mysqlx_write_timeout &&
        net_read_timeout == other.net_read_timeout &&
        net_write_timeout == other.net_write_timeout &&
        parser_max_mem_size == other.parser_max_mem_size &&
        query_alloc_block_size == other.query_alloc_block_size &&
        query_prealloc_size == other.query_prealloc_size &&
        regexp_time_limit == other.regexp_time_limit &&
        sql_mode == other.sql_mode &&
        tmp_table_size == other.tmp_table_size &&
        mysqlx_deflate_default_compression_level == other.mysqlx_deflate_default_compression_level &&
        mysqlx_deflate_max_client_compression_level == other.mysqlx_deflate_max_client_compression_level &&
        mysqlx_lz4_max_client_compression_level == other.mysqlx_lz4_max_client_compression_level &&
        mysqlx_lz4_default_compression_level == other.mysqlx_lz4_default_compression_level &&
        mysqlx_zstd_max_client_compression_level == other.mysqlx_zstd_max_client_compression_level &&
        mysqlx_zstd_default_compression_level == other.mysqlx_zstd_default_compression_level &&
        mysql_zstd_default_compression_level == other.mysql_zstd_default_compression_level &&
        sort_buffer_size == other.sort_buffer_size &&
        wait_timeout == other.wait_timeout &&
        thread_pool_dedicated_listeners == other.thread_pool_dedicated_listeners &&
        thread_pool_max_transactions_limit == other.thread_pool_max_transactions_limit &&
        time_zone == other.time_zone
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [completion_type, big_tables, connection_memory_chunk_size, connection_memory_limit, default_authentication_plugin, global_connection_memory_limit, global_connection_memory_tracking, transaction_isolation, innodb_ft_server_stopword_table, mandatory_roles, autocommit, foreign_key_checks, group_replication_consistency, innodb_ft_enable_stopword, innodb_log_writer_threads, local_infile, mysql_firewall_mode, mysqlx_enable_hello_notice, sql_require_primary_key, sql_warnings, binlog_expire_logs_seconds, binlog_row_metadata, binlog_row_value_options, binlog_transaction_compression, innodb_buffer_pool_size, innodb_ft_result_cache_limit, max_binlog_cache_size, max_connect_errors, max_heap_table_size, max_connections, max_prepared_stmt_count, connect_timeout, cte_max_recursion_depth, generated_random_password_length, information_schema_stats_expiry, innodb_buffer_pool_dump_pct, innodb_buffer_pool_instances, innodb_ddl_buffer_size, innodb_ddl_threads, innodb_ft_max_token_size, innodb_ft_min_token_size, innodb_ft_num_word_optimize, innodb_lock_wait_timeout, innodb_max_purge_lag, innodb_max_purge_lag_delay, interactive_timeout, innodb_stats_persistent_sample_pages, innodb_stats_transient_sample_pages, max_allowed_packet, max_execution_time, mysqlx_connect_timeout, mysqlx_document_id_unique_prefix, mysqlx_idle_worker_thread_timeout, mysqlx_interactive_timeout, mysqlx_max_allowed_packet, mysqlx_min_worker_threads, mysqlx_read_timeout, mysqlx_wait_timeout, mysqlx_write_timeout, net_read_timeout, net_write_timeout, parser_max_mem_size, query_alloc_block_size, query_prealloc_size, regexp_time_limit, sql_mode, tmp_table_size, mysqlx_deflate_default_compression_level, mysqlx_deflate_max_client_compression_level, mysqlx_lz4_max_client_compression_level, mysqlx_lz4_default_compression_level, mysqlx_zstd_max_client_compression_level, mysqlx_zstd_default_compression_level, mysql_zstd_default_compression_level, sort_buffer_size, wait_timeout, thread_pool_dedicated_listeners, thread_pool_max_transactions_limit, time_zone].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
