# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20190415
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Details about the HeatWave cluster properties to be updated.
  #
  class Mysql::Models::UpdateHeatWaveClusterDetails
    # A change to the shape of the nodes in the HeatWave cluster will
    # result in the entire cluster being torn down and re-created with
    # Compute instances of the new Shape. This may result in significant
    # downtime for the analytics capability while the HeatWave cluster is
    # re-provisioned.
    #
    # @return [String]
    attr_accessor :shape_name

    # A change to the number of nodes in the HeatWave cluster will result
    # in the entire cluster being torn down and re-created with the new
    # cluster of nodes. This may result in a significant downtime for the
    # analytics capability while the HeatWave cluster is
    # re-provisioned.
    #
    # @return [Integer]
    attr_accessor :cluster_size

    # Enable/disable Lakehouse for the HeatWave cluster.
    # @return [BOOLEAN]
    attr_accessor :is_lakehouse_enabled

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'shape_name': :'shapeName',
        'cluster_size': :'clusterSize',
        'is_lakehouse_enabled': :'isLakehouseEnabled'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'shape_name': :'String',
        'cluster_size': :'Integer',
        'is_lakehouse_enabled': :'BOOLEAN'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :shape_name The value to assign to the {#shape_name} property
    # @option attributes [Integer] :cluster_size The value to assign to the {#cluster_size} property
    # @option attributes [BOOLEAN] :is_lakehouse_enabled The value to assign to the {#is_lakehouse_enabled} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.shape_name = attributes[:'shapeName'] if attributes[:'shapeName']

      raise 'You cannot provide both :shapeName and :shape_name' if attributes.key?(:'shapeName') && attributes.key?(:'shape_name')

      self.shape_name = attributes[:'shape_name'] if attributes[:'shape_name']

      self.cluster_size = attributes[:'clusterSize'] if attributes[:'clusterSize']

      raise 'You cannot provide both :clusterSize and :cluster_size' if attributes.key?(:'clusterSize') && attributes.key?(:'cluster_size')

      self.cluster_size = attributes[:'cluster_size'] if attributes[:'cluster_size']

      self.is_lakehouse_enabled = attributes[:'isLakehouseEnabled'] unless attributes[:'isLakehouseEnabled'].nil?
      self.is_lakehouse_enabled = false if is_lakehouse_enabled.nil? && !attributes.key?(:'isLakehouseEnabled') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :isLakehouseEnabled and :is_lakehouse_enabled' if attributes.key?(:'isLakehouseEnabled') && attributes.key?(:'is_lakehouse_enabled')

      self.is_lakehouse_enabled = attributes[:'is_lakehouse_enabled'] unless attributes[:'is_lakehouse_enabled'].nil?
      self.is_lakehouse_enabled = false if is_lakehouse_enabled.nil? && !attributes.key?(:'isLakehouseEnabled') && !attributes.key?(:'is_lakehouse_enabled') # rubocop:disable Style/StringLiterals
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        shape_name == other.shape_name &&
        cluster_size == other.cluster_size &&
        is_lakehouse_enabled == other.is_lakehouse_enabled
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [shape_name, cluster_size, is_lakehouse_enabled].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
