# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20230501
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Request for updating Decryption Rule used in the firewall policy rules.
  # A Decryption Rule is used to define which traffic should be decrypted by the firewall, and how it should do so.
  #
  class NetworkFirewall::Models::UpdateDecryptionRuleDetails
    ACTION_ENUM = [
      ACTION_NO_DECRYPT = 'NO_DECRYPT'.freeze,
      ACTION_DECRYPT = 'DECRYPT'.freeze
    ].freeze

    # This attribute is required.
    # @return [OCI::NetworkFirewall::Models::DecryptionRuleMatchCriteria]
    attr_accessor :condition

    # **[Required]** Action:
    #
    # * NO_DECRYPT - Matching traffic is not decrypted.
    # * DECRYPT - Matching traffic is decrypted with the specified `secret` according to the specified `decryptionProfile`.
    #
    # @return [String]
    attr_reader :action

    # The name of the decryption profile to use.
    # @return [String]
    attr_accessor :decryption_profile

    # The name of a mapped secret. Its `type` must match that of the specified decryption profile.
    # @return [String]
    attr_accessor :secret

    # @return [OCI::NetworkFirewall::Models::RulePosition]
    attr_accessor :position

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'condition': :'condition',
        'action': :'action',
        'decryption_profile': :'decryptionProfile',
        'secret': :'secret',
        'position': :'position'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'condition': :'OCI::NetworkFirewall::Models::DecryptionRuleMatchCriteria',
        'action': :'String',
        'decryption_profile': :'String',
        'secret': :'String',
        'position': :'OCI::NetworkFirewall::Models::RulePosition'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [OCI::NetworkFirewall::Models::DecryptionRuleMatchCriteria] :condition The value to assign to the {#condition} property
    # @option attributes [String] :action The value to assign to the {#action} property
    # @option attributes [String] :decryption_profile The value to assign to the {#decryption_profile} property
    # @option attributes [String] :secret The value to assign to the {#secret} property
    # @option attributes [OCI::NetworkFirewall::Models::RulePosition] :position The value to assign to the {#position} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.condition = attributes[:'condition'] if attributes[:'condition']

      self.action = attributes[:'action'] if attributes[:'action']

      self.decryption_profile = attributes[:'decryptionProfile'] if attributes[:'decryptionProfile']

      raise 'You cannot provide both :decryptionProfile and :decryption_profile' if attributes.key?(:'decryptionProfile') && attributes.key?(:'decryption_profile')

      self.decryption_profile = attributes[:'decryption_profile'] if attributes[:'decryption_profile']

      self.secret = attributes[:'secret'] if attributes[:'secret']

      self.position = attributes[:'position'] if attributes[:'position']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] action Object to be assigned
    def action=(action)
      raise "Invalid value for 'action': this must be one of the values in ACTION_ENUM." if action && !ACTION_ENUM.include?(action)

      @action = action
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        condition == other.condition &&
        action == other.action &&
        decryption_profile == other.decryption_profile &&
        secret == other.secret &&
        position == other.position
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [condition, action, decryption_profile, secret, position].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
