# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20230701
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Details of the ESXi host to add to the Cluster.
  class Ocvp::Models::CreateEsxiHostDetails
    CURRENT_COMMITMENT_ENUM = [
      CURRENT_COMMITMENT_HOUR = 'HOUR'.freeze,
      CURRENT_COMMITMENT_MONTH = 'MONTH'.freeze,
      CURRENT_COMMITMENT_ONE_YEAR = 'ONE_YEAR'.freeze,
      CURRENT_COMMITMENT_THREE_YEARS = 'THREE_YEARS'.freeze
    ].freeze

    NEXT_COMMITMENT_ENUM = [
      NEXT_COMMITMENT_HOUR = 'HOUR'.freeze,
      NEXT_COMMITMENT_MONTH = 'MONTH'.freeze,
      NEXT_COMMITMENT_ONE_YEAR = 'ONE_YEAR'.freeze,
      NEXT_COMMITMENT_THREE_YEARS = 'THREE_YEARS'.freeze
    ].freeze

    # **[Required]** The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the Cluster to add the ESXi host to.
    #
    # @return [String]
    attr_accessor :cluster_id

    # A descriptive name for the ESXi host. It's changeable.
    # Esxi Host name requirements are 1-16 character length limit, Must start with a letter,
    # Must be English letters, numbers, - only, No repeating hyphens, Must be unique within the Cluster.
    #
    # If this attribute is not specified, the Cluster's `instanceDisplayNamePrefix` attribute is used
    # to name and incrementally number the ESXi host. For example, if you're creating the fourth
    # ESXi host in the Cluster, and `instanceDisplayNamePrefix` is `MyCluster`, the host's display
    # name is `MyCluster-4`.
    #
    # Avoid entering confidential information.
    #
    # @return [String]
    attr_accessor :display_name

    # The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the deleted ESXi Host with LeftOver billing cycle.
    #
    # @return [String]
    attr_accessor :billing_donor_host_id

    # The billing option currently used by the ESXi host.
    # {#list_supported_commitments list_supported_commitments}.
    #
    # @return [String]
    attr_reader :current_commitment

    # The billing option to switch to after the existing billing cycle ends.
    # If `nextCommitment` is null or empty, `currentCommitment` continues to the next billing cycle.
    # {#list_supported_commitments list_supported_commitments}.
    #
    # @return [String]
    attr_reader :next_commitment

    # The availability domain to create the ESXi host in.
    # If keep empty, for AD-specific Cluster, new ESXi host will be created in the same availability domain;
    # for multi-AD Cluster, new ESXi host will be auto assigned to the next availability domain following evenly distribution strategy.
    #
    # @return [String]
    attr_accessor :compute_availability_domain

    # The compute shape name of the ESXi host.
    # {#list_supported_host_shapes list_supported_host_shapes}.
    #
    # @return [String]
    attr_accessor :host_shape_name

    # The OCPU count of the ESXi host.
    #
    # @return [Float]
    attr_accessor :host_ocpu_count

    # The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the Capacity Reservation.
    #
    # @return [String]
    attr_accessor :capacity_reservation_id

    # The ESXi software bundle to install on the ESXi host.
    # Only versions under the same vmwareSoftwareVersion and have been validate by Oracle Cloud VMware Solution will be accepted.
    # To get a list of the available versions, use
    # {#list_supported_vmware_software_versions list_supported_vmware_software_versions}.
    #
    # @return [String]
    attr_accessor :esxi_software_version

    # Free-form tags for this resource. Each tag is a simple key-value pair with no
    # predefined name, type, or namespace. For more information, see [Resource Tags](https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm).
    #
    # Example: `{\"Department\": \"Finance\"}`
    #
    # @return [Hash<String, String>]
    attr_accessor :freeform_tags

    # Defined tags for this resource. Each key is predefined and scoped to a
    # namespace. For more information, see [Resource Tags](https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm).
    #
    # Example: `{\"Operations\": {\"CostCenter\": \"42\"}}`
    #
    # @return [Hash<String, Hash<String, Object>>]
    attr_accessor :defined_tags

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'cluster_id': :'clusterId',
        'display_name': :'displayName',
        'billing_donor_host_id': :'billingDonorHostId',
        'current_commitment': :'currentCommitment',
        'next_commitment': :'nextCommitment',
        'compute_availability_domain': :'computeAvailabilityDomain',
        'host_shape_name': :'hostShapeName',
        'host_ocpu_count': :'hostOcpuCount',
        'capacity_reservation_id': :'capacityReservationId',
        'esxi_software_version': :'esxiSoftwareVersion',
        'freeform_tags': :'freeformTags',
        'defined_tags': :'definedTags'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'cluster_id': :'String',
        'display_name': :'String',
        'billing_donor_host_id': :'String',
        'current_commitment': :'String',
        'next_commitment': :'String',
        'compute_availability_domain': :'String',
        'host_shape_name': :'String',
        'host_ocpu_count': :'Float',
        'capacity_reservation_id': :'String',
        'esxi_software_version': :'String',
        'freeform_tags': :'Hash<String, String>',
        'defined_tags': :'Hash<String, Hash<String, Object>>'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :cluster_id The value to assign to the {#cluster_id} property
    # @option attributes [String] :display_name The value to assign to the {#display_name} property
    # @option attributes [String] :billing_donor_host_id The value to assign to the {#billing_donor_host_id} property
    # @option attributes [String] :current_commitment The value to assign to the {#current_commitment} property
    # @option attributes [String] :next_commitment The value to assign to the {#next_commitment} property
    # @option attributes [String] :compute_availability_domain The value to assign to the {#compute_availability_domain} property
    # @option attributes [String] :host_shape_name The value to assign to the {#host_shape_name} property
    # @option attributes [Float] :host_ocpu_count The value to assign to the {#host_ocpu_count} property
    # @option attributes [String] :capacity_reservation_id The value to assign to the {#capacity_reservation_id} property
    # @option attributes [String] :esxi_software_version The value to assign to the {#esxi_software_version} property
    # @option attributes [Hash<String, String>] :freeform_tags The value to assign to the {#freeform_tags} property
    # @option attributes [Hash<String, Hash<String, Object>>] :defined_tags The value to assign to the {#defined_tags} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.cluster_id = attributes[:'clusterId'] if attributes[:'clusterId']

      raise 'You cannot provide both :clusterId and :cluster_id' if attributes.key?(:'clusterId') && attributes.key?(:'cluster_id')

      self.cluster_id = attributes[:'cluster_id'] if attributes[:'cluster_id']

      self.display_name = attributes[:'displayName'] if attributes[:'displayName']

      raise 'You cannot provide both :displayName and :display_name' if attributes.key?(:'displayName') && attributes.key?(:'display_name')

      self.display_name = attributes[:'display_name'] if attributes[:'display_name']

      self.billing_donor_host_id = attributes[:'billingDonorHostId'] if attributes[:'billingDonorHostId']

      raise 'You cannot provide both :billingDonorHostId and :billing_donor_host_id' if attributes.key?(:'billingDonorHostId') && attributes.key?(:'billing_donor_host_id')

      self.billing_donor_host_id = attributes[:'billing_donor_host_id'] if attributes[:'billing_donor_host_id']

      self.current_commitment = attributes[:'currentCommitment'] if attributes[:'currentCommitment']
      self.current_commitment = "HOUR" if current_commitment.nil? && !attributes.key?(:'currentCommitment') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :currentCommitment and :current_commitment' if attributes.key?(:'currentCommitment') && attributes.key?(:'current_commitment')

      self.current_commitment = attributes[:'current_commitment'] if attributes[:'current_commitment']
      self.current_commitment = "HOUR" if current_commitment.nil? && !attributes.key?(:'currentCommitment') && !attributes.key?(:'current_commitment') # rubocop:disable Style/StringLiterals

      self.next_commitment = attributes[:'nextCommitment'] if attributes[:'nextCommitment']
      self.next_commitment = "HOUR" if next_commitment.nil? && !attributes.key?(:'nextCommitment') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :nextCommitment and :next_commitment' if attributes.key?(:'nextCommitment') && attributes.key?(:'next_commitment')

      self.next_commitment = attributes[:'next_commitment'] if attributes[:'next_commitment']
      self.next_commitment = "HOUR" if next_commitment.nil? && !attributes.key?(:'nextCommitment') && !attributes.key?(:'next_commitment') # rubocop:disable Style/StringLiterals

      self.compute_availability_domain = attributes[:'computeAvailabilityDomain'] if attributes[:'computeAvailabilityDomain']

      raise 'You cannot provide both :computeAvailabilityDomain and :compute_availability_domain' if attributes.key?(:'computeAvailabilityDomain') && attributes.key?(:'compute_availability_domain')

      self.compute_availability_domain = attributes[:'compute_availability_domain'] if attributes[:'compute_availability_domain']

      self.host_shape_name = attributes[:'hostShapeName'] if attributes[:'hostShapeName']

      raise 'You cannot provide both :hostShapeName and :host_shape_name' if attributes.key?(:'hostShapeName') && attributes.key?(:'host_shape_name')

      self.host_shape_name = attributes[:'host_shape_name'] if attributes[:'host_shape_name']

      self.host_ocpu_count = attributes[:'hostOcpuCount'] if attributes[:'hostOcpuCount']

      raise 'You cannot provide both :hostOcpuCount and :host_ocpu_count' if attributes.key?(:'hostOcpuCount') && attributes.key?(:'host_ocpu_count')

      self.host_ocpu_count = attributes[:'host_ocpu_count'] if attributes[:'host_ocpu_count']

      self.capacity_reservation_id = attributes[:'capacityReservationId'] if attributes[:'capacityReservationId']

      raise 'You cannot provide both :capacityReservationId and :capacity_reservation_id' if attributes.key?(:'capacityReservationId') && attributes.key?(:'capacity_reservation_id')

      self.capacity_reservation_id = attributes[:'capacity_reservation_id'] if attributes[:'capacity_reservation_id']

      self.esxi_software_version = attributes[:'esxiSoftwareVersion'] if attributes[:'esxiSoftwareVersion']

      raise 'You cannot provide both :esxiSoftwareVersion and :esxi_software_version' if attributes.key?(:'esxiSoftwareVersion') && attributes.key?(:'esxi_software_version')

      self.esxi_software_version = attributes[:'esxi_software_version'] if attributes[:'esxi_software_version']

      self.freeform_tags = attributes[:'freeformTags'] if attributes[:'freeformTags']

      raise 'You cannot provide both :freeformTags and :freeform_tags' if attributes.key?(:'freeformTags') && attributes.key?(:'freeform_tags')

      self.freeform_tags = attributes[:'freeform_tags'] if attributes[:'freeform_tags']

      self.defined_tags = attributes[:'definedTags'] if attributes[:'definedTags']

      raise 'You cannot provide both :definedTags and :defined_tags' if attributes.key?(:'definedTags') && attributes.key?(:'defined_tags')

      self.defined_tags = attributes[:'defined_tags'] if attributes[:'defined_tags']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] current_commitment Object to be assigned
    def current_commitment=(current_commitment)
      raise "Invalid value for 'current_commitment': this must be one of the values in CURRENT_COMMITMENT_ENUM." if current_commitment && !CURRENT_COMMITMENT_ENUM.include?(current_commitment)

      @current_commitment = current_commitment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] next_commitment Object to be assigned
    def next_commitment=(next_commitment)
      raise "Invalid value for 'next_commitment': this must be one of the values in NEXT_COMMITMENT_ENUM." if next_commitment && !NEXT_COMMITMENT_ENUM.include?(next_commitment)

      @next_commitment = next_commitment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        cluster_id == other.cluster_id &&
        display_name == other.display_name &&
        billing_donor_host_id == other.billing_donor_host_id &&
        current_commitment == other.current_commitment &&
        next_commitment == other.next_commitment &&
        compute_availability_domain == other.compute_availability_domain &&
        host_shape_name == other.host_shape_name &&
        host_ocpu_count == other.host_ocpu_count &&
        capacity_reservation_id == other.capacity_reservation_id &&
        esxi_software_version == other.esxi_software_version &&
        freeform_tags == other.freeform_tags &&
        defined_tags == other.defined_tags
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [cluster_id, display_name, billing_donor_host_id, current_commitment, next_commitment, compute_availability_domain, host_shape_name, host_ocpu_count, capacity_reservation_id, esxi_software_version, freeform_tags, defined_tags].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
