# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20230701
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # A summary of the ESXi host.
  class Ocvp::Models::EsxiHostSummary
    LIFECYCLE_STATE_ENUM = [
      LIFECYCLE_STATE_CREATING = 'CREATING'.freeze,
      LIFECYCLE_STATE_UPDATING = 'UPDATING'.freeze,
      LIFECYCLE_STATE_ACTIVE = 'ACTIVE'.freeze,
      LIFECYCLE_STATE_DELETING = 'DELETING'.freeze,
      LIFECYCLE_STATE_DELETED = 'DELETED'.freeze,
      LIFECYCLE_STATE_FAILED = 'FAILED'.freeze,
      LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    CURRENT_COMMITMENT_ENUM = [
      CURRENT_COMMITMENT_HOUR = 'HOUR'.freeze,
      CURRENT_COMMITMENT_MONTH = 'MONTH'.freeze,
      CURRENT_COMMITMENT_ONE_YEAR = 'ONE_YEAR'.freeze,
      CURRENT_COMMITMENT_THREE_YEARS = 'THREE_YEARS'.freeze,
      CURRENT_COMMITMENT_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    NEXT_COMMITMENT_ENUM = [
      NEXT_COMMITMENT_HOUR = 'HOUR'.freeze,
      NEXT_COMMITMENT_MONTH = 'MONTH'.freeze,
      NEXT_COMMITMENT_ONE_YEAR = 'ONE_YEAR'.freeze,
      NEXT_COMMITMENT_THREE_YEARS = 'THREE_YEARS'.freeze,
      NEXT_COMMITMENT_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the ESXi host.
    #
    # @return [String]
    attr_accessor :id

    # A descriptive name for the ESXi host. Does not have to be unique, and it's changeable.
    # Avoid entering confidential information.
    #
    # @return [String]
    attr_accessor :display_name

    # **[Required]** The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the SDDC that the
    # ESXi host belongs to.
    #
    # @return [String]
    attr_accessor :sddc_id

    # **[Required]** The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the Cluster that the
    # ESXi host belongs to.
    #
    # @return [String]
    attr_accessor :cluster_id

    # The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the compartment that
    # contains the Cluster.
    #
    # @return [String]
    attr_accessor :compartment_id

    # In terms of implementation, an ESXi host is a Compute instance that
    # is configured with the chosen bundle of VMware software. The `computeInstanceId`
    # is the [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of that Compute instance.
    #
    # @return [String]
    attr_accessor :compute_instance_id

    # The date and time the ESXi host was created, in the format defined by
    # [RFC3339](https://tools.ietf.org/html/rfc3339).
    #
    # Example: `2016-08-25T21:10:29.600Z`
    #
    # @return [DateTime]
    attr_accessor :time_created

    # The date and time the ESXi host was updated, in the format defined by
    # [RFC3339](https://tools.ietf.org/html/rfc3339).
    #
    # @return [DateTime]
    attr_accessor :time_updated

    # The current state of the ESXi host.
    # @return [String]
    attr_reader :lifecycle_state

    # **[Required]** The billing option currently used by the ESXi host.
    # {#list_supported_commitments list_supported_commitments}.
    #
    # @return [String]
    attr_reader :current_commitment

    # **[Required]** The billing option to switch to after the current billing cycle ends.
    # If `nextCommitment` is null or empty, `currentCommitment` continues to the next billing cycle.
    # {#list_supported_commitments list_supported_commitments}.
    #
    # @return [String]
    attr_reader :next_commitment

    # **[Required]** Current billing cycle end date. If the value in `currentCommitment` and `nextCommitment` are different, the value specified in `nextCommitment`
    # becomes the new `currentCommitment` when the `contractEndDate` is reached.
    # Example: `2016-08-25T21:10:29.600Z`
    #
    # @return [DateTime]
    attr_accessor :billing_contract_end_date

    # The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the ESXi host that failed.
    #
    # @return [String]
    attr_accessor :failed_esxi_host_id

    # The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the ESXi host that
    # is newly created to replace the failed host.
    #
    # @return [String]
    attr_accessor :replacement_esxi_host_id

    # The date and time when the new esxi host should start billing cycle.
    # [RFC3339](https://tools.ietf.org/html/rfc3339).
    # Example: `2021-07-25T21:10:29.600Z`
    #
    # @return [DateTime]
    attr_accessor :grace_period_end_date

    # **[Required]** The version of VMware software that Oracle Cloud VMware Solution installed on the ESXi hosts.
    #
    # @return [String]
    attr_accessor :vmware_software_version

    # The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the ESXi host that
    # will be upgraded.
    #
    # @return [String]
    attr_accessor :non_upgraded_esxi_host_id

    # The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the ESXi host that
    # is newly created to upgrade the original host.
    #
    # @return [String]
    attr_accessor :upgraded_replacement_esxi_host_id

    # **[Required]** The availability domain of the ESXi host.
    #
    # @return [String]
    attr_accessor :compute_availability_domain

    # **[Required]** The compute shape name of the ESXi host.
    # {#list_supported_host_shapes list_supported_host_shapes}.
    #
    # @return [String]
    attr_accessor :host_shape_name

    # The OCPU count of the ESXi host.
    #
    # @return [Float]
    attr_accessor :host_ocpu_count

    # The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the deleted ESXi Host with LeftOver billing cycle.
    #
    # @return [String]
    attr_accessor :billing_donor_host_id

    # The [OCID](https://docs.cloud.oracle.com/Content/General/Concepts/identifiers.htm) of the active ESXi Host to swap billing with current host.
    #
    # @return [String]
    attr_accessor :swap_billing_host_id

    # Indicates whether this host is in the progress of billing continuation.
    #
    # @return [BOOLEAN]
    attr_accessor :is_billing_continuation_in_progress

    # Indicates whether this host is in the progress of swapping billing.
    #
    # @return [BOOLEAN]
    attr_accessor :is_billing_swapping_in_progress

    # **[Required]** Free-form tags for this resource. Each tag is a simple key-value pair with no
    # predefined name, type, or namespace. For more information, see [Resource Tags](https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm).
    #
    # Example: `{\"Department\": \"Finance\"}`
    #
    # @return [Hash<String, String>]
    attr_accessor :freeform_tags

    # **[Required]** Defined tags for this resource. Each key is predefined and scoped to a
    # namespace. For more information, see [Resource Tags](https://docs.cloud.oracle.com/Content/General/Concepts/resourcetags.htm).
    #
    # Example: `{\"Operations\": {\"CostCenter\": \"42\"}}`
    #
    # @return [Hash<String, Hash<String, Object>>]
    attr_accessor :defined_tags

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'id',
        'display_name': :'displayName',
        'sddc_id': :'sddcId',
        'cluster_id': :'clusterId',
        'compartment_id': :'compartmentId',
        'compute_instance_id': :'computeInstanceId',
        'time_created': :'timeCreated',
        'time_updated': :'timeUpdated',
        'lifecycle_state': :'lifecycleState',
        'current_commitment': :'currentCommitment',
        'next_commitment': :'nextCommitment',
        'billing_contract_end_date': :'billingContractEndDate',
        'failed_esxi_host_id': :'failedEsxiHostId',
        'replacement_esxi_host_id': :'replacementEsxiHostId',
        'grace_period_end_date': :'gracePeriodEndDate',
        'vmware_software_version': :'vmwareSoftwareVersion',
        'non_upgraded_esxi_host_id': :'nonUpgradedEsxiHostId',
        'upgraded_replacement_esxi_host_id': :'upgradedReplacementEsxiHostId',
        'compute_availability_domain': :'computeAvailabilityDomain',
        'host_shape_name': :'hostShapeName',
        'host_ocpu_count': :'hostOcpuCount',
        'billing_donor_host_id': :'billingDonorHostId',
        'swap_billing_host_id': :'swapBillingHostId',
        'is_billing_continuation_in_progress': :'isBillingContinuationInProgress',
        'is_billing_swapping_in_progress': :'isBillingSwappingInProgress',
        'freeform_tags': :'freeformTags',
        'defined_tags': :'definedTags'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'String',
        'display_name': :'String',
        'sddc_id': :'String',
        'cluster_id': :'String',
        'compartment_id': :'String',
        'compute_instance_id': :'String',
        'time_created': :'DateTime',
        'time_updated': :'DateTime',
        'lifecycle_state': :'String',
        'current_commitment': :'String',
        'next_commitment': :'String',
        'billing_contract_end_date': :'DateTime',
        'failed_esxi_host_id': :'String',
        'replacement_esxi_host_id': :'String',
        'grace_period_end_date': :'DateTime',
        'vmware_software_version': :'String',
        'non_upgraded_esxi_host_id': :'String',
        'upgraded_replacement_esxi_host_id': :'String',
        'compute_availability_domain': :'String',
        'host_shape_name': :'String',
        'host_ocpu_count': :'Float',
        'billing_donor_host_id': :'String',
        'swap_billing_host_id': :'String',
        'is_billing_continuation_in_progress': :'BOOLEAN',
        'is_billing_swapping_in_progress': :'BOOLEAN',
        'freeform_tags': :'Hash<String, String>',
        'defined_tags': :'Hash<String, Hash<String, Object>>'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :id The value to assign to the {#id} property
    # @option attributes [String] :display_name The value to assign to the {#display_name} property
    # @option attributes [String] :sddc_id The value to assign to the {#sddc_id} property
    # @option attributes [String] :cluster_id The value to assign to the {#cluster_id} property
    # @option attributes [String] :compartment_id The value to assign to the {#compartment_id} property
    # @option attributes [String] :compute_instance_id The value to assign to the {#compute_instance_id} property
    # @option attributes [DateTime] :time_created The value to assign to the {#time_created} property
    # @option attributes [DateTime] :time_updated The value to assign to the {#time_updated} property
    # @option attributes [String] :lifecycle_state The value to assign to the {#lifecycle_state} property
    # @option attributes [String] :current_commitment The value to assign to the {#current_commitment} property
    # @option attributes [String] :next_commitment The value to assign to the {#next_commitment} property
    # @option attributes [DateTime] :billing_contract_end_date The value to assign to the {#billing_contract_end_date} property
    # @option attributes [String] :failed_esxi_host_id The value to assign to the {#failed_esxi_host_id} property
    # @option attributes [String] :replacement_esxi_host_id The value to assign to the {#replacement_esxi_host_id} property
    # @option attributes [DateTime] :grace_period_end_date The value to assign to the {#grace_period_end_date} property
    # @option attributes [String] :vmware_software_version The value to assign to the {#vmware_software_version} property
    # @option attributes [String] :non_upgraded_esxi_host_id The value to assign to the {#non_upgraded_esxi_host_id} property
    # @option attributes [String] :upgraded_replacement_esxi_host_id The value to assign to the {#upgraded_replacement_esxi_host_id} property
    # @option attributes [String] :compute_availability_domain The value to assign to the {#compute_availability_domain} property
    # @option attributes [String] :host_shape_name The value to assign to the {#host_shape_name} property
    # @option attributes [Float] :host_ocpu_count The value to assign to the {#host_ocpu_count} property
    # @option attributes [String] :billing_donor_host_id The value to assign to the {#billing_donor_host_id} property
    # @option attributes [String] :swap_billing_host_id The value to assign to the {#swap_billing_host_id} property
    # @option attributes [BOOLEAN] :is_billing_continuation_in_progress The value to assign to the {#is_billing_continuation_in_progress} property
    # @option attributes [BOOLEAN] :is_billing_swapping_in_progress The value to assign to the {#is_billing_swapping_in_progress} property
    # @option attributes [Hash<String, String>] :freeform_tags The value to assign to the {#freeform_tags} property
    # @option attributes [Hash<String, Hash<String, Object>>] :defined_tags The value to assign to the {#defined_tags} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.id = attributes[:'id'] if attributes[:'id']

      self.display_name = attributes[:'displayName'] if attributes[:'displayName']

      raise 'You cannot provide both :displayName and :display_name' if attributes.key?(:'displayName') && attributes.key?(:'display_name')

      self.display_name = attributes[:'display_name'] if attributes[:'display_name']

      self.sddc_id = attributes[:'sddcId'] if attributes[:'sddcId']

      raise 'You cannot provide both :sddcId and :sddc_id' if attributes.key?(:'sddcId') && attributes.key?(:'sddc_id')

      self.sddc_id = attributes[:'sddc_id'] if attributes[:'sddc_id']

      self.cluster_id = attributes[:'clusterId'] if attributes[:'clusterId']

      raise 'You cannot provide both :clusterId and :cluster_id' if attributes.key?(:'clusterId') && attributes.key?(:'cluster_id')

      self.cluster_id = attributes[:'cluster_id'] if attributes[:'cluster_id']

      self.compartment_id = attributes[:'compartmentId'] if attributes[:'compartmentId']

      raise 'You cannot provide both :compartmentId and :compartment_id' if attributes.key?(:'compartmentId') && attributes.key?(:'compartment_id')

      self.compartment_id = attributes[:'compartment_id'] if attributes[:'compartment_id']

      self.compute_instance_id = attributes[:'computeInstanceId'] if attributes[:'computeInstanceId']

      raise 'You cannot provide both :computeInstanceId and :compute_instance_id' if attributes.key?(:'computeInstanceId') && attributes.key?(:'compute_instance_id')

      self.compute_instance_id = attributes[:'compute_instance_id'] if attributes[:'compute_instance_id']

      self.time_created = attributes[:'timeCreated'] if attributes[:'timeCreated']

      raise 'You cannot provide both :timeCreated and :time_created' if attributes.key?(:'timeCreated') && attributes.key?(:'time_created')

      self.time_created = attributes[:'time_created'] if attributes[:'time_created']

      self.time_updated = attributes[:'timeUpdated'] if attributes[:'timeUpdated']

      raise 'You cannot provide both :timeUpdated and :time_updated' if attributes.key?(:'timeUpdated') && attributes.key?(:'time_updated')

      self.time_updated = attributes[:'time_updated'] if attributes[:'time_updated']

      self.lifecycle_state = attributes[:'lifecycleState'] if attributes[:'lifecycleState']

      raise 'You cannot provide both :lifecycleState and :lifecycle_state' if attributes.key?(:'lifecycleState') && attributes.key?(:'lifecycle_state')

      self.lifecycle_state = attributes[:'lifecycle_state'] if attributes[:'lifecycle_state']

      self.current_commitment = attributes[:'currentCommitment'] if attributes[:'currentCommitment']

      raise 'You cannot provide both :currentCommitment and :current_commitment' if attributes.key?(:'currentCommitment') && attributes.key?(:'current_commitment')

      self.current_commitment = attributes[:'current_commitment'] if attributes[:'current_commitment']

      self.next_commitment = attributes[:'nextCommitment'] if attributes[:'nextCommitment']

      raise 'You cannot provide both :nextCommitment and :next_commitment' if attributes.key?(:'nextCommitment') && attributes.key?(:'next_commitment')

      self.next_commitment = attributes[:'next_commitment'] if attributes[:'next_commitment']

      self.billing_contract_end_date = attributes[:'billingContractEndDate'] if attributes[:'billingContractEndDate']

      raise 'You cannot provide both :billingContractEndDate and :billing_contract_end_date' if attributes.key?(:'billingContractEndDate') && attributes.key?(:'billing_contract_end_date')

      self.billing_contract_end_date = attributes[:'billing_contract_end_date'] if attributes[:'billing_contract_end_date']

      self.failed_esxi_host_id = attributes[:'failedEsxiHostId'] if attributes[:'failedEsxiHostId']

      raise 'You cannot provide both :failedEsxiHostId and :failed_esxi_host_id' if attributes.key?(:'failedEsxiHostId') && attributes.key?(:'failed_esxi_host_id')

      self.failed_esxi_host_id = attributes[:'failed_esxi_host_id'] if attributes[:'failed_esxi_host_id']

      self.replacement_esxi_host_id = attributes[:'replacementEsxiHostId'] if attributes[:'replacementEsxiHostId']

      raise 'You cannot provide both :replacementEsxiHostId and :replacement_esxi_host_id' if attributes.key?(:'replacementEsxiHostId') && attributes.key?(:'replacement_esxi_host_id')

      self.replacement_esxi_host_id = attributes[:'replacement_esxi_host_id'] if attributes[:'replacement_esxi_host_id']

      self.grace_period_end_date = attributes[:'gracePeriodEndDate'] if attributes[:'gracePeriodEndDate']

      raise 'You cannot provide both :gracePeriodEndDate and :grace_period_end_date' if attributes.key?(:'gracePeriodEndDate') && attributes.key?(:'grace_period_end_date')

      self.grace_period_end_date = attributes[:'grace_period_end_date'] if attributes[:'grace_period_end_date']

      self.vmware_software_version = attributes[:'vmwareSoftwareVersion'] if attributes[:'vmwareSoftwareVersion']

      raise 'You cannot provide both :vmwareSoftwareVersion and :vmware_software_version' if attributes.key?(:'vmwareSoftwareVersion') && attributes.key?(:'vmware_software_version')

      self.vmware_software_version = attributes[:'vmware_software_version'] if attributes[:'vmware_software_version']

      self.non_upgraded_esxi_host_id = attributes[:'nonUpgradedEsxiHostId'] if attributes[:'nonUpgradedEsxiHostId']

      raise 'You cannot provide both :nonUpgradedEsxiHostId and :non_upgraded_esxi_host_id' if attributes.key?(:'nonUpgradedEsxiHostId') && attributes.key?(:'non_upgraded_esxi_host_id')

      self.non_upgraded_esxi_host_id = attributes[:'non_upgraded_esxi_host_id'] if attributes[:'non_upgraded_esxi_host_id']

      self.upgraded_replacement_esxi_host_id = attributes[:'upgradedReplacementEsxiHostId'] if attributes[:'upgradedReplacementEsxiHostId']

      raise 'You cannot provide both :upgradedReplacementEsxiHostId and :upgraded_replacement_esxi_host_id' if attributes.key?(:'upgradedReplacementEsxiHostId') && attributes.key?(:'upgraded_replacement_esxi_host_id')

      self.upgraded_replacement_esxi_host_id = attributes[:'upgraded_replacement_esxi_host_id'] if attributes[:'upgraded_replacement_esxi_host_id']

      self.compute_availability_domain = attributes[:'computeAvailabilityDomain'] if attributes[:'computeAvailabilityDomain']

      raise 'You cannot provide both :computeAvailabilityDomain and :compute_availability_domain' if attributes.key?(:'computeAvailabilityDomain') && attributes.key?(:'compute_availability_domain')

      self.compute_availability_domain = attributes[:'compute_availability_domain'] if attributes[:'compute_availability_domain']

      self.host_shape_name = attributes[:'hostShapeName'] if attributes[:'hostShapeName']

      raise 'You cannot provide both :hostShapeName and :host_shape_name' if attributes.key?(:'hostShapeName') && attributes.key?(:'host_shape_name')

      self.host_shape_name = attributes[:'host_shape_name'] if attributes[:'host_shape_name']

      self.host_ocpu_count = attributes[:'hostOcpuCount'] if attributes[:'hostOcpuCount']

      raise 'You cannot provide both :hostOcpuCount and :host_ocpu_count' if attributes.key?(:'hostOcpuCount') && attributes.key?(:'host_ocpu_count')

      self.host_ocpu_count = attributes[:'host_ocpu_count'] if attributes[:'host_ocpu_count']

      self.billing_donor_host_id = attributes[:'billingDonorHostId'] if attributes[:'billingDonorHostId']

      raise 'You cannot provide both :billingDonorHostId and :billing_donor_host_id' if attributes.key?(:'billingDonorHostId') && attributes.key?(:'billing_donor_host_id')

      self.billing_donor_host_id = attributes[:'billing_donor_host_id'] if attributes[:'billing_donor_host_id']

      self.swap_billing_host_id = attributes[:'swapBillingHostId'] if attributes[:'swapBillingHostId']

      raise 'You cannot provide both :swapBillingHostId and :swap_billing_host_id' if attributes.key?(:'swapBillingHostId') && attributes.key?(:'swap_billing_host_id')

      self.swap_billing_host_id = attributes[:'swap_billing_host_id'] if attributes[:'swap_billing_host_id']

      self.is_billing_continuation_in_progress = attributes[:'isBillingContinuationInProgress'] unless attributes[:'isBillingContinuationInProgress'].nil?
      self.is_billing_continuation_in_progress = false if is_billing_continuation_in_progress.nil? && !attributes.key?(:'isBillingContinuationInProgress') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :isBillingContinuationInProgress and :is_billing_continuation_in_progress' if attributes.key?(:'isBillingContinuationInProgress') && attributes.key?(:'is_billing_continuation_in_progress')

      self.is_billing_continuation_in_progress = attributes[:'is_billing_continuation_in_progress'] unless attributes[:'is_billing_continuation_in_progress'].nil?
      self.is_billing_continuation_in_progress = false if is_billing_continuation_in_progress.nil? && !attributes.key?(:'isBillingContinuationInProgress') && !attributes.key?(:'is_billing_continuation_in_progress') # rubocop:disable Style/StringLiterals

      self.is_billing_swapping_in_progress = attributes[:'isBillingSwappingInProgress'] unless attributes[:'isBillingSwappingInProgress'].nil?
      self.is_billing_swapping_in_progress = false if is_billing_swapping_in_progress.nil? && !attributes.key?(:'isBillingSwappingInProgress') # rubocop:disable Style/StringLiterals

      raise 'You cannot provide both :isBillingSwappingInProgress and :is_billing_swapping_in_progress' if attributes.key?(:'isBillingSwappingInProgress') && attributes.key?(:'is_billing_swapping_in_progress')

      self.is_billing_swapping_in_progress = attributes[:'is_billing_swapping_in_progress'] unless attributes[:'is_billing_swapping_in_progress'].nil?
      self.is_billing_swapping_in_progress = false if is_billing_swapping_in_progress.nil? && !attributes.key?(:'isBillingSwappingInProgress') && !attributes.key?(:'is_billing_swapping_in_progress') # rubocop:disable Style/StringLiterals

      self.freeform_tags = attributes[:'freeformTags'] if attributes[:'freeformTags']

      raise 'You cannot provide both :freeformTags and :freeform_tags' if attributes.key?(:'freeformTags') && attributes.key?(:'freeform_tags')

      self.freeform_tags = attributes[:'freeform_tags'] if attributes[:'freeform_tags']

      self.defined_tags = attributes[:'definedTags'] if attributes[:'definedTags']

      raise 'You cannot provide both :definedTags and :defined_tags' if attributes.key?(:'definedTags') && attributes.key?(:'defined_tags')

      self.defined_tags = attributes[:'defined_tags'] if attributes[:'defined_tags']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] lifecycle_state Object to be assigned
    def lifecycle_state=(lifecycle_state)
      # rubocop:disable Style/ConditionalAssignment
      if lifecycle_state && !LIFECYCLE_STATE_ENUM.include?(lifecycle_state)
        OCI.logger.debug("Unknown value for 'lifecycle_state' [" + lifecycle_state + "]. Mapping to 'LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @lifecycle_state = LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE
      else
        @lifecycle_state = lifecycle_state
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] current_commitment Object to be assigned
    def current_commitment=(current_commitment)
      # rubocop:disable Style/ConditionalAssignment
      if current_commitment && !CURRENT_COMMITMENT_ENUM.include?(current_commitment)
        OCI.logger.debug("Unknown value for 'current_commitment' [" + current_commitment + "]. Mapping to 'CURRENT_COMMITMENT_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @current_commitment = CURRENT_COMMITMENT_UNKNOWN_ENUM_VALUE
      else
        @current_commitment = current_commitment
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] next_commitment Object to be assigned
    def next_commitment=(next_commitment)
      # rubocop:disable Style/ConditionalAssignment
      if next_commitment && !NEXT_COMMITMENT_ENUM.include?(next_commitment)
        OCI.logger.debug("Unknown value for 'next_commitment' [" + next_commitment + "]. Mapping to 'NEXT_COMMITMENT_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @next_commitment = NEXT_COMMITMENT_UNKNOWN_ENUM_VALUE
      else
        @next_commitment = next_commitment
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        id == other.id &&
        display_name == other.display_name &&
        sddc_id == other.sddc_id &&
        cluster_id == other.cluster_id &&
        compartment_id == other.compartment_id &&
        compute_instance_id == other.compute_instance_id &&
        time_created == other.time_created &&
        time_updated == other.time_updated &&
        lifecycle_state == other.lifecycle_state &&
        current_commitment == other.current_commitment &&
        next_commitment == other.next_commitment &&
        billing_contract_end_date == other.billing_contract_end_date &&
        failed_esxi_host_id == other.failed_esxi_host_id &&
        replacement_esxi_host_id == other.replacement_esxi_host_id &&
        grace_period_end_date == other.grace_period_end_date &&
        vmware_software_version == other.vmware_software_version &&
        non_upgraded_esxi_host_id == other.non_upgraded_esxi_host_id &&
        upgraded_replacement_esxi_host_id == other.upgraded_replacement_esxi_host_id &&
        compute_availability_domain == other.compute_availability_domain &&
        host_shape_name == other.host_shape_name &&
        host_ocpu_count == other.host_ocpu_count &&
        billing_donor_host_id == other.billing_donor_host_id &&
        swap_billing_host_id == other.swap_billing_host_id &&
        is_billing_continuation_in_progress == other.is_billing_continuation_in_progress &&
        is_billing_swapping_in_progress == other.is_billing_swapping_in_progress &&
        freeform_tags == other.freeform_tags &&
        defined_tags == other.defined_tags
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [id, display_name, sddc_id, cluster_id, compartment_id, compute_instance_id, time_created, time_updated, lifecycle_state, current_commitment, next_commitment, billing_contract_end_date, failed_esxi_host_id, replacement_esxi_host_id, grace_period_end_date, vmware_software_version, non_upgraded_esxi_host_id, upgraded_replacement_esxi_host_id, compute_availability_domain, host_shape_name, host_ocpu_count, billing_donor_host_id, swap_billing_host_id, is_billing_continuation_in_progress, is_billing_swapping_in_progress, freeform_tags, defined_tags].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
