# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20190506
require 'date'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Properties that are required to create an ODA Private Endpoint Scan Proxy.
  class Oda::Models::CreateOdaPrivateEndpointScanProxyDetails
    # **[Required]** Type indicating whether Scan listener is specified by its FQDN or list of IPs
    #
    # @return [String]
    attr_accessor :scan_listener_type

    # **[Required]** The protocol used for communication between client, scanProxy and RAC's scan listeners
    #
    # @return [String]
    attr_accessor :protocol

    # **[Required]** The FQDN/IPs and port information of customer's Real Application Cluster (RAC)'s SCAN listeners.
    #
    # @return [Array<OCI::Oda::Models::ScanListenerInfo>]
    attr_accessor :scan_listener_infos

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'scan_listener_type': :'scanListenerType',
        'protocol': :'protocol',
        'scan_listener_infos': :'scanListenerInfos'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'scan_listener_type': :'String',
        'protocol': :'String',
        'scan_listener_infos': :'Array<OCI::Oda::Models::ScanListenerInfo>'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :scan_listener_type The value to assign to the {#scan_listener_type} property
    # @option attributes [String] :protocol The value to assign to the {#protocol} property
    # @option attributes [Array<OCI::Oda::Models::ScanListenerInfo>] :scan_listener_infos The value to assign to the {#scan_listener_infos} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.scan_listener_type = attributes[:'scanListenerType'] if attributes[:'scanListenerType']

      raise 'You cannot provide both :scanListenerType and :scan_listener_type' if attributes.key?(:'scanListenerType') && attributes.key?(:'scan_listener_type')

      self.scan_listener_type = attributes[:'scan_listener_type'] if attributes[:'scan_listener_type']

      self.protocol = attributes[:'protocol'] if attributes[:'protocol']

      self.scan_listener_infos = attributes[:'scanListenerInfos'] if attributes[:'scanListenerInfos']

      raise 'You cannot provide both :scanListenerInfos and :scan_listener_infos' if attributes.key?(:'scanListenerInfos') && attributes.key?(:'scan_listener_infos')

      self.scan_listener_infos = attributes[:'scan_listener_infos'] if attributes[:'scan_listener_infos']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        scan_listener_type == other.scan_listener_type &&
        protocol == other.protocol &&
        scan_listener_infos == other.scan_listener_infos
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [scan_listener_type, protocol, scan_listener_infos].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
