# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20210621
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # Description of OpaInstance.
  class Opa::Models::OpaInstance
    CONSUMPTION_MODEL_ENUM = [
      CONSUMPTION_MODEL_UCM = 'UCM'.freeze,
      CONSUMPTION_MODEL_GOV = 'GOV'.freeze,
      CONSUMPTION_MODEL_SAAS = 'SAAS'.freeze,
      CONSUMPTION_MODEL_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    SHAPE_NAME_ENUM = [
      SHAPE_NAME_DEVELOPMENT = 'DEVELOPMENT'.freeze,
      SHAPE_NAME_PRODUCTION = 'PRODUCTION'.freeze,
      SHAPE_NAME_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    METERING_TYPE_ENUM = [
      METERING_TYPE_EXECUTION_PACK = 'EXECUTION_PACK'.freeze,
      METERING_TYPE_USERS = 'USERS'.freeze,
      METERING_TYPE_EMPLOYEE = 'EMPLOYEE'.freeze,
      METERING_TYPE_NAMED_USER = 'NAMED_USER'.freeze,
      METERING_TYPE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    LIFECYCLE_STATE_ENUM = [
      LIFECYCLE_STATE_CREATING = 'CREATING'.freeze,
      LIFECYCLE_STATE_UPDATING = 'UPDATING'.freeze,
      LIFECYCLE_STATE_ACTIVE = 'ACTIVE'.freeze,
      LIFECYCLE_STATE_DELETING = 'DELETING'.freeze,
      LIFECYCLE_STATE_DELETED = 'DELETED'.freeze,
      LIFECYCLE_STATE_FAILED = 'FAILED'.freeze,
      LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** Unique identifier that is immutable on creation
    # @return [String]
    attr_accessor :id

    # **[Required]** OpaInstance Identifier, can be renamed
    # @return [String]
    attr_accessor :display_name

    # Description of the Process Automation instance.
    # @return [String]
    attr_accessor :description

    # **[Required]** Compartment Identifier
    # @return [String]
    attr_accessor :compartment_id

    # OPA Instance URL
    # @return [String]
    attr_accessor :instance_url

    # The entitlement used for billing purposes
    # @return [String]
    attr_reader :consumption_model

    # **[Required]** Shape of the instance.
    # @return [String]
    attr_reader :shape_name

    # MeteringType Identifier
    # @return [String]
    attr_reader :metering_type

    # **[Required]** The time when OpaInstance was created. An RFC3339 formatted datetime string
    # @return [DateTime]
    attr_accessor :time_created

    # The time the OpaInstance was updated. An RFC3339 formatted datetime string
    # @return [DateTime]
    attr_accessor :time_updated

    # **[Required]** The current state of the OpaInstance.
    # @return [String]
    attr_reader :lifecycle_state

    # This property specifies the GUID of the Identity Application instance OPA has created inside the user-specified identity domain. This identity application instance may be used to host user role mappings to grant access to this OPA instance for users within the identity domain.
    # @return [String]
    attr_accessor :identity_app_guid

    # This property specifies the name of the Identity Application instance OPA has created inside the user-specified identity domain. This identity application instance may be used to host user roll mappings to grant access to this OPA instance for users within the identity domain.
    # @return [String]
    attr_accessor :identity_app_display_name

    # This property specifies the domain url of the Identity Application instance OPA has created inside the user-specified identity domain. This identity application instance may be used to host user roll mappings to grant access to this OPA instance for users within the identity domain.
    # @return [String]
    attr_accessor :identity_domain_url

    # This property specifies the OPC Service Instance GUID of the Identity Application instance OPA has created inside the user-specified identity domain. This identity application instance may be used to host user roll mappings to grant access to this OPA instance for users within the identity domain.
    # @return [String]
    attr_accessor :identity_app_opc_service_instance_guid

    # indicates if breakGlass is enabled for the opa instance.
    # @return [BOOLEAN]
    attr_accessor :is_breakglass_enabled

    # Simple key-value pair that is applied without any predefined name, type or scope. Exists for cross-compatibility only.
    # Example: `{\"bar-key\": \"value\"}`
    #
    # @return [Hash<String, String>]
    attr_accessor :freeform_tags

    # Defined tags for this resource. Each key is predefined and scoped to a namespace.
    # Example: `{\"foo-namespace\": {\"bar-key\": \"value\"}}`
    #
    # @return [Hash<String, Hash<String, Object>>]
    attr_accessor :defined_tags

    # Usage of system tag keys. These predefined keys are scoped to namespaces.
    # Example: `{\"orcl-cloud\": {\"free-tier-retained\": \"true\"}}`
    #
    # @return [Hash<String, Hash<String, Object>>]
    attr_accessor :system_tags

    # A list of associated attachments to other services
    #
    # @return [Array<OCI::Opa::Models::AttachmentDetails>]
    attr_accessor :attachments

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'id',
        'display_name': :'displayName',
        'description': :'description',
        'compartment_id': :'compartmentId',
        'instance_url': :'instanceUrl',
        'consumption_model': :'consumptionModel',
        'shape_name': :'shapeName',
        'metering_type': :'meteringType',
        'time_created': :'timeCreated',
        'time_updated': :'timeUpdated',
        'lifecycle_state': :'lifecycleState',
        'identity_app_guid': :'identityAppGuid',
        'identity_app_display_name': :'identityAppDisplayName',
        'identity_domain_url': :'identityDomainUrl',
        'identity_app_opc_service_instance_guid': :'identityAppOpcServiceInstanceGuid',
        'is_breakglass_enabled': :'isBreakglassEnabled',
        'freeform_tags': :'freeformTags',
        'defined_tags': :'definedTags',
        'system_tags': :'systemTags',
        'attachments': :'attachments'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'id': :'String',
        'display_name': :'String',
        'description': :'String',
        'compartment_id': :'String',
        'instance_url': :'String',
        'consumption_model': :'String',
        'shape_name': :'String',
        'metering_type': :'String',
        'time_created': :'DateTime',
        'time_updated': :'DateTime',
        'lifecycle_state': :'String',
        'identity_app_guid': :'String',
        'identity_app_display_name': :'String',
        'identity_domain_url': :'String',
        'identity_app_opc_service_instance_guid': :'String',
        'is_breakglass_enabled': :'BOOLEAN',
        'freeform_tags': :'Hash<String, String>',
        'defined_tags': :'Hash<String, Hash<String, Object>>',
        'system_tags': :'Hash<String, Hash<String, Object>>',
        'attachments': :'Array<OCI::Opa::Models::AttachmentDetails>'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :id The value to assign to the {#id} property
    # @option attributes [String] :display_name The value to assign to the {#display_name} property
    # @option attributes [String] :description The value to assign to the {#description} property
    # @option attributes [String] :compartment_id The value to assign to the {#compartment_id} property
    # @option attributes [String] :instance_url The value to assign to the {#instance_url} property
    # @option attributes [String] :consumption_model The value to assign to the {#consumption_model} property
    # @option attributes [String] :shape_name The value to assign to the {#shape_name} property
    # @option attributes [String] :metering_type The value to assign to the {#metering_type} property
    # @option attributes [DateTime] :time_created The value to assign to the {#time_created} property
    # @option attributes [DateTime] :time_updated The value to assign to the {#time_updated} property
    # @option attributes [String] :lifecycle_state The value to assign to the {#lifecycle_state} property
    # @option attributes [String] :identity_app_guid The value to assign to the {#identity_app_guid} property
    # @option attributes [String] :identity_app_display_name The value to assign to the {#identity_app_display_name} property
    # @option attributes [String] :identity_domain_url The value to assign to the {#identity_domain_url} property
    # @option attributes [String] :identity_app_opc_service_instance_guid The value to assign to the {#identity_app_opc_service_instance_guid} property
    # @option attributes [BOOLEAN] :is_breakglass_enabled The value to assign to the {#is_breakglass_enabled} property
    # @option attributes [Hash<String, String>] :freeform_tags The value to assign to the {#freeform_tags} property
    # @option attributes [Hash<String, Hash<String, Object>>] :defined_tags The value to assign to the {#defined_tags} property
    # @option attributes [Hash<String, Hash<String, Object>>] :system_tags The value to assign to the {#system_tags} property
    # @option attributes [Array<OCI::Opa::Models::AttachmentDetails>] :attachments The value to assign to the {#attachments} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.id = attributes[:'id'] if attributes[:'id']

      self.display_name = attributes[:'displayName'] if attributes[:'displayName']

      raise 'You cannot provide both :displayName and :display_name' if attributes.key?(:'displayName') && attributes.key?(:'display_name')

      self.display_name = attributes[:'display_name'] if attributes[:'display_name']

      self.description = attributes[:'description'] if attributes[:'description']

      self.compartment_id = attributes[:'compartmentId'] if attributes[:'compartmentId']

      raise 'You cannot provide both :compartmentId and :compartment_id' if attributes.key?(:'compartmentId') && attributes.key?(:'compartment_id')

      self.compartment_id = attributes[:'compartment_id'] if attributes[:'compartment_id']

      self.instance_url = attributes[:'instanceUrl'] if attributes[:'instanceUrl']

      raise 'You cannot provide both :instanceUrl and :instance_url' if attributes.key?(:'instanceUrl') && attributes.key?(:'instance_url')

      self.instance_url = attributes[:'instance_url'] if attributes[:'instance_url']

      self.consumption_model = attributes[:'consumptionModel'] if attributes[:'consumptionModel']

      raise 'You cannot provide both :consumptionModel and :consumption_model' if attributes.key?(:'consumptionModel') && attributes.key?(:'consumption_model')

      self.consumption_model = attributes[:'consumption_model'] if attributes[:'consumption_model']

      self.shape_name = attributes[:'shapeName'] if attributes[:'shapeName']

      raise 'You cannot provide both :shapeName and :shape_name' if attributes.key?(:'shapeName') && attributes.key?(:'shape_name')

      self.shape_name = attributes[:'shape_name'] if attributes[:'shape_name']

      self.metering_type = attributes[:'meteringType'] if attributes[:'meteringType']

      raise 'You cannot provide both :meteringType and :metering_type' if attributes.key?(:'meteringType') && attributes.key?(:'metering_type')

      self.metering_type = attributes[:'metering_type'] if attributes[:'metering_type']

      self.time_created = attributes[:'timeCreated'] if attributes[:'timeCreated']

      raise 'You cannot provide both :timeCreated and :time_created' if attributes.key?(:'timeCreated') && attributes.key?(:'time_created')

      self.time_created = attributes[:'time_created'] if attributes[:'time_created']

      self.time_updated = attributes[:'timeUpdated'] if attributes[:'timeUpdated']

      raise 'You cannot provide both :timeUpdated and :time_updated' if attributes.key?(:'timeUpdated') && attributes.key?(:'time_updated')

      self.time_updated = attributes[:'time_updated'] if attributes[:'time_updated']

      self.lifecycle_state = attributes[:'lifecycleState'] if attributes[:'lifecycleState']

      raise 'You cannot provide both :lifecycleState and :lifecycle_state' if attributes.key?(:'lifecycleState') && attributes.key?(:'lifecycle_state')

      self.lifecycle_state = attributes[:'lifecycle_state'] if attributes[:'lifecycle_state']

      self.identity_app_guid = attributes[:'identityAppGuid'] if attributes[:'identityAppGuid']

      raise 'You cannot provide both :identityAppGuid and :identity_app_guid' if attributes.key?(:'identityAppGuid') && attributes.key?(:'identity_app_guid')

      self.identity_app_guid = attributes[:'identity_app_guid'] if attributes[:'identity_app_guid']

      self.identity_app_display_name = attributes[:'identityAppDisplayName'] if attributes[:'identityAppDisplayName']

      raise 'You cannot provide both :identityAppDisplayName and :identity_app_display_name' if attributes.key?(:'identityAppDisplayName') && attributes.key?(:'identity_app_display_name')

      self.identity_app_display_name = attributes[:'identity_app_display_name'] if attributes[:'identity_app_display_name']

      self.identity_domain_url = attributes[:'identityDomainUrl'] if attributes[:'identityDomainUrl']

      raise 'You cannot provide both :identityDomainUrl and :identity_domain_url' if attributes.key?(:'identityDomainUrl') && attributes.key?(:'identity_domain_url')

      self.identity_domain_url = attributes[:'identity_domain_url'] if attributes[:'identity_domain_url']

      self.identity_app_opc_service_instance_guid = attributes[:'identityAppOpcServiceInstanceGuid'] if attributes[:'identityAppOpcServiceInstanceGuid']

      raise 'You cannot provide both :identityAppOpcServiceInstanceGuid and :identity_app_opc_service_instance_guid' if attributes.key?(:'identityAppOpcServiceInstanceGuid') && attributes.key?(:'identity_app_opc_service_instance_guid')

      self.identity_app_opc_service_instance_guid = attributes[:'identity_app_opc_service_instance_guid'] if attributes[:'identity_app_opc_service_instance_guid']

      self.is_breakglass_enabled = attributes[:'isBreakglassEnabled'] unless attributes[:'isBreakglassEnabled'].nil?

      raise 'You cannot provide both :isBreakglassEnabled and :is_breakglass_enabled' if attributes.key?(:'isBreakglassEnabled') && attributes.key?(:'is_breakglass_enabled')

      self.is_breakglass_enabled = attributes[:'is_breakglass_enabled'] unless attributes[:'is_breakglass_enabled'].nil?

      self.freeform_tags = attributes[:'freeformTags'] if attributes[:'freeformTags']

      raise 'You cannot provide both :freeformTags and :freeform_tags' if attributes.key?(:'freeformTags') && attributes.key?(:'freeform_tags')

      self.freeform_tags = attributes[:'freeform_tags'] if attributes[:'freeform_tags']

      self.defined_tags = attributes[:'definedTags'] if attributes[:'definedTags']

      raise 'You cannot provide both :definedTags and :defined_tags' if attributes.key?(:'definedTags') && attributes.key?(:'defined_tags')

      self.defined_tags = attributes[:'defined_tags'] if attributes[:'defined_tags']

      self.system_tags = attributes[:'systemTags'] if attributes[:'systemTags']

      raise 'You cannot provide both :systemTags and :system_tags' if attributes.key?(:'systemTags') && attributes.key?(:'system_tags')

      self.system_tags = attributes[:'system_tags'] if attributes[:'system_tags']

      self.attachments = attributes[:'attachments'] if attributes[:'attachments']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] consumption_model Object to be assigned
    def consumption_model=(consumption_model)
      # rubocop:disable Style/ConditionalAssignment
      if consumption_model && !CONSUMPTION_MODEL_ENUM.include?(consumption_model)
        OCI.logger.debug("Unknown value for 'consumption_model' [" + consumption_model + "]. Mapping to 'CONSUMPTION_MODEL_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @consumption_model = CONSUMPTION_MODEL_UNKNOWN_ENUM_VALUE
      else
        @consumption_model = consumption_model
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] shape_name Object to be assigned
    def shape_name=(shape_name)
      # rubocop:disable Style/ConditionalAssignment
      if shape_name && !SHAPE_NAME_ENUM.include?(shape_name)
        OCI.logger.debug("Unknown value for 'shape_name' [" + shape_name + "]. Mapping to 'SHAPE_NAME_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @shape_name = SHAPE_NAME_UNKNOWN_ENUM_VALUE
      else
        @shape_name = shape_name
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] metering_type Object to be assigned
    def metering_type=(metering_type)
      # rubocop:disable Style/ConditionalAssignment
      if metering_type && !METERING_TYPE_ENUM.include?(metering_type)
        OCI.logger.debug("Unknown value for 'metering_type' [" + metering_type + "]. Mapping to 'METERING_TYPE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @metering_type = METERING_TYPE_UNKNOWN_ENUM_VALUE
      else
        @metering_type = metering_type
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] lifecycle_state Object to be assigned
    def lifecycle_state=(lifecycle_state)
      # rubocop:disable Style/ConditionalAssignment
      if lifecycle_state && !LIFECYCLE_STATE_ENUM.include?(lifecycle_state)
        OCI.logger.debug("Unknown value for 'lifecycle_state' [" + lifecycle_state + "]. Mapping to 'LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @lifecycle_state = LIFECYCLE_STATE_UNKNOWN_ENUM_VALUE
      else
        @lifecycle_state = lifecycle_state
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        id == other.id &&
        display_name == other.display_name &&
        description == other.description &&
        compartment_id == other.compartment_id &&
        instance_url == other.instance_url &&
        consumption_model == other.consumption_model &&
        shape_name == other.shape_name &&
        metering_type == other.metering_type &&
        time_created == other.time_created &&
        time_updated == other.time_updated &&
        lifecycle_state == other.lifecycle_state &&
        identity_app_guid == other.identity_app_guid &&
        identity_app_display_name == other.identity_app_display_name &&
        identity_domain_url == other.identity_domain_url &&
        identity_app_opc_service_instance_guid == other.identity_app_opc_service_instance_guid &&
        is_breakglass_enabled == other.is_breakglass_enabled &&
        freeform_tags == other.freeform_tags &&
        defined_tags == other.defined_tags &&
        system_tags == other.system_tags &&
        attachments == other.attachments
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [id, display_name, description, compartment_id, instance_url, consumption_model, shape_name, metering_type, time_created, time_updated, lifecycle_state, identity_app_guid, identity_app_display_name, identity_domain_url, identity_app_opc_service_instance_guid, is_breakglass_enabled, freeform_tags, defined_tags, system_tags, attachments].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
