# Copyright (c) 2016, 2023, Oracle and/or its affiliates.  All rights reserved.
# This software is dual-licensed to you under the Universal Permissive License (UPL) 1.0 as shown at https://oss.oracle.com/licenses/upl or Apache License 2.0 as shown at http://www.apache.org/licenses/LICENSE-2.0. You may choose either license.

# NOTE: This class is auto generated by OracleSDKGenerator. DO NOT EDIT. API Version: 20220901
require 'date'
require 'logger'

# rubocop:disable Lint/UnneededCopDisableDirective, Metrics/LineLength
module OCI
  # An erratum associated with a managed instance.
  class OsManagementHub::Models::ManagedInstanceErratumSummary
    ADVISORY_TYPE_ENUM = [
      ADVISORY_TYPE_SECURITY = 'SECURITY'.freeze,
      ADVISORY_TYPE_BUGFIX = 'BUGFIX'.freeze,
      ADVISORY_TYPE_ENHANCEMENT = 'ENHANCEMENT'.freeze,
      ADVISORY_TYPE_OTHER = 'OTHER'.freeze,
      ADVISORY_TYPE_UNKNOWN_ENUM_VALUE = 'UNKNOWN_ENUM_VALUE'.freeze
    ].freeze

    # **[Required]** The identifier of the erratum.
    # @return [String]
    attr_accessor :name

    # **[Required]** The type of the erratum.
    # @return [String]
    attr_reader :advisory_type

    # The date and time the package was issued by a providing erratum (if available), as described in
    # [RFC 3339](https://tools.ietf.org/rfc/rfc3339), section 14.29.
    #
    # @return [DateTime]
    attr_accessor :time_issued

    # Summary description of the erratum.
    # @return [String]
    attr_accessor :synopsis

    # List of CVEs applicable to this erratum.
    # @return [Array<String>]
    attr_accessor :related_cves

    # **[Required]** The list of Packages affected by this erratum.
    # @return [Array<OCI::OsManagementHub::Models::PackageNameSummary>]
    attr_accessor :packages

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        # rubocop:disable Style/SymbolLiteral
        'name': :'name',
        'advisory_type': :'advisoryType',
        'time_issued': :'timeIssued',
        'synopsis': :'synopsis',
        'related_cves': :'relatedCves',
        'packages': :'packages'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        # rubocop:disable Style/SymbolLiteral
        'name': :'String',
        'advisory_type': :'String',
        'time_issued': :'DateTime',
        'synopsis': :'String',
        'related_cves': :'Array<String>',
        'packages': :'Array<OCI::OsManagementHub::Models::PackageNameSummary>'
        # rubocop:enable Style/SymbolLiteral
      }
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:disable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral


    # Initializes the object
    # @param [Hash] attributes Model attributes in the form of hash
    # @option attributes [String] :name The value to assign to the {#name} property
    # @option attributes [String] :advisory_type The value to assign to the {#advisory_type} property
    # @option attributes [DateTime] :time_issued The value to assign to the {#time_issued} property
    # @option attributes [String] :synopsis The value to assign to the {#synopsis} property
    # @option attributes [Array<String>] :related_cves The value to assign to the {#related_cves} property
    # @option attributes [Array<OCI::OsManagementHub::Models::PackageNameSummary>] :packages The value to assign to the {#packages} property
    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.each_with_object({}) { |(k, v), h| h[k.to_sym] = v }

      self.name = attributes[:'name'] if attributes[:'name']

      self.advisory_type = attributes[:'advisoryType'] if attributes[:'advisoryType']

      raise 'You cannot provide both :advisoryType and :advisory_type' if attributes.key?(:'advisoryType') && attributes.key?(:'advisory_type')

      self.advisory_type = attributes[:'advisory_type'] if attributes[:'advisory_type']

      self.time_issued = attributes[:'timeIssued'] if attributes[:'timeIssued']

      raise 'You cannot provide both :timeIssued and :time_issued' if attributes.key?(:'timeIssued') && attributes.key?(:'time_issued')

      self.time_issued = attributes[:'time_issued'] if attributes[:'time_issued']

      self.synopsis = attributes[:'synopsis'] if attributes[:'synopsis']

      self.related_cves = attributes[:'relatedCves'] if attributes[:'relatedCves']

      raise 'You cannot provide both :relatedCves and :related_cves' if attributes.key?(:'relatedCves') && attributes.key?(:'related_cves')

      self.related_cves = attributes[:'related_cves'] if attributes[:'related_cves']

      self.packages = attributes[:'packages'] if attributes[:'packages']
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity
    # rubocop:enable Metrics/MethodLength, Layout/EmptyLines, Style/SymbolLiteral

    # Custom attribute writer method checking allowed values (enum).
    # @param [Object] advisory_type Object to be assigned
    def advisory_type=(advisory_type)
      # rubocop:disable Style/ConditionalAssignment
      if advisory_type && !ADVISORY_TYPE_ENUM.include?(advisory_type)
        OCI.logger.debug("Unknown value for 'advisory_type' [" + advisory_type + "]. Mapping to 'ADVISORY_TYPE_UNKNOWN_ENUM_VALUE'") if OCI.logger
        @advisory_type = ADVISORY_TYPE_UNKNOWN_ENUM_VALUE
      else
        @advisory_type = advisory_type
      end
      # rubocop:enable Style/ConditionalAssignment
    end

    # rubocop:disable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines


    # Checks equality by comparing each attribute.
    # @param [Object] other the other object to be compared
    def ==(other)
      return true if equal?(other)

      self.class == other.class &&
        name == other.name &&
        advisory_type == other.advisory_type &&
        time_issued == other.time_issued &&
        synopsis == other.synopsis &&
        related_cves == other.related_cves &&
        packages == other.packages
    end
    # rubocop:enable Metrics/CyclomaticComplexity, Metrics/AbcSize, Metrics/PerceivedComplexity, Layout/EmptyLines

    # @see the `==` method
    # @param [Object] other the other object to be compared
    def eql?(other)
      self == other
    end

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Calculates hash code according to all attributes.
    # @return [Fixnum] Hash code
    def hash
      [name, advisory_type, time_issued, synopsis, related_cves, packages].hash
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # rubocop:disable Metrics/AbcSize, Layout/EmptyLines


    # Builds the object from hash
    # @param [Hash] attributes Model attributes in the form of hash
    # @return [Object] Returns the model itself
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)

      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          # check to ensure the input is an array given that the the attribute
          # is documented as an array but the input is not
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            public_method("#{key}=").call(
              attributes[self.class.attribute_map[key]]
                .map { |v| OCI::Internal::Util.convert_to_type(Regexp.last_match(1), v) }
            )
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          public_method("#{key}=").call(
            OCI::Internal::Util.convert_to_type(type, attributes[self.class.attribute_map[key]])
          )
        end
        # or else data not found in attributes(hash), not an issue as the data can be optional
      end

      self
    end
    # rubocop:enable Metrics/AbcSize, Layout/EmptyLines

    # Returns the string representation of the object
    # @return [String] String presentation of the object
    def to_s
      to_hash.to_s
    end

    # Returns the object in the form of hash
    # @return [Hash] Returns the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = public_method(attr).call
        next if value.nil? && !instance_variable_defined?("@#{attr}")

        hash[param] = _to_hash(value)
      end
      hash
    end

    private

    # Outputs non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    # @param [Object] value Any valid value
    # @return [Hash] Returns the value in the form of hash
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map { |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end
  end
end
# rubocop:enable Lint/UnneededCopDisableDirective, Metrics/LineLength
